/*----------------------------------------------------------------------------

NAME
	WintabUtils.cpp

PURPOSE
	Some general-purpose functions for accessing Wintab.

COPYRIGHT
	Copyright (c) Wacom Technology Corp., 2012-2014 All Rights Reserved
	All rights reserved.

---------------------------------------------------------------------------- */
#include "WintabUtils.h"

//////////////////////////////////////////////////////////////////////////////
HINSTANCE ghWintab = NULL;

WTINFOA gpWTInfoA = NULL;
WTOPENA gpWTOpenA = NULL;
WTGETA gpWTGetA = NULL;
WTSETA gpWTSetA = NULL;
WTCLOSE gpWTClose = NULL;
WTPACKET gpWTPacket = NULL;
WTENABLE gpWTEnable = NULL;
WTOVERLAP gpWTOverlap = NULL;
WTSAVE gpWTSave = NULL;
WTCONFIG gpWTConfig = NULL;
WTRESTORE gpWTRestore = NULL;
WTEXTSET gpWTExtSet = NULL;
WTEXTGET gpWTExtGet = NULL;
WTQUEUESIZEGET gpWTQueueSizeGet = NULL;
WTQUEUESIZESET gpWTQueueSizeSet = NULL;
WTDATAPEEK gpWTDataPeek = NULL;
WTPACKETSPEEK gpWTPacketsPeek = NULL;
WTPACKETSGET gpWTPacketsGet = NULL;
WTMGROPEN gpWTMgrOpen = NULL;
WTMGRCLOSE gpWTMgrClose = NULL;
WTMGRDEFCONTEXT gpWTMgrDefContext = NULL;
WTMGRDEFCONTEXTEX gpWTMgrDefContextEx = NULL;

// TODO - add more wintab32 function pointers as needed

char *pszProgramName = NULL;

#define GETPROCADDRESS(type, func) \
	gp##func = (type)GetProcAddress(ghWintab, #func); \
	if (!gp##func) { WACOM_ASSERT(FALSE); UnloadWintab(); return FALSE; }

//////////////////////////////////////////////////////////////////////////////
// Purpose
//		Find wintab32.dll and load it.
//		Find the exported functions we need from it.
//
//	Returns
//		TRUE on success.
//		FALSE on failure.
//
BOOL LoadWintab(void)
{
	WACOM_TRACE("LoadWintab()\n");

	ghWintab = LoadLibraryA( "Wintab32.dll" );
	if (!ghWintab)
	{
		DWORD err = GetLastError();
		WACOM_TRACE("LoadLibrary error: %i\n", err);
		//ShowError("Could not load Wintab32.dll");
		return FALSE;
	}

	// Explicitly find the exported Wintab functions in which we are interested.
	// We are using the ASCII, not unicode versions (where applicable).
	GETPROCADDRESS(WTOPENA, WTOpenA);
	GETPROCADDRESS(WTINFOA, WTInfoA);
	GETPROCADDRESS(WTGETA, WTGetA);
	GETPROCADDRESS(WTSETA, WTSetA);
	GETPROCADDRESS(WTPACKET, WTPacket);
	GETPROCADDRESS(WTCLOSE, WTClose);
	GETPROCADDRESS(WTENABLE, WTEnable);
	GETPROCADDRESS(WTOVERLAP, WTOverlap);
	GETPROCADDRESS(WTSAVE, WTSave);
	GETPROCADDRESS(WTCONFIG, WTConfig);
	GETPROCADDRESS(WTRESTORE, WTRestore);
	GETPROCADDRESS(WTEXTSET, WTExtSet);
	GETPROCADDRESS(WTEXTGET, WTExtGet);
	GETPROCADDRESS(WTQUEUESIZEGET, WTQueueSizeGet);
	GETPROCADDRESS(WTQUEUESIZESET, WTQueueSizeSet);
	GETPROCADDRESS(WTDATAPEEK, WTDataPeek);
	GETPROCADDRESS(WTPACKETSPEEK, WTPacketsPeek);
	GETPROCADDRESS(WTPACKETSGET, WTPacketsGet);
	GETPROCADDRESS(WTMGROPEN, WTMgrOpen);
	GETPROCADDRESS(WTMGRCLOSE, WTMgrClose);
	GETPROCADDRESS(WTMGRDEFCONTEXT, WTMgrDefContext);
	GETPROCADDRESS(WTMGRDEFCONTEXTEX, WTMgrDefContextEx);

	// TODO - don't forget to NULL out pointers in UnloadWintab().
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////////
// Purpose
//		Uninitializes use of wintab32.dll
//
// Returns
//		Nothing.
//
void UnloadWintab(void)
{
	WACOM_TRACE("UnloadWintab()\n");

	if (ghWintab)
	{
		FreeLibrary(ghWintab);
		ghWintab = NULL;
	}

	gpWTInfoA				= NULL;
	gpWTClose				= NULL;
	gpWTOpenA				= NULL;
	gpWTPacket				= NULL;
	gpWTEnable				= NULL;
	gpWTOverlap				= NULL;
	gpWTSave					= NULL;
	gpWTConfig				= NULL;
	gpWTGetA					= NULL;
	gpWTSetA					= NULL;
	gpWTRestore				= NULL;
	gpWTExtSet				= NULL;
	gpWTExtGet				= NULL;
	gpWTQueueSizeSet		= NULL;
	gpWTDataPeek			= NULL;
	gpWTPacketsPeek			= NULL;
	gpWTPacketsGet			= NULL;
	gpWTMgrOpen				= NULL;
	gpWTMgrClose			= NULL;
	gpWTMgrDefContext		= NULL;
	gpWTMgrDefContextEx	= NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Purpose
//		Display error to user.
//
void ShowError(const char *pszErrorMessage)
{
	WACOM_TRACE("ShowError()\n");
	WACOM_ASSERT(pszErrorMessage);
	MessageBoxA(NULL, pszErrorMessage, "Scribble", MB_OK | MB_ICONHAND);
}

//////////////////////////////////////////////////////////////////////////////
// Purpose
//		Display information in the debugger.
//
void WacomTrace(const char *lpszFormat, ...)
{
	char szTraceMessage[128] = {0};

	WACOM_ASSERT(lpszFormat);

	va_list args = NULL;
	va_start(args, lpszFormat);

	int nBytesWritten = _vsnprintf(szTraceMessage, sizeof(szTraceMessage) - 1, lpszFormat, args);
	if (nBytesWritten > 0)
	{
		OutputDebugStringA(szTraceMessage);
	}

	va_end(args);
}
