/* Seamless3d

Copyright (C) 2001-2010 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#define _WIN32_WINNT 0x0410

#include <windows.h>
#include <direct.h>
#include <process.h>

#include "seamless3d.h"
#include "resource.h"
#include "techuelife_triangle_editor.h"
#include "gui.h"
#include "scene_tree.h"

#include"robot_aid.h"

extern const char* menu_convert_all;
extern const char* menu_show_descendants;

extern const char* menu_paste;
extern const char* menu_delete;
extern const char* menu_convert;
extern const char* menu_import;
extern const char* menu_all_nodes_from_file;
extern const char* menu_replace;
extern const char* menu_part;
extern const char* menu_ownership;

static const int seamless = 1;
static const int pelvis = 3;
static const int tummy0 =	4;
static const int tummy1 =	5;
static const int tummy2 =	6;
static const int thorax =	7;
static const int neck0 =	8;
static const int neck1 =	9;
static const int head =		10;

static const int l_collar=	11;
static const int l_shoulder=	12;
static const int l_elbow=		13;
static const int l_wrist=		14;

static const int r_collar=	15;
static const int r_shoulder=	16;
static const int r_elbow=		17;
static const int r_wrist=		18;

static const int l_hip=		19;
static const int l_knee=	20;
static const int l_ankle=	21;

static const int r_hip=		22;
static const int r_knee=	23;
static const int r_ankle=	24;

static const int mirrorGroup =	34;

extern const char* name_pelvis;
extern const char* name_tummy0;
extern const char* name_tummy1;
extern const char* name_tummy2;
extern const char* name_thorax;
extern const char* name_neck0;
extern const char* name_neck1;
extern const char* name_head;

extern const char* name_l_collar;
extern const char* name_l_shoulder;
extern const char* name_l_elbow;
extern const char* name_l_wrist;

extern const char* name_l_hip;
extern const char* name_l_knee;
extern const char* name_l_ankle;
	



void InputEvent::preImmediateAvDemo(){
	g_robotDemo->fileName="poly-model-avatar";
	g_robotDemo->function=immediateAvDemo;
	g_robotDemo->name="poly model avatar";
	robotDemoStart(setupSequence);
}
void InputEvent::immediateAvDemo(){
	InputEvent* eventArray[]={
		new Section("Create a new seamless file"),

		new ClickButton(newSmls),
		new SetFieldValue("filename", "poly-model-avatar.smls"),
		new ToggleField("save"),
		new ClickYes(),
		new ClickButton(openSceneTree),

		//new TypeString(deleteOldFileAndCreateFileName()),
		//new Move_CreateNewFile_Save(),
		//new Click(),


new Section("Add floor & height guide"),
		new ClickButton(floorBuild),
		new ToggleField("zero_height"),

new Section("Add skeleton"),
	//new SelectNode(seamless),
	//new ToggleField("xRay"),
	new SelectNode(pelvis),
	new ClickButton(toggleXRayMode),
	new ClickButton(setXRayEditMode),
	//new AddCtrlPnt(.1,.5),
	new MoveToPivotPoint(pelvis),
	new Move3d(0, 0),


//new AddCtrlPnt(0,.01), //pelvis
new AddCtrlPnt(0,.224), //tummy0
new AddCtrlPnt(0,.279), //tummy1
new AddCtrlPnt(0,.331), //tummy2
new AddCtrlPnt(0,.378), //thorax
new AddCtrlPnt(0,.582), //neck0
new AddCtrlPnt(0,.614), //neck1
new AddCtrlPnt(0,.634), //head
new MoveToPivotPoint(thorax),
new Move3d(0,.378),
new AddCtrlPnt(.03, .521), //l_collar
new AddCtrlPnt(.13-.03,.521), //l_shoulder
new AddCtrlPnt(.5,.151), //l_wrist
new AddCtrlPnt(.315,.326), //l_elbow
new MoveToPivotPoint(pelvis),
new Move3d(0,0),
new AddCtrlPnt(.092,.01), //l_hip,
new AddCtrlPnt(.092,-.79), //l_ankle,
new AddCtrlPnt(.092,-.39), //l_knee,

//new Section("Name parts"),
new NameNode(pelvis, name_pelvis),
new NameNode(tummy0, name_tummy0),
new NameNode(tummy1, name_tummy1),
new NameNode(tummy2, name_tummy2),
new NameNode(thorax, name_thorax),
new NameNode(neck0, name_neck0),
new NameNode(neck1, name_neck1),
new NameNode(head, name_head),
new NameNode(l_collar, name_l_collar),
new NameNode(l_shoulder, name_l_shoulder),
new NameNode(l_elbow, name_l_elbow),
new NameNode(l_wrist, name_l_wrist),
new NameNode(l_hip, name_l_hip),
new NameNode(l_knee, name_l_knee),
new NameNode(l_ankle, "l_ankle\n"),
new ClickButton(toggleXRayMode),
new ClickButton(setXRayEditMode),

new Section("Add Lathe for central parts"),
		new SelectNode(pelvis),
		new ClickButton(buildNurbsLatheForSelPart),
new ToggleField("beginSP"),
new ToggleField("endSP"),
		new IncFieldY("translation",89),

//new ClickControlPoint(0),
//new Move3d(0,0),
new ClickControlPoint(1),
new Move3d(.233465,0),
new ClickControlPoint(2),
new Move3d(.079674,.274803),
new ClickControlPoint(3),
new Move3d(.161372,.538431),
new ClickControlPoint(4),
new Move3d(.011895,.621339),
new ClickControlPoint(5),
new Move3d(.153105,.794933),
new ClickControlPoint(6),
new Move3d(0,.878757),

new DecFieldX("bands",13),
new DecFieldX("staves",10),

/*
new Section("Remove floor & height guide"),

new MoveToNode(84),
new RightClick,
new Move_MenuItem(menu_delete),
new Click,
new SelectNode(pelvis),
*/
new Section("Convert white parts to blue"),
		new MoveToNode(seamless),
		new RightClick,
		new Move_MenuItem("convert all"),
		new Click,

new Section("Make edge vertices bright red"),
		new SelectNode(seamless+1),
		new SetFieldValue("emissive", "1"),

new Section("Drag out arm"),



new DragViewRot(.438401,-.886056,-.150694,1.368827),
new SelectTriangle(222,pelvis),
new ClickButton(splitTriangleImmediate),
new SelectVertex(41, pelvis),
new ClickButton(setFrontViewOrientation),
new IncButtonZ(viewOrientation90),

new TugVertex(41,.429717,.455433,pelvis),

new RectVert(-.352876,.46412, -.287876,.42412),
new TugVertex(42,-.262876,.489119,pelvis),

new RectPoly(-.090625,.498437,-.010625,.403438),

new DragViewRot(.86501,-.496371,.073301,2.089261),
new DragViewRot(.847764,-.527476,-.055357,3.81482),
new DragViewRot(-.349292,.12475,-.928672,5.631466),
new SelectTriangle(396,pelvis),
new ClickButton(splitTriangleImmediate),

//new DragViewRot(-.788669,.571462,-.226788,4.383112),
new DragViewRot(.771427,-.611524,.175894,2.444609),

//new DragViewRot(-.798346,.602141,.008401,2.881469),
new DragViewRot(-.791065,.609894,.047392,2.75551),

//new DragViewRot(.191774,-.403414,.894696,.616423),
new DragViewRot(0,0,1,.785398),
new Section("Drag out arm2"),
new RectVert(.05625,.501562,.12125,.404063),
new TugVertex(70,.361442,.442841,pelvis),

new Section("Drag out leg"),

new ClickButton(setFrontViewOrientation),
//new SelectVertex(5,pelvis),
new RectVert(.078125,.026562,.168125,-.058438),
new TugVertex(5,.037703,-.026145,pelvis),
new DragViewRot(-.724795,-.670823,.157065,.734041),
new SelectTriangle(42,pelvis),
new ClickButton(splitTriangleImmediate),
new SelectVertex(11,pelvis),
new ClickButton(setFrontViewOrientation),
new TugVertex(11,.082055,-1.005364,pelvis),

//////////
new Section("Drag out leg2"),

new RectVert(.115625,.092187,.205625,.009687),
new TugVertex(12,.14509,-.024582,pelvis),
new RectPoly(.028125,-.357812,.150625,-.465312),
new SelectTriangle(426,pelvis),
new ClickButton(splitTriangleImmediate),
new DragViewRot(.035408,-.999368,.003165,.7581),
new DragViewRot(.063608,-.997701,.023368,2.665506),
new DragViewRot(.014815,-.998823,-.046186,4.365444),
new ClickButton(setFrontViewOrientation),

new Section("Drag out leg3"),
new RectVert(.01875,-.492187,.16625,-.544687),
//new TugVertex(74,.114231,-.907473,pelvis),
new TugVertex(74,.114231,-.827473,pelvis),

new Section("Drag out foot"),
new ClickButton(setLeftViewOrientation),
//new SelectVertex(11,pelvis),
new TugVertex(11,-.131678,-.910624,pelvis),

//new RectVert(-.1,-.854687,0,-.937187),
//new TugVertex(74,-.041467,-.827473,pelvis),
//new TugVertex(11,-.178047,-.950364,pelvis),
//new DragViewRot(.176004,.976419,-.125013,.796332),
//new ClickButton(toggleWireFrame),
//new DragViewRot(.084192,-.995995,.030085,1.305745),
//new DragViewRot(.104491,-.991472,-.077877,1.499712),
//new ClickButton(toggleWireFrame),
//new TugVertex(11,-.171505,-.913906-.005,pelvis),
//new ClickButton(toggleWireFrame),
//new ClickButton(setLeftViewOrientation),
//new ClickButton(toggleWireFrame),
////////

new Section("rough side view"),

new TugVertex(7,.12358,.055418,pelvis),
new TugVertex(1,.105672,-.011146,pelvis),
new TugVertex(14,.070481,.175753,pelvis),
new TugVertex(20,.051373,.28312,pelvis),
new TugVertex(26,.081528,.392958,pelvis),
new TugVertex(32,.087475,.485841,pelvis),
new TugVertex(38,.083287,.537348,pelvis),
new TugVertex(10,-.10358,.055418,pelvis),
new TugVertex(4,-.080672,-.006146,pelvis),
new TugVertex(17,-.100481,.175753,pelvis),
new TugVertex(29,-.096528,.392958,pelvis),
new TugVertex(35,-.082475,.475841,pelvis),
new TugVertex(41,-.052923,.523036,pelvis),

new TugVertex(50,.039589,.585032,pelvis),
new TugVertex(49,-.026679,.561402,pelvis),

new TugVertex(48,-.040218,.561402,pelvis),
new TugVertex(51,.045284,.652111,pelvis),
new TugVertex(57,.071431,.713755,pelvis),
new TugVertex(66,-.108748,.773387,pelvis),
new TugVertex(60,-.10982,.696986,pelvis),
new Section("rough side view2"),

new MouseZoom(2.125874),
new DragViewPos(0,-1.15558,0),


new RectPoly(-.24375,.951562,.16375,.919062),
new SelectTriangle(333,pelvis),
new ClickButton(splitTriangleImmediate),

new TugVertex(83,.113197,.932747,pelvis),
new TugVertex(78,.068198,.924296,pelvis),
new TugVertex(79,-.102452,.874096,pelvis),
new TugVertex(80,-.233112,.836823,pelvis),

new TugVertex(54,-.112745,.816865,pelvis),
new TugVertex(55,-.090152,.841848,pelvis),
new TugVertex(56,.062605,.874102,pelvis),


new RectPoly(-.103125,1.270312,.109375,1.227812),
new SelectTriangle(387,pelvis),
new ClickButton(splitTriangleImmediate),

//new TugVertex(66,-.231182,1.14331,pelvis),
new TugVertex(84,-.182172,1.255888,pelvis),
new TugVertex(85,-.0677,1.23599,pelvis),
new TugVertex(86,.066889,1.227361,pelvis),
new TugVertex(87,.126128,1.252875,pelvis),
new TugVertex(69,.006244,1.295508,pelvis),

new TugVertex(62,.085548,1.004076,pelvis),
new TugVertex(63,.172024,1.130923,pelvis),
new TugVertex(68,.088938,1.109039,pelvis),
new TugVertex(67,-.101748,1.128871,pelvis),

new Section("rough face"),

new ClickButton(setFrontViewOrientation),
new TugVertex(55,.074112,.841848,pelvis),
new TugVertex(79,.107522,.874096,pelvis),
new TugVertex(62,.153901,1.017986,pelvis),
new TugVertex(61,.153901,1.017986,pelvis),
new TugVertex(67,.157393,1.128871,pelvis),
new TugVertex(85,.142136,1.23599,pelvis),
new TugVertex(48,0,.652669,pelvis),

new Section("Move arm back"),

new MouseZoom(1),
new DragViewPos(0,-.92,0),
new IncButtonZ(viewOrientation90),
new RectVert(-.30625,.526563,.44375,.409063),
new ClickButton(setLeftViewOrientation),

new TugVertex(37,.084785,.485841,pelvis),

new DragViewRot(.0226,-.999346,-.028225,1.437343),
new TugVertex(30,-.020338,.391406,pelvis),
new TugVertex(36,-.01063,.473671,pelvis),
new TugVertex(12,-.044823,-.030234,pelvis),
new DragViewRot(.023911,-.999536,-.01888,1.774457),
new TugVertex(37,.034676,.480686,pelvis),
new TugVertex(31,.039359,.385818,pelvis),
new TugVertex(25,.018946,.27761,pelvis),
new TugVertex(19,.025867,.168118,pelvis),

new Section("Insert knee"),
new ClickButton(setFrontViewOrientation),

new RectPoly(.01875,-.307812,.17375,-.442812),
new SelectTriangle(432,pelvis),
new ClickButton(splitTriangleImmediate),

new DragViewRot(.149296,-.988791,.001522,.13769),
new TugVertex(91,.053911,-.41305,pelvis),
new TugVertex(90,.117477,-.411392,pelvis),
new DragViewRot(.074044,-.996661,-.034406,1.6079),
new TugVertex(93,.046555,-.418038,pelvis),
new DragViewRot(-.021025,-.995785,-.089273,3.166683),
new TugVertex(93,-.141236,-.401911,pelvis),
new TugVertex(92,-.077112,-.405268,pelvis),
new DragViewRot(-.024965,-.996725,-.076916,4.86497),
new TugVertex(92,-.002298,-.401992,pelvis),
new TugVertex(91,.084842,-.401585,pelvis),


new Section("shin0"),
new ClickButton(setFrontViewOrientation),

new RectPoly(.03125,-.573437,.15125,-.633437),
new SelectTriangle(538,pelvis),
new ClickButton(splitTriangleImmediate),
new RectVert(.015625,-.579687,.158125,-.659687),
new TugVertex(95,.065501,-.596787,pelvis),

new RectPoly(.025,-.704687,.1525,-.757187),
new SelectTriangle(562,pelvis),
new ClickButton(splitTriangleImmediate),

new RectPoly(.028125,-.170312,.165625,-.235312),
new SelectTriangle(528,pelvis),
new ClickButton(splitTriangleImmediate),
new RectVert(.01875,-.189062,.17125,-.261562),
new TugVertex(102,.136663,-.266774,pelvis),

new RectPoly(.015625,-.110937,.180625,-.178437),
new SelectTriangle(600,pelvis),
new ClickButton(splitTriangleImmediate),

new RectPoly(.040625,-.470312,.150625,-.507812),
new SelectTriangle(552,pelvis),
new ClickButton(splitTriangleImmediate),


new RectPoly(.028125,-.310937,.155625,-.358437),
new SelectTriangle(610,pelvis),
new ClickButton(splitTriangleImmediate),
new RectVert(.01875,-.314062,.17875,-.356562),
new TugVertex(114,.129377,-.375279,pelvis),

new MouseZoom(2.143589),
new DragViewPos(0,-.455826,0),

new Section("modify leg"),
new DragViewRot(.213385,.976955,.004984,.129518),
//new SelectVertex(91,pelvis),
new TugVertex(115,.144546,.178003,pelvis),
new TugVertex(114,.281098,.187702,pelvis),
new DragViewRot(.016149,-.998776,-.046754,1.683275),
new TugVertex(117,.043441,.171156,pelvis),
new TugVertex(116,.059393,.179986,pelvis),
new DragViewRot(.00534,-.999905,-.012677,3.280599),
new TugVertex(117,-.276399,.187682,pelvis),
new TugVertex(116,-.143496,.187651,pelvis),
new DragViewRot(.004988,-.999856,.016235,4.922252),
new TugVertex(115,.14648,.177596,pelvis),
new TugVertex(100,-.014034,-.527692,pelvis),
new TugVertex(96,-.052118,-.28807,pelvis),
new TugVertex(112,-.067883,-.078238,pelvis),
new DragViewRot(-.036695,-.983642,-.176357,1.674702),
new TugVertex(101,-.102596,-.547965,pelvis),
new TugVertex(113,.057817,-.12506,pelvis),

///////////////////
new Section("modify ankle"),

new DragViewRot(.01666,.999667,.019722,1.803379),
new TugVertex(76,-.121002,-.763878,pelvis),
new TugVertex(75,.067014,-.718513,pelvis),
new TugVertex(99,.047391,-.50062,pelvis),
new TugVertex(100,-.071709,-.51085,pelvis),
new DragViewRot(.128545,-.990602,.046729,1.397766),
new TugVertex(74,.049459,-.748834,pelvis),
new TugVertex(77,.244586,-.753511,pelvis),
new TugVertex(98,.040738,-.530312,pelvis),
new TugVertex(101,.159747,-.515192,pelvis),
new DragViewRot(.809771,.524122,.263755,.240311),
new TugVertex(99,.18547,-.501864,pelvis),
new TugVertex(98,.277118,-.496971,pelvis),
new TugVertex(75,.190489,-.719324,pelvis),
new TugVertex(74,.274868,-.717047,pelvis),
new DragViewRot(-.002275,-.991769,-.128019,2.995974),
new TugVertex(77,-.233359,-.766801,pelvis),
new TugVertex(76,-.144736,-.769025,pelvis),
new TugVertex(101,-.24848,-.510934,pelvis),
new TugVertex(100,-.152633,-.51168,pelvis),


new MouseZoom(1),
new DragViewRot(.129804,-.988613,-.076132,.549113),

///////////////////
new Section("modify ankle2"),

new MouseZoom(2.143589),
new DragViewRot(.169192,-.97375,.15227,1.625639),
new RectPoly(.046875,-.823437,.201875,-.915937),
new SelectTriangle(429,pelvis),
new ClickButton(splitTriangleImmediate),

////

new RectPoly(.025,-.910937,.11,-1.030938),
new SelectTriangle(429,pelvis),
new ClickButton(splitTriangleImmediate),
new RectPoly(.05625,-.579687,.30375,-.619687),
new SelectTriangle(580,pelvis),
new ClickButton(splitTriangleImmediate),
///////////


new Section("modify foot ankle"),



new ClickButton(setRightViewOrientation),

new TugVertex(11,.28348,-.942544,pelvis),
new RectVert(.165625,-.879687,.213125,-.959687),
new TugVertex(125,.224825,-.957059,pelvis),


//new SelectVertex(121,pelvis),
new RectVert(.065625,-.817187,.143125,-.917187),
new TugVertex(121,.093134,-.953302,pelvis),


//new SelectVertex(120,pelvis),
new RectVert(.159375,-.810937,.234375,-.875937),
new TugVertex(120,.149062,-.846027,pelvis),

//new SelectVertex(129,pelvis),
new RectVert(-.0875,-.614062,-.0225,-.679062),
new TugVertex(129,-.053711,-.655503,pelvis),

//new SelectVertex(128,pelvis),
new RectVert(.065625,-.585938,.115625,-.638437),
new TugVertex(128,.089867,-.627955,pelvis),

new Section("top foot"),

new DragViewRot(.997855,.061028,.023683,.814125),

new TugVertex(124,.145229,-.786917,pelvis),
new TugVertex(123,.221254,-.791335,pelvis),
new TugVertex(119,.223892,-.685715,pelvis),
new TugVertex(120,.140603,-.68499,pelvis),

new DragViewRot(-.999954,.00493,.008264,2.018059),
new TugVertex(125,.13404,.616653,pelvis),
new TugVertex(122,.23529,.627598,pelvis),
new TugVertex(121,.112665,.49628,pelvis),
new TugVertex(118,.262298,.497595,pelvis),

new MouseZoom(1),
new DragViewRot(.129804,-.988613,-.076132,.549113),

new Section("Insert elbow ring"),
new ClickButton(setFrontViewOrientation),
new DragViewPos(0,-.92,0),
new IncButtonZ(viewOrientation90),
new MouseZoom(1.4641),
new RectPoly(-.046875,.748437,.065625,.578438),
new SelectTriangle(402,pelvis),
new ClickButton(splitTriangleImmediate),

new RectVert(.03125,.735937,.12625,.583438),
new TugVertex(131,-.002791,.633471,pelvis),

new Section("Insert wrist ring"),
//new DragViewRot(0,0,1,.785398),

new RectPoly(.2,.742188,.2875,.589688),
new SelectTriangle(778,pelvis),
new ClickButton(splitTriangleImmediate),
new RectVert(.215625,.732812,.308125,.585313),
new TugVertex(136,.352425,.64295,pelvis),

new RectPoly(.4125,.742188,.4575,.589688),
new SelectTriangle(802,pelvis),
new ClickButton(splitTriangleImmediate),
new RectPoly(.09375,.745312,.20125,.585313),
new SelectTriangle(792,pelvis),
new ClickButton(splitTriangleImmediate),
new RectPoly(-.21875,.767187,-.14125,.574688),
new SelectTriangle(768,pelvis),
new ClickButton(splitTriangleImmediate),

new RectVert(-.2375,.742188,-.1425,.582188),
new TugVertex(146,-.136552,.710869,pelvis),

new RectPoly(-.275,.742188,-.245,.607188),
new SelectTriangle(864,pelvis),
new ClickButton(splitTriangleImmediate),


new Section("Shape arm"),


new DragViewRot(.261563,-.187485,.946802,.824978),
new TugVertex(151,-.282186,.611109,pelvis),
new TugVertex(150,-.279862,.67715,pelvis),

new TugVertex(147,-.136302,.621905,pelvis),
new TugVertex(131,-.008148,.619098,pelvis),
new TugVertex(143,.157047,.625695,pelvis),
new TugVertex(135,.349741,.636883,pelvis),
new TugVertex(139,.425968,.606064,pelvis),
new TugVertex(71,.519783,.61688,pelvis),
new TugVertex(42,.609122,.59721,pelvis),
new TugVertex(70,.527975,.637109,pelvis),
new TugVertex(138,.427221,.661476,pelvis),
new TugVertex(134,.349615,.663211,pelvis),
new TugVertex(142,.156692,.670526,pelvis),
new TugVertex(130,-.008557,.677081,pelvis),
new TugVertex(146,-.137827,.677957,pelvis),
new DragViewRot(.944048,-.308629,.116286,2.260609),
new Section("Shape arm2"),
new TugVertex(150,-.143154,-.464878,pelvis),
new TugVertex(153,-.137396,-.381835,pelvis),

new TugVertex(146,-.004254,-.433814,pelvis),
new TugVertex(130,.123051,-.428445,pelvis),
new TugVertex(142,.284236,-.408237,pelvis),
new TugVertex(134,.472532,-.385885,pelvis),
new TugVertex(138,.549088,-.389151,pelvis),
//new TugVertex(70,.643926,-.374289,pelvis),
//new TugVertex(42,.71423,-.306812,pelvis),
new TugVertex(73,.648712,-.296423,pelvis),
new TugVertex(141,.550256,-.325635,pelvis),
new DragViewRot(.934474,-.355434,-.020611,3.601188),
new Section("Shape arm4"),

new TugVertex(153,-.2226,-.68283,pelvis),
new TugVertex(152,-.228373,-.625724,pelvis),
new TugVertex(149,-.082779,-.668951,pelvis),
new TugVertex(148,-.087529,-.621906,pelvis),
new TugVertex(133,.043918,-.664999,pelvis),
new TugVertex(132,.042466,-.617014,pelvis),
new TugVertex(145,.209248,-.65924,pelvis),
new TugVertex(144,.2053,-.619091,pelvis),
new TugVertex(137,.401034,-.644211,pelvis),
new TugVertex(136,.398144,-.613839,pelvis),
//new TugVertex(141,.472438,-.646242,pelvis),
new TugVertex(140,.467421,-.615906,pelvis),
new TugVertex(73,.572129,-.626374,pelvis),
new TugVertex(72,.561278,-.607716,pelvis),
//new TugVertex(42,.653836,-.552512,pelvis),
new DragViewRot(.954422,-.091188,-.284191,5.159562),

new Section("Shape arm5"),

new TugVertex(151,.040778,.3376,pelvis),
new TugVertex(147,.173132,.304154,pelvis),
new TugVertex(131,.286905,.287257,pelvis),
new TugVertex(148,.18083,.237119,pelvis),
new TugVertex(143,.437005,.259406,pelvis),
new TugVertex(135,.613676,.227519,pelvis),
new TugVertex(139,.669677,.223252,pelvis),
new TugVertex(71,.758438,.211152,pelvis),
new TugVertex(72,.764125,.125038,pelvis),
new TugVertex(140,.682287,.155164,pelvis),
new TugVertex(136,.618479,.177364,pelvis),

new Section("second elbow ring"),
new ClickButton(setFrontViewOrientation),
new IncButtonZ(viewOrientation90),

new RectPoly(-.084375,.729687,-.036875,.584688),
new SelectTriangle(874,pelvis),
new ClickButton(splitTriangleImmediate),
new RectVert(-.0875,.723437,-.0375,.590938),
new TugVertex(155,-.034043, .630813, pelvis),
///////////////
new Section("Insert extra rings for hips"),
new ClickButton(setFrontViewOrientation),
new SelectVertex(5,pelvis),
new ClickButton(centerViewToControlPoint),
new MouseZoom(4.519338),
new RectPoly(-.0375,-.189062,.5275,-.284062),
new SelectTriangle(624,pelvis),
new ClickButton(splitTriangleImmediate),
new RectPoly(-.0375,-.104687,.545,-.169687),
new SelectTriangle(936,pelvis),
new ClickButton(splitTriangleImmediate),
new RectVert(.403125,.098437,.573125,-.046563),
//new TugVertex(13,.460318,.402063,pelvis),
new TugVertex(13,.472397,.486244,pelvis),
//new SelectVertex(162,pelvis),
new RectVert(.45625,-.089062,.54125,-.189062),
//new TugVertex(162,.495993,.250042,pelvis),
new TugVertex(162,.49955,.264548,pelvis),
//new SelectVertex(158,pelvis),
new RectVert(.4375,-.229687,.5425,-.309687),
//new TugVertex(158,.504742,.007539,pelvis),
new TugVertex(158,.498185,.009002,pelvis),
new TugVertex(5,-.048732,.055113,pelvis),
new TugVertex(163,-.028644,.003041,pelvis),
new TugVertex(159,-.021955,-.053152,pelvis),
new TugVertex(4,-.170379,.066294,pelvis),
new TugVertex(10,-.115393,.493611,pelvis),

new ClickButton(setBackViewOrientation),

new TugVertex(6,.090902,.013409,pelvis),
new TugVertex(164,.045966,-.036494,pelvis),
new TugVertex(160,.004727,-.104307,pelvis),
new TugVertex(7,.195393,.488611,pelvis),


new Section("Insert extra rings for shoulder"),
new ClickButton(setFrontViewOrientation),
new MouseZoom(1.7424),
new SelectVertex(36,pelvis),
new ClickButton(centerViewToControlPoint),

new MouseZoom(6.616764),
new IncButtonZ(viewOrientation90),
new RectPoly(.1125,.451562,.3075,-.033437),
new SelectTriangle(888,pelvis),
new ClickButton(splitTriangleImmediate),
new RectPoly(.0625,.460938,.1675,-.021562),
new SelectTriangle(984,pelvis),
new ClickButton(splitTriangleImmediate),
new RectPoly(.059375,-.014062,.049375,.458437),
new SelectTriangle(1008,pelvis),
new ClickButton(splitTriangleImmediate),

new ClickButton(setFrontViewOrientation),

new RectVert(.23125,.404687,.32875,.309688),
new TugVertex(43,-.207958,.381946,pelvis),

new Section("Insert extra rings for shoulder2"),

//new SelectVertex(177,pelvis),
new RectVert(.203125,.367188,.370625,.254688),
new TugVertex(177,-.017687,.367198,pelvis),

//new SelectVertex(173,pelvis),
new RectVert(.31875,.301562,.42125,.184063),
new TugVertex(170,.157194,.321298,pelvis),
//new SelectVertex(166,pelvis),
new RectVert(.3625,.185937,.505,.085938),
new TugVertex(166,.286559,.217017,pelvis),


new TugVertex(36,.004149,-.115449,pelvis),
new TugVertex(175,.015893,-.007926,pelvis),
new TugVertex(171,.059816,.012711,pelvis),
new TugVertex(167,.127082,-.014292,pelvis),

new DragViewRot(-.999193,.040156,-.000964,2.224525),
new DragViewRot(-.997581,.003681,-.069418,2.875758),
new TugVertex(37,-.23524,-.005286,pelvis),
new TugVertex(176,-.144116,-.130924,pelvis),
new TugVertex(172,-.055546,-.184003,pelvis),
new TugVertex(168,.04447,-.167776,pelvis),


new Section("Split neck band"),
new DragViewRot(-.999304,-.036751,.006384,.4327),
new DragViewPos(-.013471,-1.526318-.05,-.030653),
new RectPoly(-.4,-.323437,.225,-.415937),
new DragViewRot(.00314,-.995673,-.092869,2.989451),
new SelectTriangle(303,pelvis),
new ClickButton(splitTriangleImmediate),
new TugVertex(178,.162201,-.309187,pelvis),
new TugVertex(179,-.06627,-.347789,pelvis),
new TugVertex(56,-.078621,-.137955,pelvis),
/////
new Section("Split neck band2"),

new DragViewRot(.010939,-.99396,-.1092,1.473366),
new TugVertex(45,.537647,-.519895,pelvis),
new TugVertex(178,.465233,-.253612,pelvis),
new TugVertex(179,.397916,-.322046,pelvis),
new TugVertex(180,-.031636,-.393724,pelvis),
new TugVertex(181,-.109505,-.403494,pelvis),
new DragViewRot(.187233,-.619888,.762025,.074127),
new TugVertex(180,.10434,-.383004,pelvis),
new TugVertex(49,.193991,-.617717,pelvis),

new Section("Split stave in face"),

//new DragViewRot(.086713,.996211,.00664,.619758),
new RectPoly(.00625,1.035937,.05375,.018438),
//new SelectTriangle(501,pelvis),
//new DragViewRot(-.35799,-.872331,-.332988,.459265),
//new SelectTriangle(473,pelvis),
//new TypeVirtKey('Y'),
new SelectTriangle(327,pelvis),
new ClickButton(splitTriangleImmediate),

new ClickButton(setFrontViewOrientation),
new TugVertex(185,.30537,.366819,pelvis),
new TugVertex(186,.315806,.747148,pelvis),
new TugVertex(187,.267061,1.07405,pelvis),
new TugVertex(184,.118194,-.116314,pelvis),
new ClickButton(setLeftViewOrientation),

new TugVertex(187,-.271042,1.099049,pelvis),
new TugVertex(186,-.440304,.767148,pelvis),
new TugVertex(185,-.46201,.311819,pelvis),
new TugVertex(184,-.434399,-.151315,pelvis),

new Section("Insert rings into hand"),
new ClickButton(setFrontViewOrientation),
new MouseZoom(1.309091),

new SelectVertex(42,pelvis),
new ClickButton(centerViewToControlPoint),
new MouseZoom(6.616764),
new IncButtonZ(viewOrientation90),
new DragViewPos(-.533223,-.991558,.018459),

new RectPoly(.3625,.151562,.4075,-.073437),
new ClickButton(splitTriangleImmediate),
new RectVert(.340625,.164063,.478125,-.103437),
new TugVertex(189,.545939,-.045038,pelvis),
new RectPoly(.36875,.160937,.40625,-.066562),
new SelectTriangle(1116,pelvis),
new ClickButton(splitTriangleImmediate),
new RectPoly(-.046875,.289063,.055625,-.045937),
new SelectTriangle(826,pelvis),
new ClickButton(splitTriangleImmediate),
new RectPoly(-.45,.348437,-.365,-.001562),
new SelectTriangle(816,pelvis),
new ClickButton(splitTriangleImmediate),

new Section("Insert rings into hand2"),

new RectVert(.384375,.145312,.444375,.042813),
new TugVertex(192,.428947,.109304,pelvis),
new RectVert(.34375,.054688,.42375,-.052813),
new TugVertex(193,.393213,.039185,pelvis),
new RectVert(.54375,.042187,.58375,-.030313),
new TugVertex(188,.593098,.003375,pelvis),
new RectVert(-.021875,.070313,.043125,.015312),
new TugVertex(197,.019273,.085587,pelvis),
new RectVert(-.009375,.248437,.055625,.168438),
new TugVertex(196,.062624,.225554,pelvis),
new RectVert(-.425,.329687,-.33,.172188),
new TugVertex(200,-.489034,.247246,pelvis),
new RectVert(-.428125,.126562,-.283125,-.038438),
new TugVertex(201,-.497588,.100313,pelvis),
new TugVertex(200,-.489033,.227246,pelvis),

new Section("Insert rings into hand3"),

new DragViewRot(.449947,-.734413,-.508119,1.824832),
new TugVertex(203,.245286,.362406,pelvis),
new TugVertex(200,-.032067,.390483,pelvis),
new TugVertex(134,-.015714,.442197,pelvis),
new TugVertex(201,-.034602,.392096,pelvis),
new TugVertex(199,.326726,-.100368,pelvis),
new TugVertex(195,.26069,-.442834,pelvis),
new TugVertex(192,-.158631,-.431623,pelvis),
new DragViewRot(.298059,-.895857,-.329547,1.505776),
new TugVertex(188,-.077509,-.580685,pelvis),
new DragViewRot(-.668017,-.250583,.700686,2.570419),
new TugVertex(189,-.10732,.564413,pelvis),
new TugVertex(193,-.135633,.40098,pelvis),
new TugVertex(194,.292074,.393082,pelvis),
new TugVertex(190,.19782,.534152,pelvis),
new TugVertex(191,.204823,.539095,pelvis),
new TugVertex(198,.297008,-.026252,pelvis),
new TugVertex(202,.183173,-.512242,pelvis),
new TugVertex(201,-.100135,-.503228,pelvis),
new TugVertex(135,-.097993,-.557727,pelvis),


new ClickButton(saveSmls),
		new Section("Complete"),
	};
	list+=EVENT_LIST(eventArray);


	return;
}
