/* Seamless3d

Copyright (C) 2001-2010 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/


#define _WIN32_WINNT 0x0400
#define WINVER 0x0500

#include <windows.h>

#include "seamless3d.h"
#include "techuelife_triangle_editor.h"

#include "gui.h"
#include "scene_tree.h"
#include "resource.h"

extern const BIProto* builtInProtoList[];

void nameOrLineNumber(const int nodeLine, String& str);


Vec2i g_mousePos;
Vec2i g_newMousePos;

Vec2i g_prevMousePos;//for drag
extern double g_time;
extern String g_upStr;

DragNode::DragNode(SceneTree& i_win):
	win(i_win),
	dropWhere(0),
	potentialDropDestLine(0),
	dragMode(none),
	item(*this),
	Button(i_win)
{
	newNodeBarList.length=4;
	command=-1;
}

void DragNode::fillNewNodeBarList(const int* dragBarTypes, const int length){
	for(int i=0, bar=0; i<length; i++){
		newNodeBarList[bar++]+=dragBarTypes[i];
		if(bar>=newNodeBarList.length){
			bar=0;
		}
	}
}
void DragNode::dragNodeFromSceneTree(Node& node){
	command = node.getType();
	dragMode = move;
	setDrag(&item);
}
void DragNode::paint(int dstWidth, int dstHeight, int srcWidth, int srcHeight, int* dstPix){
	initNewNodeBarList(g_selNode.getRaw());

	bSize=Vec2i(newNodeBarList.length * pixSymbolCX, newNodeBarList[0].length * pixSymbolCY + yOffset * 2);

	const int edgeSrcY=200;

	win.blitBlockFromHorizontalLine(dstPos, Vec2i(352, edgeSrcY+8), bSize);
	pasteSprite2( dstPos.x, dstPos.y, 352, edgeSrcY, dstWidth, dstHeight, iconSrcBufCX, iconSrcBufCY, 16*4, 9, dstPix);
	pasteSprite2( dstPos.x, dstPos.y + bSize.y - 9, 352+16*4, edgeSrcY, dstWidth, dstHeight, iconSrcBufCX, iconSrcBufCY, 16*4, 9, dstPix);

	for(int bar=0; bar<newNodeBarList.length; bar++){
		for(int i=0; i<newNodeBarList[bar].length; i++){
			int type=newNodeBarList[bar][i];
			pasteSprite2(dstPos.x+bar*pixSymbolCX, dstPos.y+i*pixSymbolCY+yOffset,
				builtInProtoList[type]->iconSrcPos.x, builtInProtoList[type]->iconSrcPos.y,
				dstWidth, dstHeight,
				srcWidth, srcHeight, pixSymbolCX, pixSymbolCY,
				dstPix
			);
		}
	}
}
inline bool DragNode::dropAsChild(const Properties& sprop, const M_Int& promptNode, int& nodeLine, int& copyNodeLine, int& clickOffset, int copySibCount, Node*& node, int& fieldIdx, const bool dropIt){
	if(CodeLine* codeLine=dynamic_cast<CodeLine*>(node)){
		if(codeLine->isIfElseForEtc(codeLine->getParentFunction())==false){
			return false;
		}
	}
	for(int i=promptNode.length; --i>=0;){
		int promptType=promptNode[i];
		int _dropWhere=DROP_CHILD;
		if(promptType==command){
			if(sprop.flags & BLUE_DOT){
				if(clickOffset >= pixSymbolCX/2){
					continue;
				}
				_dropWhere=DROP_SOURCE_CHILD;
			}
			else if(sprop.flags & RED_DOT){
				_dropWhere=DROP_DEST_CHILD;
			}

			//if((promptType == PART || promptType == IMMEDIATEPART)){
			//	if(sprop.name == "source"){
			//		if(clickOffset >= pixSymbolCX/2){
			//			continue;
			//		}
			//		_dropWhere=DROP_SOURCE_CHILD;
			//	}
			//	else if(sprop.name == "dest"){
			//		_dropWhere=DROP_DEST_CHILD;
			//	}
			//	else if(sprop.name == "part"){
			//		if(dynamic_cast<Stem*>(node)||dynamic_cast<EvolveCoord*>(node)||dynamic_cast<JoinVertex*>(node)){
			//			_dropWhere=DROP_DEST_CHILD;
			//		}
			//	}
			//}
			if(dragMode==use){
				if(sprop.blockUse()==false){
					if(dropIt==false){
						potentialDropDestLine=nodeLine;
						dropWhere=_dropWhere;
						return true;
					}
					if(dragMode==use){
						useNodeAsChild(node, g_selNode.getRaw(), fieldIdx, g_stw->selLine, 1, g_selCount);
#ifdef PROG_ROB
						g_upStr="new MoveToNode(";
						nameOrLineNumber(g_stw->selLine, g_upStr);
						g_upStr+="),\nnew VirtKeyDown(VK_LMENU),\n";
						g_upStr+="new LeftButtonDown(),\n";
						g_upStr+="new MoveToNodeAsChild(";
						nameOrLineNumber(nodeLine, g_upStr);
						g_upStr+="),\nnew LeftButtonUp(),\n";
						g_upStr+="new VirtKeyUp(VK_LMENU),";
#endif
						return true;
					}
				}
			}
			else if(sprop.blockNew()==false){
				if(dropIt==false){
					potentialDropDestLine=nodeLine;
					dropWhere=_dropWhere;
					return true;
				}
				else switch (dragMode){
					case newInstance:{
						addNewNodeAsChild(node, fieldIdx, command);
#ifdef PROG_ROB
						g_upStr="new MoveToNewNode(";
						g_upStr+=nodeLookUpStr[command].toUpperCase();
						g_upStr+="),\nnew LeftButtonDown(),\n";
						g_upStr+="new MoveToNodeAsChild(";
						nameOrLineNumber(nodeLine, g_upStr);
						g_upStr+="),\nnew LeftButtonUp(),";
#endif
						return true;
					}
					case copyOf:{
						addCopyOfAsChild(node, fieldIdx, g_stw->selLine, g_selCount);
						return true;
					}
					case move:{
						moveNodeAsChild(node, fieldIdx, nodeLine, g_stw->selLine, 1, g_selCount, g_selCount.all);
						return true;
					}
				}
			}
		}
	}
	return false;
}
inline bool DragNode::dropToReplace(const Properties& sprop, const M_Int& promptNode, Node*& node, int& nodeLine, int& copyNodeLine, int& clickOffset, int copySibCount, Node*& parent, int& fieldIdx, int& element, const bool dropIt){
	for(int i=promptNode.length; --i>=0;){
		if(promptNode[i]==command){
			if(dragMode==use){
				if(sprop.blockUse()==false){
					if(dropIt==false){
						potentialDropDestLine=nodeLine;
						dropWhere=DROP_REPLACE;
						return true;
					}
					else{
						useSiblingNode(node, nodeLine, g_stw->selLine, 1, g_selCount, DROP_REPLACE);
						return true;
					}
				}
			}
			else if(sprop.blockNew()==false){
				if(dropIt==false){
					potentialDropDestLine=nodeLine;
					dropWhere=DROP_REPLACE;
					return true;
				}
				else switch (dragMode){
					case newInstance:{
						replaceWithNewNode(parent, fieldIdx, element, command);
						return true;
					}
					case copyOf:{
						addCopyOfNodeAsSibling(nodeLine, copyNodeLine, copySibCount, DROP_REPLACE);
						return true;
					}
					case move:{
						replaceNodeWithMove(nodeLine, copyNodeLine, copySibCount, 1, 1, DROP_REPLACE);
						return true;
					}
				}
			}
		}
	}
	return false;
}
inline bool DragNode::dropAsSibling(const Properties& sprop, const M_Int& promptNode, Node*& node, int& nodeLine, int& copyNodeLine, int& clickOffset, int copySibCount, Node*& parent, int& fieldIdx, int& element, const bool dropIt){
	for(int i=promptNode.length; --i>=0;){
		if(promptNode[i]==command){
			int _dropWhere=DROP_ABOVE;
			if(clickOffset >= pixSymbolCX/2){
				_dropWhere=DROP_BELOW;
			}
			if(dragMode==use){
				if(sprop.blockUse()==false){
					if(dropIt==false){
						potentialDropDestLine=nodeLine;
						dropWhere=_dropWhere;
						return true;
					}
					else{
						useSiblingNode(node, nodeLine, g_stw->selLine, 1, g_selCount, _dropWhere);
						return true;
					}
				}
			}
			else if(sprop.blockNew()==false){
				if(dropIt==false){
					potentialDropDestLine=nodeLine;
					dropWhere=_dropWhere;
					return true;
				}
				else switch(dragMode){
					case newInstance:{
						addNewNodeAsSibling(parent, fieldIdx, element, command, _dropWhere);
						return true;
					}
					case copyOf:{
						addCopyOfNodeAsSibling(nodeLine, copyNodeLine, copySibCount, _dropWhere);
						return true;
					}
					case move:{
						replaceNodeWithMove(nodeLine, copyNodeLine, g_selCount, 1, 1, _dropWhere);
						return true;
					}
				}
			}
		}
	}
	return false;
}
void AutoScrollUp::doCallBack(){
	if(startTime+.1 < g_time){
		startTime+=.1;
		g_stw->onMouseWheel(-1);
	}
}
void AutoScrollDown::doCallBack(){
	if(startTime+.1 < g_time){
		startTime+=.1;
		g_stw->onMouseWheel(1);
	}
}
void DragNode::doDragDrop(Vec2i ptCursor, const bool dropIt){
	dropWhere=0;

	if(ptCursor.y > g_stw->size.y-12){
		if(isInList<CallBack>(g_callBackList, &autoScrollDown)==false){
			g_callBackList+=&autoScrollDown;
			autoScrollDown.startTime=g_time;
			g_stw->textCell.cellActive=false;
		}
		return;
	}
	else if(ptCursor.y < title_bar_height){
		if(isInList<CallBack>(g_callBackList, &autoScrollUp)==false){
			g_callBackList+=&autoScrollUp;
			autoScrollUp.startTime=g_time;
			g_stw->textCell.cellActive=false;
		}
		return;
	}

	removeFromList<CallBack>(g_callBackList, &autoScrollDown);
	removeFromList<CallBack>(g_callBackList, &autoScrollUp);

	int collapsedNodeLine=(ptCursor.y-SceneTree::beginYOffset)/pixSymbolCY+g_scene->scroll;
	if(collapsedNodeLine>=g_scene->sceneTreeCollapsed.length){
		return;
	}
	int clickOffset=(ptCursor.y-SceneTree::beginYOffset) % pixSymbolCY;
	int nodeLine=getLineForNonCollapsed(collapsedNodeLine);

	Node* node = g_scene->sceneTreeList[nodeLine].node;
	if(dragMode != newInstance && node!= g_selNode.getRaw()){
		if(altKeyState()){
			dragMode=use;
		}
		else if(controlKeyState()){
			dragMode=copyOf;
		}
		else{
			dragMode=move;
		}
		command=g_selNode->getType();
	}

	int indentPix = tLnOffst+g_scene->sceneTreeList[nodeLine].indent*pixSymbolCX;

	if(ptCursor.x >= indentPix + pixSymbolCX){//drop as child
		if(!g_scene->sceneTreeList[nodeLine].ref){
			const M_Properties& properties=node->getProperties();
			for(int k=0; k<properties.length; k++){
				const Properties& sprop=properties[k];
				if(node->getField(k)->getType()==MFNODE || node->getField(k)->getType()==SFNODE){
					if(dropAsChild(sprop, sprop.promptNode, nodeLine, g_stw->selLine, clickOffset, g_selCount, node, k, dropIt)){
						break;
					}
				}
			}
		}
	}
	else if(nodeLine){
		int parentLine=getParent(nodeLine);
		Node* parent=g_scene->sceneTreeList[parentLine].node.p;

		const M_Properties& parentProperties=parent->getProperties();

		int fieldIdx=0;
		int element=0;

		getFNodeElementForParent(parentLine, nodeLine, g_scene->sceneTreeList, fieldIdx, element);

		const Properties& sprop=parent->getProp(fieldIdx);

		Field* field=parent->getField(fieldIdx);
		if(ptCursor.x >= indentPix){//drop as sibling
			if(field->getType()==MFNODE){
				dropAsSibling(sprop, sprop.promptNode, node, nodeLine, g_stw->selLine, clickOffset, g_selCount, parent, fieldIdx, element, dropIt);
			}
		}
		else if(ptCursor.x >= indentPix - pixSymbolCX){//drop as replacement
			dropToReplace(sprop, sprop.promptNode, node, nodeLine, g_stw->selLine, clickOffset, g_selCount, parent, fieldIdx, element, dropIt);
		}
	}
}
void DragNode::paintDragClues(int dstWidth, int dstHeight, int srcWidth, int srcHeight, int* dstPix){
	Vec2i mousePos=g_mousePos-g_stw->pos;
	pasteSprite2(mousePos.x+6, mousePos.y+6,
		builtInProtoList[command]->iconSrcPos.x, builtInProtoList[command]->iconSrcPos.y,
		dstWidth, dstHeight,
		srcWidth, srcHeight, pixSymbolCX, pixSymbolCY,
		dstPix
	);
	if(dragMode){
		if(potentialDropDestLine < g_scene->sceneTreeList.length){
			int potentialDropDestLineCollapsed;
			getCollapsedLine(potentialDropDestLine, potentialDropDestLineCollapsed);

			int indentPix = tLnOffst+g_scene->sceneTreeList[potentialDropDestLine].indent*pixSymbolCX;
			int pixLine=SceneTree::beginYOffset+(potentialDropDestLineCollapsed-g_scene->scroll)*pixSymbolCY;
			switch(dropWhere){
				/*
				case DROP_ABOVE:
					//drawHorizontalLine2( 0, pixLine-1, 6, dstWidth-6, dstWidth, dstHeight, dstPix);

					//drawHorizontalLine2( 0, pixLine, 6, dstWidth-6, dstWidth, dstHeight, dstPix);
					//drawHorizontalLine2( 0, pixLine-3, 6, 7, dstWidth, dstHeight, dstPix);
					//drawHorizontalLine2( 0, pixLine-2, 6, 8, dstWidth, dstHeight, dstPix);
					//drawHorizontalLine2( 0, pixLine+1, 6, 8, dstWidth, dstHeight, dstPix);
					//drawHorizontalLine2( 0, pixLine+2, 6, 7, dstWidth, dstHeight, dstPix);


					//drawHorizontalLine2( 0, pixLine-3, dstWidth-6-1, dstWidth-6, dstWidth, dstHeight, dstPix);
					//drawHorizontalLine2( 0, pixLine-2, dstWidth-6-2, dstWidth-6, dstWidth, dstHeight, dstPix);
					//drawHorizontalLine2( 0, pixLine+1, dstWidth-6-2, dstWidth-6, dstWidth, dstHeight, dstPix);
					//drawHorizontalLine2( 0, pixLine+2, dstWidth-6-1, dstWidth-6, dstWidth, dstHeight, dstPix);

					pasteSprite2(indentPix-4, pixLine-5, 407, 15, dstWidth, dstHeight, srcWidth, srcHeight, 24, 10, dstPix);
					break;
				case DROP_BELOW:
					pasteSprite2(indentPix-4, pixLine+16-5, 407, 15, dstWidth, dstHeight, srcWidth, srcHeight, 24, 10, dstPix);
					break;
*/
				case DROP_ABOVE:
					pasteSprite2(indentPix, pixLine-pixSymbolCY/2, dropSiblingAbove_x, dropSiblingAbove_y, dstWidth, dstHeight, srcWidth, srcHeight, pixSymbolCX, pixSymbolCY, dstPix);
					break;
				case DROP_BELOW:
					pasteSprite2(indentPix, pixLine+pixSymbolCY/2, dropSiblingBelow_x, dropSiblingBelow_y, dstWidth, dstHeight, srcWidth, srcHeight, pixSymbolCX, pixSymbolCY, dstPix);
					break;
				case DROP_REPLACE:
					pasteSprite2(indentPix - pixSymbolCX, pixLine, dropSiblingReplace_x, dropSiblingReplace_y, dstWidth, dstHeight, srcWidth, srcHeight, pixSymbolCX, pixSymbolCY, dstPix);
					break;
				case DROP_CHILD:
					pasteSprite2(indentPix + pixSymbolCX, pixLine, dropChild_x, dropChild_y, dstWidth, dstHeight, srcWidth, srcHeight, pixSymbolCX, pixSymbolCY, dstPix);
					break;
				case DROP_SOURCE_CHILD:
					pasteSprite2(indentPix + pixSymbolCX, pixLine, dropSourceChild_x, dropSourceChild_y, dstWidth, dstHeight, srcWidth, srcHeight, pixSymbolCX, pixSymbolCY, dstPix);
					break;
				case DROP_DEST_CHILD:
					pasteSprite2(indentPix + pixSymbolCX, pixLine, dropDestChild_x, dropDestChild_y, dstWidth, dstHeight, srcWidth, srcHeight, pixSymbolCX, pixSymbolCY, dstPix);
					break;
			}
		}
	}
}
void DragNode::Item::endDrag(const Vec2i ptCursor){
	SceneTree& win=home.win;

	Vec2i localCursor=ptCursor-g_stw->pos;

	if( localCursor >= g_stw->newNodeBar.dstPos && localCursor < win.newNodeBar.dstPos + win.newNodeBar.bSize);
	else{
		home.doDragDrop(localCursor, true);
	}
	home.command = -1;
	home.dragMode = none;
	ReleaseCapture();
	g_drag = 0;
	removeFromList<CallBack>(g_callBackList, &win.newNodeBar.autoScrollDown);
	removeFromList<CallBack>(g_callBackList, &win.newNodeBar.autoScrollUp);
//	g_callBackList+=this;

	g_stw->newNodeBar.initHover(localCursor);

	home.win.paint();
}
void DragNode::Item::doDrag(Vec2i ptCursor){
	home.doDragDrop(ptCursor-g_stw->pos, false);
	home.win.paint();
}
void DragNode::action(){
	removeTypeFromList < Button>(g_callBackList);
	removeTypeFromList < DownButton>(g_callBackList);
	if(command == -1){
		setDrag(this);
		dragPos = g_mousePos - win.pos;
	}
	else{
		dragMode = newInstance;
		setDrag(&item);
	}
}
void DragNode::positionTipWin(){
	workOutSubIndex(g_mousePos-win.pos-dstPos);
	if(command!=-1){
		g_tt->autoSizeAndStuff(nodeLookUpStr[command]);
	}
	else{
		g_tt->autoSizeAndStuff("drag new node bar");
	}

	Rect rect; getClientRect(rect);

	Vec2i pos = g_mousePos;
	pos.y-=30;
	pos.x-=g_tt->size.x>>1;

	int subtract = rect.right - (pos.x + g_tt->size.x);
	if(pos.x<0){
		pos.x=0;
	}
	else if(subtract < 0){
		pos.x+=subtract;
	}
	if(pos.y<0){
		pos.y+=18 + 40;
	}
	g_tt->pos=pos;
}
void DragNode::doCallBack(){
	if(open){
		return;
	}
	if(g_time > beginHoverTime+tipTimeOut){
		open = true;
		positionTipWin();
	}
	return;
}
void DragNode::workOutSubIndex(const Vec2i ptCursor){

	int lptY = ptCursor.y - yOffset;

	if( lptY >= 0){
		for(int bar = newNodeBarList.length; --bar >= 0;){
			if( ptCursor.x >= bar * pixSymbolCX ){
				int i = lptY / pixSymbolCY;
				if( i < newNodeBarList[bar].length){
					command = newNodeBarList[bar][i];
					dragMode = newInstance;
					//g_drag=&item;
					return;
				}
				else{
					break;
				}
			}
		}
	}
	command=-1;
	return;
//	dragPos=ptCursor;
//	g_drag=this;
//	return true;
}
bool DragNode::onHoverMove(const Vec2i ptCursor){
	if( ptCursor >= dstPos && ptCursor < dstPos + bSize){
		if(win.slider.initHover(ptCursor)){
			open=false;
			return true;
		}
		int remCommand=command;
		workOutSubIndex(ptCursor-dstPos);
		if(remCommand!=command && open){
			open=false;
			beginHoverTime=g_time;
			Window::removeToolTip();
		}
		win.paint();
		return true;
	}
	else{
		open=false;
		Window::removeToolTip();
		removeTypeFromList < Button>(g_callBackList);
		win.paint();
		return false;
	}
}

void DragNode::doDrag(Vec2i ptCursor){
	Vec2i pos = ptCursor - g_stw->pos;

	Vec2i diff = ptCursor - dragPos;

	pos += diff;          // horizontal position

	dragPos += diff;

	if(dstPos.x < 0){
		dstPos.x = 0;
	}
	if(dstPos.y < title_bar_height){
		dstPos.y = title_bar_height;
	}
	g_stw->paint();
}

void DragNode::endDrag(const Vec2i ptCursor){

	ReleaseCapture();
	g_drag = 0;
	const Vec2i localCursor=ptCursor-win.pos;
	if( localCursor >= dstPos && localCursor < dstPos + bSize){
		open=true;//prevent tool tip opening
		g_callBackList+=this;
	}
}
