/* Seamless3d

Copyright (C) 2001-2010 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#define _WIN32_WINNT 0x0400
#define WINVER 0x0500

#include <windows.h>

#include "seamless3d.h"
#include "gui.h"

Drag* g_drag = 0;
bool Drag::triggered = false;
Vec2i Drag::dragPos;
Vec2i Drag::beginMousePos;

void Drag::setDrag(Drag* par){
	triggered = false;
	beginMousePos = g_mousePos;
	dragPos = g_mousePos;
	g_drag = par;
}

bool penIsDown;

#include "seamless_icons.cpp"
enum Plotter{
	down	= 0x0,
	up		= 0x10,
	south	= 0x20,
	north	= 0x30,
	west	= 0x40,
	east	= 0x50,
	southWest	=0x60,
	southEast	=0x70,
	northWest	=0x80,
	northEast	=0x90,
	end			=0xa0,
};
const unsigned char char_space[]={
	east|charWidth
};
//!"#$&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~€‚ƒ„…†‡ˆ‰Š‹ŒŽ‘’“”•–—˜™š›œžŸ ¡¢£¤¥¦§¨©ª«¬­®¯°±²³´µ¶·¸¹º»¼½¾¿ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖ×ØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõö÷øùúûüýþÿ
const unsigned char courierLookUp[]={
// ! 0x21
	0x00,
	0x00,
	0x00,
	0x00,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x00,
	0x00,
	0x08,
	0x00,
	0x00,
	0x00,
	0x00,
// " 0x22
	0x00,
	0x00,
	0x00,
	0x00,
	0x36,
	0x36,
	0x24,
	0x24,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// # 0x23
	0x00,
	0x00,
	0x00,
	0x12,
	0x12,
	0x24,
	0x7E,
	0x24,
	0x24,
	0x7E,
	0x24,
	0x48,
	0x48,
	0x00,
	0x00,
	0x00,
// $ 0x24
	0x00,
	0x00,
	0x00,
	0x08,
	0x1C,
	0x24,
	0x20,
	0x18,
	0x04,
	0x24,
	0x38,
	0x08,
	0x08,
	0x00,
	0x00,
	0x00,
// % 0x25
	0x00,
	0x00,
	0x00,
	0x00,
	0x20,
	0x50,
	0x20,
	0x0C,
	0x70,
	0x08,
	0x14,
	0x08,
	0x00,
	0x00,
	0x00,
	0x00,
// & 0x26
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x0E,
	0x10,
	0x10,
	0x18,
	0x2A,
	0x24,
	0x1E,
	0x00,
	0x00,
	0x00,
	0x00,
// ' 0x27
	0x00,
	0x00,
	0x00,
	0x00,
	0x10,
	0x10,
	0x10,
	0x10,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// ( 0x28
	0x00,
	0x00,
	0x00,
	0x00,
	0x04,
	0x04,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x04,
	0x04,
	0x00,
	0x00,
// ) 0x29
	0x00,
	0x00,
	0x00,
	0x00,
	0x20,
	0x20,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x20,
	0x20,
	0x00,
	0x00,
// * 0x2A
	0x00,
	0x00,
	0x00,
	0x00,
	0x08,
	0x3E,
	0x08,
	0x14,
	0x14,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// + 0x2B
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x08,
	0x08,
	0x08,
	0x7F,
	0x08,
	0x08,
	0x08,
	0x00,
	0x00,
	0x00,
	0x00,
// , 0x2C
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x10,
	0x30,
	0x20,
	0x00,
	0x00,
// - 0x2D
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x7E,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// . 0x2E
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x18,
	0x00,
	0x00,
	0x00,
	0x00,
// / 0x2F
	0x00,
	0x00,
	0x00,
	0x04,
	0x04,
	0x08,
	0x08,
	0x10,
	0x10,
	0x20,
	0x20,
	0x40,
	0x40,
	0x00,
	0x00,
	0x00,
// 0 0x30
	0x00,
	0x00,
	0x00,
	0x00,
	0x3C,
	0x42,
	0x42,
	0x42,
	0x42,
	0x42,
	0x42,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
// 1 0x31
	0x00,
	0x00,
	0x00,
	0x00,
	0x08,
	0x38,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// 2 0x32
	0x00,
	0x00,
	0x00,
	0x00,
	0x38,
	0x44,
	0x04,
	0x08,
	0x10,
	0x20,
	0x44,
	0x7C,
	0x00,
	0x00,
	0x00,
	0x00,
// 3 0x33
	0x00,
	0x00,
	0x00,
	0x00,
	0x38,
	0x44,
	0x04,
	0x18,
	0x04,
	0x04,
	0x44,
	0x38,
	0x00,
	0x00,
	0x00,
	0x00,
// 4 0x34
	0x00,
	0x00,
	0x00,
	0x00,
	0x0C,
	0x14,
	0x24,
	0x24,
	0x7E,
	0x04,
	0x04,
	0x0E,
	0x00,
	0x00,
	0x00,
	0x00,
// 5 0x35
	0x00,
	0x00,
	0x00,
	0x00,
	0x3E,
	0x20,
	0x20,
	0x3C,
	0x02,
	0x02,
	0x42,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
// 6 0x36
	0x00,
	0x00,
	0x00,
	0x00,
	0x0E,
	0x10,
	0x20,
	0x3C,
	0x22,
	0x22,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// 7 0x37
	0x00,
	0x00,
	0x00,
	0x00,
	0x7E,
	0x42,
	0x02,
	0x04,
	0x04,
	0x08,
	0x08,
	0x08,
	0x00,
	0x00,
	0x00,
	0x00,
// 8 0x38
	0x00,
	0x00,
	0x00,
	0x00,
	0x3C,
	0x42,
	0x42,
	0x3C,
	0x42,
	0x42,
	0x42,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
// 9 0x39
	0x00,
	0x00,
	0x00,
	0x00,
	0x3C,
	0x42,
	0x42,
	0x42,
	0x3E,
	0x02,
	0x04,
	0x78,
	0x00,
	0x00,
	0x00,
	0x00,
// : 0x3A
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x18,
	0x00,
	0x00,
	0x18,
	0x18,
	0x00,
	0x00,
	0x00,
	0x00,
// ; 0x3B
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x18,
	0x00,
	0x00,
	0x18,
	0x30,
	0x20,
	0x00,
	0x00,
	0x00,
// < 0x3C
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x02,
	0x0C,
	0x10,
	0x60,
	0x10,
	0x0C,
	0x02,
	0x00,
	0x00,
	0x00,
	0x00,
// = 0x3D
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x7E,
	0x00,
	0x7E,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// > 0x3E
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x40,
	0x30,
	0x08,
	0x06,
	0x08,
	0x30,
	0x40,
	0x00,
	0x00,
	0x00,
	0x00,
// ? 0x3F
	0x00,
	0x00,
	0x00,
	0x00,
	0x1C,
	0x22,
	0x02,
	0x02,
	0x04,
	0x08,
	0x00,
	0x18,
	0x00,
	0x00,
	0x00,
	0x00,
// @ 0x40
	0x00,
	0x00,
	0x00,
	0x38,
	0x44,
	0x44,
	0x4C,
	0x54,
	0x54,
	0x4C,
	0x40,
	0x44,
	0x38,
	0x00,
	0x00,
	0x00,
// A 0x41
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x08,
	0x14,
	0x14,
	0x14,
	0x1C,
	0x22,
	0x77,
	0x00,
	0x00,
	0x00,
	0x00,
// B 0x42
	0x00,
	0x00,
	0x00,
	0x00,
	0x7C,
	0x22,
	0x22,
	0x3C,
	0x22,
	0x22,
	0x22,
	0x7C,
	0x00,
	0x00,
	0x00,
	0x00,
// C 0x43
	0x00,
	0x00,
	0x00,
	0x00,
	0x1E,
	0x22,
	0x40,
	0x40,
	0x40,
	0x40,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// D 0x44
	0x00,
	0x00,
	0x00,
	0x00,
	0x78,
	0x24,
	0x22,
	0x22,
	0x22,
	0x22,
	0x24,
	0x78,
	0x00,
	0x00,
	0x00,
	0x00,
// E 0x45
	0x00,
	0x00,
	0x00,
	0x00,
	0x7E,
	0x22,
	0x28,
	0x38,
	0x28,
	0x20,
	0x22,
	0x7E,
	0x00,
	0x00,
	0x00,
	0x00,
// F 0x46
	0x00,
	0x00,
	0x00,
	0x00,
	0x7E,
	0x22,
	0x28,
	0x38,
	0x28,
	0x20,
	0x20,
	0x70,
	0x00,
	0x00,
	0x00,
	0x00,
// G 0x47
	0x00,
	0x00,
	0x00,
	0x00,
	0x1E,
	0x22,
	0x40,
	0x40,
	0x47,
	0x42,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// H 0x48
	0x00,
	0x00,
	0x00,
	0x00,
	0x77,
	0x22,
	0x22,
	0x3E,
	0x22,
	0x22,
	0x22,
	0x77,
	0x00,
	0x00,
	0x00,
	0x00,
// I 0x49
	0x00,
	0x00,
	0x00,
	0x00,
	0x3E,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// J 0x4A
	0x00,
	0x00,
	0x00,
	0x00,
	0x1E,
	0x04,
	0x04,
	0x04,
	0x44,
	0x44,
	0x44,
	0x38,
	0x00,
	0x00,
	0x00,
	0x00,
// K 0x4B
	0x00,
	0x00,
	0x00,
	0x00,
	0x77,
	0x22,
	0x24,
	0x28,
	0x38,
	0x24,
	0x22,
	0x73,
	0x00,
	0x00,
	0x00,
	0x00,
// L 0x4C
	0x00,
	0x00,
	0x00,
	0x00,
	0x70,
	0x20,
	0x20,
	0x20,
	0x20,
	0x22,
	0x22,
	0x7E,
	0x00,
	0x00,
	0x00,
	0x00,
// M 0x4D
	0x00,
	0x00,
	0x00,
	0x00,
	0x77,
	0x36,
	0x36,
	0x2A,
	0x2A,
	0x22,
	0x22,
	0x77,
	0x00,
	0x00,
	0x00,
	0x00,
// N 0x4E
	0x00,
	0x00,
	0x00,
	0x00,
	0xE7,
	0x62,
	0x52,
	0x52,
	0x4A,
	0x4A,
	0x46,
	0xE6,
	0x00,
	0x00,
	0x00,
	0x00,
// O 0x4F
	0x00,
	0x00,
	0x00,
	0x00,
	0x1C,
	0x22,
	0x41,
	0x41,
	0x41,
	0x41,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// P 0x50
	0x00,
	0x00,
	0x00,
	0x00,
	0x7C,
	0x22,
	0x22,
	0x22,
	0x3C,
	0x20,
	0x20,
	0x70,
	0x00,
	0x00,
	0x00,
	0x00,
// Q 0x51
	0x00,
	0x00,
	0x00,
	0x00,
	0x1C,
	0x22,
	0x41,
	0x41,
	0x41,
	0x41,
	0x22,
	0x1C,
	0x1F,
	0x00,
	0x00,
	0x00,
// R 0x52
	0x00,
	0x00,
	0x00,
	0x00,
	0x7C,
	0x22,
	0x22,
	0x22,
	0x3C,
	0x24,
	0x22,
	0x71,
	0x00,
	0x00,
	0x00,
	0x00,
// S 0x53
	0x00,
	0x00,
	0x00,
	0x00,
	0x3A,
	0x46,
	0x40,
	0x3C,
	0x02,
	0x02,
	0x62,
	0x5C,
	0x00,
	0x00,
	0x00,
	0x00,
// T 0x54
	0x00,
	0x00,
	0x00,
	0x00,
	0x7F,
	0x49,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// U 0x55
	0x00,
	0x00,
	0x00,
	0x00,
	0x77,
	0x22,
	0x22,
	0x22,
	0x22,
	0x22,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// V 0x56
	0x00,
	0x00,
	0x00,
	0x00,
	0xE7,
	0x42,
	0x42,
	0x24,
	0x24,
	0x24,
	0x18,
	0x18,
	0x00,
	0x00,
	0x00,
	0x00,
// W 0x57
	0x00,
	0x00,
	0x00,
	0x00,
	0x77,
	0x22,
	0x22,
	0x2A,
	0x2A,
	0x2A,
	0x2A,
	0x14,
	0x00,
	0x00,
	0x00,
	0x00,
// X 0x58
	0x00,
	0x00,
	0x00,
	0x00,
	0x77,
	0x22,
	0x14,
	0x08,
	0x08,
	0x14,
	0x22,
	0x77,
	0x00,
	0x00,
	0x00,
	0x00,
// Y 0x59
	0x00,
	0x00,
	0x00,
	0x00,
	0x77,
	0x22,
	0x14,
	0x14,
	0x08,
	0x08,
	0x08,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// Z 0x5A
	0x00,
	0x00,
	0x00,
	0x00,
	0x3E,
	0x22,
	0x04,
	0x08,
	0x08,
	0x10,
	0x22,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// [ 0x5B
	0x00,
	0x00,
	0x00,
	0x00,
	0x1C,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x1C,
	0x00,
	0x00,
// \ 0x5C
	0x00,
	0x00,
	0x00,
	0x40,
	0x40,
	0x20,
	0x20,
	0x10,
	0x10,
	0x10,
	0x08,
	0x08,
	0x08,
	0x00,
	0x00,
	0x00,
// ] 0x5D
	0x00,
	0x00,
	0x00,
	0x00,
	0x70,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x70,
	0x00,
	0x00,
// ^ 0x5E
	0x00,
	0x00,
	0x00,
	0x10,
	0x10,
	0x28,
	0x44,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// _ 0x5F
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0xFF,
// ` 0x60
	0x00,
	0x00,
	0x00,
	0x00,
	0x10,
	0x08,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// a 0x61
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x3C,
	0x42,
	0x3E,
	0x42,
	0x46,
	0x3B,
	0x00,
	0x00,
	0x00,
	0x00,
// b 0x62
	0x00,
	0x00,
	0x00,
	0x00,
	0xC0,
	0x40,
	0x5C,
	0x62,
	0x42,
	0x42,
	0x62,
	0xDC,
	0x00,
	0x00,
	0x00,
	0x00,
// c 0x63
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x3A,
	0x46,
	0x40,
	0x40,
	0x42,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
// d 0x64
	0x00,
	0x00,
	0x00,
	0x00,
	0x06,
	0x02,
	0x3A,
	0x46,
	0x42,
	0x42,
	0x46,
	0x3B,
	0x00,
	0x00,
	0x00,
	0x00,
// e 0x65
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x3C,
	0x42,
	0x7E,
	0x40,
	0x40,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// f 0x66
	0x00,
	0x00,
	0x00,
	0x00,
	0x0E,
	0x10,
	0x7E,
	0x10,
	0x10,
	0x10,
	0x10,
	0x7E,
	0x00,
	0x00,
	0x00,
	0x00,
// g 0x67
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x3B,
	0x46,
	0x42,
	0x42,
	0x46,
	0x3A,
	0x02,
	0x3C,
	0x00,
	0x00,
// h 0x68
	0x00,
	0x00,
	0x00,
	0x00,
	0x60,
	0x20,
	0x2C,
	0x32,
	0x22,
	0x22,
	0x22,
	0x77,
	0x00,
	0x00,
	0x00,
	0x00,
// i 0x69
	0x00,
	0x00,
	0x00,
	0x00,
	0x08,
	0x00,
	0x38,
	0x08,
	0x08,
	0x08,
	0x08,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// j 0x6A
	0x00,
	0x00,
	0x00,
	0x00,
	0x08,
	0x00,
	0x3C,
	0x04,
	0x04,
	0x04,
	0x04,
	0x04,
	0x04,
	0x78,
	0x00,
	0x00,
// k 0x6B
	0x00,
	0x00,
	0x00,
	0x00,
	0x60,
	0x20,
	0x2F,
	0x24,
	0x38,
	0x28,
	0x24,
	0x67,
	0x00,
	0x00,
	0x00,
	0x00,
// l 0x6C
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// m 0x6D
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0xD2,
	0x6D,
	0x49,
	0x49,
	0x49,
	0xED,
	0x00,
	0x00,
	0x00,
	0x00,
// n 0x6E
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x6C,
	0x32,
	0x22,
	0x22,
	0x22,
	0x77,
	0x00,
	0x00,
	0x00,
	0x00,
// o 0x6F
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x3C,
	0x42,
	0x42,
	0x42,
	0x42,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
// p 0x70
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x6C,
	0x32,
	0x22,
	0x22,
	0x22,
	0x3C,
	0x20,
	0x70,
	0x00,
	0x00,
// q 0x71
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x3B,
	0x46,
	0x42,
	0x42,
	0x46,
	0x3A,
	0x02,
	0x07,
	0x00,
	0x00,
// r 0x72
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x6E,
	0x30,
	0x20,
	0x20,
	0x20,
	0x7C,
	0x00,
	0x00,
	0x00,
	0x00,
// s 0x73
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x3E,
	0x42,
	0x3C,
	0x02,
	0x42,
	0x7C,
	0x00,
	0x00,
	0x00,
	0x00,
// t 0x74
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x20,
	0x7C,
	0x20,
	0x20,
	0x20,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// u 0x75
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x66,
	0x22,
	0x22,
	0x22,
	0x26,
	0x1B,
	0x00,
	0x00,
	0x00,
	0x00,
// v 0x76
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0xE7,
	0x42,
	0x24,
	0x24,
	0x18,
	0x18,
	0x00,
	0x00,
	0x00,
	0x00,
// w 0x77
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x77,
	0x22,
	0x2A,
	0x2A,
	0x2A,
	0x14,
	0x00,
	0x00,
	0x00,
	0x00,
// x 0x78
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x66,
	0x24,
	0x18,
	0x18,
	0x24,
	0x66,
	0x00,
	0x00,
	0x00,
	0x00,
// y 0x79
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x77,
	0x22,
	0x22,
	0x14,
	0x14,
	0x08,
	0x08,
	0x38,
	0x00,
	0x00,
// z 0x7A
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x3E,
	0x04,
	0x08,
	0x10,
	0x22,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// { 0x7B
	0x00,
	0x00,
	0x00,
	0x00,
	0x08,
	0x10,
	0x10,
	0x10,
	0x20,
	0x10,
	0x10,
	0x10,
	0x08,
	0x00,
	0x00,
	0x00,
// | 0x7C
	0x00,
	0x00,
	0x00,
	0x00,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x00,
	0x00,
// } 0x7D
	0x00,
	0x00,
	0x00,
	0x00,
	0x20,
	0x10,
	0x10,
	0x10,
	0x08,
	0x10,
	0x10,
	0x10,
	0x20,
	0x00,
	0x00,
	0x00,
// ~ 0x7E
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x32,
	0x4C,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x7F
	0x00,
	0x00,
	0x00,
	0x00,
	0xFE,
	0x82,
	0x82,
	0x82,
	0x82,
	0x82,
	0x82,
	0xFE,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x80
	0x00,
	0x00,
	0x00,
	0x00,
	0x3E,
	0x42,
	0xFC,
	0x40,
	0xFC,
	0x40,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x81
	0x00,
	0x00,
	0x00,
	0x00,
	0xFE,
	0x82,
	0x82,
	0x82,
	0x82,
	0x82,
	0x82,
	0xFE,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x82
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x10,
	0x30,
	0x20,
	0x00,
	0x00,
//  0x83
	0x00,
	0x00,
	0x00,
	0x00,
	0x07,
	0x08,
	0x08,
	0x3E,
	0x08,
	0x08,
	0x08,
	0x08,
	0x70,
	0x00,
	0x00,
	0x00,
//  0x84
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x12,
	0x24,
	0x48,
	0x00,
	0x00,
	0x00,
// 0x85
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x54,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x86
	0x00,
	0x00,
	0x00,
	0x00,
	0x10,
	0x10,
	0x7C,
	0x10,
	0x10,
	0x10,
	0x10,
	0x10,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x87
	0x00,
	0x00,
	0x00,
	0x00,
	0x10,
	0x10,
	0x7C,
	0x10,
	0x10,
	0x7C,
	0x10,
	0x10,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x88
	0x00,
	0x00,
	0x00,
	0x00,
	0x10,
	0x28,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x89
	0x00,
	0x00,
	0x00,
	0x00,
	0x40,
	0xA0,
	0x5C,
	0x70,
	0x00,
	0x12,
	0x2D,
	0x12,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x8A
	0x00,
	0x14,
	0x08,
	0x00,
	0x3A,
	0x46,
	0x40,
	0x3C,
	0x02,
	0x02,
	0x62,
	0x5C,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x8B
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x10,
	0x20,
	0x40,
	0x40,
	0x20,
	0x10,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x8C
	0x00,
	0x00,
	0x00,
	0x00,
	0x3F,
	0x49,
	0x8A,
	0x8E,
	0x8A,
	0x88,
	0x49,
	0x3F,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x8D
	0x00,
	0x00,
	0x00,
	0x00,
	0xFE,
	0x82,
	0x82,
	0x82,
	0x82,
	0x82,
	0x82,
	0xFE,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x8E
	0x00,
	0x14,
	0x08,
	0x00,
	0x3E,
	0x22,
	0x04,
	0x08,
	0x08,
	0x10,
	0x22,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x8F
	0x00,
	0x00,
	0x00,
	0x00,
	0xFE,
	0x82,
	0x82,
	0x82,
	0x82,
	0x82,
	0x82,
	0xFE,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x90
	0x00,
	0x00,
	0x00,
	0x00,
	0xFE,
	0x82,
	0x82,
	0x82,
	0x82,
	0x82,
	0x82,
	0xFE,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x91
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x08,
	0x0C,
	0x04,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x92
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x10,
	0x30,
	0x20,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x93
	0x00,
	0x00,
	0x00,
	0x00,
	0x48,
	0x24,
	0x12,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x94
	0x00,
	0x00,
	0x00,
	0x00,
	0x12,
	0x24,
	0x48,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x95
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x1C,
	0x1C,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x96
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x7E,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x97
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0xFF,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x98
	0x00,
	0x00,
	0x00,
	0x00,
	0x34,
	0x2C,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x99
	0x00,
	0x00,
	0x00,
	0x00,
	0xFB,
	0x5B,
	0x55,
	0xFB,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x9A
	0x00,
	0x00,
	0x00,
	0x14,
	0x08,
	0x00,
	0x3E,
	0x42,
	0x3C,
	0x02,
	0x42,
	0x7C,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x9B
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x08,
	0x04,
	0x02,
	0x02,
	0x04,
	0x08,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x9C
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x76,
	0x89,
	0x8F,
	0x88,
	0x88,
	0x77,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x9D
	0x00,
	0x00,
	0x00,
	0x00,
	0xFE,
	0x82,
	0x82,
	0x82,
	0x82,
	0x82,
	0x82,
	0xFE,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x9E
	0x00,
	0x00,
	0x00,
	0x14,
	0x08,
	0x00,
	0x3E,
	0x24,
	0x08,
	0x10,
	0x22,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
//  0x9F
	0x00,
	0x00,
	0x24,
	0x00,
	0x77,
	0x22,
	0x14,
	0x14,
	0x08,
	0x08,
	0x08,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
//   0xA0
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// ¡ 0xA1
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x08,
	0x00,
	0x00,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x00,
	0x00,
// ¢ 0xA2
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x08,
	0x1C,
	0x22,
	0x20,
	0x22,
	0x1C,
	0x08,
	0x00,
	0x00,
	0x00,
	0x00,
// £ 0xA3
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x24,
	0x20,
	0x20,
	0x78,
	0x10,
	0x22,
	0x7E,
	0x00,
	0x00,
	0x00,
	0x00,
// ¤ 0xA4
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x42,
	0x3C,
	0x24,
	0x24,
	0x3C,
	0x42,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// ¥ 0xA5
	0x00,
	0x00,
	0x00,
	0x00,
	0x77,
	0x22,
	0x14,
	0x3E,
	0x08,
	0x3E,
	0x08,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// ¦ 0xA6
	0x00,
	0x00,
	0x00,
	0x00,
	0x10,
	0x10,
	0x10,
	0x10,
	0x00,
	0x00,
	0x10,
	0x10,
	0x10,
	0x10,
	0x00,
	0x00,
// § 0xA7
	0x00,
	0x00,
	0x00,
	0x00,
	0x3E,
	0x22,
	0x70,
	0x48,
	0x66,
	0x12,
	0x0E,
	0x44,
	0x7C,
	0x00,
	0x00,
	0x00,
// ¨ 0xA8
	0x00,
	0x00,
	0x00,
	0x00,
	0x24,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// © 0xA9
	0x00,
	0x00,
	0x00,
	0x00,
	0x3C,
	0x42,
	0x9D,
	0xA1,
	0xA1,
	0x9D,
	0x42,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
// ª 0xAA
	0x00,
	0x00,
	0x00,
	0x00,
	0x38,
	0x08,
	0x38,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// « 0xAB
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x12,
	0x24,
	0x48,
	0x24,
	0x12,
	0x00,
	0x00,
	0x00,
	0x00,
// ¬ 0xAC
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0xFE,
	0x02,
	0x02,
	0x02,
	0x00,
	0x00,
	0x00,
	0x00,
// ­ 0xAD
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x7E,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// ® 0xAE
	0x00,
	0x00,
	0x00,
	0x00,
	0x3C,
	0x42,
	0xBD,
	0x95,
	0x99,
	0xBD,
	0x42,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
// ¯ 0xAF
	0x00,
	0x00,
	0xFF,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// ° 0xB0
	0x00,
	0x00,
	0x00,
	0x18,
	0x24,
	0x24,
	0x18,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// ± 0xB1
	0x00,
	0x00,
	0x00,
	0x00,
	0x08,
	0x08,
	0x08,
	0x7F,
	0x08,
	0x08,
	0x08,
	0x7F,
	0x00,
	0x00,
	0x00,
	0x00,
// ² 0xB2
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x28,
	0x10,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// ³ 0xB3
	0x00,
	0x00,
	0x00,
	0x00,
	0x3C,
	0x0C,
	0x04,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// ´ 0xB4
	0x00,
	0x00,
	0x00,
	0x00,
	0x08,
	0x10,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// µ 0xB5
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0xCC,
	0x44,
	0x44,
	0x44,
	0x4C,
	0x76,
	0x40,
	0x40,
	0x00,
	0x00,
// ¶ 0xB6
	0x00,
	0x00,
	0x00,
	0x00,
	0x3F,
	0x4A,
	0x4A,
	0x4A,
	0x3A,
	0x0A,
	0x0A,
	0x0A,
	0x3B,
	0x00,
	0x00,
	0x00,
// · 0xB7
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x18,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// ¸ 0xB8
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x08,
	0x04,
	0x1C,
	0x00,
	0x00,
// ¹ 0xB9
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x08,
	0x08,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// º 0xBA
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x24,
	0x24,
	0x18,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// » 0xBB
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x48,
	0x24,
	0x12,
	0x24,
	0x48,
	0x00,
	0x00,
	0x00,
	0x00,
// ¼ 0xBC
	0x00,
	0x00,
	0x00,
	0x00,
	0xC0,
	0x40,
	0x41,
	0x46,
	0xFE,
	0x2A,
	0x0F,
	0x07,
	0x00,
	0x00,
	0x00,
	0x00,
// ½ 0xBD
	0x00,
	0x00,
	0x00,
	0x00,
	0xC0,
	0x42,
	0x44,
	0xF8,
	0x27,
	0x41,
	0x02,
	0x07,
	0x00,
	0x00,
	0x00,
	0x00,
// ¾ 0xBE
	0x00,
	0x00,
	0x00,
	0x00,
	0xE0,
	0x61,
	0x22,
	0xE4,
	0x1E,
	0x2A,
	0x4F,
	0x07,
	0x00,
	0x00,
	0x00,
	0x00,
// ¿ 0xBF
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x00,
	0x10,
	0x20,
	0x40,
	0x40,
	0x44,
	0x38,
	0x00,
	0x00,
// À 0xC0
	0x00,
	0x10,
	0x08,
	0x00,
	0x18,
	0x08,
	0x14,
	0x14,
	0x14,
	0x1C,
	0x22,
	0x77,
	0x00,
	0x00,
	0x00,
	0x00,
// Á 0xC1
	0x00,
	0x04,
	0x08,
	0x00,
	0x18,
	0x08,
	0x14,
	0x14,
	0x14,
	0x1C,
	0x22,
	0x77,
	0x00,
	0x00,
	0x00,
	0x00,
// Â 0xC2
	0x00,
	0x08,
	0x14,
	0x00,
	0x18,
	0x08,
	0x14,
	0x14,
	0x14,
	0x1C,
	0x22,
	0x77,
	0x00,
	0x00,
	0x00,
	0x00,
// Ã 0xC3
	0x00,
	0x34,
	0x2C,
	0x00,
	0x18,
	0x08,
	0x14,
	0x14,
	0x14,
	0x1C,
	0x22,
	0x77,
	0x00,
	0x00,
	0x00,
	0x00,
// Ä 0xC4
	0x00,
	0x00,
	0x24,
	0x00,
	0x18,
	0x08,
	0x14,
	0x14,
	0x14,
	0x1C,
	0x22,
	0x77,
	0x00,
	0x00,
	0x00,
	0x00,
// Å 0xC5
	0x1C,
	0x14,
	0x1C,
	0x00,
	0x18,
	0x08,
	0x14,
	0x14,
	0x14,
	0x1C,
	0x22,
	0x77,
	0x00,
	0x00,
	0x00,
	0x00,
// Æ 0xC6
	0x00,
	0x00,
	0x00,
	0x00,
	0x3F,
	0x19,
	0x2A,
	0x2E,
	0x3A,
	0x48,
	0x49,
	0xFF,
	0x00,
	0x00,
	0x00,
	0x00,
// Ç 0xC7
	0x00,
	0x00,
	0x00,
	0x00,
	0x1E,
	0x22,
	0x40,
	0x40,
	0x40,
	0x40,
	0x22,
	0x1C,
	0x08,
	0x04,
	0x1C,
	0x00,
// È 0xC8
	0x00,
	0x10,
	0x08,
	0x00,
	0x7E,
	0x22,
	0x28,
	0x38,
	0x28,
	0x20,
	0x22,
	0x7E,
	0x00,
	0x00,
	0x00,
	0x00,
// É 0xC9
	0x00,
	0x04,
	0x08,
	0x00,
	0x7E,
	0x22,
	0x28,
	0x38,
	0x28,
	0x20,
	0x22,
	0x7E,
	0x00,
	0x00,
	0x00,
	0x00,
// Ê 0xCA
	0x00,
	0x08,
	0x14,
	0x00,
	0x7E,
	0x22,
	0x28,
	0x38,
	0x28,
	0x20,
	0x22,
	0x7E,
	0x00,
	0x00,
	0x00,
	0x00,
// Ë 0xCB
	0x00,
	0x00,
	0x24,
	0x00,
	0x7E,
	0x22,
	0x28,
	0x38,
	0x28,
	0x20,
	0x22,
	0x7E,
	0x00,
	0x00,
	0x00,
	0x00,
// Ì 0xCC
	0x00,
	0x10,
	0x08,
	0x00,
	0x3E,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// Í 0xCD
	0x00,
	0x04,
	0x08,
	0x00,
	0x3E,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// Î 0xCE
	0x00,
	0x08,
	0x14,
	0x00,
	0x3E,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// Ï 0xCF
	0x00,
	0x00,
	0x24,
	0x00,
	0x3E,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x08,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// Ð 0xD0
	0x00,
	0x00,
	0x00,
	0x00,
	0x78,
	0x24,
	0x22,
	0x72,
	0x22,
	0x22,
	0x24,
	0x78,
	0x00,
	0x00,
	0x00,
	0x00,
// Ñ 0xD1
	0x00,
	0x34,
	0x2C,
	0x00,
	0xE7,
	0x62,
	0x52,
	0x52,
	0x4A,
	0x4A,
	0x46,
	0xE6,
	0x00,
	0x00,
	0x00,
	0x00,
// Ò 0xD2
	0x00,
	0x00,
	0x08,
	0x04,
	0x1C,
	0x22,
	0x41,
	0x41,
	0x41,
	0x41,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// Ó 0xD3
	0x00,
	0x04,
	0x08,
	0x00,
	0x1C,
	0x22,
	0x41,
	0x41,
	0x41,
	0x41,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// Ô 0xD4
	0x00,
	0x08,
	0x14,
	0x00,
	0x1C,
	0x22,
	0x41,
	0x41,
	0x41,
	0x41,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// Õ 0xD5
	0x00,
	0x1A,
	0x16,
	0x00,
	0x1C,
	0x22,
	0x41,
	0x41,
	0x41,
	0x41,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// Ö 0xD6
	0x00,
	0x00,
	0x12,
	0x00,
	0x1C,
	0x22,
	0x41,
	0x41,
	0x41,
	0x41,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// × 0xD7
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x24,
	0x18,
	0x18,
	0x24,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
// Ø 0xD8
	0x00,
	0x00,
	0x00,
	0x00,
	0x1D,
	0x22,
	0x45,
	0x49,
	0x51,
	0x51,
	0x22,
	0x5C,
	0x00,
	0x00,
	0x00,
	0x00,
// Ù 0xD9
	0x00,
	0x10,
	0x08,
	0x00,
	0x77,
	0x22,
	0x22,
	0x22,
	0x22,
	0x22,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// Ú 0xDA
	0x00,
	0x04,
	0x08,
	0x00,
	0x77,
	0x22,
	0x22,
	0x22,
	0x22,
	0x22,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// Û 0xDB
	0x00,
	0x08,
	0x14,
	0x00,
	0x77,
	0x22,
	0x22,
	0x22,
	0x22,
	0x22,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// Ü 0xDC
	0x00,
	0x00,
	0x24,
	0x00,
	0x77,
	0x22,
	0x22,
	0x22,
	0x22,
	0x22,
	0x22,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// Ý 0xDD
	0x00,
	0x04,
	0x08,
	0x00,
	0x77,
	0x22,
	0x14,
	0x14,
	0x08,
	0x08,
	0x08,
	0x1C,
	0x00,
	0x00,
	0x00,
	0x00,
// Þ 0xDE
	0x00,
	0x00,
	0x00,
	0x00,
	0x70,
	0x20,
	0x3C,
	0x22,
	0x22,
	0x3C,
	0x20,
	0x70,
	0x00,
	0x00,
	0x00,
	0x00,
// ß 0xDF
	0x00,
	0x00,
	0x00,
	0x00,
	0x18,
	0x24,
	0x24,
	0x28,
	0x26,
	0x22,
	0x2A,
	0x7C,
	0x00,
	0x00,
	0x00,
	0x00,
// à 0xE0
	0x00,
	0x00,
	0x00,
	0x10,
	0x08,
	0x00,
	0x3C,
	0x42,
	0x3E,
	0x42,
	0x46,
	0x3B,
	0x00,
	0x00,
	0x00,
	0x00,
// á 0xE1
	0x00,
	0x00,
	0x00,
	0x04,
	0x08,
	0x00,
	0x3C,
	0x42,
	0x3E,
	0x42,
	0x46,
	0x3B,
	0x00,
	0x00,
	0x00,
	0x00,
// â 0xE2
	0x00,
	0x00,
	0x00,
	0x08,
	0x14,
	0x00,
	0x3C,
	0x42,
	0x3E,
	0x42,
	0x46,
	0x3B,
	0x00,
	0x00,
	0x00,
	0x00,
// ã 0xE3
	0x00,
	0x00,
	0x00,
	0x34,
	0x2C,
	0x00,
	0x3C,
	0x42,
	0x3E,
	0x42,
	0x46,
	0x3B,
	0x00,
	0x00,
	0x00,
	0x00,
// ä 0xE4
	0x00,
	0x00,
	0x00,
	0x00,
	0x24,
	0x00,
	0x3C,
	0x42,
	0x3E,
	0x42,
	0x46,
	0x3B,
	0x00,
	0x00,
	0x00,
	0x00,
// å 0xE5
	0x00,
	0x00,
	0x1C,
	0x14,
	0x1C,
	0x00,
	0x3C,
	0x42,
	0x3E,
	0x42,
	0x46,
	0x3B,
	0x00,
	0x00,
	0x00,
	0x00,
// æ 0xE6
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x6E,
	0x11,
	0x7F,
	0x90,
	0x90,
	0x7F,
	0x00,
	0x00,
	0x00,
	0x00,
// ç 0xE7
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x3A,
	0x46,
	0x40,
	0x40,
	0x42,
	0x3C,
	0x08,
	0x04,
	0x1C,
	0x00,
// è 0xE8
	0x00,
	0x00,
	0x00,
	0x10,
	0x08,
	0x00,
	0x3C,
	0x42,
	0x7E,
	0x40,
	0x40,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// é 0xE9
	0x00,
	0x00,
	0x00,
	0x08,
	0x10,
	0x00,
	0x3C,
	0x42,
	0x7E,
	0x40,
	0x40,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// ê 0xEA
	0x00,
	0x00,
	0x00,
	0x08,
	0x14,
	0x00,
	0x3C,
	0x42,
	0x7E,
	0x40,
	0x40,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// ë 0xEB
	0x00,
	0x00,
	0x00,
	0x00,
	0x24,
	0x00,
	0x3C,
	0x42,
	0x7E,
	0x40,
	0x40,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// ì 0xEC
	0x00,
	0x00,
	0x00,
	0x10,
	0x08,
	0x00,
	0x38,
	0x08,
	0x08,
	0x08,
	0x08,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// í 0xED
	0x00,
	0x00,
	0x00,
	0x08,
	0x10,
	0x00,
	0x38,
	0x08,
	0x08,
	0x08,
	0x08,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// î 0xEE
	0x00,
	0x00,
	0x00,
	0x08,
	0x14,
	0x00,
	0x38,
	0x08,
	0x08,
	0x08,
	0x08,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// ï 0xEF
	0x00,
	0x00,
	0x00,
	0x00,
	0x24,
	0x00,
	0x38,
	0x08,
	0x08,
	0x08,
	0x08,
	0x3E,
	0x00,
	0x00,
	0x00,
	0x00,
// ð 0xF0
	0x00,
	0x00,
	0x00,
	0x00,
	0x14,
	0x18,
	0x24,
	0x3A,
	0x46,
	0x42,
	0x42,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
// ñ 0xF1
	0x00,
	0x00,
	0x00,
	0x34,
	0x2C,
	0x00,
	0x6C,
	0x32,
	0x22,
	0x22,
	0x22,
	0x77,
	0x00,
	0x00,
	0x00,
	0x00,
// ò 0xF2
	0x00,
	0x00,
	0x00,
	0x10,
	0x08,
	0x00,
	0x3C,
	0x42,
	0x42,
	0x42,
	0x42,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
// ó 0xF3
	0x00,
	0x00,
	0x00,
	0x08,
	0x10,
	0x00,
	0x3C,
	0x42,
	0x42,
	0x42,
	0x42,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
// ô 0xF4
	0x00,
	0x00,
	0x00,
	0x08,
	0x14,
	0x00,
	0x3C,
	0x42,
	0x42,
	0x42,
	0x42,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
// õ 0xF5
	0x00,
	0x00,
	0x00,
	0x34,
	0x2C,
	0x00,
	0x3C,
	0x42,
	0x42,
	0x42,
	0x42,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
// ö 0xF6
	0x00,
	0x00,
	0x00,
	0x00,
	0x24,
	0x00,
	0x3C,
	0x42,
	0x42,
	0x42,
	0x42,
	0x3C,
	0x00,
	0x00,
	0x00,
	0x00,
// ÷ 0xF7
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x10,
	0x00,
	0x00,
	0x7C,
	0x00,
	0x00,
	0x10,
	0x00,
	0x00,
	0x00,
	0x00,
// ø 0xF8
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x3A,
	0x44,
	0x4A,
	0x52,
	0x22,
	0x5C,
	0x00,
	0x00,
	0x00,
	0x00,
// ù 0xF9
	0x00,
	0x00,
	0x00,
	0x10,
	0x08,
	0x00,
	0x66,
	0x22,
	0x22,
	0x22,
	0x26,
	0x1B,
	0x00,
	0x00,
	0x00,
	0x00,
// ú 0xFA
	0x00,
	0x00,
	0x00,
	0x04,
	0x08,
	0x00,
	0x66,
	0x22,
	0x22,
	0x22,
	0x26,
	0x1B,
	0x00,
	0x00,
	0x00,
	0x00,
// û 0xFB
	0x00,
	0x00,
	0x00,
	0x08,
	0x14,
	0x00,
	0x66,
	0x22,
	0x22,
	0x22,
	0x26,
	0x1B,
	0x00,
	0x00,
	0x00,
	0x00,
// ü 0xFC
	0x00,
	0x00,
	0x00,
	0x00,
	0x24,
	0x00,
	0x66,
	0x22,
	0x22,
	0x22,
	0x26,
	0x1B,
	0x00,
	0x00,
	0x00,
	0x00,
// ý 0xFD
	0x00,
	0x00,
	0x00,
	0x04,
	0x08,
	0x00,
	0x77,
	0x22,
	0x22,
	0x14,
	0x14,
	0x08,
	0x08,
	0x38,
	0x00,
	0x00,
// þ 0xFE
	0x00,
	0x00,
	0x00,
	0x00,
	0xC0,
	0x40,
	0x5C,
	0x62,
	0x42,
	0x42,
	0x62,
	0x5C,
	0x40,
	0xE0,
	0x00,
	0x00,
// ÿ 0xFF
	0x00,
	0x00,
	0x00,
	0x00,
	0x12,
	0x00,
	0x77,
	0x22,
	0x22,
	0x14,
	0x14,
	0x08,
	0x08,
	0x38,
	0x00,
	0x00,
};
inline void pasteChar8_162(const unsigned char* char_bits, unsigned color, int x, int y, int dstWidth, int dstHeight, int* dstPix){
	for(int r=16; --r>=0;){
		char ch=char_bits[r];
		for(int b=0x80, a=0; b; a++){
			if(b&ch){
				plotPixel2(color, x+a, y+r, dstWidth, dstHeight, dstPix);
			}
			b>>=1;
		}
	}
}
void plotChar2(unsigned char charecter, unsigned color, int x, int y, int dstWidth, int dstHeight, int* dstPix){
	if(charecter<=' ' || charecter-' '>=sizeof(courierLookUp)/sizeof(const unsigned char)*16){
		return;
	}
	const unsigned char* charPtr = &courierLookUp[(charecter-'!')*16];
	pasteChar8_162(charPtr, color, x,y-11, dstWidth, dstHeight, dstPix);
}
void plotPixel2(unsigned color, int x, int y, int dstWidth, int dstHeight, int* pixBuf){
	if((unsigned)x>=(unsigned)dstWidth || (unsigned)y>=(unsigned)dstHeight){
		return;
	}
	pixBuf[y*dstWidth+x]=color;
}
void drawSouthEast(unsigned color, int x, int from, int to, int dstWidth, int dstHeight, int* pixBuf){
	int diff=to-from;
	if((unsigned)x>=(unsigned)dstWidth || x + diff >= dstWidth){
		return;
	}
	if(from<0){
		from=0;
	}
	else if(from>=dstHeight){
		return;
	}
	if(to<0){
		return;
	}
	if(to>=dstHeight){
		to=dstHeight-1;
	}
	unsigned* p=(unsigned*)pixBuf+(dstHeight-from-1)*dstWidth+x;
	for(int i=to-from+1; --i>=0;){
		*p=color;
		p-=dstWidth-1;
	}
}
void drawSouthWest(unsigned color, int x, int from, int to, int dstWidth, int dstHeight, int* pixBuf){
	int diff=to-from;
	if((unsigned)x>=(unsigned)dstWidth || x-diff<0){
		return;
	}
	if(from<0){
		from=0;
	}
	else if(from>=dstHeight){
		return;
	}
	if(to<0){
		return;
	}
	if(to>=dstHeight){
		to=dstHeight-1;
	}
	unsigned* p=(unsigned*)pixBuf+(dstHeight-from-1)*dstWidth+x;
	for(int i=to-from+1; --i>=0;){
		*p=color;
		p-=dstWidth+1;
	}
}
void drawTextR(const String& str, unsigned color, int dx, int dy, int dstWidth, int dstHeight, int* dstPix){
	for(int i=str.length; --i>=0;){
		plotChar2(str[i], color, dx, dy, dstWidth, dstHeight, dstPix);
		dx -= charWidth;
	}
}
void drawText2(const String& str, unsigned color, Vec2i dstPos, int dstWidth, int dstHeight, int* dstPix){
	for(int i=0; i<str.length; i++){
		plotChar2(str[i], color, dstPos.x, dstPos.y, dstWidth, dstHeight, dstPix);
		dstPos.x += charWidth;
	}
}
/*
void drawText(const String& str, unsigned color, int dx, int dy, int dstWidth, int dstHeight, int* dstPix){
	for(int i=0; i<str.length; i++){
		plotChar(str[i], color, dx, dy, dstWidth, dstHeight, dstPix);
		dx += charWidth;
	}
}
void drawText(const String& str, int fromChar, int toChar, unsigned color, int dx, int dy, int dstWidth, int dstHeight, int* dstPix){
	for(int i=fromChar; i<toChar; i++){
		plotChar(str[i], color, dx, dy, dstWidth, dstHeight, dstPix);
		dx += charWidth;
	}
}
*/
void drawText2(const String& str, int fromChar, int toChar, unsigned color, Vec2i dstPos, int dstWidth, int dstHeight, int* dstPix){
	for(int i=fromChar; i<toChar; i++){
		plotChar2(str[i], color, dstPos.x, dstPos.y, dstWidth, dstHeight, dstPix);
		dstPos.x += charWidth;
	}
}
void changePixelColorInHorizontalLine(unsigned fromColor, unsigned toColor, unsigned y, unsigned from, unsigned to, unsigned dstWidth, unsigned dstHeight, int* pixBuf){
	if(y>=dstHeight||from>=dstWidth)return;
	if(to>=dstWidth){
		to=dstWidth-1;
	}

	unsigned* p=(unsigned*)pixBuf+y*dstWidth+from;
	for(int i=to-from; --i>=0;){
		if(*p==fromColor){
			*p=toColor;
		}
		p++;
	}
}
void changePixelColorInBlock(unsigned fromColor, unsigned toColor, int tx, int ty, int bx, int by, int dstWidth, int dstHeight, int* pixBuf){
	for(int i=by-ty; --i>=0; ){
		changePixelColorInHorizontalLine(fromColor, toColor, ty+i, tx, bx, dstWidth, dstHeight, pixBuf);
	}
}
void invertedHorizontalLine(unsigned y, unsigned from, unsigned to, unsigned dstWidth, unsigned dstHeight, int* pixBuf){
	if(y>=dstHeight||from>=dstWidth)return;
	if(to>=dstWidth){
		to=dstWidth;
	}

	unsigned* p=(unsigned*)pixBuf+y*dstWidth+from;
	for(int i=to-from; --i>=0;){
		*p=~(*p);
		p++;
	}
}
void invertBlock( int tx, int ty, int bx, int by, int dstWidth, int dstHeight, int* pixBuf){
	for(int i=by-ty; --i>=0; ){
		invertedHorizontalLine(ty+i, tx, bx, dstWidth, dstHeight, pixBuf);
	}
}
void paintBlock( unsigned color, int tx, int ty, int bx, int by, int dstWidth, int dstHeight, int* pixBuf){
	for(int i=by-ty; --i>=0; ){
		drawHorizontalLine(color, ty+i, tx, bx, dstWidth, dstHeight, pixBuf);
	}
}
void paintBlock2( unsigned color, int tx, int ty, int bx, int by, int dstWidth, int dstHeight, int* pixBuf){
	for(int i = by - ty; --i>=0; ){
		drawHorizontalLine2(color, ty + i, tx, bx, dstWidth, dstHeight, pixBuf);
	}
}
/*
const unsigned char verdanaChar_A[]={
	east| 1,
	down,
	north| 1,
	northEast| 1,
	north| 2,
	northEast| 1,
	north| 1,
	northEast| 1,
	north| 1,
	up,

	south | 8,
	east | 3,
	down,
	north| 1,
	northWest| 1,
	north| 2,
	northWest| 1,
	north| 1,
	up,
	south | 4,
	down,
	west | 2,
	up,
	east | 6,
	south | 2,
	end

};
const unsigned char verdanaChar_B[]={
	east|5,
	down,
	west | 4,
	north | 8,
	east | 3,
	southEast | 1,
	south | 3,
	west | 3,
	east | 3,
	southEast | 1,
	south | 2,
	up,
	east | 2,
	south | 1,
	end
};
const unsigned char verdanaChar_C[]={
	east|7,
	north | 1,
	down,
	southWest | 1,
	west | 3,
	northWest | 2,
	north | 4,
	northEast | 2,
	east | 3,
	southEast | 1,
	up,
	south | 7,
	east | 2,
	end
};
const unsigned char verdanaChar_D[]={
	east|6,
	north | 1,
	down,
	southWest | 1,
	west | 4,
	north | 8,
	east | 4,
	southEast | 2,
	south | 4,
	up,
	south | 2,
	east | 2,
	end
};

const unsigned char verdanaChar_E[]={
	east|6,
	down,
	west|5,
	north|4,
	east|5,
	west|5,
	north|4,
	east|5,
	up,
	east|2,
	south|8,
	end
};
*/
void drawHorizontalLine(unsigned color, unsigned y, unsigned from, unsigned to, unsigned dstWidth, unsigned dstHeight, int* pixBuf){
	if(y>=dstHeight||from>=dstWidth)return;
	if(to>=dstWidth){
		to=dstWidth-1;
	}

	unsigned* p=(unsigned*)pixBuf+(dstHeight-y-1)*dstWidth+from;
	for(int i=to-from; --i>=0;){
		*p=color;
		p++;
	}
}
void drawVerticalLine(unsigned color, int x, int from, int to, int dstWidth, int dstHeight, int* pixBuf){
	if((unsigned)x>=(unsigned)dstWidth){
		return;
	}
	if(from < 0){
		from = 0;
	}
	else if(from>=dstHeight){
		return;
	}
	if(to < 0){
		return;
	}
	if(to>=dstHeight){
		to = dstHeight - 1;
	}
	unsigned* p=(unsigned*)pixBuf+(dstHeight-from-1)*dstWidth+x;
	for(int i = to - from; --i >= 0;){
		*p = color;
		p -= dstWidth;
	}
}
void drawHorizontalLine2(unsigned color, int y, int from, int to, int dstWidth, int dstHeight, int* pixBuf){
	if(static_cast<unsigned>(y) >= static_cast<unsigned>(dstHeight) || from >= dstWidth){
		return;
	}
	if(from < 0){
		from = 0;
	}
	if(to >= dstWidth){
		to = dstWidth;
	}

	unsigned* p=(unsigned*) pixBuf + y * dstWidth + from;
	for(int i = to - from; --i >= 0;){
		*p = color;
		p++;
	}
}
void drawVerticalLine2(unsigned color, int x, int from, int to, int dstWidth, int dstHeight, int* pixBuf){
	if((unsigned)x>=(unsigned)dstWidth){
		return;
	}
	if(from<0){
		from=0;
	}
	else if(from>=dstHeight){
		return;
	}
	if(to<0){
		return;
	}
	if(to>=dstHeight){
		to=dstHeight-1;
	}
	unsigned* p=(unsigned*)pixBuf+from*dstWidth+x;
	for(int i=to-from; --i>=0;){
		*p=color;
		p+=dstWidth;
	}
}

void line(unsigned color, int fromX, int fromY, int toX, int toY, int dstWidth, int dstHeight, int* pixBuf){
	if(fromX < 0 || fromX >= dstWidth){
		return;
	}
	if(toX < 0 || toX >= dstWidth){
		return;
	}
	if(fromY < 0 || fromY >= dstHeight){
		return;
	}
	if(toY < 0 || toY >= dstHeight){
		return;
	}
	float diffX = toX - fromX;
	float diffY = toY - fromY;

	if(abs(diffX) > abs(diffY)){
		int steps = abs(toX - fromX);
		float stepYAmount = diffY / diffX;
		float stepY = 0;
		float halfPixel = (stepYAmount > 0) ? .5 : -.5;

		unsigned* p = (unsigned*)pixBuf + fromY * dstWidth + fromX;
		for(int i = 0; i <= steps; i++){
			int yOffset = static_cast<int>(stepY + halfPixel) * dstWidth;
			p[i + yOffset] = color;
			stepY += stepYAmount;
		}
	}
	else{
		int steps = toY - fromY;
		int dir = (steps < 0) ? -1: 1;
		steps = abs(steps);

		float stepXAmount = diffX / abs(diffY);
		float stepX = 0;
		float halfPixel = (stepXAmount > 0) ? .5 : -.5;

		unsigned* p = (unsigned*)pixBuf + fromY * dstWidth + fromX;
		for(int i = 0, y = 0; i <= steps; i++){
			int yOffset = static_cast<int>(stepX + halfPixel);
			p[y * dstWidth + yOffset] = color;
			stepX += stepXAmount;
			y += dir;
		}
	}
}

void pasteSprite( int dx, int dy, int sx, int sy, int dstWidth, int dstHeight, int srcWidth, int srcHeight, int iconWidth, int iconHeight, int* dstPix){

	if(dx>dstWidth-iconWidth){
		if(dx>=dstWidth){
			return;
		}
		iconWidth=dstWidth-dx;
	}
	if(dy>dstHeight-iconHeight){
		if(dy>=dstHeight){
			return;
		}
		iconHeight=dstHeight-dy;
	}
	else if(dy<0){
		sy-=dy;
		iconHeight+=dy;
		if(iconHeight<=0){
			return;
		}
		dy=0;
	}
	unsigned dstP=(dstHeight-dy)*dstWidth+dx;
	for(int r=iconHeight; --r>=0;){
		dstP-=dstWidth;
		for(int c=0; c<iconWidth; c++){
			unsigned ipix=iconSrcBuf[(srcHeight-iconHeight-sy+r)*srcWidth+sx+c];
			if(ipix!=0xffffff){
				dstPix[dstP+c]=ipix;
			}
		}
	}
}

void pasteSprite( int dx, int dy, int sx, int sy, int dstWidth, int dstHeight, int srcWidth, int srcHeight, int iconWidth, int iconHeight, Pix24* dstPix){

	if(dx>dstWidth - iconWidth){
		if(dx >= dstWidth){
			return;
		}
		iconWidth = dstWidth - dx;
	}
	if(dy>dstHeight - iconHeight){
		if(dy >= dstHeight){
			return;
		}
		iconHeight = dstHeight - dy;
	}
	else if(dy < 0){
		sy -= dy;
		iconHeight += dy;
		if(iconHeight <= 0){
			return;
		}
		dy=0;
	}
	unsigned dstP = (dy) * dstWidth+dx;
	for(int r=iconHeight; --r>=0;){
		for(int c = 0; c < iconWidth; c++){
			unsigned ipix = iconSrcBuf[(srcHeight - iconHeight - sy + r) * srcWidth + sx + c];

			unsigned alpha = ipix >> 24;
			unsigned ialpha = 0xff - alpha;

			Pix24 bkgrnd = dstPix[dstP + c];

			unsigned f0 = (ipix & 0x0ff) * alpha;
			unsigned f1 = ((ipix >> 8) & 0x0ff) * alpha;
			unsigned f2 = ((ipix >> 16) & 0x0ff) * alpha;

			unsigned b0 = bkgrnd.b * ialpha;
			unsigned b1 = bkgrnd.g * ialpha;
			unsigned b2 = bkgrnd.r * ialpha;

			f0 += b0;
			f1 += b1;
			f2 += b2;

			f0 >>= 8;
			f1 >>= 8;
			f2 >>= 8;

			dstPix[dstP + c] = Pix24(f2, f1, f0);
		}
		dstP += dstWidth;
	}
}


void pasteRectangle2(unsigned color, int dx, int dy, int dstWidth, int dstHeight, int iconWidth, int iconHeight, int* dstPix){

	if(dx>dstWidth-iconWidth){
		if(dx>=dstWidth){
			return;
		}
		iconWidth=dstWidth-dx;
	}
	if(dy>dstHeight-iconHeight){
		if(dy>=dstHeight){
			return;
		}
		iconHeight=dstHeight-dy;
	}
	else if(dy<0){
		iconHeight+=dy;
		if(iconHeight<=0){
			return;
		}
		dy=0;
	}
	unsigned dstP=(dy)*dstWidth+dx;
	for(int r=iconHeight; --r>=0;){
		for(int c=0; c<iconWidth; c++){
			dstPix[dstP+c]=color;
		}
		dstP+=dstWidth;
	}
}
void pasteSprite2(int dx, int dy, int sx, int sy, int dstWidth, int dstHeight, int srcWidth, int srcHeight, int iconWidth, int iconHeight, int* dstPix){

	if(dx > dstWidth - iconWidth){
		if(dx >= dstWidth){
			return;
		}
		iconWidth = dstWidth - dx;
	}
	else if(dx < 0){
		sx -= dx;
		iconWidth += dx;
		if(iconWidth <= 0){
			return;
		}
		dx = 0;
	}
	if(dy>dstHeight-iconHeight){
		if(dy>=dstHeight){
			return;
		}
		iconHeight=dstHeight-dy;
	}
	else if(dy < 0){
		sy -= dy;
		iconHeight += dy;
		if(iconHeight <= 0){
			return;
		}
		dy = 0;
	}
	unsigned dstP = (dy) * dstWidth+dx;
	for(int r = iconHeight; --r>=0;){
		for(int c = 0; c < iconWidth; c++){
			unsigned ipix = iconSrcBuf[(srcHeight - iconHeight - sy + r) * srcWidth + sx + c];

			unsigned alpha = ipix >> 24;
			unsigned ialpha = 0xff - alpha;

			unsigned bkgrnd = dstPix[dstP + c];

			unsigned f0 = (ipix & 0x0ff) * alpha;
			unsigned f1 = ((ipix >> 8) & 0x0ff) * alpha;
			unsigned f2 = ((ipix >> 16) & 0x0ff) * alpha;

			unsigned b0 = (bkgrnd & 0x0ff) * ialpha;
			unsigned b1 = ((bkgrnd >> 8) & 0x0ff) * ialpha;
			unsigned b2 = ((bkgrnd >> 16) & 0x0ff) * ialpha;

			f0 += b0;
			f1 += b1;
			f2 += b2;

			f0 >>= 8;
			f1 >>= 8;
			f2 >>= 8;

			f1 <<= 8;
			f2 <<= 16;

			f0 += f1;
			f0 += f2;
////////////////

			//unsigned f0 = (ipix & 0x000000ff) * alpha;
			//unsigned f1 = (ipix & 0x0000ff00) * alpha;
			//unsigned f2 = (ipix & 0x00ff0000) * alpha;

			//unsigned b0 = (bkgrnd & 0x000000ff) * ialpha;
			//unsigned b1 = (bkgrnd & 0x0000ff00) * ialpha;
			//unsigned b2 = (bkgrnd & 0x00ff0000) * ialpha;


			//f0 = f0 & 0x0000ff00;
			//f1 = f1 & 0x00ff0000;
			//f2 = f2 & 0xff000000;

			//b0 = b0 & 0x0000ff00;
			//b1 = b1 & 0x00ff0000;
			//b2 = b2 & 0xff000000;

			//f0 = (f0 + f1 + f2 + b0 + b1 + b2) >> 8;

////
			//unsigned f0 = (ipix & 0x00ffffff) * alpha;
			//unsigned b0 = (bkgrnd & 0x00ffffff) * ialpha;
			//f0 = (f0 + b0) >> 8;

			dstPix[dstP + c] = f0;
		}
		dstP += dstWidth;
	}
}



void pasteRectangle(unsigned color, int dx, int dy, int dstWidth, int dstHeight, int iconWidth, int iconHeight, int* dstPix){

	if(dx>dstWidth-iconWidth){
		if(dx>=dstWidth){
			return;
		}
		iconWidth=dstWidth-dx;
	}
	if(dy>dstHeight-iconHeight){
		if(dy>=dstHeight){
			return;
		}
		iconHeight=dstHeight-dy;
	}
	else if(dy<0){
		iconHeight+=dy;
		if(iconHeight<=0){
			return;
		}
		dy=0;
	}
	unsigned dstP=(dstHeight-dy)*dstWidth+dx;
	for(int r=iconHeight; --r>=0;){
		dstP-=dstWidth;
		for(int c=0; c<iconWidth; c++){
			dstPix[dstP+c]=color;
		}
	}
}
bool CaretWindow::deleteHighlight(){
	if(caretPos<highlightPos){
		cellStr=cellStr.replace(caretPos, highlightPos-caretPos, "");
		highlightPos=caretPos;
		//onChange();
		return true;
	}
	else if(caretPos > highlightPos){
		cellStr=cellStr.replace(highlightPos, caretPos-highlightPos, "");
		caretPos = highlightPos;
		updateCaret();
		//onChange();
		return true;
	}
	return false;
}
bool TextCell::deleteHighlight(){
	if(caretPos<highlightPos){
		str=str.replace(caretPos, highlightPos-caretPos, "");
		highlightPos=caretPos;
		return true;
	}
	else if(caretPos > highlightPos){
		str=str.replace(highlightPos, caretPos-highlightPos, "");
		caretPos = highlightPos;
		//updateCaret();
		return true;
	}
	return false;
}
void CaretWindow::decCaretPos(){
	if(caretPos>0){
		caretPos--;
	}
	updateCaret();
	win.paint();
}
void CaretWindow::incCaretPos(){
	if(caretPos<cellStr.length){
		caretPos++;
	}
	updateCaret();
	win.paint();
}
void CaretWindow::decCaretPosNH(){
//	bool flag=caretPos!=highlightPos;
	if(caretPos>0){
		caretPos--;
	}
	highlightPos=caretPos;
	updateCaret();
//	if(flag){
		win.paint();
//	}
}
void CaretWindow::incCaretPosNH(){
//	bool flag=caretPos!=highlightPos;
	if(caretPos<cellStr.length){
		caretPos++;
	}
	highlightPos=caretPos;
	updateCaret();
//	if(flag){
		win.paint();
//	}
}
void CaretWindow::doHome(){
	caretPos=highlightPos=0;
	updateCaret();
	win.paint();
}
void CaretWindow::doEnd(){
	caretPos=highlightPos=cellStr.length;
	updateCaret();
	win.paint();
}
void CaretWindow::setVisCharLen(){
	if(caretPos<firstVisChar){
		firstVisChar=caretPos;
		return;
	}
	int visCharLen = cellSize.x / charWidth;
	if(caretPos >= firstVisChar + visCharLen){
		firstVisChar = caretPos - visCharLen;
	}
};
void CaretWindow::updateCaret(){
	if(caretPos >  cellStr.length){
		caretPos=cellStr.length;
	}
	if( highlightPos > cellStr.length){
		highlightPos=cellStr.length;
	}
	if(caretPos<0){
		caretPos=0;
	}
	if(highlightPos<0){
		highlightPos=0;
	}
	setVisCharLen();
}
void CaretWindow::doHomeShift(){
	caretPos=0;
	updateCaret();
	win.paint();
}
void CaretWindow::doEndShift(){
	caretPos=cellStr.length;
	updateCaret();
	win.paint();
}
void CaretWindow::doDelete(){
	if(deleteHighlight()){
		onChange();
		win.paint();
	}
	else if(caretPos < cellStr.length){
		cellStr.deleteChar(caretPos);
		onChange();
		win.paint();
	}
}
void CaretWindow::doDeleteShift(){
	cellStr="";
	updateCaret();
	onChange();
	win.paint();
}

void TextCell::incCaretPos(){
}
void TextCell::decCaretPos(){
	if(caretPos>0){
		caretPos--;
	}
//	SendMessage(hWnd, WM_PAINT, 0, 0);
//	updateCaret();
}
void TextCell::incCaretPosNH(){
	bool flag=caretPos!=highlightPos;
	if(caretPos<str.length){
		caretPos++;
	}
	highlightPos=caretPos;
//	updateCaret();
	if(flag){
//		SendMessage(hWnd, WM_PAINT, 0, 0);
	}
}
void TextCell::decCaretPosNH(){
	bool flag=caretPos!=highlightPos;
	if(caretPos>0){
		caretPos--;
	}
	highlightPos=caretPos;
//	updateCaret();
	if(flag){
//		SendMessage(hWnd, WM_PAINT, 0, 0);
	}
}

void CaretWindow::paintCellStr(){
	const int visCharLen = cellSize.x / charWidth;
	int lastChar = cellStr.length;
	if(lastChar - firstVisChar > visCharLen){
		lastChar = firstVisChar + visCharLen;
	}
	Vec2i textPos = this->cellPos;
	textPos.y += 11;
	drawText2(cellStr, firstVisChar, lastChar, 0, textPos, win.size.x, win.size.y, win.pixBuf.getArray());
	highLightBlock();
}
////////
void insertShort(M_<char>& buf, short data);
void insertInt32(M_<char>& buf, int data);

void createBMP24(String filename, int width, int height, M_<Pix24>& pixels){
	M_<char> header;
	const int header_size = 0x36;
	const int file_size = height * width * sizeof(Pix24) + header_size;
	header += 'B';
	header += 'M';
	insertInt32(header, file_size);
	header.length += 4; //reserved
	insertInt32(header, header_size); //pixels offset
	insertInt32(header, 40); //the size of this header 
	insertInt32(header, width);
	insertInt32(header, height);
	insertShort(header, 1); //number of color planes being used.
	insertShort(header, 24); //number of bits per pixel
	insertInt32(header, 0); //Compression method none;
	insertInt32(header, height * width * sizeof(Pix24)); //The image size field;
	insertInt32(header, 256); //horizontal resolution of the image. (pixel per meter, signed integer)
	insertInt32(header, 256); //vertical resolution of the image. (pixel per meter, signed integer)
	header.length += 8;

	FILE* stream = fopen(filename.c_str(), "w+b");
	fwrite(header.getArray(), header_size, 1, stream);
	fwrite(pixels.getArray(), pixels.length, sizeof(Pix24), stream);
	fclose(stream);
}
void convertRGB_to_BGR(M_<Pix24>& pixels){
	for(int i = pixels.length; --i>=0;){
		char r = pixels[i].r;
		pixels[i].r = pixels[i].b;
		pixels[i].b = r;
	}
}
void convertRGB_to_BGR(M_<Pix32>& pixels){
	for(int i = pixels.length; --i>=0;){
		Pix32 pix32 = pixels[i];
		pixels[i].r = pix32.a;
		pixels[i].g = pix32.b;
		pixels[i].b = pix32.g;
		pixels[i].a = pix32.r;
	}
}
void _convert_raw_bitmap_to_cpp(){
	M_<Pix24> pixels;
	int width = iconSrcBufCX;
	int height = iconSrcBufCY;
	pixels.length = width * height;

	FILE* stream;
	stream = fopen("C:\\Documents and Settings\\thyme\\My Documents\\My Pictures\\toolbar.raw","r+b");
	fread(pixels.getArray(), iconSrcBufCX*iconSrcBufCY, 3, stream);
	fclose(stream);

	convertRGB_to_BGR(pixels);

	createBMP24("myfile.bmp", width, height, pixels);

}

void convert_raw_bitmap_to_cpp(){
	int headerSize = 0x36;
	//int headerSize = 122; //gimp
	unsigned char* raw=new unsigned char[3 * iconSrcBufCX * iconSrcBufCY + headerSize];
	FILE* stream;
	stream = fopen("C:\\Documents and Settings\\thyme\\My Documents\\My Pictures\\toolbar44.bmp","r+b");
	fread(raw, iconSrcBufCX * iconSrcBufCY + headerSize, 3, stream);
	fclose(stream);
	raw += headerSize;

	M_<Pix24> pixels;
	int width = iconSrcBufCX;
	int height = iconSrcBufCY;
	pixels.length = width * height;

	for(int i = pixels.length; --i>=0;){
		pixels[i] = *reinterpret_cast<Pix24*>(raw + i * 3);
	}
	convertRGB_to_BGR(pixels);
	raw = reinterpret_cast<unsigned char*>(pixels.getArray());

	stream = fopen("seamless_icons.cpp","w+t");
	fputs("const unsigned iconSrcBuf[]={\n", stream);

	int newLineCntr=20;

	for(int i = 0; i < iconSrcBufCX*iconSrcBufCY; i++){
		unsigned int pixel = 0;
		pixel|=*raw;
		raw++;
		pixel<<=8;
		pixel|=*raw;
		raw++;
		pixel<<=8;
		pixel|=*raw;
		raw++;
		fprintf(stream, "0x%x,", pixel);
		if(--newLineCntr==0){
			newLineCntr=20;
			fputs("\n", stream);
		}
	}
	fputs("};\n", stream);
	fclose(stream);
}

void convert_raw_alpha_bitmap_to_cpp(){
	int headerSize = 0x36; //0x36 == 54
	unsigned char* raw = new unsigned char[4 * iconSrcBufCX * iconSrcBufCY + headerSize];
	FILE* stream;
	stream = fopen("F:\\Pictures\\toolbar\\toolbar-v.bmp","r+b");
	//stream = fopen("F:\\Pictures\\toolbar\\toolbar-h.bmp","r+b");
	//int length = fread(raw, 4, 100, stream);
	fread(raw, 4, iconSrcBufCX * iconSrcBufCY + headerSize, stream);
	fclose(stream);
	raw += headerSize;

	M_<Pix32> pixels;
	int width = iconSrcBufCX;
	int height = iconSrcBufCY;
	pixels.length = width * height;

	for(int i = pixels.length; --i>=0;){
		pixels[i] = *reinterpret_cast<Pix32*>(raw + i * 4);
	}
	convertRGB_to_BGR(pixels);
	raw = reinterpret_cast<unsigned char*>(pixels.getArray());

	stream = fopen("seamless_icons.cpp","w+t");
	fputs("const unsigned iconSrcBuf[]={\n", stream);

	int newLineCntr=20;

	for(int i = 0; i < iconSrcBufCX * iconSrcBufCY; i++){
		unsigned int pixel = 0;
		pixel |= *raw;
		raw++;
		pixel <<= 8;
		pixel |= *raw;
		raw++;
		pixel <<= 8;
		pixel |= *raw;
		raw++;
		pixel <<= 8;
		pixel |= *raw;
		raw++;
		fprintf(stream, "0x%x,", pixel);
		if(--newLineCntr==0){
			newLineCntr=20;
			fputs("\n", stream);
		}
	}
	fputs("};\n", stream);
	fclose(stream);
}

Window::~Window(){
	if(workSurface){
		releaseWindow();
	}
}
void Window::reInitAllVisable(){
	for(int i=visWinList.length; --i>=0;){
		visWinList[i]->initWindow();
	}
}
void Window::setWinSize(Vec2i& par){
	pixBuf.length = par.x * par.y;
/*
	if(workSurface){
		workSurface->Release();
	}
	*/
	if(false == initWindow()){
		throw;
	}
}
