/* Seamless3d

Copyright (C) 2001-2017 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/


#include "seamless3d.h"
#include "techuelife_triangle_editor.h"


extern Ptr<TransformCP> g_transformCP;
extern Vec2i g_mousePos;

void houseKeepAndPaintCp();
void paintCp();
Seamless* getSeamless2();

extern Ptr<PartCageClue> g_partCageClue;
extern ScenePtr g_scene;
extern ShapePtr g_transformLeverClue;
extern ShapePtr g_translateDimensionClue;

extern GeomRenderPtr g_redConePoint;
extern GeomRenderPtr g_greenConePoint;
extern GeomRenderPtr g_blueConePoint;

extern TransformPtr g_redConeBase;
extern TransformPtr g_greenConeBase;
extern TransformPtr g_blueConeBase;

extern Ptr<Settings> g_settings;
Hair* getHair();
void commonPostTug();
bool checkWithInSights(const bool perspective, Vec3f sr);
extern Ptr<RadicalPatch> g_radicalPatch;

void TransformCP::SelectObject::operator=(const bool& par){
	refValue() = par;
	home.selectRing = false;
	home.selectRings = false;
	if(par){
		MultiSelect::pointList.length = 0;
	}
	houseKeepAndPaintCp();
}

void TransformCP::LeverNeutral::operator=(const bool& par){
	refValue() = par;
	home.leverRotate.refValue() = false;
	home.leverStretch.refValue() = false;
	home.leverRotateAndStretch.refValue() = false;
	home.leverScale.refValue() = false;
	home.leverTranslate.refValue() = false;
}

void TransformCP::LeverRotate::operator=(const bool& par){
	refValue() = par;
	home.leverNeutral.refValue() = false;
	home.leverStretch.refValue() = false;
	home.leverRotateAndStretch.refValue() = false;
	home.leverScale.refValue() = false;
	home.leverTranslate.refValue() = false;
}

void TransformCP::LeverStretch::operator=(const bool& par){
	refValue() = par;
	home.leverNeutral.refValue() = false;
	home.leverRotate.refValue() = false;
	home.leverRotateAndStretch.refValue() = false;
	home.leverScale.refValue() = false;
	home.leverTranslate.refValue() = false;
}

void TransformCP::LeverRotateAndStretch::operator=(const bool& par){
	refValue() = par;
	home.leverNeutral.refValue() = false;
	home.leverRotate.refValue() = false;
	home.leverStretch.refValue() = false;
	home.leverScale.refValue() = false;
	home.leverTranslate.refValue() = false;
}

void TransformCP::LeverScale::operator=(const bool& par){
	refValue() = par;
	home.leverNeutral.refValue() = false;
	home.leverRotate.refValue() = false;
	home.leverStretch.refValue() = false;
	home.leverRotateAndStretch.refValue() = false;
	home.leverTranslate.refValue() = false;
}

void TransformCP::LeverTranslate::operator=(const bool& par){
	refValue() = par;
	home.leverNeutral.refValue() = false;
	home.leverRotate.refValue() = false;
	home.leverStretch.refValue() = false;
	home.leverRotateAndStretch.refValue() = false;
	home.leverScale.refValue() = false;
}

void TransformCP::SameScreenPosition::operator=(const bool& par){
	refValue() = par;
	home.selectRings = false;
	paintCp();
}

void TransformCP::RotAxisX::operator=(const bool& par){
	refValue() = par;
	home.rotAxisY.refValue() = false;
	home.rotAxisZ.refValue() = false;
}

void TransformCP::RotAxisY::operator=(const bool& par){
	refValue() = par;
	home.rotAxisX.refValue() = false;
	home.rotAxisZ.refValue() = false;
}

void TransformCP::RotAxisZ::operator=(const bool& par){
	refValue() = par;
	home.rotAxisY.refValue() = false;
	home.rotAxisX.refValue() = false;
}

void TransformCP::houseKeep(){
	if(MultiSelect::pointList.length == 0){
		return;
	}

	if(showDimFilterVisual){
		float inverseSize = (1 / g_scene->getZoom());
		float d = inverseSize * .2;
		float pl = inverseSize * .12;
		Vec3f pp = Vec3f();
		IndexedLineSet* ils = dynamic_cast<IndexedLineSet*>(g_translateDimensionClue->geometry.p);
		M_Vec3f& coord = dynamic_cast<Coordinate*>(ils->coord.p)->point;
		M_Color3f& color = dynamic_cast<Color*>(ils->color.p)->color;
		M_Int& coordIndex = ils->coordIndex.refValue();

		Color3f red = xTug ? Color3f(1, 0, 0) : Color3f(.7, .5, .5);
		Color3f green = yTug ? Color3f(0, 1, 0) : Color3f(.5, .7, .5);
		Color3f blue = zTug ? Color3f(0, 0, 1) : Color3f(.5, .5, .7);

		coord.length = 0;
		color.length = 0;
		coordIndex.length = 0;

		if(xTug){
			coordIndex += 0;
			coord += pp;
			color += red;
			coordIndex += 1;
			coord += Vec3f(d, 0, 0);
			color += red;
			coordIndex += -1;
		}

		if(yTug){
			coordIndex += coord.length;
			coord += pp;
			color += green;
			coordIndex += coord.length;
			coord += Vec3f(0, d, 0);
			color += green;
			coordIndex += -1;
		}

		if(zTug){
			coordIndex += coord.length;
			coord += pp;
			color += blue;
			coordIndex += coord.length;
			coord += Vec3f(0, 0, d);
			color += blue;
			coordIndex += -1;
		}

		if(xTug && yTug){
			coordIndex += coord.length;
			coord += Vec3f(0, pl, 0);
			color += green;
			coordIndex += coord.length;
			coord += Vec3f(pl, pl, 0);
			color += green;
			coordIndex += -1;

			coordIndex += coord.length;
			coord += Vec3f(pl, 0, 0);
			color += red;
			coordIndex += coord.length;
			coord += Vec3f(pl, pl, 0);
			color += red;
			coordIndex += -1;
		}
		if(yTug && zTug){
			coordIndex += coord.length;
			coord += Vec3f(0, pl, 0);
			color += green;
			coordIndex += coord.length;
			coord += Vec3f(0, pl, pl);
			color += green;
			coordIndex += -1;

			coordIndex += coord.length;
			coord += Vec3f(0, 0, pl);
			color += blue;
			coordIndex += coord.length;
			coord += Vec3f(0, pl, pl);
			color += blue;
			coordIndex += -1;
		}
		if(xTug && zTug){
			coordIndex += coord.length;
			coord += Vec3f(pl, 0, 0);
			color += red;
			coordIndex += coord.length;
			coord += Vec3f(pl, 0, pl);
			color += red;
			coordIndex += -1;

			coordIndex += coord.length;
			coord += Vec3f(0, 0, pl);
			color += blue;
			coordIndex += coord.length;
			coord += Vec3f(pl, 0, pl);
			color += blue;
			coordIndex += -1;
		}

		ils->convertToRenderFormat(g_translateDimensionClue->render);


		g_redConePoint->genColorList(red);
		g_greenConePoint->genColorList(green);
		g_blueConePoint->genColorList(blue);

		g_redConePoint->convertToRenderFormat(g_redConePoint->render, g_redConePoint->appearance);
		g_greenConePoint->convertToRenderFormat(g_greenConePoint->render, g_greenConePoint->appearance);
		g_blueConePoint->convertToRenderFormat(g_blueConePoint->render, g_blueConePoint->appearance);

		g_redConeBase->scale = Vec3f(inverseSize, inverseSize, inverseSize);
		g_greenConeBase->scale = Vec3f(inverseSize, inverseSize, inverseSize);
		g_blueConeBase->scale = Vec3f(inverseSize, inverseSize, inverseSize);

		g_redConeBase->translation = pivotPoint + translation + Vec3f(d, 0, 0);
		g_greenConeBase->translation = pivotPoint + translation + Vec3f(0, d, 0);
		g_blueConeBase->translation = pivotPoint + translation + Vec3f(0, 0, d);
	}
	if(MultiSelect::pointList.length >= leverSelection){
		Vec3f pp = g_scene->getViewMatrix() * (pivotPoint + translation);

		IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_transformLeverClue->geometry.p);
		M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
		M_Int& coordIndex = polyLine->coordIndex.refValue();

		coord.length = 0;
		coordIndex.length = 0;

		if(g_settings->multiPointLever){
			for(int k = 0; k < rotCtrlPnt.length; k++){
				coordIndex += coord.length;
				coord += pp;
				coord += rotCtrlPnt[k];
				coordIndex += coord.length;
				coord += rotCtrlPnt3[k];
				coord += rotCtrlPnt2[k];
				coord += rotCtrlPnt4[k];
				coord += rotCtrlPnt5[k];
				coordIndex += -1;

				coordIndex += coord.length - 5;
				coordIndex += coord.length - 3;
				coordIndex += -1;
				coordIndex += coord.length - 2;
				coordIndex += coord.length - 1;
				coordIndex += -1;
			}
		}
		else{
			for(int k = 0; k < rotCtrlPnt.length; k++){
				coordIndex += coord.length;
				coord += pp;
				coordIndex += coord.length;
				coord += rotCtrlPnt[k];
				coordIndex += -1;
			}
		}
		polyLine->convertToRenderFormat(g_transformLeverClue->render);
	}
}

void TransformCP::selectObjectPoints(M_<NurbsPatch*>& contigList){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Seamless* smls = patch->smls;
		M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
		for(int k = 0; k < cpNodeList.length; k++){
			NurbsPatch* patch = cpNodeList[k];
			if(patch->object){
				int len = patch->dpoint.length;
				for(int i = 0; i < len; i++){
					contigList[patch->fromPoint + i] = patch;
				}
			}
		}
	}
}

void TransformCP::refreshRemPointsMin(){
	MultiSelect::beginDragPos.length = MultiSelect::pointList.length;
	if(MultiSelect::pointList.length){
		Seamless* smls = MultiSelect::pointList[0].p->smls;
		M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
		for(int k = cpNodeList.length; --k >= 0;){
			cpNodeList[k]->container->remTransform();
		}
		for(int i = MultiSelect::pointList.length; --i >= 0;){
			NurbsPatch* patch = MultiSelect::pointList[i].p;
			int idx = MultiSelect::pointList[i].i;
			MultiSelect::beginDragPos[i] = patch->dpoint[idx];
		}
	}
}

void TransformCP::refreshRemPoints(){
	index[0] = 0;
	scaleOrientation.refValue() = Rotation();
	scale.refValue() = Vec3f(1, 1, 1);
	rotation.refValue() = Rotation();
	pivotPoint.refValue() += translation.refValue();
	translation.refValue() = Vec3f();
	remRotCtrlPnt = rotCtrlPnt[leverIdx];
	//setRemRotCtrlPnt();
	refreshRemPointsMin();
}

void TransformCP::calcPivotPoint(){

	Vec3f& pivotPoint = this->pivotPoint.refValue();

	pivotPoint = Vec3f(0, 0, 0);
	index[0] = 0;
	scale.refValue() = Vec3f(1, 1, 1);
	scaleOrientation.refValue() = Rotation();
	rotation.refValue() = Rotation();
	translation.refValue() = Vec3f();

	if(MultiSelect::pointList.length == 0){
		return;
	}

	pivotPoint = Vec3f();
	MultiSelect::beginDragPos.length = MultiSelect::pointList.length;
	for(int i = MultiSelect::pointList.length; --i >= 0;){
		NurbsPatch* patch = MultiSelect::pointList[i].p;
		int idx = MultiSelect::pointList[i].i;
		Vec3f v = patch->dpoint[idx];
		MultiSelect::beginDragPos[i] = v;
		pivotPoint += v;
	}

	if(g_scene->mode & SINGLE_ACTIVE_MODE){
		NurbsPatch* patch = MultiSelect::pointList[MultiSelect::clickedIndex].p;
		int idx = MultiSelect::pointList[MultiSelect::clickedIndex].i;
		pivotPoint = patch->dpoint[idx];
	}
	else{
		pivotPoint = pivotPoint / MultiSelect::beginDragPos.length;
	}
}

bool SurfacePoint(
				  float u, float v, int uOrder, int vOrder, int uDimension, int vDimension,
				  const float *uKnot, const float * vKnot,
				  const Vec3f *P,const float *W,
				  Vec3f *dest // result
				  );

void TransformCP::doNurbsRadicalChange(const M_Vec3f& diff, const M_<NurbsPatch*>& srcP, const M_Int& srcI, const M_Vec2f& mapVec, const M_<NurbsPatch*>& m_mapP, Seamless* smls){

	for(int i = mapVec.length; --i>=0;){
		NurbsPatch* srcPatch = srcP[i];
		if(dynamic_cast<NurbsPatch*>(srcPatch->container)){
			int idx = srcI[i];
			Vec3f svec;

			NurbsPatch* mapP = m_mapP[i];

			int uDim = mapP->dim.x;
			int vDim = mapP->dim.y;


			const float* uKnotPtr = mapP->uKnot.refValue().getArray();
			const float* vKnotPtr = mapP->vKnot.refValue().getArray();
			const Vec3f* pointPtr = mapP->point.refValue().getArray();
			const float* weightPtr = mapP->weight.refValue().getArray();


			SurfacePoint(mapVec[i].x, mapVec[i].y, mapP->order.x, mapP->order.y, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &svec);

			svec -= diff[i];


			setFilteredDimension(srcPatch->point.refValue(), smls->cp.refValue(), srcPatch->cpIndex.refValue(), idx, svec * srcPatch->weight[idx]);
			srcPatch->dpoint.refValue()[idx] = srcPatch->point[idx] / srcPatch->weight[idx];
			srcPatch->mirrorSingleCP(idx);
		}
	}
	if(false == (g_scene->mode.refValue() & BOX_PATCH_MODELLING_MODE)){
		smls->calcAllPolyMidPoints();
	}
}

Matrix& TransformCP::multiplyScaleSequence(Matrix& matrix) const {

	const M_Rotation& scaleOrientation = this->scaleOrientation.refValue();
	const M_Vec3f& scale = this->scale.refValue();

	for (int k = 0; k < scaleOrientation.length; k++) {
		matrix = matrix * rotationMatrix(-scaleOrientation[k]) * scaleMatrix(scale[k]) * rotationMatrix(scaleOrientation[k]);
	}
	return matrix;
}

Matrix& TransformCP::setMatrix(Matrix& matrix) const {
	matrix = translationMatrix(-pivotPoint);
	matrix = multiplyScaleSequence(matrix) * rotationMatrix(rotation) * translationMatrix(pivotPoint) * translationMatrix(translation);
	return matrix;
}

void TransformCP::processFields(){
 	if(MultiSelect::beginDragPos.length != MultiSelect::pointList.length){
		refreshRemPoints();
	}
	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		processFieldsForAnim(part);
		return;
	}
	Build* bld = dynamic_cast<Build*>(g_selNode.getRaw());
	if(bld == 0){
		return;
	}
	
	Seamless* smls = 0;
	NurbsPatch* patch = bld->getPatch();
	if(patch){
		smls = patch->smls;
	}
	else{
		smls = getSeamless2();
	}

	if(smls){
#ifdef CPINDEX
		M_Vec3f& cp = smls->cp.refValue();
#endif

		//Matrix matrix = translationMatrix(-pivotPoint) * rotationMatrix(-scaleOrientation) * scaleMatrix(scale) * rotationMatrix(scaleOrientation) * rotationMatrix(rotation) * translationMatrix(translation) * translationMatrix(pivotPoint);
		Matrix matrix;
		setMatrix(matrix);
			
		if(MultiSelect::beginDragPos.length){
			M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
			for(int k = 0; k < cpNodeList.length;){
				Build* container = cpNodeList[k]->container;
				container->doCPTransform();
				k += container->numOfPatches();
			}
			//if(selectObject){
			//	M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
			//	for(int k = 0; k < cpNodeList.length; k++){
			//		NurbsPatch* patch = cpNodeList[k];
			//		if(patch->object){
			//			for(int i = patch->dpoint.refValue().length; --i>=0;){
			//				Vec3f svec = matrix * MultiSelect::beginDragPos[patch->fromPoint + i];
			//				patch->setFilteredDimension(patch->point.refValue(), i, svec * patch->weight[i]);
			//				patch->dpoint.refValue()[i] = patch->point[i] / patch->weight[i];
			//				patch->mirrorSingleCP(i);
			//			}
			//		}
			//	}
			//}
			for(int i = MultiSelect::pointList.length; --i>=0;){
				NurbsPatch* patch = MultiSelect::pointList[i].p;
				if(dynamic_cast<NurbsPatch*>(patch->container)){
					int idx = MultiSelect::pointList[i].i;
					Vec3f svec = matrix * MultiSelect::beginDragPos[i];
					Vec3f pnt;
					float w = patch->weight[idx];
					setFilteredDimension(patch->point.refValue(), smls->cp.refValue(), patch->cpIndex.refValue(), idx, svec * patch->weight[idx]);
					patch->dpoint.refValue()[idx] = patch->point[idx] / patch->weight[idx];
					patch->mirrorSingleCP(idx);
				}
			}
		}
		if(smls->bnspe){
			if(Hair* hair = getHair()){
				hair->doLocksUpdateStrands();
				hair->processFields();
				//M_Strand& m_strand = hair->strand.refValue();
				smls->doBuild();
				smls->ctrf = true;
			}
		}
		houseKeep();
		if(false == (g_scene->mode.refValue() & BOX_PATCH_MODELLING_MODE)){
			smls->calcAllPolyMidPoints();
		}
		if(g_radicalPatch->active){
			doNurbsRadicalChange(g_radicalPatch->diff, g_radicalPatch->srcP, g_radicalPatch->srcI, g_radicalPatch->mapVec, g_radicalPatch->mapP, smls);
		}
		commonPostTug();
		if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
			if(part->smls == smls){
				g_partCageClue->genCage();
			}
		}
		else{
			smls->houseKeepNurbsCages();
		}

		return;
	}
}


void TransformCP::setRemRotCtrlPnt(){
	switch(overWhatPoint){
		case TransformCP::ROTATE:
			remRotCtrlPnt = rotCtrlPnt[leverIdx];
			break;
		case TransformCP::NEUTRAL:
			remRotCtrlPnt = rotCtrlPnt2[leverIdx];
			break;
		case TransformCP::SCALE1D:
			remRotCtrlPnt = rotCtrlPnt3[leverIdx];
			break;
		case TransformCP::TRANSLATE:
			remRotCtrlPnt = rotCtrlPnt4[leverIdx];
			break;
		case TransformCP::SCALE3D:
			remRotCtrlPnt = rotCtrlPnt5[leverIdx];
			break;
	}
}

void TransformCP::setMode(int newMode){
	if(newMode != mode){
		mode = newMode;
		//setRemRotCtrlPnt();
		remRotation = rotation;
		switch(mode){
		case ROTATE:
			remRotCtrlPnt = rotCtrlPnt[leverIdx];
		case SCALE3D:
		case ROTATE_AND_SCALE1D:
		case SCALE2D:
		case SCALE1D:
		//case TRANSLATE:
		//case ROTATE:
			//this->refreshRemPoints();
			break;
		}
//		if(mode == TRANSLATE){
//			remTranslation = translation;
//		}
	}
}

void TransformCP::XTug::operator=(const bool& par){
	refValue() = par;
	home.houseKeep();
}
void TransformCP::YTug::operator=(const bool& par){
	refValue() = par;
	home.houseKeep();
}
void TransformCP::ZTug::operator=(const bool& par){
	refValue() = par;
	home.houseKeep();
}

void TransformCP::PivotPoint::operator=(const Vec3f& par){
	refValue() = par;
	home.calcRotCtrlPnts();
	home.processFields();
}

void TransformCP::Translation::operator=(const Vec3f& par){
	refValue() = par;
	home.calcRotCtrlPnts();
	home.processFields();
}

void TransformCP::calcScaleOrientation() {

	Vec3f pp = g_scene->getViewMatrix() * (pivotPoint + translation);

	Vec3f d = rotCtrlPnt[leverIdx];
	pp.z = d.z;
	pp = g_scene->getIViewMatrix() * pp;

	Vec3f bv = g_scene->getIViewMatrix() * rotCtrlPnt[leverIdx] - pp;
	float bdis = bv.length();
	if (bdis > .00001) {
		Vec3f baxis = bv.normalize();
		Rotation so = Rotation(baxis, Vec3f(0, 1, 0));
		scaleOrientation.refValue()[index[0]] = -so;
	}
}

void TransformCP::calcLeverAngle() {
	Vec3f pp = g_scene->getViewMatrix() * (pivotPoint + translation);

	Vec3f v = rotCtrlPnt[leverIdx] - pp;
	if (v.length() > .00001) {
		Vec3f la = leverAxis[leverIdx];
		leverAngle = atan2(v.y, v.x) - atan2(la.y, la.x);
		leverRot = Rotation(0, 0, 1, leverAngle);
	}
}

void TransformCP::scalePoints(bool lock, bool x, bool z) {
	if (g_mousePos == Vec2i(663, 426 + 15)) {
		int a = 0;
	}

	Vec3f pp = g_scene->getViewMatrix() * (pivotPoint + translation);
	Vec3f d = remRotCtrlPnt;
	pp.z = d.z;
	//	g_pointer.translation.z = pp.z;
	pp = g_scene->getIViewMatrix() * pp;
	pp = pivotPoint + translation;

	Vec3f newPt = g_scene->getIViewMatrix() * g_catchIMatrix * (g_pointer.translation + mouseOffset);
	Vec3f ev = newPt - pp;
	Vec3f bv = g_scene->getIViewMatrix() * remRotCtrlPnt - pp;
	float bdis = bv.length();
	float edis = ev.length();
	Vec3f baxis = bv.normalize();
	Vec3f eaxis = ev.normalize();
	Rotation so = Rotation(baxis, Vec3f(0, 1, 0));
	//Rotation so = -scaleOrientation;


	float s = 0;

	if (bdis >= .00001) {
		s = edis / bdis;
	}

	Vec3f v = remRotCtrlPnt - g_scene->getViewMatrix() * (pivotPoint + translation);
	Vec3f la = leverAxis[1];
	float angle = atan2(v.y, v.x) - atan2(la.y, la.x);
	Rotation rot = Rotation(0, 0, 1, angle);


	Vec3f iPointer = -rot * ((g_pointer.translation + mouseOffset) - g_scene->getViewMatrix() * (pivotPoint + translation));

	if (iPointer.y <= 0) {
		s = 0;
	}

	{
		if (x) {
			scale.refValue()[g_transformCP->index[0]].x = s;
		}
		scale.refValue()[g_transformCP->index[0]].y = s;
		if (z) {
			scale.refValue()[g_transformCP->index[0]].z = s;
		}
		if (lock == false) {
			rotation.refValue() = Rotation(baxis, eaxis);
			rotCtrlPnt[leverIdx] = g_pointer.translation + mouseOffset;
		}
		else {
			//Vec3f& rotPoint = (overWhatPoint == TransformCP::ROTATE) ? rotCtrlPnt[leverIdx] : rotCtrlPnt3[leverIdx];
			if (s == 0) {
				rotCtrlPnt[leverIdx] = g_scene->getViewMatrix() * pivotPoint;
			}
			else {
				rotCtrlPnt[leverIdx] = g_scene->getViewMatrix() * (rotation * -so * Vec3f(0, edis, 0) + pp);
			}
		}
		calcNonSelectedRotCtrlPnts();
		//scaleOrientation = -so;
		//rotCtrlPnt = g_pointer.translation;
//		g_pointer.translation.z = 0;

		processFields();
	}
}

void TransformCP::postRotPointDrag(){
	restoreCursorPoint();

	if (scale[index[0]] != Vec3f(1, 1, 1)) {
		index[0] = scaleOrientation.length;
		scaleOrientation.refValue() += Rotation();
		scale.refValue() += Vec3f(1, 1, 1);
	}
}
