/* Seamless3d

Copyright (C) 2001-2010 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/


#include <windows.h>
#include <direct.h>

#include "seamless3d.h"
#include "techuelife_triangle_editor.h"
#include "gui.h"
#include "scene_tree.h"
#include "resource.h"
#include "input.h"
#include "robot_aid.h"
#include "seamless3d_gui_render.h"
#include "../lpng/png.h"

bool g_editVideoFrames = true;
extern const BIProto* builtInProtoList[];

bool Video::audioTimeMode = false;

bool readPngFile(const char* file_name, int width, int height);
extern Ptr<PasteOptions> g_pasteOptions;

Ptr<VideoControlPanel> g_vcp;

static png_bytep* row_pointers = 0;

static png_byte color_type = 0;
static png_byte bit_depth = 0;

static png_structp png_ptr = 0;
static png_infop info_ptr = 0;
static int number_of_passes = 0;

void abort_(const char * s){
	dLog(s);
	throw;
}
extern int g_fps;

extern int g_counter;

void Video::getFilename(String& filename, int frame, String& path, int numOfDigits){
	static String cntrStr;
	cntrStr = "";
	filename = path;

	cntrStr = frame;

	while(cntrStr.length < numOfDigits){
		cntrStr = '0' + cntrStr;
	}
	filename += cntrStr;
	
	filename += ".png";
}


void VideoControlPanel::getFilename(String& filename){
	static String cntrStr;
	cntrStr = "";
	//filename = '\"';
	filename = video->path;
	M_Int& frameRef = video->frameRef.refValue();

	int frame = 0;

	if(video->audioTimeMode){
		frame = g_counter * (25 / g_fps) + firstFrame;
		if(frame < frameRef.length){
			frame = frameRef[frame];
		}
		cntrStr = frame;
	}
	else{
		frame = video->frame + firstFrame;

		if(frame < frameRef.length){
			frame = frameRef[frame];
		}
		
	}
	cntrStr = frame;

	//dLog(g_counter);
	
	while(cntrStr.length < video->numOfDigits){
		cntrStr = '0' + cntrStr;
	}
	filename += cntrStr;
	
	static String pcmString;

	video->pcmFilename = filename;

	filename += ".png";
}

void VideoControlPanel::setupNodeWindow(Node* node){
	this->node=node;
	video = dynamic_cast<Video*>(node);

	processFilename();

	const M_Properties& properties = node->getProperties();

	g_controlPanelOpen = true;
	g_propWinLine.length = 0;
	buttonList.length = 0;
	int yPos = title_bar_height + 1;
	int lastYPos = yPos;

	size = Vec2i(frameWidth, frameHeight + 25);
	pos = Vec2i(0, 100);

	openButton=dynamic_cast<SFBoolButton*>(video->open.setUpControl(*node, video->open, properties[0], 0, 340+48, 0, *this));
	openButton->bSize=Vec2i(24, 24);
	openButton->dstPos=Vec2i(130, 1);
	openButton->srcPos=Vec2i(openSmls_x,openSmls_y);
	openButton->iconSize=Vec2i(24, 24);
	buttonList+=openButton;
	g_propWinLine+=PropWinLine(yPos, openButton);

	//FieldButton* button = video->file.setUpControl(*node, video->file, properties[1], 0, size.y - 30, 1, *this);
	//button->bSize=Vec2i(24, 24);
	//button->srcPos=Vec2i(268, 191);
	//button->iconSize=Vec2i(24, 18);
	//button->dstPos.x = 190;
	//button->valPos.x = button->dstPos.x + 24 + 4;
	//button->valSize.x = 8*7;
	//buttonList+=button;
	//g_propWinLine+=PropWinLine(yPos, button);

	houseKeepVisualClue();
	paint();
}

void VideoControlPanel::setWinSize(Vec2i& raw){
	pixBuf.length = raw.x * raw.y;
	if(false == initWindow()){
		throw;
	}
}


int readpng_init(FILE *infile, unsigned long *pWidth, unsigned long *pHeight);
unsigned char *readpng_get_image(double display_exponent, int *pChannels, unsigned long *pRowbytes);
int kmain();


void copyRect(int dx, int dy, int sx, int sy, int dstWidth, int dstHeight, int srcWidth, int srcHeight, int iconWidth, int iconHeight, png_bytep* row_pointers, int* dstPix){
	if (info_ptr->color_type != PNG_COLOR_TYPE_RGB){
		return;
	}

	if(dx>dstWidth-iconWidth){
		if(dx>=dstWidth){
			return;
		}
		iconWidth=dstWidth-dx;
	}
	if(dy>dstHeight-iconHeight){
		if(dy>=dstHeight){
			return;
		}
		iconHeight=dstHeight-dy;
	}
	else if(dy<0){
		sy-=dy;
		iconHeight+=dy;
		if(iconHeight<=0){
			return;
		}
		dy=0;
	}
	unsigned dstP=(dy)*dstWidth+dx;

	for(int y = 0; y < iconHeight; y++){
		png_byte* row = row_pointers[y];
		for(int x = 0; x < iconWidth; x++) {
			png_byte* ptr = &(row[x * 3]);

			unsigned int pix = ptr[0];
			pix <<= 8;
			pix = pix | ptr[1];
			pix <<= 8;
			pix = pix | ptr[2];

			dstPix[dstP + x] = pix;
		}
		dstP += dstWidth;
	}
}

void VideoControlPanel::paint(){
	if(node==0)return;

	int dstWidth=size.x;
	int dstHeight=size.y;
	int* dstPix=pixBuf.getArray();
	const M_Properties& properties=node->getProperties();


	//colorBlockColorRgn(dstPix);
	blitHorizontalColorSweep(Vec2i(10, 0), Vec2i(470, 0), Vec2i(471, 0), Vec2i(dstWidth-9-10, title_bar_height));
	pasteSprite2( 0, 0, 461, 0, dstWidth, dstHeight, iconSrcBufCX, iconSrcBufCY, 10, 25, dstPix);
	pasteSprite2( dstWidth-9, 0, 471, 0, dstWidth, dstHeight, iconSrcBufCX, iconSrcBufCY, 9, 25, dstPix);


	static String str;
	int i = 0;
	int j = 0;
	int currentGroup=-1;

	int g=str.length*8+15;
	int type=node->getType();
	int iconNodeType = type;
	if(type>=g_userProtoBegin){
		iconNodeType=PROTOINSTANCE;
	}
	int iconExtraX = (builtInProtoList[iconNodeType]->iconSize.x-16)>>1;
	//pasteSprite2( 5-iconExtraX, 5-((nodeIconLookUp[iconNodeType].size.y-16)>>1), nodeIconLookUp[iconNodeType].srcPos.x, nodeIconLookUp[iconNodeType].srcPos.y, dstWidth, dstHeight, iconSrcBufCX, iconSrcBufCY, nodeIconLookUp[iconNodeType].size.x, nodeIconLookUp[iconNodeType].size.y, dstPix);

	str = "frame: ";

	str += video->frame;
	drawText2(str, 0x0, Vec2i(5-iconExtraX, 16), dstWidth, dstHeight, dstPix);

	str = video->filenameOnly;
	drawText2(str, 0x0, Vec2i(160, 16), dstWidth, dstHeight, dstPix);

	exitWindow.dstPos=Vec2i(size.x-22, 4);
	exitWindow.paint();

	for(int i=buttonList.length; --i>=0;){
		FieldButton* button = buttonList[i];

		if(button->sprop.doNotDisplayName() == false && button->sprop.push() == false && button->sprop.link() == false){
			button->paintFieldName();
		}
		
		button->paint();
		Vec2i valPos=button->valPos;
		valPos.y+=16;
		if(button->sprop.doNotDisplayValue()==false){
			if(button->editMode){
				drawText2(textCell.cellStr, textCell.firstVisChar, textCell.cellStr.length, 0, valPos, size.x, size.y, dstPix);
				textCell.highLightBlock();
			}
			else{
				button->toString(Button::str="");
				drawText2(Button::str, 0, valPos, size.x, size.y, dstPix);
			}
		}
	}
	static String filename;

	getFilename(filename);
	if(readPngFile(filename.c_str(), frameWidth, frameHeight)){
		copyRect(0, 25, 0, 0, dstWidth, dstHeight, frameWidth, frameHeight, frameWidth, frameHeight, row_pointers, dstPix);
	}


	if(g_settings->invertWindowColor){
		invert();
	}
	loadSurfaceFromMemory();
	if(textCell.cellActive==true){
		textCell.doNoThinkWink();
	}
}

/*
 * void read_png_file(const char* file_name)
 * void write_png_file(char* file_name)
 * addapted from Guillaume Cottenceau's code
 * http://zarb.org/~gc/html/libpng.html
 *
 * These functions are under the terms
 * of the X11 license.
 *
 */




bool readPngFile(const char* file_name, int width, int height){

	char header[8];	// 8 is the maximum size that can be checked

	FILE *fp = fopen(file_name, "rb");
	if (!fp){
		dLog("failed to open png file");
		return false;
	}
	fread(header, 1, 8, fp);
	if (png_sig_cmp((png_bytep)header, 0, 8)){
		dLog("not recognized as a PNG file");
		return false;
	}



	/* initialize stuff */
	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	
	if(!png_ptr){
		dLog("png_create_read_struct failed");
		return false;
	}

	info_ptr = png_create_info_struct(png_ptr);
	if(!info_ptr){
		dLog("png_create_info_struct failed");
		return false;
	}

	if(setjmp(png_jmpbuf(png_ptr))){
		dLog("Error during init_io");
		return false;
	}

	png_init_io(png_ptr, fp);
	png_set_sig_bytes(png_ptr, 8);

	png_read_info(png_ptr, info_ptr);

	if(width != info_ptr->width || height != info_ptr->height || info_ptr->color_type != PNG_COLOR_TYPE_RGB){
		fclose(fp);
		return false;
	}
	//bit_depth = info_ptr->bit_depth;

	//int number_of_passes = png_set_interlace_handling(png_ptr);
	png_read_update_info(png_ptr, info_ptr);

	/* read file */
	if(setjmp(png_jmpbuf(png_ptr))){
		dLog("Error during read_image");
		fclose(fp);
		return false;
	}

	png_read_image(png_ptr, row_pointers);

	fclose(fp);
	return true;
}


void write_png_file(char* file_name, int width, int height, int bit_depth){
	/* create file */
	FILE *fp = fopen(file_name, "wb");
	if (!fp)
		abort_("[write_png_file] File %s could not be opened for writing");


	/* initialize stuff */
	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	
	if (!png_ptr)
		abort_("[write_png_file] png_create_write_struct failed");

	info_ptr = png_create_info_struct(png_ptr);
	if (!info_ptr)
		abort_("[write_png_file] png_create_info_struct failed");

	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[write_png_file] Error during init_io");

	png_init_io(png_ptr, fp);


	/* write header */
	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[write_png_file] Error during writing header");

	png_set_IHDR(png_ptr, info_ptr, width, height,
		     bit_depth, color_type, PNG_INTERLACE_NONE,
		     PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);

	png_write_info(png_ptr, info_ptr);


	/* write bytes */
	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[write_png_file] Error during writing bytes");

	png_write_image(png_ptr, row_pointers);


	/* end write */
	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[write_png_file] Error during end of write");

	png_write_end(png_ptr, NULL);

        /* cleanup heap allocation */
	for(int y = 0; y < height; y++){
		free(row_pointers[y]);
	}
	free(row_pointers);

	fclose(fp);
}

void Video::setPath(){
	String ext;
	String drive;
	String dir;

	splitPath(file, drive, dir, filenameOnly, ext);
	path = drive;
	path += dir;

	numOfDigits = 0;
	int i = i = filenameOnly.length;
	for(; --i >= 0;){
		char ch = filenameOnly[i];
		if(ch > '9' || ch < '0' ){
			break;
		}
		numOfDigits++;
	}
	i++;
	path += filenameOnly.substr(0, i);
	if(i >= filenameOnly.length){
		return;
	}
	filenameOnly += ext;
}

void Video::processFields(){
	setPath();
}

void VideoControlPanel::processFilename(){
	video->frame = 0;
	frameWidth = 256;
	frameHeight = 256;
	String ext;
	String drive;
	String dir;

	splitPath(video->file, drive, dir, video->filenameOnly, ext);
	video->path = drive;
	video->path += dir;

	video->numOfDigits = 0;
	int i = i = video->filenameOnly.length;
	for(; --i >= 0;){
		char ch = video->filenameOnly[i];
		if(ch > '9' || ch < '0' ){
			break;
		}
		video->numOfDigits++;
	}
	i++;
	video->path += video->filenameOnly.substr(0, i);
	if(i >= video->filenameOnly.length){
		return;
	}
	firstFrame = getInt(video->filenameOnly, i);
	video->filenameOnly += ext;

	char header[8];	// 8 is the maximum size that can be checked

	/* open file and test for it being a png */
	FILE *fp = fopen(video->file.refValue().c_str(), "rb");
	if(!fp){
		dLog("failed to open png file");
		return;
	}
	fread(header, 1, 8, fp);
	if(png_sig_cmp((png_bytep)header, 0, 8)){
		fclose(fp);
		dLog("not recognized as a PNG file");
		return;
	}


	/* initialize stuff */
	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	
	if(!png_ptr){
		fclose(fp);
		dLog("png_create_read_struct failed");
		return;
	}

	info_ptr = png_create_info_struct(png_ptr);
	if(!info_ptr){
		fclose(fp);
		dLog("png_create_info_struct failed");
		fclose(fp);
	}

	if(setjmp(png_jmpbuf(png_ptr))){
		fclose(fp);
		dLog("Error during init_io");
		return;
	}

	png_init_io(png_ptr, fp);
	png_set_sig_bytes(png_ptr, 8);

	png_read_info(png_ptr, info_ptr);

	frameWidth = info_ptr->width;
	frameHeight = info_ptr->height;
	//color_type = info_ptr->color_type;
	//bit_depth = info_ptr->bit_depth;

	//number_of_passes = png_set_interlace_handling(png_ptr);
	//png_read_update_info(png_ptr, info_ptr);


	fclose(fp);

	if(frameBuffer){
		delete[] frameBuffer;
	}

	frameBuffer = new char[frameHeight * sizeof(png_bytep) + sizeof(png_byte) * info_ptr->rowbytes * frameHeight];
	row_pointers = (png_bytep*)frameBuffer;

	for(int y = 0; y < frameHeight; y++){
		row_pointers[y] = (png_bytep)&frameBuffer[frameHeight * sizeof(png_bytep) + info_ptr->rowbytes * y];
	}
}

void VideoControlPanel::openMovieFile(const char* filename){
	video->file = filename;
	processFilename();
	size = Vec2i(frameWidth, frameHeight + 25);
	paint();
}


void VideoControlPanel::setPeriodFromVideo(Anim* anim){
	if(anim->video.p != video){
		return;
	}
	String cntrStr;
	String filename;
	int frame = 0;

	FILE *fp = 0;

	M_Int& frameRef = video->frameRef.refValue();
	frameRef.length = 0;

	while(true){
		cntrStr = "";
		filename = video->path;
		cntrStr = frame + firstFrame;
		while(cntrStr.length < video->numOfDigits){
			cntrStr = '0' + cntrStr;
		}
		filename += cntrStr;
		
		filename += ".png";

		if(FILE *fp = fopen(filename.c_str(), "rb")){
			fclose(fp);
			frameRef += frame;
			frame++;
		}
		else{
			break;
		}
		
	};

	anim->period = static_cast<float>(frame - 1) / static_cast<float>(anim->stepsPerSecond);
}

void setPeriodFromVideo(){
	Anim* anim = Window::animMode();
	if(g_vcp){
		g_vcp->setPeriodFromVideo(anim);
	}
}

void createPNG24(const char* filename, int width, int height, Pix24* pixels);

void createSculptiePng(Art* part, String filename){
	filename += ".png";
	int width = 64;
	int height = 64;
	M_Vec3f& coord = part->coord;
	if(coord.length == width * height){
		Pix24* pixels = new Pix24[width * height];
		for(int r = 0; r < height; r++){
			for(int c = 0; c < width; c++){
				int i = r * width + c;
				Vec3f v = coord[i];
				pixels[i] = Pix24(
					-v.x * 127 + 127,
					v.z * 127 + 127,
					v.y * 127 + 127);
			}
		}
		createPNG24(filename.c_str(), width, height, pixels);
	}
}

void exportSculpties(M_Node& m_node){
	for(int k = m_node.length; --k >= 0 ;){
		Node* node = m_node[k];
		if(Seamless* smls = dynamic_cast<Seamless*>(node)){
			M_Art& partList = smls->partList;
			for(int k = partList.length; --k >= 0;){
				Art* part = partList[k];
				if(part->name.length){
					createSculptiePng(part, part->name);
				}
			}
		}
	}
}

//void Video::slowFrames(int frameIndex, int numOf){
//}
//
//void Video::repeatFrame(int frameIndex, int numOf){
//	M_Int& frameRef = this->frameRef.refValue();
//
//	if(frameRef.length == 0){
//		return;
//	}
//
//	if(frameIndex >= frameRef.length){
//		return;
//	}
//	if(g_pasteOptions->insert){
//		frameRef.length += numOf;
//	}
//
//	for(int k = 0; k < numOf; k += 5){
//		int fRef = frameRef[frameIndex + k];
//		for(int i = 0; i < 5 && k + i < frameRef.length; i++){
//			frameRef[frameIndex + k + i] = fRef;
//		}
//	}
//}

void Video::repeatFrame(int frameIndex, int numOf){
	M_Int& frameRef = this->frameRef.refValue();

	if(frameRef.length == 0){
		return;
	}

	if(frameIndex >= frameRef.length){
		return;
	}
	int fRef = frameRef[frameIndex];

	frameRef.length += numOf;

	for(int i = frameRef.length; --i >= frameIndex + numOf;){
		frameRef[i] = frameRef[i - numOf];
	}

	for(int k = 0; k < numOf; k++){
		frameRef[frameIndex + k] = fRef;
	}
}


void Video::deleteFrame(int frameIndex, int numOf){
	//repeatFrame(frameIndex, numOf);
	//return;
	M_Int& frameRef = this->frameRef.refValue();

	if(frameRef.length == 0){
		return;
	}

	if(frameIndex >= frameRef.length){
		return;
	}

	int fRef = frameRef[frameIndex];
	frameRef.deleteElement(frameIndex, numOf);
}

void Video::reduceFrames(int frameIndex, int toFrame, int keepCaptureForEvery){
	M_Int& frameRef = this->frameRef.refValue();

	if(frameRef.length == 0){
		return;
	}

	if(frameIndex >= frameRef.length){
		return;
	}
	int s = frameIndex;

	while(s < toFrame){
		frameRef[frameIndex++] = frameRef[s];
		s += keepCaptureForEvery;
	}
	//frameRef.deleteElement(frameIndex, s - frameIndex);
}

void removeFilename(String& str){
	int i = str.length;
	while(str[--i] != '\\');
	str = str.substr(0, i);
}

void Video::outputFrames(int contigFrame, const int numOfFrames){

	String srcFilename;
	String dstFilename;
	String dstPath;

	Pix24* dstPix = new Pix24[g_vcp->frameWidth * g_vcp->frameHeight];
	

	for(int frame = 0; frame < numOfFrames; frame++){

		getFilename(srcFilename, frameRef[frame], path, numOfDigits);

		if(readPngFile(srcFilename.c_str(), g_vcp->frameWidth, g_vcp->frameHeight)){
//			copyRect(0, 25, 0, 0, g_vcp->frameWidth, g_vcp->frameHeight, g_vcp->frameWidth, g_vcp->frameHeight, g_vcp->frameWidth, g_vcp->frameHeight, row_pointers, dstPix);
		}

		dstPath = g_currentFilename;
		removeFilename(dstPath);
		
		dstPath += "\\output\\frame";

		//dstPath = srcFilename;
		//dstPath = "C:\\Users\\thyme\\Documents\\march-vlog\\clip0005\\output\\frame";

		getFilename(dstFilename, contigFrame, dstPath, numOfDigits);

		png_bytep* dstPtr = row_pointers + 240 * 3;

		createPNG24(dstFilename.c_str(), g_vcp->frameWidth, g_vcp->frameHeight, (Pix24*)dstPtr);

		contigFrame++;
	}
	delete[] dstPix;
}

Pix24 mixPixels(Pix32 ipix, Pix24 bkgrnd){

	unsigned alpha = ipix.a;
	unsigned ialpha = 0xff - alpha;

	if(alpha != 0){
		int g = 0;
	}

	unsigned f0 = ipix.r * alpha;
	unsigned f1 = ipix.g * alpha;
	unsigned f2 = ipix.b * alpha;

	unsigned b0 = bkgrnd.r * ialpha;
	unsigned b1 = bkgrnd.g * ialpha;
	unsigned b2 = bkgrnd.b * ialpha;

	f0 += b0;
	f1 += b1;
	f2 += b2;

	f0 >>= 8;
	f1 >>= 8;
	f2 >>= 8;

	return Pix24(f0, f1, f2);
}

extern AnimCopy g_animCopy;

void convertBGR_to_GBR2(M_<Pix32>& pixels){
	for(int i = pixels.length; --i>=0;){
		Pix32 pix32 = pixels[i];
		pixels[i].r = pix32.b;
		pixels[i].b = pix32.r;
	}
}


void Video::mixLayers(){
	int width = g_vcp->frameWidth;
	int height = g_vcp->frameHeight;

	int headerSize = 0x36; //0x36 == 54
	unsigned char* raw = new unsigned char[4 * width * height + headerSize];
	FILE* stream;
	//stream = fopen("C:\\Documents and Settings\\thyme\\My Documents\\My Pictures\\toolbar-g.bmp","r+b");
	//stream = fopen("C:\\Documents and Settings\\thyme\\My Documents\\converted-2008.bmp","r+b");
	stream = fopen ("C:\\Users\\thyme\\Documents\\nurbs-stitched-animated-character\\seamless3d.com.alpha.bmp", "rb");
	//stream = fopen("C:\\Users\\thyme\\Documents\\fred.bmp","r+b");
	fread(raw, width * height + headerSize, 4, stream);
	fclose(stream);
	raw += headerSize;

	M_<Pix32> pixels;
	pixels.length = width * height;

	for(int i = pixels.length; --i>=0;){
		pixels[i] = *reinterpret_cast<Pix32*>(raw + i * 4);
	}
	convertBGR_to_GBR2(pixels);
	raw = reinterpret_cast<unsigned char*>(pixels.getArray());

	String frameFilename;
	M_Bool flag;
	flag.length = frameRef.length;

	Anim* anim = Window::getAnimBar();

	int steps = (anim->selectTo - anim->selectFrom) * anim->period / (1.0 / anim->stepsPerSecond) + .5;

	for(int q = 0; q <= steps; q++){
		int f = this->frame + q;
		f = frameRef[f];
		if(flag[f]){
			continue;
		}
		flag[f] = true;

		getFilename(frameFilename, f, path, numOfDigits);

		if(readPngFile(frameFilename.c_str(), width, height)){
		}

		png_bytep* dstPtr = row_pointers + 240 * 3;

		Pix24* framePixels = reinterpret_cast<Pix24*>(dstPtr);

		for(int r = 0; r < height; r++){
			for(int c = 0; c < width; c++){
				int si = width * (height - 1 - r) + c;
				int i = width * r + c;

		
				Pix24 p24 = mixPixels(pixels[si], framePixels[i]);

				framePixels[i] = p24;
			}
		}

		createPNG24(frameFilename.c_str(), width, height, framePixels);
	}
}
