/* Seamless3d

Copyright (C) 2001-2017 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#include "seamless3d.h"
#include "gui.h"

extern const char* tip_cCLathe_curve;
extern const char* tip_cCLathe_distance;
extern const char* tip_cCLathe_baseRad;
extern const char* tip_cCLathe_autoClose;
const char* tip_stem_add = "add generated triangles to existing triangles";
extern const char* tip_stem_slide;
extern const char* tip_stem_orientation;
extern const char* tip_stem_staves;
extern const char* tip_stem_xDistance;
extern const char* tip_stem_bands;
extern const char* tip_stem_yDistance;
extern const char* tip_stem_height;
extern const char* tip_stem_sync;
extern const char* tip_stem_mould;
extern const char* tip_stem_insideOut;
extern const char* tip_stem_joinEnds;
extern const char* tip_stem_endSP;
extern const char* tip_stem_beginSP;
extern const char* tip_rectangleBuild_staves;
extern const char* tip_rectangleBuild_bands;
extern const char* tip_rectangleBuild_endSP;
extern const char* tip_rectangleBuild_beginSP;

struct InitPromptNode{
	const int* list;
	const int numOf;
	InitPromptNode(const int* list, const int numOf):
		list(list),
		numOf(numOf)
	{}
};
#define INIT_PROMPT_NODE(x) InitPromptNode(x, sizeof(x)/sizeof(int))

class BIField{//built in field
public:
	const int type;
	const char* str;
	const unsigned int flags;
	const InitPromptNode initPromptNode;
	const char* tip;
	BIField(const int type, const char* str, unsigned int flags, const InitPromptNode initPromptNode, const char* tip=0):
		type(type),
		str(str),
		flags(flags),
		initPromptNode(initPromptNode),
		tip(tip)
	{}
	BIField(const int type, const char* str, unsigned int flags, const char* tip=0):
		type(type),
		str(str),
		flags(flags),
		initPromptNode(0,0),
		tip(tip)
	{}
};
class BIProto0 : public BIProto{//built in proto
public:
	BIProto0(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 0, 0)
	{}
};
class BIProto1 : public BIProto{//built in proto
public:
	const BIField bifield0;
	BIProto1(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 1, &this->bifield0),
		bifield0(bifield0)
	{}
};
class BIProto2 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	BIProto2(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 2, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1)
	{}
};
class BIProto3 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	BIProto3(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 3, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2)
	{}
};
class BIProto4 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	BIProto4(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 4, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3)
	{}
};
class BIProto5 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField* my_bifieldRef;
	BIProto5(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 5, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		my_bifieldRef(&this->bifield0)
	{}
};
class BIProto6 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	BIProto6(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 6, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5)
	{}
};
class BIProto7 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	BIProto7(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 7, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6)
	{}
};
class BIProto8 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	BIProto8(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 8, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7)
	{}
};
class BIProto9 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	BIProto9(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 9, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8)
	{}
};
class BIProto10 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	BIProto10(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 10, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9)
	{}
};
class BIProto11 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	BIProto11(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 11, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10)
	{}
};
class BIProto12 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	BIProto12(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 12, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11)
	{}
};
class BIProto13 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	BIProto13(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 13, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12)
	{}
};
class BIProto14 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	BIProto14(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 14, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13)
	{}
};
class BIProto15 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	BIProto15(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 15, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14)
	{}
};
class BIProto16 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	BIProto16(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 16, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15)
	{}
};
class BIProto17 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	BIProto17(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 17, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16)
	{}
};
class BIProto18 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	BIProto18(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 18, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17)
	{}
};
class BIProto19 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	BIProto19(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 19, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18)
	{}
};
class BIProto20 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	BIProto20(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 20, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19)
	{}
};
class BIProto21 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	BIProto21(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 21, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20)
	{}
};
class BIProto22 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	BIProto22(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 22, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21)
	{}
};
class BIProto23 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	BIProto23(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 23, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22)
	{}
};
class BIProto24 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	BIProto24(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 24, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23)
	{}
};
class BIProto25 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	BIProto25(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 25, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24)
	{}
};

class BIProto26 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	BIProto26(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 26, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25)
	{}
};
class BIProto27 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	BIProto27(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 27, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26)
	{}
};
class BIProto28 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	BIProto28(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 28, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27)
	{}
};
class BIProto29 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	BIProto29(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 29, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28)
	{}
};
class BIProto30 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	const BIField bifield29;
	BIProto30(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28, BIField bifield29):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 30, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28),
		bifield29(bifield29)
	{}
};
class BIProto31 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	const BIField bifield29;
	const BIField bifield30;
	BIProto31(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28, BIField bifield29, BIField bifield30):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 31, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28),
		bifield29(bifield29),
		bifield30(bifield30)
	{}
};
class BIProto32 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	const BIField bifield29;
	const BIField bifield30;
	const BIField bifield31;
	BIProto32(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28, BIField bifield29, BIField bifield30, BIField bifield31):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 32, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28),
		bifield29(bifield29),
		bifield30(bifield30),
		bifield31(bifield31)
	{}
};
class BIProto33 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	const BIField bifield29;
	const BIField bifield30;
	const BIField bifield31;
	const BIField bifield32;
	BIProto33(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28, BIField bifield29, BIField bifield30, BIField bifield31, BIField bifield32):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 33, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28),
		bifield29(bifield29),
		bifield30(bifield30),
		bifield31(bifield31),
		bifield32(bifield32)
	{}
};

class BIProto34 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	const BIField bifield29;
	const BIField bifield30;
	const BIField bifield31;
	const BIField bifield32;
	const BIField bifield33;
	BIProto34(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28, BIField bifield29, BIField bifield30, BIField bifield31, BIField bifield32, BIField bifield33):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 34, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28),
		bifield29(bifield29),
		bifield30(bifield30),
		bifield31(bifield31),
		bifield32(bifield32),
		bifield33(bifield33)
	{}
};


class BIProto35 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	const BIField bifield29;
	const BIField bifield30;
	const BIField bifield31;
	const BIField bifield32;
	const BIField bifield33;
	const BIField bifield34;
	BIProto35(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28, BIField bifield29, BIField bifield30, BIField bifield31, BIField bifield32, BIField bifield33, BIField bifield34):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 35, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28),
		bifield29(bifield29),
		bifield30(bifield30),
		bifield31(bifield31),
		bifield32(bifield32),
		bifield33(bifield33),
		bifield34(bifield34)
	{}
};

class BIProto36 : public BIProto{//built in proto
public:
	const BIField bifield0;
	const BIField bifield1;
	const BIField bifield2;
	const BIField bifield3;
	const BIField bifield4;
	const BIField bifield5;
	const BIField bifield6;
	const BIField bifield7;
	const BIField bifield8;
	const BIField bifield9;
	const BIField bifield10;
	const BIField bifield11;
	const BIField bifield12;
	const BIField bifield13;
	const BIField bifield14;
	const BIField bifield15;
	const BIField bifield16;
	const BIField bifield17;
	const BIField bifield18;
	const BIField bifield19;
	const BIField bifield20;
	const BIField bifield21;
	const BIField bifield22;
	const BIField bifield23;
	const BIField bifield24;
	const BIField bifield25;
	const BIField bifield26;
	const BIField bifield27;
	const BIField bifield28;
	const BIField bifield29;
	const BIField bifield30;
	const BIField bifield31;
	const BIField bifield32;
	const BIField bifield33;
	const BIField bifield34;
	const BIField bifield35;
	BIProto36(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, BIField bifield0, BIField bifield1, BIField bifield2, BIField bifield3, BIField bifield4, BIField bifield5, BIField bifield6, BIField bifield7, BIField bifield8, BIField bifield9, BIField bifield10, BIField bifield11, BIField bifield12, BIField bifield13, BIField bifield14, BIField bifield15, BIField bifield16, BIField bifield17, BIField bifield18, BIField bifield19, BIField bifield20, BIField bifield21, BIField bifield22, BIField bifield23, BIField bifield24, BIField bifield25, BIField bifield26, BIField bifield27, BIField bifield28, BIField bifield29, BIField bifield30, BIField bifield31, BIField bifield32, BIField bifield33, BIField bifield34, BIField bifield35):
		BIProto(ntype, containerField, iconSize, iconSrcPos, 36, &this->bifield0),
		bifield0(bifield0),
		bifield1(bifield1),
		bifield2(bifield2),
		bifield3(bifield3),
		bifield4(bifield4),
		bifield5(bifield5),
		bifield6(bifield6),
		bifield7(bifield7),
		bifield8(bifield8),
		bifield9(bifield9),
		bifield10(bifield10),
		bifield11(bifield11),
		bifield12(bifield12),
		bifield13(bifield13),
		bifield14(bifield14),
		bifield15(bifield15),
		bifield16(bifield16),
		bifield17(bifield17),
		bifield18(bifield18),
		bifield19(bifield19),
		bifield20(bifield20),
		bifield21(bifield21),
		bifield22(bifield22),
		bifield23(bifield23),
		bifield24(bifield24),
		bifield25(bifield25),
		bifield26(bifield26),
		bifield27(bifield27),
		bifield28(bifield28),
		bifield29(bifield29),
		bifield30(bifield30),
		bifield31(bifield31),
		bifield32(bifield32),
		bifield33(bifield33),
		bifield34(bifield34),
		bifield35(bifield35)
	{}
};

extern int g_userProtoBegin;

static const int promptNode_cameraSelctor_keyValue[] = {VIEWPOINT};
static const int promptNode_scene_children[] = {ANCHOR, ANIM, ANIMBYANKLE, ANIMBYWRIST, BACKGROUND, BILLBOARD, BOOLEANFILTER, CAMERASELECTOR, BOOLEANTOGGLE, BOOLEANTRIGGER, BSPTREE, CCPATH, COLLISION, COLORINTERPOLATOR, COMPOSITION, COORDINATEINTERPOLATOR, CODELINE, CYLINDERSENSOR, DIRECTIONALLIGHT, FLOATACCELERATOR, FOG, FOG2, FUNCTION, _GROUP, GESTUREGROUP, HANIMHUMANOID, INLINE, INTEGERSEQUENCER, INTEGERTRIGGER, LOD_TYPE, MODELMORPHER, MOVIE, MULTIUSER, NAVIGATIONINFO, NOOUTPUTGROUP, NORMALINTERPOLATOR, NURBSTRACK, ORIENTATIONINTERPOLATOR, PLANESENSOR, POINTLIGHT, POSACCELERATOR, POSITIONINTERPOLATOR, POVFOG, POVLIGHT, PROTODECLARE, PROXIMITYSENSOR, ROTACCELERATOR, ROTFILTER, ROTMULT, SEAMLESS, SCALARINTERPOLATOR, SHAPE, SCRIPT, SOUND, SPHERESENSOR, SPOTLIGHT, STATICGROUP, SWITCH, TIMESENSOR, TIMETRIGGER, TOUCHSENSOR, TRANSFORM, VIEWPOINT, VISIBILITYSENSOR, WAVETOPOLY, MIDIFORMANT, WORLDINFO, 	G_S_BOOL, G_S_COLOR, G_S_FLOAT, G_S_IMAGE, G_S_INT32, G_NODEPTR, G_S_ROTATION, G_S_STRING, G_S_TIME, G_S_VEC2F, G_S_VEC3F, G_M_BOOL, G_M_COLOR, G_M_FLOAT, G_M_IMAGE, G_M_INT32, G_M_NODE, G_M_ROTATION, G_M_STRING, G_M_TIME, G_M_VEC2F, G_M_VEC3F};
static const int promptNode_shape_appearance[] = {APPEARANCE};
static const int promptNode_shape_geometry[] = {BOX, CONE, CYLINDER, ELEVATIONGRID, EXTRUSION, NURBSSURFACE, SPHERE, TEXT, INDEXEDLINESET, INDEXEDFACESET, INDEXEDTRIANGLESET, POINTSET, POLYLINE2D};
static const int promptNode_part_effect[] = {TEXTUREEFFECT, COLOREFFECT, MONOCOLOREFFECT};
static const int promptNode_part_morpher[] = {MORPHER};
static const int promptNode_part_part[] = {PART, IMMEDIATEPART};
static const int promptNode_appearance_material[] = {MATERIAL};
static const int promptNode_appearance_texture[] = {IMAGETEXTURE, MOVIETEXTURE};
static const int promptNode_appearance_textureTransform[] = {TEXTURETRANSFORM};
static const int promptNode_indexedFaceSet_coord[] = {COORDINATE};
static const int promptNode_indexedFaceSet_normal[] = {NORMAL};
static const int promptNode_indexedFaceSet_color[] = {COLOR};
static const int promptNode_indexedFaceSet_texCoord[] = {TEXTURECOORDINATE};
static const int promptNode_gestureGroup_children[] = {ANIM};
static const int promptNode_gestureGroup_mobility[] = {MOBILE};
static const int promptNode_surfaceGenerator_generator[] = {BEZIERBEND, BEZIERLATHE, BEZIERPATCH, BEZIERSCALE, BEZIERCIRCUMSCALE, BEZIERSHIFT, BEZIERTAPE, BEZIERTWIST, CCLATHE, CCBEND, CCSCALE, CCTRANSLATE, CCRINGSCALE, ECBEND, ECLATHE, ECSCALE, ECSHIFT, ECSYNCBEND, ECTWIST, RADIALTUG, NURBSPATCH, SINETUG, SURFACETRANSFORM};
static const int promptNode_stem_source[] = {PART, IMMEDIATEPART};
static const int promptNode_stem_part[] = {PART};
static const int promptNode_stem_setup[] = {MEMBERFUNCTION};
static const int promptNode_stem_textureMapper[] = {COLORMAPPER, COLORSWEEP, TEXTUREMAPPER};
static const int promptNode_sphereBuild_textureMapper[] = {STEM, COLORSWEEP, TEXTUREMAPPER};
static const int promptNode_stem_surface[] = {SURFACEGENERATOR};
static const int promptNode_stem_fuse[] = {FUSESURFACE};
static const int promptNode_place_part[] = {PART};
static const int promptNode_place_setup[] = {MEMBERFUNCTION};
static const int promptNode_seamless_effect[] = {TEXTUREEFFECT, COLOREFFECT, MONOCOLOREFFECT};
static const int promptNode_seamless_skeleton[] = {PART, IMMEDIATEPART};
static const int promptNode_seamless_build[] = {BENDYLATHE, BEZIERLATHEBUILD, BEZIERPATCH, BOXBUILD, BRANCHJOINNEGX, BRANCHJOINPOSX, BRANCHJOINNEGY, BRANCHJOINPOSY, BRANCHJOINNEGZ, BRANCHJOINPOSZ, CENTERJOINT, COLORSWEEP, CONEBUILD, COPYPART, COPYTOMORPH, CYLINDERBUILD, DELPLANE, DELPOLY, DELSPHERE, EVOLVECOORD, EXITBUILD, GENPOLY, GROUPBUILD, HAIR, JOINVERTEX, NURBSLATHE, NURBSLINE, NURBSPATCH, PLACE, PLACETEXTURE, POLYMAP, POLYTONURBS, PROJECTONTOTRI, RECTANGLEBUILD, RELATEMORPHER, RINGBUILD, SCGROUPBUILD, SPHEREBUILD, SPLITTRI, STEM, STEM2, SUBDIVISION, SWAPTRIDIVIDE, TEXTUREMAPPER, TORUSBUILD, TRANSFERBCYL, TRANSFEROWNERSHIP, TRANSFERVERTEX};
static const int promptNode_seamless_mirrorGroup[] = {MIRRORGROUP};
static const int promptNode_groupBuild_setup[] = {MEMBERFUNCTION};
static const int promptNode_joinCloseVertex_source[] = {PART};
static const int promptNode_joinCloseVertex_part[] = {PART};
static const int promptNode_joinCloseVertex_setup[] = {MEMBERFUNCTION};
static const int promptNode_centerJoint_part[] = {PART};
static const int promptNode_centerJoint_setup[] = {MEMBERFUNCTION};
static const int promptNode_delPoly_part[] = {PART};
static const int promptNode_delPoly_setup[] = {MEMBERFUNCTION};
static const int promptNode_joinVertex_source[] = {PART};
static const int promptNode_joinVertex_part[] = {PART};
static const int promptNode_joinVertex_setup[] = {MEMBERFUNCTION};
static const int promptNode_copyPart_source[] = {PART, IMMEDIATEPART};
static const int promptNode_copyPart_dest[] = {PART};
static const int promptNode_copyPart_setup[] = {MEMBERFUNCTION};
static const int promptNode_transferBCyl_source[] = {PART};
static const int promptNode_transferBCyl_dest[] = {PART};
static const int promptNode_transferBCyl_setup[] = {MEMBERFUNCTION};
static const int promptNode_transferOwnership_source[] = {STEM, PART};
static const int promptNode_transferVertex_source[] = {PART};
static const int promptNode_transferVertex_dest[] = {PART};
static const int promptNode_transferVertex_setup[] = {MEMBERFUNCTION};
static const int promptNode_genPoly_part[] = {PART};
static const int promptNode_genPoly_setup[] = {MEMBERFUNCTION};
static const int promptNode_branchJoinNegX_part[] = {PART};
static const int promptNode_branchJoinNegX_setup[] = {MEMBERFUNCTION};
static const int promptNode_branchJoinNegX_surface[] = {SURFACEGENERATOR};
static const int promptNode_branchJoinPosX_part[] = {PART};
static const int promptNode_branchJoinPosX_setup[] = {MEMBERFUNCTION};
static const int promptNode_branchJoinPosX_surface[] = {SURFACEGENERATOR};
static const int promptNode_branchJoinNegY_part[] = {PART};
static const int promptNode_branchJoinNegY_setup[] = {MEMBERFUNCTION};
static const int promptNode_branchJoinNegY_surface[] = {SURFACEGENERATOR};
static const int promptNode_branchJoinPosY_part[] = {PART};
static const int promptNode_branchJoinPosY_setup[] = {MEMBERFUNCTION};
static const int promptNode_branchJoinPosY_surface[] = {SURFACEGENERATOR};
static const int promptNode_branchJoinNegZ_part[] = {PART};
static const int promptNode_branchJoinNegZ_setup[] = {MEMBERFUNCTION};
static const int promptNode_branchJoinNegZ_surface[] = {SURFACEGENERATOR};
static const int promptNode_branchJoinPosZ_part[] = {PART};
static const int promptNode_branchJoinPosZ_setup[] = {MEMBERFUNCTION};
static const int promptNode_branchJoinPosZ_surface[] = {SURFACEGENERATOR};
static const int promptNode_colorSweep_part[] = {PART, IMMEDIATEPART};
static const int promptNode_colorSweep_setup[] = {MEMBERFUNCTION};
static const int promptNode_evolveCoord_source[] = {PART, IMMEDIATEPART};
static const int promptNode_evolveCoord_part[] = {PART};
static const int promptNode_evolveCoord_setup[] = {MEMBERFUNCTION};
static const int promptNode_function_codeLine[] = {CODELINE};
static const int promptNode_memberFunction_codeLine[] = {CODELINE};
static const int promptNode_codeLine_codeLine[] = {CODELINE};
static const int promptNode_morphPart_part[] = {PART};
static const int promptNode_morphPart_bank[] = {MORPHER};
static const int promptNode_anim_soundTrack[] = {COMPOSITION, AUDIO};
static const int promptNode_anim_video[] = {VIDEO};
static const int promptNode_anim_children[] = {ANIMBYANKLE, ANIMBYWRIST, BOOLEANFILTER, CAMERASELECTOR, BOOLEANTOGGLE, BOOLEANTRIGGER, CCPATH, COLORINTERPOLATOR, COORDINATEINTERPOLATOR, CODELINE, CPMORPHER, FLOATACCELERATOR, FUNCTION, _GROUP, INTEGERSEQUENCER, INTEGERTRIGGER, NORMALINTERPOLATOR, NURBSTRACK, NIKINTERPOLATOR, ORIENTATIONINTERPOLATOR, POSACCELERATOR, POSCOMPENSATOR, POSITIONINTERPOLATOR, ROTACCELERATOR, ROTFILTER, ROTMULT, SCALARINTERPOLATOR, SOUND, TIMETRIGGER};
static const int promptNode_anim_onFrame[] = {MEMBERFUNCTION};
static const int promptNode_movie_clip[] = {ANIM, VIDEO};
static const int promptNode_timeSensor_on_fraction_changed[] = {MEMBERFUNCTION};
static const int promptNode_elevationGrid_color[] = {COLOR};
static const int promptNode_elevationGrid_normal[] = {NORMAL};
static const int promptNode_elevationGrid_texCoord[] = {TEXTURECOORDINATE};
static const int promptNode_indexedLineSet_coord[] = {COORDINATE};
static const int promptNode_indexedLineSet_color[] = {COLOR};
static const int promptNode_pointSet_coord[] = {COORDINATE};
static const int promptNode_pointSet_color[] = {COLOR};
static const int promptNode_sound_source[] = {AUDIOCLIP, MOVIETEXTURE};
static const int promptNode_text_fontStyle[] = {FONTSTYLE};
static const int promptNode_exitBuild_setup[] = {MEMBERFUNCTION};
static const int promptNode_splitTri_part[] = {PART};
static const int promptNode_splitTri_setup[] = {MEMBERFUNCTION};
static const int promptNode_swapTriDivide_part[] = {PART};
static const int promptNode_swapTriDivide_setup[] = {MEMBERFUNCTION};
static const int promptNode_placeTexture_part[] = {PART};
static const int promptNode_placeTexture_setup[] = {MEMBERFUNCTION};
static const int promptNode_hAnimHumanoid_skeleton[] = {HANIMJOINT};
static const int promptNode_hAnimHumanoid_joints[] = {HANIMJOINT};
static const int promptNode_hAnimHumanoid_skin[] = {SHAPE};
static const int promptNode_hAnimHumanoid_skinCoord[] = {COORDINATE};
static const int promptNode_hAnimHumanoid_skinNormal[] = {NORMAL};
static const int promptNode_hAnimJoint_children[] = {HANIMJOINT, HANIMSEGMENT, HANIMSITE};
static const int promptNode_hAnimJoint_displacers[] = {HANIMDISPLACER};
static const int promptNode_copyToMorph_part[] = {PART, IMMEDIATEPART};
static const int promptNode_copyToMorph_morpher[] = {MORPHER};
static const int promptNode_copyToMorph_setup[] = {MEMBERFUNCTION};
static const int promptNode_relateMorpher_part[] = {PART};
static const int promptNode_relateMorpher_setup[] = {MEMBERFUNCTION};
static const int promptNode_immediatePart_effect[] = {TEXTUREEFFECT, COLOREFFECT, MONOCOLOREFFECT};
static const int promptNode_immediatePart_part[] = {PART, IMMEDIATEPART};
static const int promptNode_delPlane_part[] = {PART};
static const int promptNode_delPlane_setup[] = {MEMBERFUNCTION};
static const int promptNode_tapeMeasure_part[] = {PART, IMMEDIATEPART};
static const int promptNode_tapeMeasure_setup[] = {MEMBERFUNCTION};
static const int promptNode_animByAnkle_pelvis[] = {PART};
static const int promptNode_animByAnkle_hip[] = {PART};
static const int promptNode_animByAnkle_knee[] = {PART};
static const int promptNode_animByAnkle_ankle[] = {PART};
static const int promptNode_animByWrist_sholder[] = {PART};
static const int promptNode_animByWrist_elbow[] = {PART};
static const int promptNode_animByWrist_wrist[] = {PART};
static const int promptNode_indexedTriangleSet_coord[] = {COORDINATE};
static const int promptNode_indexedTriangleSet_normal[] = {NORMAL};
static const int promptNode_indexedTriangleSet_color[] = {COLOR};
static const int promptNode_indexedTriangleSet_texCoord[] = {TEXTURECOORDINATE};
static const int promptNode_sphereBuild_part[] = {PART};
static const int promptNode_sphereBuild_setup[] = {MEMBERFUNCTION};
static const int promptNode_cylinderBuild_part[] = {PART};
static const int promptNode_cylinderBuild_setup[] = {MEMBERFUNCTION};
static const int promptNode_coneBuild_part[] = {PART};
static const int promptNode_coneBuild_setup[] = {MEMBERFUNCTION};
static const int promptNode_torusBuild_part[] = {PART};
static const int promptNode_torusBuild_setup[] = {MEMBERFUNCTION};
static const int promptNode_boxBuild_part[] = {PART};
static const int promptNode_boxBuild_setup[] = {MEMBERFUNCTION};
static const int promptNode_rectangleBuild_part[] = {PART};
static const int promptNode_rectangleBuild_setup[] = {MEMBERFUNCTION};
static const int promptNode_bezierLatheBuild_part[] = {PART};
static const int promptNode_bezierLatheBuild_setup[] = {MEMBERFUNCTION};
static const int promptNode_mirrorGroup_mirrorEdit[] = {MIRROREDIT};
static const int promptNode_mirrorEdit_part[] = {PART, IMMEDIATEPART};
static const int promptNode_polyToNurbs_mimic[] = {NURBSPATCH, POLYMAP};
static const int promptNode_polyToNurbs_original[] = {PART, IMMEDIATEPART};
static const int promptNode_projectOntoTri_feedback[] = {FEEDBACK};
static const int promptNode_feedback_source[] = {PART, IMMEDIATEPART};
//static const int promptNode_nurbsPatch_fuseSurface[] = {FUSESURFACE};
static const int promptNode_nurbsPatch_stem[] = {STEM, STEM2};
static const int promptNode_nurbsPatch_mapper[] = {COLORMAPPER, TEXTUREMAPPER};
static const int promptNode_fuseSurface_patch[] = {NURBSPATCH};
static const int promptNode_NikIntpolator_nurbsLine[] = {NURBSLINE};
static const int promptNode_envelope_chords[] = {CHORDS};
static const int promptNode_larynx_chords[] = {CHORDS, ENVELOPE};
static const int promptNode_chords_phaseRef[] = {CHORDS};
static const int promptNode_oscillator_larynx[] = {LARYNX};
static const int promptNode_oscillator_melody[] = {NURBSLINE};
static const int promptNode_oscillator_filter[] = {FILTER, HARMONIC, HARMONICPATCH, SAMPLE, NURBSPATCH};
static const int promptNode_midiFormant_patch[] = {FILTER, NURBSPATCH};
static const int promptNode_filter_formant[] = {FORMANT, LOWPASS, MULTIFORMANT};
static const int promptNode_chords_patch[] = {NURBSPATCH};
static const int promptNode_chords_amplitudeModulator[] = {AMPLITUDEMODULATOR, FREQUENCYMODULATOR};
static const int promptNode_chords_envelope[] = {ENVELOPE};
static const int promptNode_modulator_oscillator[] = {OSCILLATOR};
static const int promptNode_track_sound[] = {OSCILLATOR, PHRASE};
static const int promptNode_composition_track[] = {TRACK};
static const int promptNode_modelSequence_seamless[] = {SEAMLESS};
static const int promptNode_modelMorpher_sequence[] = {MODELSEQUENCE};
static const int promptNode_fuseSurface_stem[] = {STEM};
static const int promptNode_settings_commonSpec[] = {COMMON_SPEC};

static const int promptNode_hair_strand[] = {STRAND};
static const int promptNode_hair_lock[] = {LOCK, NURBSPATCH};

static const BIProto6 mesh_ = BIProto6(
	"Mesh", "source", Vec2i(16,16), Vec2i(224,160),
	BIField(MFVEC3F,	"coord []", 0),
	BIField(MFVEC3F,	"normal []", 0),
	BIField(MFCOLOR,	"color []", 0),
	BIField(MFVEC2F,	"texCoord []", 0),
	BIField(MFINT32,	"coordIndex []{rangeFrom 0}", UNREADABLE | UNWRITEABLE),
	BIField(MFINT32,	"rc []", UNREADABLE | UNWRITEABLE)
);
static const BIProto16 scene_ = BIProto16(
	"Scene", 0, Vec2i(16,16), Vec2i(192,112),
	BIField(SFFLOAT,	"version 0", UNREADABLE | UNWRITEABLE | DISABLE),

	BIField(SFCOLOR,	"backgroundColor 0 0 0", 0),
	BIField(SFCOLOR,	"scene3DFontColor 1 1 0", 0),

	BIField(SFINT32, 	"mode 3", UNREADABLE | UNWRITEABLE | DISABLE), //force it to allways save by having a value that can never be
	//BIField(SFBOOL,		"xRayEdit FALSE", UNREADABLE | UNWRITEABLE | DISABLE),
	//BIField(SFBOOL,		"wireframe TRUE", UNREADABLE | UNWRITEABLE | DISABLE),

	BIField(SFINT32, 	"selLine 0", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(SFINT32,	"scroll 0", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(MFINT32,	"collapse []", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(MFINT32,	"wOpen []", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(MFVEC2F,	"wPos []", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(MFVEC2F,	"wSize []", UNREADABLE | UNWRITEABLE | DISABLE),

	BIField(SFNODE,	"grid NULL", DISABLE,	0),
	BIField(SFNODE,	"exportMovie NULL", DISABLE,	0),
	BIField(SFNODE,	"exportVrmlX3d NULL", DISABLE,	0),
	BIField(SFNODE,	"brush NULL", DISABLE,	0),
	BIField(SFNODE,	"fill NULL", DISABLE,	0),
	BIField(MFNODE,	"children []", 0,	INIT_PROMPT_NODE(promptNode_scene_children))
);
static const BIProto24 seamless_ = BIProto24(
	"Seamless", "children", Vec2i(16,16), Vec2i(0,112),
	BIField(SFFLOAT,	"version 0", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(MFVEC3F,	"cp []{beginGroup TRUE}", DISABLE | DO_NOT_SAVE),
	BIField(MFFLOAT,	"cw []", DISABLE | DO_NOT_SAVE),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFBOOL,	"track TRUE", DISABLE),
	BIField(SFINT32, "tesselate 4", 0),
	BIField(SFBOOL,	"autoJoin TRUE",  0),
	BIField(SFBOOL,	"autoStitch TRUE", 0),
	BIField(SFBOOL,	"bnspe TRUE", 0),
	BIField(SFBOOL,	"xRay FALSE", 0),
	BIField(SFBOOL,	"autoIn FALSE", 0),
	BIField(SFBOOL,	"in FALSE", DISABLE),
	BIField(SFBOOL,	"animCP FALSE", 0),
	BIField(SFBOOL,	"buildNormals FALSE", DISABLE),
	BIField(SFBOOL,	"surfaceNormals FALSE", 0),
	BIField(SFBOOL,	"ignoreWireFrame FALSE", 0),
	BIField(SFBOOL,	"shadow TRUE", 0),
	BIField(MFINT32,"allot []", DISABLE), //0 1 0 27, 1 1 27 27 partIndex, number of groups, point index, num of points (point index & num of points = group)
	BIField(MFINT32,"stitchIndex []", DISABLE), //destination, source
	BIField(SFNODE,	"effect NULL", 0,	INIT_PROMPT_NODE(promptNode_seamless_effect)),
	BIField(MFNODE,	"skeleton []", BLOCK_USE,	INIT_PROMPT_NODE(promptNode_seamless_skeleton)),
	BIField(MFNODE,	"build []", 0,	INIT_PROMPT_NODE(promptNode_seamless_build))
	//BIField(SFNODE,	"mirrorGroup NULL", 0,	INIT_PROMPT_NODE(promptNode_seamless_mirrorGroup))
);
static const BIProto22 colorEffect_ = BIProto22(
	"ColorEffect", "effect", Vec2i(16,16), Vec2i(128,160), //ColorEffect
	BIField(SFCOLOR,	"ambient .2 .2 .2", 0),
	BIField(SFCOLOR,	"diffuse .8 .8 .8", 0),
	BIField(SFFLOAT,	"brilliance 1", 0),
	BIField(SFCOLOR,	"emissive 0 0 0", 0),
	BIField(SFFLOAT,	"shininess .2{rangeFrom 0 rangeTo 1}", 0),
	BIField(SFCOLOR,	"specular 0 0 0", 0),
	BIField(SFBOOL,		"backfaceRender FALSE", 0),
	BIField(SFFLOAT,	"phong 0", 0),
	BIField(SFFLOAT,	"phong_size 40", 0),
	BIField(SFFLOAT,	"roughness .05", 0),
	BIField(SFFLOAT,	"crand 0", 0),
	BIField(SFBOOL,		"conserve_energy FALSE", 0),
	BIField(SFCOLOR,	"reflection 0 0 0", 0),
	BIField(SFBOOL,		"fresnel FALSE", 0),
	BIField(SFFLOAT,	"falloff 0", 0),
	BIField(SFFLOAT,	"exponent 1", 0),
	BIField(SFFLOAT,	"metallic 1", 0),
	BIField(SFFLOAT,	"iridescence 0", 0),
	BIField(SFFLOAT,	"thickness 0", 0),
	BIField(SFFLOAT,	"turbulence 0", 0),
	BIField(SFFLOAT,	"transparency 0{rangeFrom 0 rangeTo 1}", 0),
	BIField(MFCOLOR,	"color []", DISABLE_LEN)
);
static const BIProto23 textureEffect_ = BIProto23(
	"TextureEffect", "effect", Vec2i(16,16), Vec2i(320,96), //TextureEffect
	BIField(SFCOLOR,	"ambient .2 .2 .2", 0),
	BIField(SFCOLOR,	"diffuse .8 .8 .8", 0),
	BIField(SFFLOAT,	"brilliance 1", 0),
	BIField(SFCOLOR,	"emissive 0 0 0", 0),
	BIField(SFFLOAT,	"shininess .2{rangeFrom 0 rangeTo 1}", 0),
	BIField(SFCOLOR,	"specular 0 0 0", 0),
	BIField(SFBOOL,		"backfaceRender FALSE", 0),
	BIField(SFFLOAT,	"phong 0", 0),
	BIField(SFFLOAT,	"phong_size 40", 0),
	BIField(SFFLOAT,	"roughness .05", 0),
	BIField(SFFLOAT,	"crand 0", 0),
	BIField(SFBOOL,		"conserve_energy FALSE", 0),
	BIField(SFCOLOR,	"reflection 0 0 0", 0),
	BIField(SFBOOL,		"fresnel FALSE", 0),
	BIField(SFFLOAT,	"falloff 0", 0),
	BIField(SFFLOAT,	"exponent 1", 0),
	BIField(SFFLOAT,	"metallic 1", 0),
	BIField(SFFLOAT,	"iridescence 0", 0),
	BIField(SFFLOAT,	"thickness 0", 0),
	BIField(SFFLOAT,	"turbulence 0", 0),
	BIField(SFFLOAT,	"transparency 0{rangeFrom 0 rangeTo 1}", 0),
	BIField(SFSTRING,	"url \"\"", 0),
	BIField(MFVEC2F,	"texCoord []", DISABLE_LEN)
);
static const BIProto22 monoColorEffect_ = BIProto22(
	"MonoColorEffect", "effect", Vec2i(16,16), Vec2i(112,96), //MonoColorEffect
	BIField(SFCOLOR,	"ambient .2 .2 .2", 0),
	BIField(SFCOLOR,	"diffuse .8 .8 .8", 0),
	BIField(SFFLOAT,	"brilliance 0", 0),
	BIField(SFCOLOR,	"emissive 0 0 0", 0),
	BIField(SFFLOAT,	"shininess .2{rangeFrom 0 rangeTo 1}", 0),
	BIField(SFCOLOR,	"specular 0 0 0", 0),
	BIField(SFBOOL,		"backfaceRender FALSE", 0),
	BIField(SFFLOAT,	"phong 0", 0),
	BIField(SFFLOAT,	"phong_size 40", 0),
	BIField(SFFLOAT,	"roughness .05", 0),
	BIField(SFFLOAT,	"crand 0", 0),
	BIField(SFBOOL,		"conserve_energy FALSE", 0),
	BIField(SFCOLOR,	"reflection 0 0 0", 0),
	BIField(SFBOOL,		"fresnel FALSE", 0),
	BIField(SFFLOAT,	"falloff 0", 0),
	BIField(SFFLOAT,	"exponent 1", 0),
	BIField(SFFLOAT,	"metallic 1", 0),
	BIField(SFFLOAT,	"iridescence 0", 0),
	BIField(SFFLOAT,	"thickness 0", 0),
	BIField(SFFLOAT,	"turbulence 0", 0),
	BIField(SFFLOAT,	"transparency 0{rangeFrom 0 rangeTo 1}", 0),
	BIField(MFCOLOR,	"color []", DISABLE_LEN)
);
static const BIProto22 part_ = BIProto22(
	"Part", "_part", Vec2i(16,16), Vec2i(0,96),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFBOOL,		"track TRUE", DISABLE),
	BIField(SFBOOL,		"mirror TRUE", 0),
	BIField(SFBOOL,		"ik FALSE", 0),
	BIField(SFBOOL,		"pin FALSE", 0),
	BIField(SFBOOL,		"rh FALSE", 0),
	BIField(SFBOOL,		"nik FALSE", PUSH | DO_NOT_SAVE),
	BIField(SFROTATION,	"ikRot 0 0 1 0", 0),
	BIField(SFVEC3F,	"ikPos 0 0 0", 0),
	BIField(SFFLOAT,	"fka 0", 0), //ik blend angle
	BIField(SFFLOAT,	"fkb 0", 0), //ik blend interpolation
	BIField(MFVEC3F,	"coord []", DISABLE_LEN),
	BIField(MFVEC3F,	"normal []", DISABLE | DISABLE_LEN | DO_NOT_SAVE),
	BIField(MFINT32,	"coordGroup []{beginGroup TRUE rangeFrom 0 initElVal \"10\"}", SAME_AS_LAST | DISABLE),
	BIField(MFFLOAT,	"weight []{initElVal \"1\"}", SAME_AS_LAST | DISABLE),
	BIField(SFVEC3F,	"pivotPoint 0 0 0", 0),
	BIField(MFVEC2F,	"uvCoord []", DISABLE),
	BIField(SFNODE,		"effect NULL", 0,	INIT_PROMPT_NODE(promptNode_part_effect)),
	BIField(MFNODE,	"morpher []", BLOCK_USE,	INIT_PROMPT_NODE(promptNode_part_morpher)),
	BIField(MFNODE,	"part []", BLOCK_USE,	INIT_PROMPT_NODE(promptNode_part_part))
);
static const BIProto22 immediatePart_ = BIProto22(
	"ImmediatePart", "_part", Vec2i(16,16), Vec2i(128,96), //ImmediatePart
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFBOOL,		"track TRUE", DISABLE),
	BIField(SFBOOL,		"mirror TRUE", 0),
	BIField(SFBOOL,		"ik FALSE", 0),
	BIField(SFBOOL,		"pin FALSE", 0),
	BIField(SFBOOL,		"rh FALSE", 0),
	BIField(SFBOOL,		"nik FALSE", PUSH | DO_NOT_SAVE),
	BIField(SFROTATION,	"ikRot 0 0 1 0", 0),
	BIField(SFVEC3F,	"ikPos 0 0 0", 0),
	BIField(SFFLOAT,	"fka 0", 0), //ik blend angle
	BIField(SFFLOAT,	"fkb 0", 0), //ik blend interpolation
	BIField(MFVEC3F,	"coord []", DISABLE_LEN),
	BIField(MFVEC3F,	"normal []", DISABLE | DISABLE_LEN),
	BIField(MFINT32,	"coordGroup []{beginGroup TRUE rangeFrom 0 initElVal \"10\"}", SAME_AS_LAST | DISABLE),
	BIField(MFFLOAT,	"weight []{initElVal \"1\"}", SAME_AS_LAST | DISABLE),
	BIField(SFVEC3F,	"pivotPoint 0 0 0", 0),
	BIField(MFVEC2F,	"uvCoord []", DISABLE),
	BIField(SFNODE,	"effect NULL", 0,	INIT_PROMPT_NODE(promptNode_part_effect)),
	BIField(MFNODE,	"morpher []", BLOCK_USE,	INIT_PROMPT_NODE(promptNode_part_morpher)),
	BIField(MFNODE,	"part []", BLOCK_USE,	INIT_PROMPT_NODE(promptNode_part_part))
);
static const BIProto4 groupBuild_ = BIProto4(
	"GroupBuild", "children", Vec2i(16,16), Vec2i(304,112),
	BIField(SFCOLOR,"marker 1 1 1", 0),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_groupBuild_setup)),
	BIField(SFNODE, 	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_bezierLatheBuild_part)),
	BIField(MFNODE,	"build []", 0,		INIT_PROMPT_NODE(promptNode_seamless_build))
);
static const BIProto32 stem_ = BIProto32(
	"Stem", "build", Vec2i(16,16), Vec2i(16,112),
	BIField(SFVEC2F,	"cells 0 0", 0),
	BIField(SFBOOL,	"add TRUE", 0, tip_stem_add),
	BIField(SFBOOL,	"mirror FALSE", 0),
	BIField(SFBOOL,	"store FALSE", 0),

	BIField(SFVEC2F,	"slide 0 0", 0,	tip_stem_slide),
	BIField(SFROTATION,	"orientation 0 0 1 0", DISABLE,	tip_stem_orientation),
	BIField(MFINT32,	"staves []{beginGroup TRUE rangeFrom 0}", 0,	tip_stem_staves),
	BIField(MFFLOAT,	"xDistance []{beginGroup TRUE}", 0,	tip_stem_xDistance),
	BIField(MFFLOAT,	"height []{beginGroup TRUE initElVal \"[0 1]\"}", SAME_AS_LAST,	tip_stem_height),
	BIField(MFFLOAT,	"yDistance []{beginGroup TRUE initElVal \"1\"}", SAME_AS_LAST ,	tip_stem_yDistance),
	BIField(MFINT32,	"bands []{beginGroup TRUE rangeFrom 0 initElVal \"16\"}", SAME_AS_LAST,	tip_stem_bands),
	//BIField(MFINT32,	"ownership []{beginGroup TRUE initElVal \"1\" rangeFrom 0}", SAME_AS_LAST),
	BIField(SFBOOL,	"sync FALSE", 0,	tip_stem_sync),
	BIField(SFBOOL,	"mould TRUE", 0,	tip_stem_mould),
	BIField(SFBOOL,	"northPole FALSE", 0),
	BIField(SFBOOL,	"southPole FALSE", 0),
	BIField(SFBOOL,	"insideOut FALSE", 0,	tip_stem_insideOut),
	BIField(SFBOOL,	"xCentred TRUE", 0),
	BIField(SFBOOL,	"uWrap FALSE", 0,	tip_stem_joinEnds),
	BIField(SFBOOL,	"vWrap FALSE", 0,	tip_stem_joinEnds),
	BIField(SFBOOL,	"endSP FALSE", 0,	tip_stem_endSP),
	BIField(SFBOOL,	"beginSP FALSE", 0,	tip_stem_beginSP),

	BIField(MFVEC2F,	"coord []", DISABLE),
	BIField(MFCOLOR,	"color []", DISABLE),
	BIField(MFVEC2F,	"texCoord []", DISABLE),
	BIField(MFINT32,	"coordIndex []{rangeFrom 0}", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(MFINT32,	"rc []", UNREADABLE | UNWRITEABLE | DISABLE),

	BIField(MFNODE,	"source []", BLOCK_NEW | BLUE_DOT,	INIT_PROMPT_NODE(promptNode_stem_source)),
	BIField(MFNODE,	"part []", BLOCK_NEW | RED_DOT,	INIT_PROMPT_NODE(promptNode_stem_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_stem_setup)),
	BIField(MFNODE,	"textureMapper []", 0,	INIT_PROMPT_NODE(promptNode_stem_textureMapper)),
	BIField(SFNODE,	"surface NULL", 0,	INIT_PROMPT_NODE(promptNode_stem_surface)),
	BIField(MFNODE,	"fuse []", 0,	INIT_PROMPT_NODE(promptNode_stem_fuse))
);
static const BIProto10 textureMapper_ = BIProto10(
	"TextureMapper", "textureMapper", Vec2i(16,16), Vec2i(0,160), //TextureMapper
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFBOOL,	"autoSize3d FALSE", 0),
	BIField(SFVEC2F,	"size3d 1 1", 0),
	BIField(SFVEC3F,	"position3d 0 0 0", 0),
	BIField(SFVEC2F,	"size2d 1 1", 0),
	BIField(SFVEC2F,	"position2d 0 0", 0),
	BIField(SFBOOL,	"flat TRUE", 0),
	BIField(SFBOOL,	"spherical FALSE", 0),
	BIField(SFBOOL,	"cylindrical FALSE", 0),
	BIField(SFBOOL,	"clear FALSE", PUSH)
);
static const BIProto11 colorSweep_ = BIProto11(
	"ColorSweep", "build", Vec2i(16,16), Vec2i(colorSweepIcon_x, colorSweepIcon_y), //ColorSweep
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(MFCOLOR,	"color []{beginGroup TRUE initElVal \"[1 1 1 1 1 1]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"distance []{startAtIndex 1 initElVal \"1\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"opacity []{initElVal \"[1 1]\"}", SAME_AS_LAST),
	BIField(SFBOOL,	"spherical FALSE", 0),
	BIField(SFBOOL,	"mirror FALSE", DISABLE),
	BIField(SFBOOL,	"cylindrical FALSE", DISABLE),
	BIField(SFBOOL,	"straight FALSE", 0),
	BIField(MFNODE,	"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_colorSweep_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_colorSweep_setup))
);
static const BIProto1 colorMapper_ = BIProto1(
	"ColorMapper", "build", Vec2i(16,16), Vec2i(colorSweepIcon_x, colorSweepIcon_y), //ColorSweep
	BIField(MFCOLOR,	"color []", 0)
);
static const BIProto1 surfaceGenerator_ = BIProto1(
	"SurfaceGenerator", "surface", Vec2i(16,16), Vec2i(0,128),
	BIField(MFNODE,	"generator []", 0,	INIT_PROMPT_NODE(promptNode_surfaceGenerator_generator))
);
static const BIProto4 cCLathe_ = BIProto4(
	"CCLathe", "generator", Vec2i(16,16), Vec2i(48,112),
	BIField(MFFLOAT,	"curve []{beginGroup TRUE initElVal \"[-1.5708 1.5708 ]\"}", SAME_AS_LAST,	tip_cCLathe_curve),
	BIField(MFFLOAT,	"distance []{startAtIndex 1 initElVal \"1.5708\"}", SAME_AS_LAST,	tip_cCLathe_distance),
	BIField(SFFLOAT,	"baseRad 0", 0,	tip_cCLathe_baseRad),
	BIField(SFBOOL,	"autoClose FALSE", 0,	tip_cCLathe_autoClose)
);
static const BIProto24 nurbsPatch_ = BIProto24(
	"NurbsPatch", "build", Vec2i(16,16), Vec2i(160,192), //NurbsPatch
	BIField(MFVEC3F,	"point []{beginGroup TRUE}", DISABLE), //DO_NOT_SAVE for new smls cp format
	BIField(MFVEC3F,	"point []", DISABLE_LEN | DO_NOT_SAVE),
	BIField(MFFLOAT,	"weight []", 0), //DO_NOT_SAVE for new smls cp format
	BIField(MFINT32,	"cpIndex []", DISABLE | DO_NOT_SAVE),
	BIField(MFFLOAT,	"uKnot []{beginGroup TRUE}", DISABLE),
	BIField(MFFLOAT,	"vKnot []{beginGroup TRUE}", DISABLE),
	BIField(SFVEC2F,	"order 3 3", DISABLE),

	BIField(SFBOOL,		"object FALSE", DISABLE | DO_NOT_SAVE),
	BIField(SFBOOL,		"cage TRUE", DISABLE),
	BIField(SFBOOL,		"showAll FALSE", 0),
	BIField(SFBOOL,		"mirror FALSE", 0),
	BIField(SFVEC2F,	"dim 0 0", 0),

	BIField(SFBOOL,		"endSP FALSE", 0),
	BIField(SFBOOL,		"uJoinEnds TRUE", 0),
	BIField(SFBOOL,		"beginSP FALSE", 0),

	BIField(SFBOOL,		"sync FALSE", 0),
	BIField(SFBOOL,		"insideOut FALSE", 0),

	BIField(SFVEC3F,	"scale 1 1 1", DISABLE),
	BIField(SFROTATION,	"rotation 0 0 1 0", DISABLE),
	BIField(SFVEC3F,	"translation 0 0 0", DISABLE),
	BIField(SFNODE,		"fuseSurface NULL", 0),	//INIT_PROMPT_NODE(promptNode_nurbsPatch_fuseSurface)),
	BIField(MFNODE,		"stem []", 0,	INIT_PROMPT_NODE(promptNode_nurbsPatch_stem)),
	BIField(SFNODE,		"mapper NULL", 0,	INIT_PROMPT_NODE(promptNode_nurbsPatch_mapper)),
	BIField(SFNODE, 	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_bezierLatheBuild_setup))
);
static const BIProto25 colorPatch_ = BIProto25(
	"ColorPatch", "build", Vec2i(16,16), Vec2i(160,192), //NurbsPatch
	BIField(MFVEC3F,	"point []{beginGroup TRUE}", DISABLE), //DO_NOT_SAVE for new smls cp format
	BIField(MFVEC3F,	"point []", DISABLE_LEN | DO_NOT_SAVE),
	BIField(MFFLOAT,	"weight []", 0), //DO_NOT_SAVE for new smls cp format
	BIField(MFCOLOR,	"color []", 0),
	BIField(MFINT32,	"cpIndex []", DISABLE | DO_NOT_SAVE),
	BIField(MFFLOAT,	"uKnot []{beginGroup TRUE}", DISABLE),
	BIField(MFFLOAT,	"vKnot []{beginGroup TRUE}", DISABLE),
	BIField(SFVEC2F,	"order 3 3", DISABLE),

	BIField(SFBOOL,		"object FALSE", DISABLE | DO_NOT_SAVE),
	BIField(SFBOOL,		"cage TRUE", DISABLE),
	BIField(SFBOOL,		"showAll FALSE", 0),
	BIField(SFBOOL,		"mirror FALSE", 0),
	BIField(SFVEC2F,	"dim 0 0", 0),

	BIField(SFBOOL,		"endSP FALSE", 0),
	BIField(SFBOOL,		"uJoinEnds TRUE", 0),
	BIField(SFBOOL,		"beginSP FALSE", 0),

	BIField(SFBOOL,		"sync FALSE", 0),
	BIField(SFBOOL,		"insideOut FALSE", 0),

	BIField(SFVEC3F,	"scale 1 1 1", DISABLE),
	BIField(SFROTATION,	"rotation 0 0 1 0", DISABLE),
	BIField(SFVEC3F,	"translation 0 0 0", DISABLE),
	BIField(SFNODE,		"fuseSurface NULL", 0),	//INIT_PROMPT_NODE(promptNode_nurbsPatch_fuseSurface)),
	BIField(MFNODE,		"stem []", 0,	INIT_PROMPT_NODE(promptNode_nurbsPatch_stem)),
	BIField(SFNODE,		"mapper NULL", 0,	INIT_PROMPT_NODE(promptNode_nurbsPatch_mapper)),
	BIField(SFNODE, 	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_bezierLatheBuild_setup))
);
static const BIProto2 shape_ = BIProto2(
	"Shape", "children", Vec2i(16,16), Vec2i(224,128),
	BIField(SFNODE,	"appearance NULL", 0,	INIT_PROMPT_NODE(promptNode_shape_appearance)),
	BIField(SFNODE,	"geometry NULL", 0,	INIT_PROMPT_NODE(promptNode_shape_geometry))
);
static const BIProto3 appearance_ = BIProto3(
	"Appearance", "appearance", Vec2i(16,16), Vec2i(256,112),
	BIField(SFNODE,	"material NULL", 0,	INIT_PROMPT_NODE(promptNode_appearance_material)),
	BIField(SFNODE,	"texture NULL", 0,	INIT_PROMPT_NODE(promptNode_appearance_texture)),
	BIField(SFNODE,	"textureTransform NULL", 0,	INIT_PROMPT_NODE(promptNode_appearance_textureTransform))
);
static const BIProto6 material_ = BIProto6(
	"Material", "material", Vec2i(16,16), Vec2i(272,112),
	BIField(SFFLOAT,	"ambientIntensity .2{rangeFrom 0 rangeTo 1}", 0),
	BIField(SFCOLOR,	"diffuseColor .8 .8 .8", 0),
	BIField(SFCOLOR,	"emissiveColor 0 0 0", 0),
	BIField(SFFLOAT,	"shininess .2{rangeFrom 0 rangeTo 1}", 0),
	BIField(SFCOLOR,	"specularColor 0 0 0", 0),
	BIField(SFFLOAT,	"transparency 0{rangeFrom 0 rangeTo 1}", 0)
);
static const BIProto7 indexedLineSet_ = BIProto7(
	"IndexedLineSet", "geometry", Vec2i(16,16), Vec2i(432,144), //IndexedLineSet
	BIField(MFINT32,	"set_colorIndex", UNREADABLE),
	BIField(MFINT32,	"set_coordIndex", UNREADABLE),
	BIField(SFNODE,	"coord NULL", 0,	INIT_PROMPT_NODE(promptNode_indexedLineSet_coord)),
	BIField(SFNODE,	"color NULL", 0,	INIT_PROMPT_NODE(promptNode_indexedLineSet_color)),
	BIField(MFINT32,	"colorIndex []{rangeFrom -1}", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"colorPerVertex TRUE", UNREADABLE | UNWRITEABLE),
	BIField(MFINT32,	"coordIndex []{rangeFrom -1}", UNREADABLE | UNWRITEABLE)
);
static const BIProto1 coordinate_ = BIProto1(
	"Coordinate", "coord", Vec2i(16,16), Vec2i(256,128),
	BIField(MFVEC3F,	"point []", 0)
);
static const BIProto1 color_ = BIProto1(
	"Color", "color", Vec2i(16,16), Vec2i(288,128),
	BIField(MFCOLOR,	"color []", 0)
);


















static const BIProto18 indexedFaceSet_ = BIProto18(
	"IndexedFaceSet", "geometry", Vec2i(16,16), Vec2i(448,144),
	BIField(MFINT32,	"set_coordIndex", UNREADABLE),
	BIField(MFINT32,	"set_normalIndex", UNREADABLE),
	BIField(MFINT32,	"set_colorIndex", UNREADABLE),
	BIField(MFINT32,	"set_texCoordIndex", UNREADABLE),
	BIField(SFNODE,	"coord NULL", 0,	INIT_PROMPT_NODE(promptNode_indexedFaceSet_coord)),
	BIField(SFNODE,	"normal NULL", 0,	INIT_PROMPT_NODE(promptNode_indexedFaceSet_normal)),
	BIField(SFNODE,	"color NULL", 0,	INIT_PROMPT_NODE(promptNode_indexedFaceSet_color)),
	BIField(SFNODE,	"texCoord NULL", 0,	INIT_PROMPT_NODE(promptNode_indexedFaceSet_texCoord)),
	BIField(SFBOOL,	"ccw TRUE", UNREADABLE | UNWRITEABLE),
	BIField(MFINT32,	"coordIndex []{rangeFrom -1}", UNREADABLE | UNWRITEABLE),
	BIField(MFINT32,	"normalIndex []{rangeFrom -1}", UNREADABLE | UNWRITEABLE),
	BIField(MFINT32,	"colorIndex []{rangeFrom -1}", UNREADABLE | UNWRITEABLE),
	BIField(MFINT32,	"texCoordIndex []{rangeFrom -1}", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"normalPerVertex TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"colorPerVertex TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"convex TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFFLOAT,	"creaseAngle 0", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"solid TRUE", UNREADABLE | UNWRITEABLE)
);

























static const BIProto0 nullNode_ = BIProto0(
	"NullNode", 0, Vec2i(16,16), Vec2i(0,96)
);
/*
static const BIProto8 grid_ = BIProto8(
	"Grid", 0,
	BIField(SFFLOAT,	"h_xDistance 1", 0),
	BIField(SFFLOAT,	"h_yDistance 1", 0),
	BIField(SFINT32,	"h_staves 4{rangeFrom 1}", 0),
	BIField(SFINT32,	"h_bands 4{rangeFrom 1}", 0),
	BIField(SFVEC3F,	"h_position 0 0 0", 0),
	BIField(SFBOOL,		"h_tiled TRUE", 0),

	BIField(SFFLOAT,	"verticalStep .25", 0),
	BIField(SFINT32,	"numbOfVerticalSteps 8", 0),
	//BIField(SFCOLOR,	"floorColor .5 .5 .5", 0),
	BIField(SFVEC3F,	"baseHeight 0", 0)
);
*/
static const BIProto8 avatarSocket_ = BIProto8(
	"AvatarSocket", 0, Vec2i(16,16), Vec2i(448,128),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"position 0 0 0", 0),
	BIField(MFNODE,	"children []", 0),
	BIField(SFSTRING,	"nickname \"\"", 0),
	BIField(SFSTRING,	"avatarUrl \"\"", 0),
	BIField(SFBOOL,	"male FALSE", 0),
	BIField(SFSTRING,	"voice \"\"", 0),
	BIField(SFINT32,	"id 0", UNREADABLE | UNWRITEABLE | DISABLE)
);
static const BIProto3 imageTexture_ = BIProto3(
	"ImageTexture", "texture", Vec2i(16,16), Vec2i(256,144),
	BIField(MFSTRING,	"url []", 0),
	BIField(SFBOOL,	"repeatS TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"repeatT TRUE", UNREADABLE | UNWRITEABLE)
);
static const BIProto7 navigationInfo_ = BIProto7(
	"NavigationInfo", "navigationInfo", Vec2i(16,16), Vec2i(288,96),
	BIField(SFBOOL,	"set_bind", UNREADABLE),
	BIField(MFFLOAT,	"avatarSize [.25 1.6 .75 ]", 0),
	BIField(SFBOOL,	"headlight TRUE", 0),
	BIField(SFFLOAT,	"speed 1", 0),
	BIField(MFSTRING,	"type [\"WALK\" \"ANY\"]", 0),
	BIField(SFFLOAT,	"visibilityLimit 0", 0),
	BIField(SFBOOL,	"isBound", UNWRITEABLE)
);
static const BIProto5 group_ = BIProto5(
	"Group", "children", Vec2i(16,16), Vec2i(304,112),
	BIField(MFNODE,	"addChildren", UNREADABLE),
	BIField(MFNODE,	"removeChildren", UNREADABLE),
	BIField(MFNODE,	"children []", 0,	INIT_PROMPT_NODE(promptNode_scene_children)),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE)
);
static const BIProto5 noOutputGroup_ = BIProto5(
	"NoOutputGroup", "children", Vec2i(16,16), Vec2i(32,96),
	BIField(MFNODE,	"addChildren", UNREADABLE),
	BIField(MFNODE,	"removeChildren", UNREADABLE),
	BIField(MFNODE,	"children []", 0,	INIT_PROMPT_NODE(promptNode_scene_children)),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE | DISABLE)
);
static const BIProto8 gestureGroup_ = BIProto8(
	"GestureGroup", "children", Vec2i(16,16), Vec2i(368,96),
	BIField(MFNODE,	"addChildren", UNREADABLE),
	BIField(MFNODE,	"removeChildren", UNREADABLE),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(MFNODE,	"children []", 0,	INIT_PROMPT_NODE(promptNode_gestureGroup_children)),
	BIField(MFNODE,	"mobility []", 0,	INIT_PROMPT_NODE(promptNode_gestureGroup_mobility))
);
static const BIProto3 mobility_ = BIProto3(
	"Mobility", "mobility", Vec2i(16,16), Vec2i(112,96),
	BIField(MFNODE,	"children []", 0,	INIT_PROMPT_NODE(promptNode_scene_children)),
	BIField(SFNODE,	"onRot NULL", 0,	INIT_PROMPT_NODE(promptNode_timeSensor_on_fraction_changed)),
	BIField(SFNODE,	"onTran NULL", 0,	INIT_PROMPT_NODE(promptNode_timeSensor_on_fraction_changed))
);
static const BIProto5 cCBend_ = BIProto5(
	"CCBend", "generator", Vec2i(16,16), Vec2i(80,96),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(MFFLOAT,	"curve []{beginGroup TRUE initElVal \"[0 .0001 ]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"distance []{startAtIndex 1 initElVal \"1.5708\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"yAngle []{startAtIndex 1 initElVal \"0\"}", SAME_AS_LAST)
);

static const BIProto12 stem2_ = BIProto12(
	"Stem2", "build", Vec2i(16,16), Vec2i(16,112),
	BIField(SFVEC2F,	"slide 0 0", 0,	tip_stem_slide),
	BIField(MFFLOAT,	"xDistance []", 0,	0),
	BIField(MFFLOAT,	"yDistance []", 0,	0),
	BIField(MFINT32,	"staves []{rangeFrom 0}", 0,	0),
	BIField(MFINT32,	"bands []{rangeFrom 0}", 0,	0),
	BIField(SFBOOL,		"sync FALSE", 0,	tip_stem_sync),
	BIField(SFBOOL,		"mould TRUE", 0,	tip_stem_mould),
	BIField(SFVEC2F,	"cells 0 0", 0),
	BIField(MFNODE,	"part []", BLOCK_NEW | RED_DOT,	INIT_PROMPT_NODE(promptNode_stem_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_stem_setup)),
	BIField(MFNODE,	"textureMapper []", 0,	INIT_PROMPT_NODE(promptNode_stem_textureMapper)),
	BIField(MFNODE,	"fuse []", 0,	INIT_PROMPT_NODE(promptNode_stem_fuse))
);

static const BIProto10 polymap_ = BIProto10(
	"Polymap", "build", Vec2i(16,16), Vec2i(16,112),
	BIField(SFBOOL,	"add TRUE", 0, tip_stem_add),
	BIField(MFVEC2F,	"coord []", DISABLE),
	BIField(MFCOLOR,	"color []", DISABLE),
	BIField(MFVEC2F,	"texCoord []", DISABLE),
	BIField(MFINT32,	"coordIndex []{rangeFrom 0}", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(MFINT32,	"rc []", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(MFINT32,	"patchOwner []", DISABLE),
	BIField(MFNODE,	"surface []", 0,	INIT_PROMPT_NODE(promptNode_surfaceGenerator_generator)),
	BIField(SFNODE,	"part NULL", BLOCK_NEW ,	INIT_PROMPT_NODE(promptNode_stem_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_stem_setup))
);

static const BIProto0 branchJoin_ = BIProto0(
	"BranchJoin", "build", Vec2i(16,16), Vec2i(16,128)
);
static const BIProto0 generator_ = BIProto0(
	"Generator", 0, Vec2i(16,16), Vec2i(16,128)
);
static const BIProto3 surfaceTransform_ = BIProto3(
	"SurfaceTransform", "generator", Vec2i(16,16), Vec2i(64,96),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0)
);
static const BIProto12 place_ = BIProto12(
	"Place", "build", Vec2i(16,16), Vec2i(placeIcon_x, placeIcon_y),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFINT32,	"tranVertex 0{rangeFrom -1}", 0),
	BIField(SFVEC3F,	"multiply 0 0 0", DISABLE),
	BIField(SFBOOL,	"alignX FALSE", DO_NOT_SAVE),
	BIField(SFBOOL,	"alignY FALSE", DO_NOT_SAVE),
	BIField(SFBOOL,	"alignZ FALSE", DO_NOT_SAVE),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFBOOL,	"descendants TRUE", 0),
	BIField(SFBOOL,	"staticTransform TRUE", 0),
	BIField(MFNODE,	"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_place_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_place_setup))
);
static const BIProto6 directionalLight_ = BIProto6(
	"DirectionalLight", "children", Vec2i(16,16), Vec2i(304,96),
	BIField(SFFLOAT,	"ambientIntensity 0", 0),
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFVEC3F,	"direction 0 0 -1", 0),
	BIField(SFBOOL,	"global FALSE", 0),
	BIField(SFFLOAT,	"intensity 1", 0),
	BIField(SFBOOL,	"on TRUE", 0)
);
static const BIProto13 cCScale_ = BIProto13(
	"CCScale", "generator", Vec2i(16,16), Vec2i(64,144),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFFLOAT,	"baseRad 1", 0),
	BIField(MFFLOAT,	"curve []{beginGroup TRUE initElVal \"[0 .0001 ]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"distance []{startAtIndex 1 initElVal \"[1.5708]\"}", SAME_AS_LAST),
	BIField(SFBOOL,	"bNegX FALSE", 0),
	BIField(SFBOOL,	"bPosX FALSE", 0),
	BIField(SFBOOL,	"bNegZ FALSE", 0),
	BIField(SFBOOL,	"bPosZ FALSE", 0),
	BIField(SFFLOAT,	"negX 1", 0),
	BIField(SFFLOAT,	"posX 1", 0),
	BIField(SFFLOAT,	"negZ 1", 0),
	BIField(SFFLOAT,	"posZ 1", 0)
);
static const BIProto5 cCTranslate_ = BIProto5(
	"CCTranslate", "generator", Vec2i(16,16), Vec2i(96,96),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFFLOAT,	"baseRad 0", 0),
	BIField(MFFLOAT,	"curve []{beginGroup TRUE initElVal \"[-1.5708 1.5708 ]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"distance []{startAtIndex 1 initElVal \"1.5708\"}", SAME_AS_LAST)
);
static const BIProto11 cCRingScale_ = BIProto11(
	"CCRingScale", "generator", Vec2i(16,16), Vec2i(240,112),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFFLOAT,	"centerOffset 0", 0),
	BIField(SFFLOAT,	"baseRad 1", 0),
	BIField(MFFLOAT,	"curve []{beginGroup TRUE initElVal \"[-1.5708 1.5708 ]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"distance []{startAtIndex 1 initElVal \"1.5708\"}", SAME_AS_LAST),
	BIField(SFFLOAT,	"autoEnd 1", 0),
	BIField(SFFLOAT,	"negX 1", 0),
	BIField(SFFLOAT,	"posX 1", 0),
	BIField(SFFLOAT,	"negZ 1", 0),
	BIField(SFFLOAT,	"posZ 1", 0)
);
static const BIProto3 bezierLathe_ = BIProto3(
	"BezierLathe", "generator", Vec2i(16,16), Vec2i(32,144),
	BIField(MFVEC2F,	"point []{beginGroup TRUE}", 0),
	BIField(MFBOOL,	"fix []", DO_NOT_SAVE),
	BIField(MFINT32,	"nopfs []", DISABLE)
);
static const BIProto2 eCLathe_ = BIProto2(
	"ECLathe", "generator", Vec2i(16,16), Vec2i(48,112),
	BIField(MFVEC2F,	"point []{beginGroup TRUE initElVal \"[0 0 1 1]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"tangent []{initElVal \"[0 1.570796]\"}", SAME_AS_LAST)
);
static const BIProto12 eCScale_ = BIProto12(
	"ECScale", "generator", Vec2i(16,16), Vec2i(64,144),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(MFVEC2F,	"point []{beginGroup TRUE initElVal \"[0 0 .1 .5]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"tangent []{initElVal \"[1.570796 1.089909]\"}", SAME_AS_LAST),
	BIField(SFBOOL,	"bNegX FALSE", 0),
	BIField(SFBOOL,	"bPosX FALSE", 0),
	BIField(SFBOOL,	"bNegZ FALSE", 0),
	BIField(SFBOOL,	"bPosZ FALSE", 0),
	BIField(SFFLOAT,	"negX 1", 0),
	BIField(SFFLOAT,	"posX 1", 0),
	BIField(SFFLOAT,	"negZ 1", 0),
	BIField(SFFLOAT,	"posZ 1", 0)
);
static const BIProto6 eCShift_ = BIProto6(
	"ECShift", "generator", Vec2i(16,16), Vec2i(96,96),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(MFVEC2F,	"point []{beginGroup TRUE initElVal \"[0 0 .1 .5]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"tangent []{initElVal \"[1.570796 1.089909]\"}", SAME_AS_LAST),
	BIField(SFFLOAT,	"negDecay 1", 0),
	BIField(SFFLOAT,	"posDecay 1", 0)
);
static const BIProto5 eCBend_ = BIProto5(
	"ECBend", "generator", Vec2i(16,16), Vec2i(80,96),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(MFVEC2F,	"point []{beginGroup TRUE initElVal \"[0 0 .1 .5]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"tangent []{initElVal \"[1.570796 1.089909]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"yAngle []{startAtIndex 1 initElVal \"0\"}", SAME_AS_LAST)
);
static const BIProto2 eCSyncBend_ = BIProto2(
	"ECSyncBend", "generator", Vec2i(16,16), Vec2i(80,96),
	BIField(MFVEC2F,	"point []{beginGroup TRUE initElVal \"[0 0 .1 .5]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"tangent []{initElVal \"[1.570796 1.089909]\"}", SAME_AS_LAST)
);
static const BIProto4 eCTwist_ = BIProto4(
	"ECTwist", 0, Vec2i(16,16), Vec2i(80,96),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(MFVEC2F,	"point []{beginGroup TRUE initElVal \"[0 0 1 2]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"tangent []{initElVal \"[.995382 1.29955]\"}", SAME_AS_LAST)
);
static const BIProto12 background_ = BIProto12(
	"Background", "children", Vec2i(16,16), Vec2i(272,96),
	BIField(SFBOOL,	"set_bind", UNREADABLE),
	BIField(MFFLOAT,	"groundAngle []", 0),
	BIField(MFCOLOR,	"groundColor []", 0),
	BIField(MFFLOAT,	"skyAngle []", 0),
	BIField(MFCOLOR,	"skyColor 0 0 0", 0),
	BIField(MFSTRING,	"backUrl []", 0),
	BIField(MFSTRING,	"bottomUrl []", 0),
	BIField(MFSTRING,	"frontUrl []", 0),
	BIField(MFSTRING,	"leftUrl []", 0),
	BIField(MFSTRING,	"rightUrl []", 0),
	BIField(MFSTRING,	"topUrl []", 0),
	BIField(SFBOOL,	"isBound", UNWRITEABLE)
);

static const BIProto4 sCGroupBuild_ = BIProto4(//Script Controlled GroupBuild
	"SCGroupBuild", "children", Vec2i(16,16), Vec2i(224,96+16),
	BIField(SFCOLOR,"marker 1 1 1", 0),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_groupBuild_setup)),
	BIField(SFNODE, 	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_bezierLatheBuild_part)),
	BIField(MFNODE,	"build []", 0,		INIT_PROMPT_NODE(promptNode_seamless_build))
);

static const BIProto6 joinCloseVertex_ = BIProto6(
	"JoinCloseVertex", "build", Vec2i(16,16), Vec2i(224,96),
	BIField(SFBOOL,	"srcColor TRUE", 0),
	BIField(SFBOOL,	"dstColor TRUE", 0),
	BIField(SFFLOAT,	"joinRange .001", 0),
	BIField(MFNODE,	"source []", BLOCK_NEW | BLUE_DOT,	INIT_PROMPT_NODE(promptNode_joinCloseVertex_source)),
	BIField(MFNODE,	"part []", BLOCK_NEW | RED_DOT,	INIT_PROMPT_NODE(promptNode_joinCloseVertex_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_joinCloseVertex_setup))
);
static const BIProto5 centerJoint_ = BIProto5(
	"CenterJoint", "build", Vec2i(16,16), Vec2i(centerJoint_x,centerJoint_y), //CenterJoint
	BIField(SFINT32,	"fromVertex 0{rangeFrom 0}", 0),
	BIField(SFINT32,	"numOf 0{rangeFrom 0}", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_centerJoint_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_centerJoint_setup))
);
static const BIProto3 delPoly_ = BIProto3(
	"DelPoly", "build", Vec2i(16,16), Vec2i(64,112),
	BIField(MFINT32,	"poly []", 0),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_delPoly_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_delPoly_setup))
);
static const BIProto10 joinVertex_ = BIProto10(
	"JoinVertex", "build", Vec2i(16,16), Vec2i(48,160),
	BIField(SFBOOL,	"move FALSE", 0),
	BIField(SFBOOL,	"srcColor TRUE", 0),
	BIField(SFBOOL,	"dstColor TRUE", 0),
	BIField(MFINT32,	"srcCoord []{beginGroup TRUE}", 0),
	BIField(MFINT32,	"dstCoord []", 0),
	BIField(SFBOOL,	"in FALSE", 0),
	BIField(SFFLOAT,	"range .001", 0),
	BIField(SFNODE,	"source NULL", BLOCK_NEW | BLUE_DOT,	INIT_PROMPT_NODE(promptNode_joinVertex_source)),
	BIField(SFNODE,	"part NULL", BLOCK_NEW | RED_DOT,	INIT_PROMPT_NODE(promptNode_joinVertex_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_joinVertex_setup))
);
static const BIProto14 copyPart_ = BIProto14(
	"CopyPart", "build", Vec2i(16,16), Vec2i(copyPartIcon_x, copyPartIcon_y),
	BIField(SFBOOL,	"mirrorX FALSE", 0),
	BIField(SFBOOL,	"mirrorY FALSE", 0),
	BIField(SFBOOL,	"mirrorZ FALSE", 0),
	BIField(SFBOOL,	"add FALSE", 0),
	BIField(SFBOOL,	"insideOut FALSE", 0),
	BIField(SFBOOL,	"staticTransform FALSE", DISABLE),
	BIField(SFBOOL,	"copyTransform FALSE", DISABLE),
	BIField(SFBOOL,	"resetTransform FALSE", DISABLE),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(MFNODE,	"source []", BLOCK_NEW | BLUE_DOT,	INIT_PROMPT_NODE(promptNode_copyPart_source)),
	BIField(MFNODE,	"dest []", BLOCK_NEW | RED_DOT,	INIT_PROMPT_NODE(promptNode_copyPart_dest)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_copyPart_setup))
);
static const BIProto18 viewpoint_ = BIProto18(
	"Viewpoint", "children", Vec2i(16,16), Vec2i(240,144),
	BIField(SFBOOL,	"set_bind", UNREADABLE),
	BIField(SFBOOL,	"live FALSE", 0, "set as active camera"),
	BIField(SFBOOL,	"aim FALSE", 0, "look at type"),
	BIField(SFBOOL,	"perspective TRUE", DO_NOT_OUTPUT),
	BIField(SFFLOAT,	"zoom 1", DO_NOT_OUTPUT),
	BIField(SFFLOAT,	"fieldOfView .785398", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"lookAt 0 0 0", 0),
	BIField(SFVEC3F,	"up 0 1 10", 0),
	BIField(SFVEC3F,	"position 0 0 10", 0),
	BIField(SFBOOL,	"track TRUE", DISABLE),
	BIField(SFBOOL,	"jump TRUE", DISABLE),
	BIField(SFSTRING,	"description \"\"", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(SFBOOL,	"capture FALSE", PUSH),
	BIField(SFBOOL,	"animCapture FALSE", PUSH),
	BIField(SFBOOL,	"animDelete FALSE", PUSH),
	BIField(SFTIME,	"bindTime", UNWRITEABLE),
	BIField(SFBOOL,	"isBound", UNWRITEABLE)
);
static const BIProto14 transferBCyl_ = BIProto14(
	"TransferBCyl", "build", Vec2i(16,16), Vec2i(336,112),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFFLOAT,	"height .5{rangeFrom 0}", 0),
	BIField(SFFLOAT,	"radius .25{rangeFrom 0}", 0),
	BIField(SFBOOL,		"indexPosition TRUE", 0),
	BIField(SFBOOL,		"insert FALSE", 0),
	BIField(SFFLOAT,	"groupingHeight 0{rangeFrom 0}", 0),
	BIField(SFINT32,	"numOfGroups 3{rangeFrom 0}", 0),
	BIField(SFINT32,	"group -1{rangeFrom -1}", 0),
	BIField(MFINT32,	"vertex []", 0),
	BIField(SFBOOL,	"in TRUE", 0),
	BIField(SFNODE,	"source NULL", BLOCK_NEW | BLUE_DOT,	INIT_PROMPT_NODE(promptNode_transferBCyl_source)),
	BIField(SFNODE,	"dest NULL", BLOCK_NEW | RED_DOT,	INIT_PROMPT_NODE(promptNode_transferBCyl_dest)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_transferBCyl_setup))
);
static const BIProto6 transferOwnership_ = BIProto6(
	"TransferOwnership", "build", Vec2i(16,16), Vec2i(256,96),
	BIField(MFINT32,	"numOf []{beginGroup TRUE initElVal \"0\"}", SAME_AS_LAST),
	BIField(MFINT32,	"qty []{initElVal \"1\"}", SAME_AS_LAST),
	BIField(SFBOOL,	"beginSP FALSE", 0,	tip_rectangleBuild_beginSP),
	BIField(SFNODE,	"source NULL", BLOCK_NEW | BLUE_DOT,	INIT_PROMPT_NODE(promptNode_transferOwnership_source)),
	BIField(MFNODE,	"dest NULL", BLOCK_NEW | RED_DOT,	INIT_PROMPT_NODE(promptNode_transferVertex_dest)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_transferVertex_setup))
);
static const BIProto7 transferVertex_ = BIProto7(
	"TransferVertex", "build", Vec2i(16,16), Vec2i(256,96),
	BIField(SFINT32,	"fromVertex 0{rangeFrom 0}", 0),
	BIField(SFINT32,	"numOf -1{rangeFrom -1}", 0),
	BIField(SFINT32,	"qty 1", 0),
	BIField(SFBOOL,	"beginSP FALSE", 0,	tip_rectangleBuild_beginSP),
	BIField(SFNODE,	"source NULL", BLOCK_NEW | BLUE_DOT,	INIT_PROMPT_NODE(promptNode_transferVertex_source)),
	BIField(SFNODE,	"dest NULL", BLOCK_NEW | RED_DOT,	INIT_PROMPT_NODE(promptNode_transferVertex_dest)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_transferVertex_setup))
);
static const BIProto1 normal_ = BIProto1(
	"Normal", "normal", Vec2i(16,16), Vec2i(272,128),
	BIField(MFVEC3F,	"vector []", 0)
);
static const BIProto1 textureCoordinate_ = BIProto1(
	"TextureCoordinate", "texCoord", Vec2i(16,16), Vec2i(304,128),
	BIField(MFVEC2F,	"point []", 0)
);
static const BIProto6 sineTug_ = BIProto6(
	"SineTug", "generator", Vec2i(16,16), Vec2i(32,160),
	BIField(MFROTATION,	"orientation []{beginGroup TRUE}", SAME_AS_LAST),
	BIField(MFVEC3F,	"location []", SAME_AS_LAST),
	BIField(MFFLOAT,	"radius []{initElVal \"1\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"distance []", SAME_AS_LAST),
	BIField(MFFLOAT,	"decay []{rangeFrom 0 initElVal \"1\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"xScale []{initElVal \"1\"}", SAME_AS_LAST)
);
static const BIProto4 simulSineTug_ = BIProto4(
	"SimulSineTug", "generator", Vec2i(16,16), Vec2i(32,160),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(MFROTATION,	"orientation []{beginGroup TRUE}", SAME_AS_LAST),
	BIField(MFFLOAT,	"distance []", SAME_AS_LAST),
	BIField(SFFLOAT,	"radius 0", 0)
);
static const BIProto3 genPoly_ = BIProto3(
	"GenPoly", "build", Vec2i(16,16), Vec2i(64,128),
	BIField(MFINT32,	"vertex []{initElVal \"[0 0 0]\" incElVal \"[0 0 0]\"}", 0),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_genPoly_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_genPoly_setup))
);

static const BIProto3 script_ = BIProto3(
	"Script", "children", Vec2i(16,16), Vec2i(288,112),
	BIField(MFSTRING,	"url []", 0),
	BIField(SFBOOL,	"directOutput FALSE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"mustEvaluate FALSE", UNREADABLE | UNWRITEABLE)
);
static const BIProto10 branchJoinNegX_ = BIProto10(
	"BranchJoinNegX", "build", Vec2i(16,16), Vec2i(16,128),

	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFBOOL,	"descendants FALSE", 0),
	BIField(SFFLOAT,"decay 0", 0),
	BIField(SFBOOL,	"sawn FALSE", 0),
	BIField(SFBOOL,	"backward FALSE", 0),
	BIField(SFINT32,"numOf -1{rangeFrom -1}", 0),
	BIField(MFNODE,	"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_branchJoinNegX_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_branchJoinNegX_setup)),
	BIField(SFNODE,	"surface NULL", 0,	INIT_PROMPT_NODE(promptNode_branchJoinNegX_surface))
);
static const BIProto10 branchJoinPosX_ = BIProto10(
	"BranchJoinPosX", "build", Vec2i(16,16), Vec2i(16,144), //BranchJoinPosX
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFBOOL,	"descendants FALSE", 0),
	BIField(SFFLOAT,"decay 0", 0),
	BIField(SFBOOL,	"sawn FALSE", 0),
	BIField(SFBOOL,	"backward FALSE", 0),
	BIField(SFINT32,"numOf -1{rangeFrom -1}", 0),
	BIField(MFNODE,	"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_branchJoinPosX_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_branchJoinPosX_setup)),
	BIField(SFNODE,	"surface NULL", 0,	INIT_PROMPT_NODE(promptNode_branchJoinPosX_surface))
);
static const BIProto10 branchJoinNegY_ = BIProto10(
	"BranchJoinNegY", "build", Vec2i(16,16), Vec2i(16,160), //BranchJoinNegY
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFBOOL,	"descendants FALSE", 0),
	BIField(SFFLOAT,"decay 0", 0),
	BIField(SFBOOL,	"sawn FALSE", 0),
	BIField(SFBOOL,	"backward FALSE", 0),
	BIField(SFINT32,"numOf -1{rangeFrom -1}", 0),
	BIField(MFNODE,	"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_branchJoinNegY_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_branchJoinNegY_setup)),
	BIField(SFNODE,	"surface NULL", 0,	INIT_PROMPT_NODE(promptNode_branchJoinNegY_surface))
);
static const BIProto10 branchJoinPosY_ = BIProto10(
	"BranchJoinPosY", "build", Vec2i(16,16), Vec2i(208,96), //BranchJoinPosY
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFBOOL,	"descendants FALSE", 0),
	BIField(SFFLOAT,"decay 0", 0),
	BIField(SFBOOL,	"sawn FALSE", 0),
	BIField(SFBOOL,	"backward FALSE", 0),
	BIField(SFINT32,"numOf -1{rangeFrom -1}", 0),
	BIField(MFNODE,	"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_branchJoinPosY_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_branchJoinPosY_setup)),
	BIField(SFNODE,	"surface NULL", 0,	INIT_PROMPT_NODE(promptNode_branchJoinPosY_surface))
);
static const BIProto10 branchJoinNegZ_ = BIProto10(
	"BranchJoinNegZ", "build", Vec2i(16,16), Vec2i(208,112), //BranchJoinNegZ
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFBOOL,	"descendants FALSE", 0),
	BIField(SFFLOAT,"decay 0", 0),
	BIField(SFBOOL,	"sawn FALSE", 0),
	BIField(SFBOOL,	"backward FALSE", 0),
	BIField(SFINT32,"numOf -1{rangeFrom -1}", 0),
	BIField(MFNODE,	"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_branchJoinNegZ_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_branchJoinNegZ_setup)),
	BIField(SFNODE,	"surface NULL", 0,	INIT_PROMPT_NODE(promptNode_branchJoinNegZ_surface))
);
static const BIProto10 branchJoinPosZ_ = BIProto10(
	"BranchJoinPosZ", "build", Vec2i(16,16), Vec2i(208,128), //BranchJoinPosZ
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFBOOL,	"descendants FALSE", 0),
	BIField(SFFLOAT,"decay 0", 0),
	BIField(SFBOOL,	"sawn FALSE", 0),
	BIField(SFBOOL,	"backward FALSE", 0),
	BIField(SFINT32,"numOf -1{rangeFrom -1}", 0),
	BIField(MFNODE,	"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_branchJoinPosZ_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_branchJoinPosZ_setup)),
	BIField(SFNODE,	"surface NULL", 0,	INIT_PROMPT_NODE(promptNode_branchJoinPosZ_surface))
);
static const BIProto11 projectOntoTri_ = BIProto11(
	"ProjectOntoTri", "build", Vec2i(16,16), Vec2i(16,160), //ProjectOntoTri
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFBOOL,	"descendants FALSE", 0),
	BIField(SFFLOAT,"decay 0", 0),
	BIField(SFBOOL,	"sawn FALSE", 0),
	BIField(SFBOOL,	"backward FALSE", 0),
	BIField(SFINT32,"numOf -1{rangeFrom -1}", 0),
	BIField(MFNODE,	"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_stem_part)),
	BIField(SFNODE,	"surface NULL", 0,	INIT_PROMPT_NODE(promptNode_stem_part)),
	BIField(SFNODE,	"feedback NULL", 0,	INIT_PROMPT_NODE(promptNode_projectOntoTri_feedback)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_stem_setup))
);
static const BIProto7 evolveCoord_ = BIProto7(
	"EvolveCoord", "build", Vec2i(16,16), Vec2i(336,128), //EvolveCoord
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(MFVEC2F,	"point []{beginGroup TRUE initElVal \"[0 0 1 2]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"tangent []{initElVal \"[.995382 1.29955]\"}", SAME_AS_LAST),
	BIField(MFNODE,	"source []", BLOCK_NEW | BLUE_DOT,	INIT_PROMPT_NODE(promptNode_evolveCoord_source)),
	BIField(MFNODE,	"part []", BLOCK_NEW | RED_DOT,	INIT_PROMPT_NODE(promptNode_evolveCoord_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_evolveCoord_setup))
);
static const BIProto1 function_ = BIProto1(
	"Function", 0,Vec2i(16,16), Vec2i(160,96), //Function
 	BIField(MFNODE,	"codeLine []", BLOCK_USE,	INIT_PROMPT_NODE(promptNode_function_codeLine))
);
static const BIProto1 memberFunction_ = BIProto1(
 	"MemberFunction", 0, Vec2i(16,16), Vec2i(0,176), //MemberFunction
	BIField(MFNODE,	"codeLine []", BLOCK_USE,	INIT_PROMPT_NODE(promptNode_memberFunction_codeLine))
);
static const BIProto1 codeLine_ = BIProto1(
	"CodeLine", 0,Vec2i(16,16), Vec2i(112,96), //CodeLine
 	BIField(MFNODE,	"codeLine []", BLOCK_USE,	INIT_PROMPT_NODE(promptNode_codeLine_codeLine))
);
static const BIProto6 morphPart_ = BIProto6(
	"MorphPart", 0,Vec2i(16,16), Vec2i(80,128), //MorphPart
 	BIField(SFFLOAT,	"set_fraction", UNREADABLE),
	BIField(MFVEC3F,	"value_changed", UNWRITEABLE),
	BIField(MFFLOAT,	"key []{beginGroup TRUE}", 0),
	BIField(MFFLOAT,	"weight []", 0),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_morphPart_part)),
	BIField(MFNODE,	"bank []", 0,	INIT_PROMPT_NODE(promptNode_morphPart_bank))
);
static const BIProto2 morpher_ = BIProto2(
	"Morpher", 0,Vec2i(16,16), Vec2i(96,128), //Morpher
 	BIField(SFFLOAT,	"weight", UNREADABLE),
	BIField(MFVEC3F,	"displacement []", 0)
);
static const BIProto4 textureTransform_ = BIProto4(
	"TextureTransform", "textureTransform", Vec2i(16,16), Vec2i(192,128), //TextureTransform
 	BIField(SFVEC2F,	"center 0 0", 0),
	BIField(SFVEC2F,	"scale 1 1", 0),
	BIField(SFFLOAT,	"rotation 0", 0),
	BIField(SFVEC2F,	"translation 0 0", 0)
);
static const BIProto2 protoDeclare_ = BIProto2(
	"ProtoDeclare", 0, Vec2i(16,16), Vec2i(80,112), //ProtoDeclare
	BIField(SFNODE,	"protoInterface NULL", 0),
	BIField(SFNODE,	"protoBody NULL", 0)
);
static const BIProto0 protoInterface_ = BIProto0(
	"ProtoInterface", 0, Vec2i(16,16), Vec2i(288,160) //ProtoInterface
);
static const BIProto1 protoBody_ = BIProto1(
	"ProtoBody", 0, Vec2i(16,16), Vec2i(272,160), //ProtoBody
	BIField(MFNODE,	"children []", 0,	INIT_PROMPT_NODE(promptNode_scene_children))
);
static const BIProto0 protoInstance_ = BIProto0(
	"ProtoInstance", 0, Vec2i(16,16), Vec2i(336,160) //ProtoInstance
);

static const BIProto1 movie_ = BIProto1(
	"Movie", "children", Vec2i(16,16), Vec2i(176,144+16), //Movie
	BIField(MFNODE,	"clip []", 0,	INIT_PROMPT_NODE(promptNode_movie_clip))
);

static const BIProto3 video_ = BIProto3(
	"Video", "clip", Vec2i(16,16), Vec2i(176,144), //Video
	BIField(SFBOOL,	"open FALSE", DO_NOT_SAVE | PUSH | DO_NOT_DISPLY_NAME, "open png file sequence"),
	BIField(SFSTRING,	"file \"\"", DISABLE),
	BIField(MFINT32,	"frameRef []{beginGroup TRUE}", 0)
);

static const BIProto8 audio_ = BIProto8(
	"Audio", "soundTrack", Vec2i(16,16), Vec2i(112,192), //Sample
	BIField(SFBOOL,	"open FALSE", DO_NOT_SAVE | PUSH | DO_NOT_DISPLY_NAME, "open png file sequence"),
	BIField(SFSTRING,	"file \"\"", 0),
	BIField(SFBOOL,	"mute FALSE", 0),
	BIField(SFFLOAT,	"delay 0", 0),
	BIField(SFBOOL,	"videoSync TRUE", 0),
	BIField(MFVEC2F,	"lPoint []{beginGroup TRUE initElVal \"0 1\"}", SAME_AS_LAST),
	BIField(MFVEC2F,	"rPoint []{beginGroup TRUE initElVal \"0 1\"}", SAME_AS_LAST),
	BIField(MFINT32,	"frameRef []{beginGroup TRUE}", 0)
);

static const BIProto11 timeSensor_ = BIProto11(
	"TimeSensor", "children", Vec2i(16,16), Vec2i(112,112), //TimeSensor
	BIField(SFTIME,	"cycleInterval 1", 0),
	BIField(SFBOOL,	"enabled TRUE", 0),
	BIField(SFBOOL,	"loop FALSE", 0),
	BIField(SFTIME,	"startTime 0", 0),
	BIField(SFTIME,	"stopTime 0", 0),
	BIField(SFTIME,	"cycleTime", UNWRITEABLE),
	BIField(SFFLOAT,	"fraction_changed", UNWRITEABLE),
	BIField(SFBOOL,	"isActive", UNWRITEABLE),
	BIField(SFTIME,	"time", UNWRITEABLE),
	BIField(MFNODE,	"children []", 0),
	BIField(SFNODE,	"on_fraction_changed NULL", 0,	INIT_PROMPT_NODE(promptNode_timeSensor_on_fraction_changed))
);
static const BIProto4 orientationInterpolator_ = BIProto4(
	"OrientationInterpolator", "children", Vec2i(16,16), Vec2i(96,144), //OrientationInterpolator
	BIField(SFFLOAT,	"set_fraction", UNREADABLE),
	BIField(MFFLOAT,	"key []{beginGroup TRUE}", 0),
	BIField(MFROTATION,	"keyValue []", 0),
	BIField(SFROTATION,	"value_changed", UNWRITEABLE)
);
static const BIProto4 positionInterpolator_ = BIProto4(
	"PositionInterpolator", "children", Vec2i(16,16), Vec2i(112,144), //PositionInterpolator
	BIField(SFFLOAT,	"set_fraction", UNREADABLE),
	BIField(MFFLOAT,	"key []{beginGroup TRUE}", 0),
	BIField(MFVEC3F,	"keyValue []", 0),
	BIField(SFVEC3F,	"value_changed", UNWRITEABLE)
);
static const BIProto5 nurbsTrack_ = BIProto5(
	"NurbsTrack", "children", Vec2i(16,16), Vec2i(112,144), //NurbsPath
	BIField(SFFLOAT,	"frame", UNREADABLE),
	BIField(MFFLOAT,	"key []{beginGroup TRUE}", 0),
	BIField(MFVEC3F,	"keyValue []{beginGroup TRUE}", 0),
	BIField(MFINT32,	"dim []{beginGroup TRUE}", 0),
	BIField(SFVEC3F,	"pos", UNWRITEABLE)
);

/*
static const BIProto4 animCurver_ = BIProto4(
	"AnimCurver", "children",
	BIField(MFVEC3F,	"point []", 0),
	BIField(SFFLOAT, "curveDistance 0", 0),
	BIField(SFNODE,	"source NULL", BLOCK_NEW | BLUE_DOT,	INIT_PROMPT_NODE(promptNode_joinVertex_source)),
	BIField(SFNODE,	"dest NULL", BLOCK_NEW | BLUE_DOT,	INIT_PROMPT_NODE(promptNode_joinVertex_source))
);
*/

static const BIProto4 scalarInterpolator_ = BIProto4(
	"ScalarInterpolator", "children", Vec2i(16,16), Vec2i(112,160), //ScalarInterpolator
	BIField(SFFLOAT,	"set_fraction", UNREADABLE),
	BIField(MFFLOAT,	"key []{beginGroup TRUE}", 0),
	BIField(MFFLOAT,	"keyValue []", 0),
	BIField(SFFLOAT,	"value_changed", UNWRITEABLE)
);
static const BIProto2 posCompensator_ = BIProto2(
	"PosCompensator", "children", Vec2i(16,16), Vec2i(112,96), //PosCompensator
	BIField(MFVEC2F,	"moment []{beginGroup TRUE}", 0),
	BIField(MFVEC3F,	"relPos []", 0)
);

static const BIProto12 nikInterpolator_ = BIProto12(
	"NikInterpolator", "children", Vec2i(16,16), Vec2i(96,160), //CoordinateInterpolator
	BIField(SFBOOL,		"selectShort FALSE", DO_NOT_SAVE),
	BIField(SFBOOL,		"calcLongOnly FALSE", 0),
	BIField(SFBOOL,		"snapTwin FALSE", PUSH | DO_NOT_SAVE),
	BIField(SFBOOL,		"autoBeginTwin FALSE", 0),
	BIField(SFBOOL,		"autoEndTwin TRUE", 0),
	BIField(SFBOOL,		"autoStraightShort TRUE", 0),
	BIField(SFBOOL,		"relativeEnd TRUE", 0),
	BIField(MFFLOAT,	"key []", 0),
	BIField(MFVEC3F,	"keyValue []", 0),
	BIField(MFFLOAT,	"weight []", 0),
	BIField(MFFLOAT,	"knot []", 0),
	BIField(MFROTATION, "defaultRot []", 0)
);

static const BIProto8 collision_ = BIProto8(
	"Collision", "children", Vec2i(16,16), Vec2i(48,144), //Collision
	BIField(MFNODE,	"addChildren", UNREADABLE),
	BIField(MFNODE,	"removeChildren", UNREADABLE),
	BIField(MFNODE,	"children []", 0,	INIT_PROMPT_NODE(promptNode_scene_children)),
	BIField(SFBOOL,	"collide TRUE", 0),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE),
	BIField(SFNODE,	"proxy NULL", UNREADABLE | UNWRITEABLE),
	BIField(SFTIME,	"collideTime", UNWRITEABLE)
);
static const BIProto10 transform_ = BIProto10(
	"Transform", "children", Vec2i(16,16), Vec2i(32,112), //Transform
	BIField(MFNODE,	"addChildren", UNREADABLE),
	BIField(MFNODE,	"removeChildren", UNREADABLE),
	BIField(SFVEC3F,	"center 0 0 0", 0),
	BIField(SFROTATION,	"scaleOrientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE),
	BIField(MFNODE,	"children []", 0,	INIT_PROMPT_NODE(promptNode_scene_children))
);
static const BIProto4 colorInterpolator_ = BIProto4(
	"ColorInterpolator", "children", Vec2i(16,16), Vec2i(80,144), //ColorInterpolator
	BIField(SFFLOAT,	"set_fraction", UNREADABLE),
	BIField(MFFLOAT,	"key []{beginGroup TRUE}", 0),
	BIField(MFCOLOR,	"keyValue []", 0),
	BIField(SFCOLOR,	"value_changed", UNWRITEABLE)
);
static const BIProto4 coordinateInterpolator_ = BIProto4(
	"CoordinateInterpolator", "children", Vec2i(16,16), Vec2i(96,160), //CoordinateInterpolator
	BIField(SFFLOAT,	"set_fraction", UNREADABLE),
	BIField(MFFLOAT,	"key []", 0),
	BIField(MFVEC3F,	"keyValue []", 0),
	BIField(MFVEC3F,	"value_changed", UNWRITEABLE)
);
static const BIProto4 normalInterpolator_ = BIProto4(
	"NormalInterpolator", "children", Vec2i(16,16), Vec2i(80,160), //NormalInterpolator
	BIField(SFFLOAT,	"set_fraction", UNREADABLE),
	BIField(MFFLOAT,	"key []", 0),
	BIField(MFVEC3F,	"keyValue []", 0),
	BIField(MFVEC3F,	"value_changed", UNWRITEABLE)
);
static const BIProto3 modelSequence_ = BIProto3(
	"ModelSequence", "modelSequence", Vec2i(16,16), Vec2i(224,176), //ModelSequence
	BIField(SFFLOAT,	"frame", UNREADABLE),
	BIField(MFFLOAT,	"key []", 0),
	BIField(MFNODE,	"model []", 0, INIT_PROMPT_NODE(promptNode_modelSequence_seamless))
);
static const BIProto1 modelMorpher_ = BIProto1(
	"ModelMorpher", "children", Vec2i(16,16), Vec2i(192,192), //ModelMorpher
	BIField(MFNODE,	"sequence []", 0, INIT_PROMPT_NODE(promptNode_modelMorpher_sequence))
);
static const BIProto5 worldInfo_ = BIProto5(
	"WorldInfo", "children", Vec2i(16,16), Vec2i(144,112), //WorldInfo
	BIField(SFSTRING,	"title \"\"", UNREADABLE | UNWRITEABLE),
	BIField(SFSTRING,	"author \"\"", UNREADABLE | UNWRITEABLE | DO_NOT_OUTPUT),
	BIField(SFSTRING,	"website \"\"", UNREADABLE | UNWRITEABLE | DO_NOT_OUTPUT),
	BIField(MFSTRING,	"info []", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,		"netSave FALSE", UNREADABLE | UNWRITEABLE | DO_NOT_OUTPUT)
);
static const BIProto9 touchSensor_ = BIProto9(
	"TouchSensor", "children", Vec2i(16,16), Vec2i(320,160), //TouchSensor
	BIField(SFSTRING,	"description \"\"", 0),
	BIField(SFBOOL,	"enabled TRUE", 0),
	BIField(SFVEC3F,	"hitNormal_changed", UNWRITEABLE),
	BIField(SFVEC3F,	"hitPoint_changed", UNWRITEABLE),
	BIField(SFVEC2F,	"hitTexCoord_changed", UNWRITEABLE),
	BIField(SFBOOL,	"isActive", UNWRITEABLE),
	BIField(SFBOOL,	"isOver", UNWRITEABLE),
	BIField(SFTIME,	"touchTime", UNWRITEABLE),
	BIField(SFNODE,	"onTouchTime NULL", DO_NOT_OUTPUT,	INIT_PROMPT_NODE(promptNode_timeSensor_on_fraction_changed))
);
static const BIProto9 cCPath_ = BIProto9(
	"CCPath", 0, Vec2i(16,16), Vec2i(192,128), //CCPath
	BIField(SFFLOAT,	"set_fraction", UNREADABLE),
	BIField(SFFLOAT,	"fraction_changed", UNWRITEABLE),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(MFFLOAT,	"curve []{beginGroup TRUE}", 0),
	BIField(MFFLOAT,	"distance []{startAtIndex 1}", 0),
	BIField(MFFLOAT,	"zAngle []{startAtIndex 1}", 0),
	BIField(SFROTATION,	"rot_changed", UNWRITEABLE),
	BIField(SFVEC3F,	"pos_changed", UNWRITEABLE)
);
static const BIProto4 bspTree_ = BIProto4(
	"BspTree", 0, Vec2i(16,16), Vec2i(0,144), //BspTree
	BIField(SFROTATION,	"plane 0 0 1 0", 0),
	BIField(SFNODE,	"front NULL", 0,	INIT_PROMPT_NODE(promptNode_scene_children)),
	BIField(SFNODE,	"overlap NULL", 0,	INIT_PROMPT_NODE(promptNode_scene_children)),
	BIField(SFNODE,	"back NULL", 0,	INIT_PROMPT_NODE(promptNode_scene_children))
);
static const BIProto3 inline_ = BIProto3(
	"Inline", "children", Vec2i(16,16), Vec2i(224,144), //Inline
	BIField(MFSTRING,	"url []", 0),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE)
);
static const BIProto5 fog_ = BIProto5(
	"Fog", "children", Vec2i(16,16), Vec2i(416,96), //Fog
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFSTRING,	"fogType \"LINEAR\"", 0),
	BIField(SFFLOAT,	"visibilityRange 0", 0),
	BIField(SFBOOL,	"set_bind", UNREADABLE),
	BIField(SFBOOL,	"isBound", UNWRITEABLE)
);
static const BIProto7 fog2_ = BIProto7(
	"Fog2", "children", Vec2i(16,16), Vec2i(432,96), //Fog2
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFSTRING,	"fogType \"LINEAR\"", 0),
	BIField(SFFLOAT,	"visibilityRange 0", 0),
	BIField(SFFLOAT,	"visibilityStart 0", 0),
	BIField(SFFLOAT,	"density 0", 0),
	BIField(SFBOOL,	"set_bind", UNREADABLE),
	BIField(SFBOOL,	"isBound", UNWRITEABLE)
);
static const BIProto8 anchor_ = BIProto8(
	"Anchor", "children", Vec2i(16,16), Vec2i(384,96), //Anchor
	BIField(MFNODE,	"addChildren", UNREADABLE),
	BIField(MFNODE,	"removeChildren", UNREADABLE),
	BIField(MFNODE,	"children []", 0,	INIT_PROMPT_NODE(promptNode_scene_children)),
	BIField(SFSTRING,	"description \"\"", 0),
	BIField(MFSTRING,	"parameter []", 0),
	BIField(MFSTRING,	"url []", 0),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE)
);
static const BIProto8 audioClip_ = BIProto8(
	"AudioClip", "children", Vec2i(16,16), Vec2i(304,160), //AudioClip
	BIField(SFSTRING,	"description \"\"", 0),
	BIField(SFBOOL,	"loop FALSE", 0),
	BIField(SFFLOAT,	"pitch 1", 0),
	BIField(SFTIME,	"startTime 0", 0),
	BIField(SFTIME,	"stopTime 0", 0),
	BIField(MFSTRING,	"url []", 0),
	BIField(SFTIME,	"duration_changed", UNWRITEABLE),
	BIField(SFBOOL,	"isActive", UNWRITEABLE)
);
static const BIProto6 billboard_ = BIProto6(
	"Billboard", "children", Vec2i(16,16), Vec2i(400,96), //Billboard
	BIField(MFNODE,	"addChildren", UNREADABLE),
	BIField(MFNODE,	"removeChildren", UNREADABLE),
	BIField(SFVEC3F,	"axisOfRotation 0 1 0", 0),
	BIField(MFNODE,	"children []", 0,	INIT_PROMPT_NODE(promptNode_scene_children)),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE)
);
static const BIProto2 box_ = BIProto2(
	"Box", "geometry", Vec2i(16,16), Vec2i(boxBuildIcon_x,boxBuildIcon_y), //Box
	BIField(SFVEC3F,	"size 2 2 2", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"solid TRUE", UNREADABLE | UNWRITEABLE)
);
static const BIProto5 cone_ = BIProto5(
	"Cone", "geometry", Vec2i(16,16), Vec2i(coneBuildIcon_x,coneBuildIcon_y), //Cone
	BIField(SFFLOAT,	"height 2", UNREADABLE | UNWRITEABLE),
	BIField(SFFLOAT,	"bottomRadius 1", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"side TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"bottom TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"solid TRUE", UNREADABLE | UNWRITEABLE)
);
static const BIProto6 cylinder_ = BIProto6(
	"Cylinder", "geometry", Vec2i(16,16), Vec2i(cylinderBuildIcon_x,cylinderBuildIcon_y), //Cylinder
	BIField(SFFLOAT,	"height 2", UNREADABLE | UNWRITEABLE),
	BIField(SFFLOAT,	"radius 1", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"side TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"top TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"bottom TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"solid TRUE", UNREADABLE | UNWRITEABLE)
);
static const BIProto9 cylinderSensor_ = BIProto9(
	"CylinderSensor", "geometry", Vec2i(16,16), Vec2i(160,112), //CylinderSensor
	BIField(SFBOOL,	"autoOffset TRUE", 0),
	BIField(SFFLOAT,	"diskAngle .262", 0),
	BIField(SFBOOL,	"enabled TRUE", 0),
	BIField(SFFLOAT,	"maxAngle -1", 0),
	BIField(SFFLOAT,	"minAngle 0", 0),
	BIField(SFFLOAT,	"offset 0", 0),
	BIField(SFBOOL,	"isActive", UNWRITEABLE),
	BIField(SFROTATION,	"rotation_changed", UNWRITEABLE),
	BIField(SFVEC3F,	"trackPoint_changed", UNWRITEABLE)
);
static const BIProto14 elevationGrid_ = BIProto14(
	"ElevationGrid", "geometry", Vec2i(16,16), Vec2i(352,160), //ElevationGrid
	BIField(MFFLOAT,	"set_height", UNREADABLE),
	BIField(SFNODE,	"color NULL", 0,	INIT_PROMPT_NODE(promptNode_elevationGrid_color)),
	BIField(SFNODE,	"normal NULL", 0,	INIT_PROMPT_NODE(promptNode_elevationGrid_normal)),
	BIField(SFNODE,	"texCoord NULL", 0,	INIT_PROMPT_NODE(promptNode_elevationGrid_texCoord)),
	BIField(MFFLOAT,	"height []", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"ccw TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"colorPerVertex TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFFLOAT,	"creaseAngle 0", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"normalPerVertex TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"solid TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFINT32,	"xDimension 0{rangeFrom 0}", UNREADABLE | UNWRITEABLE),
	BIField(SFFLOAT,	"xSpacing 1", UNREADABLE | UNWRITEABLE),
	BIField(SFINT32,	"zDimension 0{rangeFrom 0}", UNREADABLE | UNWRITEABLE),
	BIField(SFFLOAT,	"zSpacing 1", UNREADABLE | UNWRITEABLE)
);
static const BIProto14 extrusion_ = BIProto14(
	"Extrusion", "geometry", Vec2i(16,16), Vec2i(176,128), //Extrusion
	BIField(MFVEC2F,	"set_crossSection", UNREADABLE),
	BIField(MFROTATION,	"set_orientation", UNREADABLE),
	BIField(MFVEC2F,	"set_scale", UNREADABLE),
	BIField(MFVEC3F,	"set_spine", UNREADABLE),
	BIField(SFBOOL,	"beginCap TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"ccw TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"convex TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFFLOAT,	"creaseAngle 0", UNREADABLE | UNWRITEABLE),
	BIField(MFVEC2F,	"crossSection [1 1 1 -1 -1 -1 -1 1 1 1 ]", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"endCap TRUE", UNREADABLE | UNWRITEABLE),
	BIField(MFROTATION,	"orientation 0 0 1 0", UNREADABLE | UNWRITEABLE),
	BIField(MFVEC2F,	"scale 1 1", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"solid TRUE", UNREADABLE | UNWRITEABLE),
	BIField(MFVEC3F,	"spine [0 0 0 0 1 0 ]", UNREADABLE | UNWRITEABLE)
);
static const BIProto9 fontStyle_ = BIProto9(
	"FontStyle", "fontStyle", Vec2i(16,16), Vec2i(416,144), //FontStyle
	BIField(MFSTRING,	"family \"SERIF\"", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"horizontal TRUE", UNREADABLE | UNWRITEABLE),
	BIField(MFSTRING,	"justify \"BEGIN\"", UNREADABLE | UNWRITEABLE),
	BIField(SFSTRING,	"language \"\"", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"leftToRight TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFFLOAT,	"size 1", UNREADABLE | UNWRITEABLE),
	BIField(SFFLOAT,	"spacing 1", UNREADABLE | UNWRITEABLE),
	BIField(SFSTRING,	"style \"PLAIN\"", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"topToBottom TRUE", UNREADABLE | UNWRITEABLE)
);
static const BIProto13 nurbsSurface_ = BIProto13(
	"NurbsSurface", "geometry", Vec2i(16,16), Vec2i(112,96), //NurbsSurface
	BIField(MFVEC3F,	"controlPoint []{beginGroup TRUE}", 0),
	BIField(MFFLOAT,	"weight []", 0),
	BIField(MFFLOAT,	"uKnot []{beginGroup TRUE}", DISABLE),
	BIField(MFFLOAT,	"vKnot []{beginGroup TRUE}", DISABLE),
	BIField(SFNODE,		"texCoord NULL", 0,	INIT_PROMPT_NODE(promptNode_indexedFaceSet_texCoord)),
	BIField(SFFLOAT,	"uDimension 0", 0),
	BIField(SFFLOAT,	"vDimension 0", 0),
	BIField(SFFLOAT,	"uOrder 3", 0),
	BIField(SFFLOAT,	"vOrder 3", 0),
	BIField(SFFLOAT,	"uTessellation 0", 0),
	BIField(SFFLOAT,	"vTessellation 0", 0),
	BIField(SFBOOL,		"ccw TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,		"solid TRUE", 0)
);
static const BIProto7 lOD_ = BIProto7(
	"LOD", "children", Vec2i(16,16), Vec2i(224,160), //LOD
	BIField(MFNODE,	"addChildren", UNREADABLE),
	BIField(MFNODE,	"removeChildren", UNREADABLE),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC3F,	"center 0 0 0", UNREADABLE | UNWRITEABLE),
	BIField(MFFLOAT,	"range []", UNREADABLE | UNWRITEABLE),
	BIField(MFNODE,	"children []", 0,	INIT_PROMPT_NODE(promptNode_scene_children))
);
static const BIProto9 movieTexture_ = BIProto9(
	"MovieTexture", "texture", Vec2i(16,16), Vec2i(336,96), //MovieTexture
	BIField(SFBOOL,	"loop FALSE", 0),
	BIField(SFFLOAT,	"speed 1", 0),
	BIField(SFTIME,	"startTime 0", 0),
	BIField(SFTIME,	"stopTime 0", 0),
	BIField(MFSTRING,	"url []", 0),
	BIField(SFBOOL,	"repeatS TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"repeatT TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFTIME,	"duration_changed", UNWRITEABLE),
	BIField(SFBOOL,	"isActive", UNWRITEABLE)
);
static const BIProto2 pixelTexture_ = BIProto2(
	"PixelTexture", "texture", Vec2i(16,16), Vec2i(224,160), //PixelTexture
	BIField(SFBOOL,	"repeatS TRUE", UNREADABLE | UNWRITEABLE),
	BIField(SFBOOL,	"repeatT TRUE", UNREADABLE | UNWRITEABLE)
);
static const BIProto8 planeSensor_ = BIProto8(
	"PlaneSensor", "children", Vec2i(16,16), Vec2i(320,144), //PlaneSensor
	BIField(SFBOOL,	"autoOffset TRUE", 0),
	BIField(SFBOOL,	"enabled TRUE", 0),
	BIField(SFVEC2F,	"maxPosition -1 -1", 0),
	BIField(SFVEC2F,	"minPosition 0 0", 0),
	BIField(SFVEC3F,	"offset 0 0 0", 0),
	BIField(SFBOOL,	"isActive", UNWRITEABLE),
	BIField(SFVEC3F,	"trackPoint_changed", UNWRITEABLE),
	BIField(SFVEC3F,	"translation_changed", UNWRITEABLE)
);
static const BIProto8 pointLight_ = BIProto8(
	"PointLight", "children", Vec2i(16,16), Vec2i(464,96), //PointLight
	BIField(SFFLOAT,	"ambientIntensity 0", 0),
	BIField(SFVEC3F,	"attenuation 1 0 0", 0),
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFBOOL,	"global TRUE", 0),
	BIField(SFFLOAT,	"intensity 1", 0),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFBOOL,	"on TRUE", 0),
	BIField(SFFLOAT,	"radius 100", 0)
);
static const BIProto2 pointSet_ = BIProto2(
	"PointSet", "children", Vec2i(16,16), Vec2i(224,160), //PointSet
	BIField(SFNODE,	"coord NULL", 0,	INIT_PROMPT_NODE(promptNode_pointSet_coord)),
	BIField(SFNODE,	"color NULL", 0,	INIT_PROMPT_NODE(promptNode_pointSet_color))
);
static const BIProto8 proximitySensor_ = BIProto8(
	"ProximitySensor", "children", Vec2i(16,16), Vec2i(160,128), //ProximitySensor
	BIField(SFVEC3F,	"center 0 0 0", 0),
	BIField(SFVEC3F,	"size 0 0 0", 0),
	BIField(SFBOOL,	"enabled TRUE", 0),
	BIField(SFBOOL,	"isActive", UNWRITEABLE),
	BIField(SFVEC3F,	"position_changed", UNWRITEABLE),
	BIField(SFROTATION,	"orientation_changed", UNWRITEABLE),
	BIField(SFTIME,	"enterTime", UNWRITEABLE),
	BIField(SFTIME,	"exitTime", UNWRITEABLE)
);
static const BIProto10 sound_ = BIProto10(
	"Sound", "children", Vec2i(16,16), Vec2i(192,160), //Sound
	BIField(SFVEC3F,	"direction 0 0 1", 0),
	BIField(SFFLOAT,	"intensity 1", 0),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFFLOAT,	"maxBack 10", 0),
	BIField(SFFLOAT,	"maxFront 10", 0),
	BIField(SFFLOAT,	"minBack 1", 0),
	BIField(SFFLOAT,	"minFront 1", 0),
	BIField(SFFLOAT,	"priority 0", 0),
	BIField(SFNODE,	"source NULL", 0,	INIT_PROMPT_NODE(promptNode_sound_source)),
	BIField(SFBOOL,	"spatialize TRUE", UNREADABLE | UNWRITEABLE)
);
static const BIProto1 sphere_ = BIProto1(
	"Sphere", "geometry", Vec2i(16,16), Vec2i(sphereBuildIcon_x, sphereBuildIcon_y), //Sphere
	BIField(SFFLOAT,	"radius 1", UNREADABLE | UNWRITEABLE)
);
static const BIProto6 sphereSensor_ = BIProto6(
	"SphereSensor", "children", Vec2i(16,16), Vec2i(176,96), //SphereSensor
	BIField(SFBOOL,	"autoOffset TRUE", 0),
	BIField(SFBOOL,	"enabled TRUE", 0),
	BIField(SFROTATION,	"offset 0 0 1 0", 0),
	BIField(SFBOOL,	"isActive", UNWRITEABLE),
	BIField(SFROTATION,	"rotation_changed", UNWRITEABLE),
	BIField(SFVEC3F,	"trackPoint_changed", UNWRITEABLE)
);
static const BIProto11 spotLight_ = BIProto11(
	"SpotLight", "children", Vec2i(16,16), Vec2i(384,144), //SpotLight
	BIField(SFFLOAT,	"ambientIntensity 0", 0),
	BIField(SFVEC3F,	"attenuation 1 0 0", 0),
	BIField(SFFLOAT,	"beamWidth 1.570796", 0),
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFFLOAT,	"cutOffAngle .785398", 0),
	BIField(SFVEC3F,	"direction 0 0 -1", 0),
	BIField(SFBOOL,	"global TRUE", 0),
	BIField(SFFLOAT,	"intensity 1", 0),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFBOOL,	"on TRUE", 0),
	BIField(SFFLOAT,	"radius 100", 0)
);
static const BIProto6 switch_ = BIProto6(
	"Switch", "children", Vec2i(16,16), Vec2i(80,128), //Switch
	BIField(MFNODE,	"addChildren", UNREADABLE),
	BIField(MFNODE,	"removeChildren", UNREADABLE),
	BIField(SFINT32,	"whichChoice -1", 0),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE),
	BIField(MFNODE,	"children []", 0,	INIT_PROMPT_NODE(promptNode_scene_children))
);
static const BIProto4 text_ = BIProto4(
	"Text", "geometry", Vec2i(16,16), Vec2i(400,144), //Text
	BIField(MFSTRING,	"string []", 0),
	BIField(MFFLOAT,	"length []", 0),
	BIField(SFFLOAT,	"maxExtent 0", 0),
	BIField(SFNODE,	"fontStyle NULL", 0,	INIT_PROMPT_NODE(promptNode_text_fontStyle))
);
static const BIProto6 visibilitySensor_ = BIProto6(
	"VisibilitySensor", "children", Vec2i(16,16), Vec2i(128,128), //VisibilitySensor
	BIField(SFVEC3F,	"center 0 0 0", 0),
	BIField(SFBOOL,	"enabled TRUE", 0),
	BIField(SFVEC3F,	"size 0 0 0", 0),
	BIField(SFTIME,	"enterTime", UNWRITEABLE),
	BIField(SFTIME,	"exitTime", UNWRITEABLE),
	BIField(SFBOOL,	"isActive", UNWRITEABLE)
);
static const BIProto2 exitBuild_ = BIProto2(
	"ExitBuild", "build", Vec2i(16,16), Vec2i(128,144), //ExitBuild
	BIField(SFBOOL,	"enabled TRUE", 0),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_exitBuild_setup))
);
static const BIProto3 splitTri_ = BIProto3(
	"SplitTri", "build", Vec2i(16,16), Vec2i(320,112), //SplitTri
	BIField(MFINT32,	"corner []", 0),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_splitTri_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_splitTri_setup))
);
static const BIProto3 swapTriDivide_ = BIProto3(
	"SwapTriDivide", "build", Vec2i(16,16), Vec2i(swapDivide_x,swapDivide_y), //SwapTriDivide
	BIField(MFINT32,	"corner []{initElVal \"[0 0]\" incElVal \"[0 0]\"}", 0),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_swapTriDivide_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_swapTriDivide_setup))
);
static const BIProto5 placeTexture_ = BIProto5(
	"PlaceTexture", "build", Vec2i(16,16), Vec2i(192,144), //PlaceTexture
	BIField(SFVEC2F,	"scale 1 1", 0),
	BIField(SFFLOAT,	"rotation 0", 0),
	BIField(SFVEC2F,	"translation 0 0", 0),
	BIField(MFNODE,	"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_placeTexture_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_placeTexture_setup))
);
static const BIProto18 hAnimHumanoid_ = BIProto18(
	"HAnimHumanoid", "children", Vec2i(16,16), Vec2i(448,96), //HAnimHumanoid
	BIField(SFSTRING,	"version \"\"", 0),
	BIField(SFSTRING,	"name \"\"", 0),
	BIField(MFSTRING,	"info []", 0),
	BIField(SFVEC3F,	"center 0 0 0", 0),
	BIField(SFROTATION,	"scaleOrientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE),
	BIField(MFNODE,	"skin []", 0,	INIT_PROMPT_NODE(promptNode_hAnimHumanoid_skin)),
	BIField(SFNODE,	"skinCoord NULL", 0,	INIT_PROMPT_NODE(promptNode_hAnimHumanoid_skinCoord)),
	BIField(SFNODE,	"skinNormal NULL", 0,	INIT_PROMPT_NODE(promptNode_hAnimHumanoid_skinNormal)),
	BIField(MFNODE,	"skeleton []", 0,	INIT_PROMPT_NODE(promptNode_hAnimHumanoid_skeleton)),
	BIField(MFNODE,	"joints []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_hAnimHumanoid_joints)),
	BIField(MFNODE,	"segments []", 0),
	BIField(MFNODE,	"sites []", 0),
	BIField(MFNODE,	"viewpoints []", 0)
);
static const BIProto19 hAnimJoint_ = BIProto19(
	"HAnimJoint", "", Vec2i(16,16), Vec2i(0,96), //HAnimJoint	//add back "children" when contact no longer needs it specified for xml
	BIField(MFNODE,	"addChildren", UNREADABLE),
	BIField(MFNODE,	"removeChildren", UNREADABLE),
	BIField(SFSTRING,	"name \"\"", 0),
	BIField(SFVEC3F,	"center 0 0 0", 0),
	BIField(SFROTATION,	"scaleOrientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE),
	BIField(MFVEC3F,	"skinCoordLocal []{beginGroup TRUE}", 0),
	BIField(MFINT32,	"skinCoordIndex []{beginGroup TRUE}", 0),
	BIField(MFFLOAT,	"skinCoordWeight []", 0),
	BIField(SFROTATION,	"limitOrientation 0 0 1 0", 0),
	BIField(MFFLOAT,	"llimit []", 0),
	BIField(MFFLOAT,	"stiffness [0 0 0]", 0),
	BIField(MFFLOAT,	"ulimit []", 0),
	BIField(MFNODE,	"children []", BLOCK_USE,	INIT_PROMPT_NODE(promptNode_hAnimJoint_children)),
	BIField(MFNODE,	"displacers []", BLOCK_USE,	INIT_PROMPT_NODE(promptNode_hAnimJoint_displacers))
);
static const BIProto4 hAnimDisplacer_ = BIProto4(
	"HAnimDisplacer", "displacers", Vec2i(16,16), Vec2i(96,128), //HAnimDisplacer
	BIField(MFINT32,	"coordIndex []{beginGroup TRUE}", 0),
	BIField(MFVEC3F,	"displacements []", 0),
	BIField(SFSTRING,	"name \"\"", 0),
	BIField(SFFLOAT,	"weight 0", 0)
);
static const BIProto11 hAnimSegment_ = BIProto11(
	"HAnimSegment", "children", Vec2i(16,16), Vec2i(112,96), //HAnimSegment
	BIField(MFNODE,		"addChildren", UNREADABLE),
	BIField(MFNODE,		"removeChildren", UNREADABLE),
	BIField(SFVEC3F,	"centerOfMass 0 0 0", 0),
	BIField(MFNODE,		"children []", 0,	INIT_PROMPT_NODE(promptNode_scene_children)),
	BIField(MFVEC3F,	"coord []", 0),
	BIField(MFNODE,		"displacers []", BLOCK_USE,	INIT_PROMPT_NODE(promptNode_hAnimJoint_displacers)),
	BIField(SFFLOAT,	"mass 0", 0),
	BIField(MFFLOAT,	"momentsOfInertia [0 0 0 0 0 0 0 0 0]", 0),
	BIField(SFSTRING,	"name \"\"", 0),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE)
);
static const BIProto11 hAnimSite_ = BIProto11(
	"HAnimSite", "children", Vec2i(16,16), Vec2i(112,96), //HAnimSite
	BIField(MFNODE,		"addChildren", UNREADABLE),
	BIField(MFNODE,		"removeChildren", UNREADABLE),
	BIField(MFNODE,		"children []", 0,	INIT_PROMPT_NODE(promptNode_scene_children)),
	BIField(SFSTRING,	"name \"\"", 0),
	BIField(SFVEC3F,	"center 0 0 0", 0),
	BIField(SFROTATION,	"scaleOrientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE)
);
static const BIProto4 booleanFilter_ = BIProto4(
	"BooleanFilter", "children", Vec2i(16,16), Vec2i(112,96), //BooleanFilter
	BIField(SFBOOL,	"set_boolean", UNREADABLE),
	BIField(SFBOOL,	"inputFalse", UNWRITEABLE),
	BIField(SFBOOL,	"inputNegate", UNWRITEABLE),
	BIField(SFBOOL,	"inputTrue", UNWRITEABLE)
);
static const BIProto3 CameraSelector_ = BIProto3(
	"CameraSelector", "children", Vec2i(16,16), Vec2i(112,96), //CameraSelector
	BIField(SFFLOAT,	"set_fraction", UNREADABLE),
	BIField(MFFLOAT,	"key []{beginGroup TRUE}", 0),
	BIField(MFNODE,	"keyValue []", 0, INIT_PROMPT_NODE(promptNode_cameraSelctor_keyValue))
);
static const BIProto2 booleanToggle_ = BIProto2(
	"BooleanToggle", "children", Vec2i(16,16), Vec2i(112,96), //BooleanToggle
	BIField(SFBOOL,	"set_boolean", UNREADABLE),
	BIField(SFBOOL,	"toggle FALSE", 0)
);
static const BIProto2 booleanTrigger_ = BIProto2(
	"BooleanTrigger", "children", Vec2i(16,16), Vec2i(112,96), //BooleanTrigger
	BIField(SFTIME,	"set_triggerTime", UNREADABLE),
	BIField(SFBOOL,	"triggerTrue", UNWRITEABLE)
);
static const BIProto6 integerSequencer_ = BIProto6(
	"IntegerSequencer", "children", Vec2i(16,16), Vec2i(112,96), //IntegerSequencer
	BIField(SFBOOL,	"next", UNREADABLE),
	BIField(SFBOOL,	"previous", UNREADABLE),
	BIField(SFFLOAT,	"set_fraction", UNREADABLE),
	BIField(MFFLOAT,	"key []{beginGroup TRUE}", 0),
	BIField(MFINT32,	"keyValue []", 0),
	BIField(SFINT32,	"value_changed", UNWRITEABLE)
);
static const BIProto3 integerTrigger_ = BIProto3(
	"IntegerTrigger", "children", Vec2i(16,16), Vec2i(112,96), //IntegerTrigger
	BIField(SFBOOL,	"set_boolean", UNREADABLE),
	BIField(SFINT32,	"integerKey -1", 0),
	BIField(SFINT32,	"triggerValue", UNWRITEABLE)
);
static const BIProto2 timeTrigger_ = BIProto2(
	"TimeTrigger", "children", Vec2i(16,16), Vec2i(112,96), //TimeTrigger
	BIField(SFBOOL,	"set_boolean", UNREADABLE),
	BIField(SFTIME,	"triggerTime", UNWRITEABLE)
);
static const BIProto3 staticGroup_ = BIProto3(
	"StaticGroup", "children", Vec2i(16,16), Vec2i(368,96), //StaticGroup
	BIField(MFNODE,	"children []", UNREADABLE | UNWRITEABLE,	INIT_PROMPT_NODE(promptNode_scene_children)),
	BIField(SFVEC3F,	"bboxCenter 0 0 0", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC3F,	"bboxSize -1 -1 -1", UNREADABLE | UNWRITEABLE)
);
static const BIProto10 floatAccelerator_ = BIProto10(
	"FloatAccelerator", 0, Vec2i(16,16), Vec2i(208,160), //FloatAccelerator
	BIField(SFFLOAT,	"set_fraction", UNREADABLE),
	BIField(SFFLOAT,	"fraction_changed", UNWRITEABLE),
	BIField(SFFLOAT,	"value_changed", UNWRITEABLE),
	BIField(SFFLOAT,	"fracMult 1", 0),
	BIField(SFFLOAT,	"init 0", 0),
	BIField(MFFLOAT,	"delay []{beginGroup TRUE initElVal \"1\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"accelerate []{initElVal \"1\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"attack []{initElVal \"1\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"sustain []{initElVal \"0\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"decay []{initElVal \"1\"}", SAME_AS_LAST)
);
static const BIProto13 rotAccelerator_ = BIProto13(
	"RotAccelerator", 0, Vec2i(16,16), Vec2i(208,144), //RotAccelerator
	BIField(SFFLOAT,	"set_fraction", UNREADABLE),
	BIField(SFFLOAT,	"fraction_changed", UNWRITEABLE),
	BIField(SFROTATION,	"value_changed", UNWRITEABLE),
	BIField(SFFLOAT,	"fracMult 1", 0),
	BIField(SFROTATION,	"init 0 0 1 0", 0),
	BIField(MFVEC3F,	"axis []{beginGroup TRUE initElVal \"0 1 0\"}", SAME_AS_LAST | AXIS3F),
	BIField(MFFLOAT,	"delay []{initElVal \"1\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"accelerate []{initElVal \"1\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"attack []{initElVal \"1\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"sustain []{initElVal \"0\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"decay []{initElVal \"1\"}", SAME_AS_LAST),
	BIField(SFBOOL,	"calcForReturn FALSE", PUSH),
	BIField(SFBOOL,	"calcForPose FALSE", PUSH)
);
static const BIProto3 copyToMorph_ = BIProto3(
	"CopyToMorph", "build", Vec2i(16,16), Vec2i(144,96), //CopyToMorph
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_copyToMorph_part)),
	BIField(SFNODE,	"morpher NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_copyToMorph_morpher)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_copyToMorph_setup))
);
static const BIProto2 relateMorpher_ = BIProto2(
	"RelateMorpher", "build", Vec2i(16,16), Vec2i(416,128), //RelateMorpher
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_relateMorpher_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_relateMorpher_setup))
);
static const BIProto12 delPlane_ = BIProto12(
	"DelPlane", "build", Vec2i(16,16), Vec2i(256,160), //DelPlane
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFBOOL,	"negX FALSE", 0),
	BIField(SFBOOL,	"posX FALSE", 0),
	BIField(SFBOOL,	"negY FALSE", 0),
	BIField(SFBOOL,	"posY FALSE", 0),
	BIField(SFBOOL,	"negZ FALSE", 0),
	BIField(SFBOOL,	"posZ FALSE", 0),
	BIField(SFBOOL,	"full TRUE", 0),
	BIField(SFBOOL,	"align FALSE", 0),
	BIField(MFNODE,	"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_delPlane_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_delPlane_setup))
);
static const BIProto4 delSphere_ = BIProto4(
	"DelSphere", "build", Vec2i(16,16), Vec2i(256,160), //DelSphere
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFFLOAT,	"radius .25", 0),
	BIField(MFNODE,	"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_delPlane_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_delPlane_setup))
);
static const BIProto4 radialTug_ = BIProto4(
	"RadialTug", "build", Vec2i(16,16), Vec2i(336,144), //RadialTug
	BIField(MFVEC3F,	"location []{beginGroup TRUE}", 0),
	BIField(MFVEC3F,	"displacement []", 0),
	BIField(MFFLOAT,	"slope []{rangeFrom .0001 initElVal \"1\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"radius []{initElVal \"1\"}", SAME_AS_LAST)
);
static const BIProto5 tapeMeasure_ = BIProto5(
	"TapeMeasure", "build", Vec2i(16,16), Vec2i(144,144), //TapeMeasure
	BIField(SFFLOAT,	"measurement 0", 0),
	BIField(SFINT32,	"baseVertex 0{rangeFrom 0}", 0),
	BIField(SFINT32,	"staves 0{rangeFrom 0}", 0),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_tapeMeasure_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_tapeMeasure_setup))
);
static const BIProto11 posAccelerator_ = BIProto11(
	"PosAccelerator", 0, Vec2i(16,16), Vec2i(400,128), //PosAccelerator
	BIField(SFFLOAT,	"set_fraction", UNREADABLE),
	BIField(SFFLOAT,	"fraction_changed", UNWRITEABLE),
	BIField(SFVEC3F,	"value_changed", UNWRITEABLE),
	BIField(SFFLOAT,	"fracMult 1", 0),
	BIField(SFVEC3F,	"init 0 0 0", 0),
	BIField(MFVEC3F,	"direction []{beginGroup TRUE initElVal \"0 1 0\"}", SAME_AS_LAST | AXIS3F),
	BIField(MFFLOAT,	"delay []{initElVal \"1\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"accelerate []{initElVal \"1\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"attack []{initElVal \"1\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"sustain []{initElVal \"0\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"decay []{initElVal \"1\"}", SAME_AS_LAST)
);
static const BIProto8 animByAnkle_ = BIProto8(
	"AnimByAnkle", 0, Vec2i(16,16), Vec2i(288,144), //AnimByAnkle
	BIField(SFVEC3F,	"position", UNREADABLE),
	BIField(SFROTATION,	"catAnkle", UNREADABLE),
	BIField(SFBOOL,	"outAnkle TRUE", 0),
	BIField(SFFLOAT,	"swivel 0", 0),
	BIField(SFNODE,	"pelvis NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_animByAnkle_pelvis)),
	BIField(SFNODE,	"hip NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_animByAnkle_hip)),
	BIField(SFNODE,	"knee NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_animByAnkle_knee)),
	BIField(SFNODE,	"ankle NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_animByAnkle_ankle))
);
static const BIProto7 animByWrist_ = BIProto7(
	"AnimByWrist", 0, Vec2i(16,16), Vec2i(304,144), //AnimByWrist
	BIField(SFVEC3F,	"position", UNREADABLE),
	BIField(SFROTATION,	"catWrist", UNREADABLE),
	BIField(SFBOOL,	"outWrist TRUE", 0),
	BIField(SFFLOAT,	"swivel 0", 0),
	BIField(SFNODE,	"sholder NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_animByWrist_sholder)),
	BIField(SFNODE,	"elbow NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_animByWrist_elbow)),
	BIField(SFNODE,	"wrist NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_animByWrist_wrist))
);
static const BIProto4 rotFilter_ = BIProto4(
	"RotFilter", 0, Vec2i(16,16), Vec2i(240,160), //RotFilter
	BIField(SFROTATION,	"inRot", UNREADABLE),
	BIField(SFVEC3F,	"axis 1 1 1", 0),
	BIField(SFFLOAT,	"angle 1", 0),
	BIField(SFROTATION,	"outRot", UNWRITEABLE)
);
static const BIProto3 rotMult_ = BIProto3(
	"RotMult", 0, Vec2i(16,16), Vec2i(272,144), //RotMult
	BIField(SFROTATION,	"inA", UNREADABLE),
	BIField(SFROTATION,	"inB", UNREADABLE),
	BIField(SFROTATION,	"catenation", UNWRITEABLE)
);
static const BIProto1 multiUser_ = BIProto1(
	"MultiUser", 0, Vec2i(16,16), Vec2i(432,128), //MultiUser
	BIField(MFSTRING,	"ip []", 0)
);
static const BIProto10 indexedTriangleSet_ = BIProto10(
	"IndexedTriangleSet", "geometry", Vec2i(16,16), Vec2i(240,128), //IndexedTriangleSet
	BIField(MFINT32,	"set_index", UNREADABLE),
	BIField(SFNODE,	"coord NULL", 0,	INIT_PROMPT_NODE(promptNode_indexedTriangleSet_coord)),
	BIField(SFNODE,	"normal NULL", 0,	INIT_PROMPT_NODE(promptNode_indexedTriangleSet_normal)),
	BIField(SFNODE,	"color NULL", 0,	INIT_PROMPT_NODE(promptNode_indexedTriangleSet_color)),
	BIField(SFNODE,	"texCoord NULL", 0,	INIT_PROMPT_NODE(promptNode_indexedTriangleSet_texCoord)),
	BIField(SFBOOL,	"ccw TRUE", 0),
	BIField(SFBOOL,	"normalPerVertex TRUE", 0),
	BIField(SFBOOL,	"colorPerVertex TRUE", 0),
	BIField(SFBOOL,	"solid TRUE", 0),
	BIField(MFINT32,	"index []{rangeFrom 0 initElVal \"[0 0 0]\" incElVal \"[0 0 0]\"}", 0)
);
static const BIProto1 polyline2D_ = BIProto1(
	"Polyline2D", "geometry", Vec2i(16,16), Vec2i(432,144), //Polyline2D
	BIField(MFVEC2F,	"lineSegments []", 0)
);
static const BIProto14 sphereBuild_ = BIProto14(
	"SphereBuild", 0, Vec2i(16,16), Vec2i(sphereBuildIcon_x, sphereBuildIcon_y), //SphereBuild
	BIField(SFFLOAT,	"radius .25{rangeFrom 0}", 0),
	BIField(SFINT32,	"staves 16{rangeFrom 2}", 0),
	BIField(SFINT32,	"bands 8{rangeFrom 1}", 0),
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFBOOL,	"yCentred TRUE", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFBOOL, 	"mirror FALSE", 0),
	BIField(SFBOOL, 	"insideOut FALSE", 0),
	BIField(SFBOOL, 	"descendants FALSE", DISABLE),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_sphereBuild_part)),
	BIField(SFNODE,	"textureMapper NULL", 0,	INIT_PROMPT_NODE(promptNode_sphereBuild_textureMapper)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_sphereBuild_setup))
);
static const BIProto21 cylinderBuild_ = BIProto21(
	"CylinderBuild", 0, Vec2i(16,16), Vec2i(cylinderBuildIcon_x, cylinderBuildIcon_y), //CylinderBuild
	BIField(SFFLOAT,	"height .5", 0),
	BIField(SFFLOAT,	"radius .25", 0),
	BIField(SFINT32,	"staves 16{rangeFrom 2}", 0),
	BIField(SFINT32,	"bands 4{rangeFrom 1}", 0),
	BIField(SFBOOL,	"topCap TRUE", 0),
	BIField(SFBOOL,	"baseCap TRUE", 0),
	BIField(SFFLOAT,	"topHeight 0{rangeFrom 0}", 0),
	BIField(SFFLOAT,	"baseHeight 0{rangeFrom 0}", 0),
	BIField(SFINT32,	"topBands 4{rangeFrom 1}", 0),
	BIField(SFINT32,	"baseBands 4{rangeFrom 1}", 0),
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFBOOL,	"yCentred FALSE", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFBOOL, 	"mirror FALSE", 0),
	BIField(SFBOOL, 	"insideOut FALSE", 0),
	BIField(SFBOOL, 	"descendants FALSE", DISABLE),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_cylinderBuild_part)),
	BIField(MFNODE,	"textureMapper []", 0,	INIT_PROMPT_NODE(promptNode_sphereBuild_textureMapper)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_cylinderBuild_setup))
);
static const BIProto20 coneBuild_ = BIProto20(
	"ConeBuild", 0, Vec2i(16,16), Vec2i(coneBuildIcon_x, coneBuildIcon_y), //ConeBuild
	BIField(SFFLOAT,	"height .5{rangeFrom 0}", 0),
	BIField(SFFLOAT,	"topRad 0{rangeFrom 0}", 0),
	BIField(SFFLOAT,	"baseRad .25{rangeFrom 0}", 0),
	BIField(SFINT32,	"staves 16{rangeFrom 2}", 0),
	BIField(SFINT32,	"bands 7{rangeFrom 1}", 0),
	BIField(SFBOOL,	"topCap TRUE", 0),
	BIField(SFBOOL,	"baseCap TRUE", 0),
	BIField(SFINT32,	"topBands 1{rangeFrom 1}", 0),
	BIField(SFINT32,	"baseBands 1{rangeFrom 1}", 0),
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFBOOL,	"yCentred FALSE", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFBOOL, 	"mirror FALSE", 0),
	BIField(SFBOOL, 	"insideOut FALSE", 0),
	BIField(SFBOOL, 	"descendants FALSE", DISABLE),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_coneBuild_part)),
	BIField(MFNODE,	"textureMapper []", 0,	INIT_PROMPT_NODE(promptNode_sphereBuild_textureMapper)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_coneBuild_setup))
);
static const BIProto15 torusBuild_ = BIProto15(
	"TorusBuild", 0, Vec2i(16,16), Vec2i(torusBuildIcon_x, torusBuildIcon_y), //TorusBuild
	BIField(SFFLOAT,	"holeRad .1", 0),
	BIField(SFFLOAT,	"tubeRad .1", 0),
	BIField(SFINT32,	"staves 32{rangeFrom 2}", 0),
	BIField(SFINT32,	"bands 16{rangeFrom 1}", 0),
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFBOOL,	"yCentred FALSE", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFBOOL, 	"mirror FALSE", 0),
	BIField(SFBOOL, 	"insideOut FALSE", 0),
	BIField(SFBOOL, 	"descendants FALSE", DISABLE),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_torusBuild_part)),
	BIField(MFNODE,		"textureMapper []", 0,	INIT_PROMPT_NODE(promptNode_sphereBuild_textureMapper)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_torusBuild_setup))
);
static const BIProto13 boxBuild_ = BIProto13(
	"BoxBuild", 0, Vec2i(16,16), Vec2i(boxBuildIcon_x, boxBuildIcon_y), //BoxBuild
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFBOOL,	"xCentred TRUE", 0),
	BIField(SFBOOL,	"yCentred FALSE", 0),
	BIField(SFBOOL,	"zCentred TRUE", 0),
	BIField(SFVEC3F,	"scale .25 .25 .25", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFBOOL, 	"mirror FALSE", 0),
	BIField(SFBOOL, 	"insideOut FALSE", 0),
	BIField(SFBOOL, 	"descendants FALSE", DISABLE),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_boxBuild_part)),
	BIField(MFNODE,	"textureMapper []", 0,	INIT_PROMPT_NODE(promptNode_sphereBuild_textureMapper)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_boxBuild_setup))
);
static const BIProto20 rectangleBuild_ = BIProto20(
	"RectangleBuild", 0, Vec2i(16,16), Vec2i(rectangleBuildIcon_x, rectangleBuildIcon_y), //RectangleBuild
	BIField(SFFLOAT,	"xDistance 1", 0),
	BIField(SFFLOAT,	"yDistance 1", 0),
	BIField(SFINT32,	"staves 4{rangeFrom 1}", 0,	tip_rectangleBuild_staves),
	BIField(SFINT32,	"bands 4{rangeFrom 1}", 0,	tip_rectangleBuild_bands),
	BIField(SFBOOL,	"endSP FALSE", 0,	tip_rectangleBuild_endSP),
	BIField(SFBOOL,	"beginSP FALSE", 0,	tip_rectangleBuild_beginSP),
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFBOOL,	"xCentred TRUE", 0),
	BIField(SFBOOL,	"yCentred FALSE", 0),
	BIField(SFBOOL,	"xNegSide FALSE", 0),
	BIField(SFBOOL,	"yNegSide FALSE", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFBOOL, 	"mirror FALSE", 0),
	BIField(SFBOOL, 	"insideOut FALSE", 0),
	BIField(SFBOOL, 	"descendants FALSE", DISABLE),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_rectangleBuild_part)),
	BIField(MFNODE,	"textureMapper []", 0,	INIT_PROMPT_NODE(promptNode_sphereBuild_textureMapper)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_rectangleBuild_setup))
);
static const BIProto14 bezierLatheBuild_ = BIProto14(
	"BezierLatheBuild", 0, Vec2i(16,16), Vec2i(bezierLatheBuildIcon_x, bezierLatheBuildIcon_y), //BezierLatheBuild
	BIField(MFVEC2F,	"point []{beginGroup TRUE}", 0),
	BIField(MFBOOL,		"fix []", DO_NOT_SAVE),
	BIField(MFINT32,	"dbands 16{rangeFrom 1}", DO_NOT_SAVE),
	BIField(MFINT32,	"nopfs []", DISABLE),
	BIField(MFINT32,	"bands 16{rangeFrom 1 beginGroup TRUE}", 0),
	BIField(SFINT32,	"staves 16{rangeFrom 2}", 0),
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFBOOL, 	"descendants FALSE", DISABLE),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_bezierLatheBuild_part)),
	BIField(MFNODE,	"textureMapper []", 0,	INIT_PROMPT_NODE(promptNode_sphereBuild_textureMapper)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_bezierLatheBuild_setup))
);
static const BIProto1 mirrorEdit_ = BIProto1(
	"MirrorEdit", 0, Vec2i(16,16), Vec2i(16*2,112+16), //MirrorEdit
	BIField(MFNODE,	"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_mirrorEdit_part))
);
static const BIProto1 mirrorGroup_ = BIProto1(
	"MirrorGroup", 0, Vec2i(16,16), Vec2i(16*3,112+16), //MirrorGroup
	BIField(MFNODE,	"mirrorEdit []", BLOCK_USE,	INIT_PROMPT_NODE(promptNode_mirrorGroup_mirrorEdit))
);
static const BIProto16 polyToNurbs_ = BIProto16(
	"PolyToNurbs", 0, Vec2i(16,16), Vec2i(224,160),
	BIField(SFBOOL,		"cycle FALSE", PUSH | DO_NOT_SAVE),
	BIField(SFFLOAT,	"sumDiff 0", 0),
	BIField(SFFLOAT,	"averageDiff 0", 0),
	BIField(SFBOOL,		"bond FALSE", 0),
	BIField(SFFLOAT,	"coordChange .01", 0),
	BIField(SFBOOL,		"evolve FALSE", 0),
	BIField(SFFLOAT,	"pointChange .001", 0),
	BIField(SFBOOL,		"spread FALSE", PUSH | DO_NOT_SAVE),
	BIField(SFINT32,	"spreadRes 100", 0),
	BIField(SFINT32,	"offset, 0", DO_NOT_SAVE),

	BIField(SFINT32,	"selOriginalVertex, 0", 0),
	BIField(SFINT32,	"selMimicVertex, 0{rangeFrom 0}", 0),
	BIField(SFBOOL,		"mimicOriginalIndex FALSE", DO_NOT_SAVE),

	BIField(SFNODE,		"original NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_polyToNurbs_original)),
	BIField(SFNODE,		"mimic NULL", 0,	INIT_PROMPT_NODE(promptNode_polyToNurbs_mimic)),
	BIField(SFNODE,		"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_sphereBuild_setup))
);
static const BIProto15 ringBuild_ = BIProto15(
	"RingBuild", 0,  Vec2i(16,16), Vec2i(128+16, 176), //RingBuild
	BIField(SFINT32,"numOfRingsToInsert 1{rangeFrom 1}", 0),
	BIField(MFVEC2F,	"scaleX []{beginGroup TRUE initElVal \".25 .25\"}", SAME_AS_LAST),
	BIField(MFVEC2F,	"scaleZ []{initElVal \".25 .25\"}", SAME_AS_LAST),
	BIField(MFROTATION,	"orientation []{initElVal \"0 0 1 0\"}", SAME_AS_LAST),
	BIField(MFVEC3F,	"location []{endGroup TRUE initElVal \"0 0 0\"}", SAME_AS_LAST),

	BIField(MFINT32,	"ownership []{beginGroup TRUE initElVal \"1\" rangeFrom 0}", SAME_AS_LAST),

	BIField(SFINT32,"staves 16{rangeFrom 1}", 0,	tip_stem_staves),
	//BIField(SFBOOL,	"insideOut FALSE", 0,	tip_stem_insideOut),

	BIField(SFBOOL,	"dragLocationY TRUE", 0),
	BIField(SFBOOL,	"dragLocationXZ TRUE", 0),
	BIField(SFBOOL,	"dragOrientation TRUE", 0),

	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(MFNODE,	"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_sphereBuild_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_sphereBuild_setup))
);
static const BIProto13 bezierScale_ = BIProto13(
	"BezierScale", "children", Vec2i(16,16), Vec2i(64,144), //BezierScale
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(MFVEC2F,	"point []{beginGroup TRUE}", 0),
	BIField(MFBOOL,		"fix []", DO_NOT_SAVE),
	BIField(MFINT32,	"nopfs []", DISABLE),
	BIField(SFBOOL,		"bNegX FALSE", 0),
	BIField(SFBOOL,		"bPosX FALSE", 0),
	BIField(SFBOOL,		"bNegZ FALSE", 0),
	BIField(SFBOOL,		"bPosZ FALSE", 0),
	BIField(SFFLOAT,	"negX 1", 0),
	BIField(SFFLOAT,	"posX 1", 0),
	BIField(SFFLOAT,	"negZ 1", 0),
	BIField(SFFLOAT,	"posZ 1", 0)
);
static const BIProto7 bezierCircumScale_ = BIProto7(
	"BezierCircumScale", "children", Vec2i(16,16), Vec2i(240,112), //BezierCircumScale
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(MFVEC2F,	"point []{beginGroup TRUE}", 0),
	BIField(MFBOOL,		"fix []", DO_NOT_SAVE),
	BIField(MFINT32,	"nopfs []", DISABLE),
	BIField(SFFLOAT,	"centerOffset 0", 0),
	BIField(SFFLOAT,	"level 1", 0)
);
/*
static const BIProto11 bezierPatch_ = BIProto11(
	"BezierPatch", "build",
	BIField(MFVEC3F,	"point []{beginGroup TRUE}", 0),
	BIField(MFBOOL,		"fix []", DO_NOT_SAVE),
	BIField(MFINT32,	"fixIdx []{beginGroup TRUE}", 0),
	BIField(SFINT32,	"numOfU 2", 0),
	BIField(SFINT32,	"numOfV 2", 0),
	BIField(SFINT32,	"staves 10", 0),
	BIField(SFINT32,	"bands 10", 0),
	BIField(MFNODE,		"source []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_stem_source)),
	BIField(MFNODE,		"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_stem_part)),
	BIField(MFNODE,		"task []", BLOCK_USE,	INIT_PROMPT_NODE(promptNode_bezierPatch_task)),
	BIField(SFNODE,		"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_stem_setup))
);
*/
static const BIProto5 bezierPatch_ = BIProto5(
	"BezierPatch",		"build", Vec2i(16,16), Vec2i(112,96), //BezierPatch
	BIField(MFVEC3F,	"point []{beginGroup TRUE}", 0),
	BIField(MFFLOAT,	"weight []", 0),
//	BIField(MFINT32,	"fixIdx []{beginGroup TRUE}", DISABLE),
	BIField(MFINT32,	"link []{beginGroup TRUE}", DISABLE),

	BIField(MFVEC2F,	"dim []{beginGroup TRUE}", 0),
	BIField(MFVEC2F,	"zone []", 0)
//	BIField(MFINT32,	"staves []", 0),
//	BIField(MFINT32,	"bands []", 0),
//	BIField(MFINT32,	"fixIdxIdx []", DISABLE)

//	BIField(SFBOOL,	"mimicX []{beginGroup TRUE}", 0),
//	BIField(SFBOOL,	"mimicY []", 0),
//	BIField(SFBOOL,	"mimicZ []", 0),
//	BIField(MFINT32,	"mimicSrc []", 0),
//	BIField(MFINT32,	"mimicDst []", 0)

//	BIField(MFNODE,		"source []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_stem_source)),
//	BIField(MFNODE,		"part []", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_stem_part)),
//	BIField(MFNODE,		"task []", BLOCK_USE,	INIT_PROMPT_NODE(promptNode_bezierPatch_task)),
//	BIField(SFNODE,		"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_stem_setup))
);
static const BIProto3 feedback_ = BIProto3(
	"Feedback", "feedback", Vec2i(16,16), Vec2i(224,160),
	BIField(SFVEC2F,	"multiply .5 .5", 0),
	BIField(SFINT32,	"from 0", 0),
	BIField(SFNODE,		"source NULL", 0,	INIT_PROMPT_NODE(promptNode_feedback_source))
);
static const BIProto11 povLight_ = BIProto11(
	"PovLight", "children", Vec2i(16,16), Vec2i(464,96), //PovLight
	BIField(SFVEC3F,	"location 0 0 100", 0),
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFBOOL,		"pointlight FALSE", 0),
	BIField(SFBOOL,		"spotlight FALSE", 0),
	BIField(SFBOOL,		"shadowless FALSE", 0),
	BIField(SFFLOAT,	"radius 0.523598776", 0),
	BIField(SFFLOAT,	"falloff 0.785398163", 0),
	BIField(SFFLOAT,	"tightness 0", 0),
    BIField(SFVEC3F,	"point_at 0 0 0", 0),
	BIField(SFFLOAT,	"fade_distance 0", 0),
    BIField(SFFLOAT,	"fade_power 0", 0)
);
static const BIProto9 povFog_ = BIProto9(
	"PovFog", "children", Vec2i(16,16), Vec2i(416,96), //PovFog
 	BIField(SFFLOAT,	"lambda 2", 0),
	BIField(SFINT32,	"fog_type 1", 0),
	BIField(SFFLOAT,	"fog_offset 0", 0),
	BIField(SFFLOAT,	"fog_alt 0", 0),
	BIField(SFINT32,	"octaves 6", 0),
	BIField(SFFLOAT,	"omega 5", 0),
	BIField(SFVEC3F,	"turbulence 0 0 0", 0),
	BIField(SFFLOAT,	"turb_depth .5", 0),
	BIField(SFVEC3F,	"up 0 1 0", 0)
);
static const BIProto7 bezierShift_ = BIProto7(
	"BezierShift", "children", Vec2i(16,16), Vec2i(96,96), //BezierShift
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(MFVEC2F,	"point []{beginGroup TRUE}", 0),
	BIField(MFBOOL,		"fix []", DO_NOT_SAVE),
	BIField(MFINT32,	"nopfs []", DISABLE),
	BIField(SFFLOAT,	"negDecay 1", 0),
	BIField(SFFLOAT,	"posDecay 1", 0)
);
static const BIProto5 bezierTwist_ = BIProto5(
	"BezierTwist", "children", Vec2i(16,16), Vec2i(80,96), //BezierTwist
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(MFVEC2F,	"point []{beginGroup TRUE}", 0),
	BIField(MFBOOL,		"fix []", DO_NOT_SAVE),
	BIField(MFINT32,	"nopfs []", DISABLE)
);
static const BIProto3 bezierTape_ = BIProto3(
	"BezierTape", "children", Vec2i(16,16), Vec2i(80,96), //BezierTape
	BIField(MFVEC2F,	"point []{beginGroup TRUE}", 0),
	BIField(MFBOOL,		"fix []", DO_NOT_SAVE),
	BIField(MFINT32,	"nopfs []", DISABLE)
);
static const BIProto5 bezierBend_ = BIProto5(
	"BezierBend", "children", Vec2i(16,16), Vec2i(80,96), //BezierBend
	BIField(SFVEC3F,	"location 0 0 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", 0),
	BIField(MFVEC3F,	"point []{beginGroup TRUE}", 0),
	BIField(MFBOOL,		"fix []", DO_NOT_SAVE),
	BIField(MFINT32,	"nopfs []", DISABLE)
);

static const BIProto9 nurbsLine_ = BIProto9(
	"NurbsLine", 0, Vec2i(16,16), Vec2i(11 * 16, 112), //NurbsLine
	BIField(MFVEC3F,	"point []{beginGroup TRUE}", DISABLE), //DO_NOT_SAVE for new smls cp format
	BIField(MFVEC3F,	"point []", DISABLE_LEN | DO_NOT_SAVE),
	BIField(MFFLOAT,	"weight []", 0),
	BIField(MFCOLOR,	"color []", 0),
	BIField(MFFLOAT,	"uKnot []{beginGroup TRUE}", DISABLE),
	BIField(SFBOOL,		"mirror FALSE", 0),
	BIField(SFBOOL,		"joinEnds FALSE", 0),
	BIField(SFINT32,	"steps 64", 0),
	BIField(SFBOOL,		"cage TRUE", DISABLE)
);

static const BIProto19 nurbsLathe_ = BIProto19(
	"NurbsLathe", 0, Vec2i(16,16), Vec2i(160+16,192), //NurbsLathe
	BIField(MFVEC2F,	"point []{beginGroup TRUE}", DISABLE),
	BIField(MFVEC2F,	"point []", DO_NOT_SAVE | DISABLE_LEN),
	BIField(MFFLOAT,	"weight []", 0),
	BIField(MFFLOAT,	"knot []{beginGroup TRUE}", DISABLE),
	BIField(SFINT32,	"order 3{rangeFrom 1}", DISABLE),
	BIField(SFINT32,	"staves 16{rangeFrom 2}", 0),
	BIField(SFINT32,	"bands 16{rangeFrom 1}", 0),
	//BIField(SFINT32,	"ctrlStaves 4{rangeFrom 3}", DO_NOT_SAVE),
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFBOOL,	"endSP TRUE", 0,	tip_rectangleBuild_endSP),
	BIField(SFBOOL,	"beginSP TRUE", 0,	tip_rectangleBuild_beginSP),
	BIField(SFBOOL, 	"mirror FALSE", 0),
	BIField(SFBOOL, 	"insideOut FALSE", 0),
	BIField(SFBOOL, 	"descendants FALSE", DISABLE),
	BIField(SFNODE, 	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_bezierLatheBuild_part)),
	BIField(SFNODE,		"textureMapper NULL", 0,	INIT_PROMPT_NODE(promptNode_sphereBuild_textureMapper)),
	BIField(SFNODE, 	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_bezierLatheBuild_setup))
);
static const BIProto20 bendyLathe_ = BIProto20(
	"BendyLathe", 0, Vec2i(16,16), Vec2i(160+16*3,192), //BendyLathe
	BIField(SFBOOL, 	"showRingControlPoints FALSE", 0),
	BIField(MFVEC3F,	"point []{beginGroup TRUE}", DISABLE),
	BIField(MFVEC3F,	"point []", DO_NOT_SAVE),
	BIField(MFFLOAT,	"weight []", DISABLE),
	BIField(MFFLOAT,	"knot []{beginGroup TRUE}", DISABLE),
	BIField(SFINT32,	"order 3{rangeFrom 1}", DISABLE),
	BIField(MFFLOAT,	"radius []{beginGroup TRUE}", 0),
	BIField(MFFLOAT,	"at []", 0),
	BIField(SFINT32,	"staves 16{rangeFrom 2}", 0),
	BIField(SFINT32,	"bands 25{rangeFrom 1}", 0),
	//BIField(SFINT32,	"ctrlStaves 4{rangeFrom 3}", DO_NOT_SAVE),
	BIField(SFCOLOR,	"color 1 1 1", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFBOOL, 	"mirror FALSE", 0),
	BIField(SFBOOL, 	"insideOut FALSE", 0),
	BIField(SFBOOL, 	"descendants FALSE", DISABLE),
	BIField(SFNODE, 	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_bezierLatheBuild_part)),
	BIField(MFNODE,		"textureMapper []", 0,	INIT_PROMPT_NODE(promptNode_sphereBuild_textureMapper)),
	BIField(SFNODE, 	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_bezierLatheBuild_setup))
);
static const BIProto22 fuseSurface_ = BIProto22(
	"FuseSurface",		"fuseSurface", Vec2i(16,16), Vec2i(240,192), //FuseSurface
	BIField(MFVEC2F,	"baseCoord []{beginGroup TRUE initElVal \"[.01]\"}", SAME_AS_LAST),

	BIField(SFFLOAT,	"diff 0", DO_NOT_SAVE),
	BIField(SFFLOAT,	"sumDiff 0", DO_NOT_SAVE),
	BIField(SFFLOAT,	"averageDiff 0", DO_NOT_SAVE),

	BIField(SFFLOAT,	"vSpan .5", 0),
	BIField(SFINT32,	"numOf 1", 0),
	BIField(SFINT32,	"qty 0", 0),
	BIField(SFROTATION,	"orientation 0 0 1 0", DISABLE | DO_NOT_SAVE),
	BIField(SFBOOL,		"bond TRUE", 0),
	BIField(SFFLOAT,	"coordChange .01", 0),
	BIField(SFBOOL,		"spread FALSE", PUSH | DO_NOT_SAVE),
	BIField(SFINT32,	"spreadRes 100", 0),
	BIField(SFBOOL,		"polyMapAid FALSE", 0),
	BIField(SFBOOL,		"polyMapAidMirror FALSE", DISABLE | DO_NOT_SAVE),

	BIField(MFINT32,	"srcMatch []{beginGroup TRUE}", DISABLE_LEN),
	BIField(MFINT32,	"dstMatch []", 0),
	BIField(SFBOOL,		"in FALSE", 0),
	BIField(SFFLOAT,	"range .001", 0),
	BIField(SFBOOL,		"snap FALSE", 0),
	BIField(SFBOOL,		"evolve FALSE", 0),
	BIField(SFFLOAT,	"pointChange .001", 0),

	BIField(SFNODE,		"patch NULL", 0,		INIT_PROMPT_NODE(promptNode_fuseSurface_patch))
);
static const BIProto5 subdivision_ = BIProto5(
	"Subdivision", "build", Vec2i(16,16), Vec2i(144,192), //Subdivision
	BIField(MFVEC3F,	"point []{beginGroup TRUE}", 0),
	BIField(SFINT32,	"cycle 2", 0),
	BIField(MFINT32,	"coordIndex []{beginGroup TRUE initElVal \"[0 1 2 3]\"}", SAME_AS_LAST),
//	BIField(MFINT32,	"coordIndex []{beginGroup TRUE", DISABLE_LEN),
//	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_stem_part)),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_centerJoint_part)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_stem_setup))
);
static const BIProto14 oscillator_ = BIProto14(
	"Oscillator", "sound", Vec2i(16,16), Vec2i(80,192), //Oscillator
	BIField(MFVEC2F,	"note []{beginGroup TRUE}", DISABLE),
	BIField(MFVEC2F,	"note []", DO_NOT_SAVE),
	BIField(MFFLOAT,	"fWeight []", DISABLE),
	BIField(MFFLOAT,	"fKnot []{beginGroup TRUE}", DISABLE),
	BIField(SFINT32,	"fOrder 3", DISABLE),
	BIField(SFFLOAT,	"volume 1", 0),
	BIField(SFFLOAT,	"delay 0", 0),
	BIField(SFINT32,	"staves 32", 0),
	BIField(MFINT32,	"limit []", 0),
	BIField(MFVEC2F,	"vCoord []{beginGroup TRUE}", DO_NOT_SAVE),
	BIField(MFVEC3F,	"coord []{endGroup TRUE}", DO_NOT_SAVE),
	BIField(SFNODE,		"larynx NULL", 0,	INIT_PROMPT_NODE(promptNode_oscillator_larynx)),
	BIField(SFNODE,		"melody NULL", 0,	INIT_PROMPT_NODE(promptNode_oscillator_melody)),
	BIField(MFNODE,		"filter []", 0,	INIT_PROMPT_NODE(promptNode_oscillator_filter))
);

static const BIProto2 envelope_ = BIProto2(
	"Envelope", "envelope", Vec2i(16,16), Vec2i(240,112), //Envelope
	BIField(MFVEC2F,	"point []", 0),
	BIField(MFNODE,		"chords NULL", 0,	INIT_PROMPT_NODE(promptNode_envelope_chords))
);

static const BIProto2 amplitudeModulator_ = BIProto2(
	"AmplitudeModulator", "modulator", Vec2i(16,16), Vec2i(144,192), //AmplitudeModulator
	BIField(MFVEC2F,	"wet []", 0),
	BIField(SFNODE,		"oscillator NULL", 0,	INIT_PROMPT_NODE(promptNode_modulator_oscillator))
);

static const BIProto2 frequencyModulator_ = BIProto2(
	"FrequencyModulator", "modulator", Vec2i(16,16), Vec2i(80,96), //FrequencyModulator
	BIField(MFVEC2F,	"wet []", 0),
	BIField(SFNODE,		"oscillator NULL", 0,	INIT_PROMPT_NODE(promptNode_modulator_oscillator))
);

static const BIProto16 chords_ = BIProto16(
	"Chords", "chords", Vec2i(16,16), Vec2i(96,192), //Chords
	BIField(MFVEC2F,	"spectrum []{beginGroup TRUE}", 0),
	BIField(MFFLOAT,	"multiple []{beginGroup TRUE}", 0),
	BIField(MFFLOAT,	"amplitude []", 0),
	BIField(MFFLOAT,	"phase  []", 0),
	BIField(SFBOOL,		"sawTooth true", 0),
	BIField(SFBOOL,		"square false", 0),
	BIField(SFFLOAT,	"randomize 0", 0),
	BIField(SFINT32,	"harmonics 92", 0),
	BIField(SFFLOAT,	"increment 1", 0),
	BIField(SFBOOL,		"enable TRUE", 0),
	BIField(SFFLOAT,	"interval 0", 0),
	BIField(SFFLOAT,	"baseMultiple 1", 0),
	BIField(SFFLOAT,	"level 1", 0),
	//BIField(SFNODE,		"envelope NULL", 0,	INIT_PROMPT_NODE(promptNode_chords_envelope)),
	BIField(SFNODE,		"patials NULL", 0,	INIT_PROMPT_NODE(promptNode_chords_patch)),
	BIField(SFNODE,		"modulator NULL", 0,	INIT_PROMPT_NODE(promptNode_chords_amplitudeModulator)),
	BIField(SFNODE,		"phaseRef NULL", 0,	INIT_PROMPT_NODE(promptNode_chords_phaseRef))
);

static const BIProto1 larynx_ = BIProto1(
	"Larynx", "larynx", Vec2i(16,16), Vec2i(96,192), //Larynx
	/*
	BIField(MFBOOL,		"enable TRUE{beginGroup TRUE}", 0),
	BIField(MFBOOL,		"saw TRUE", 0),
	BIField(MFINT32,	"qty 92", 0),
	BIField(MFFLOAT,	"increment 1", 0),
	BIField(MFFLOAT,	"randomize 0", 0),
	BIField(MFFLOAT,	"level 1{endGroup TRUE}", 0),
	
	BIField(MFNODE,		"osc []", 0,	INIT_PROMPT_NODE(promptNode_larynx_chords)),
	*/
	BIField(MFNODE,		"chords NULL", 0,	INIT_PROMPT_NODE(promptNode_larynx_chords))
);

static const BIProto3 phrase_ = BIProto3(
	"Phrase", "sound", Vec2i(16,16), Vec2i(128,192), //Phrase
	BIField(SFFLOAT,	"volume 1", 0),
	BIField(SFFLOAT,	"delay 0", 0),
	BIField(MFNODE,		"sound []", 0,	INIT_PROMPT_NODE(promptNode_track_sound))
);
static const BIProto7 harmonic_ = BIProto7(
	"Harmonic", "timbre", Vec2i(16,16), Vec2i(96,192), //Harmonic
	BIField(MFVEC2F,	"point []{beginGroup TRUE}", DISABLE),
	BIField(MFVEC2F,	"point []", DO_NOT_SAVE),
	BIField(MFFLOAT,	"weight []", 0),
	BIField(MFFLOAT,	"knot []{beginGroup TRUE}", 0),
	BIField(SFINT32,	"order 3", 0),
	//BIField(MFINT32,	"numOf []{beginGroup TRUE initElVal \"[1]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"multiply []{beginGroup TRUE initElVal \"[1]\"}", SAME_AS_LAST),
	BIField(MFFLOAT,	"phase []{initElVal \"[0]\"}", SAME_AS_LAST)
);
static const BIProto11 harmonicPatch_ = BIProto11(
	"HarmonicPatch", "timbre", Vec2i(16,16), Vec2i(96,192), //HarmonicPatch
	BIField(MFVEC3F,	"point []{beginGroup TRUE}", DISABLE),
	BIField(MFVEC3F,	"point []", DISABLE_LEN | DO_NOT_SAVE),
	BIField(MFFLOAT,	"weight []", 0),
	BIField(MFFLOAT,	"uKnot []{beginGroup TRUE}", 0),
	BIField(MFFLOAT,	"vKnot []{beginGroup TRUE}", 0),
	BIField(SFVEC2F,	"dim 2 2", 0),
	BIField(SFVEC2F,	"order 3 3", 0),
	BIField(SFINT32,	"staves 100", SAME_AS_LAST),
	BIField(MFFLOAT,	"vCoord []{beginGroup TRUE}", SAME_AS_LAST),
	BIField(MFVEC3F,	"coord []{endGroup TRUE}", SAME_AS_LAST),
	BIField(SFINT32,	"harmonics 30", SAME_AS_LAST)
);
static const BIProto1 sample_ = BIProto1(
	"Sample", "timbre",  Vec2i(16,16), Vec2i(112,192), //Sample
	BIField(SFSTRING,	"url \"\"", 0)
);
static const BIProto19 waveToPoly_ = BIProto19(
	"WaveToPoly", "children", Vec2i(16,16), Vec2i(224,160), //WaveToPoly
	BIField(SFBOOL,	"open FALSE", DO_NOT_SAVE | PUSH | DO_NOT_DISPLY_NAME, "open wav file"),
	BIField(SFSTRING,	"file \"\"", 0),
	BIField(SFINT32,	"viewFrom 0", 0),
	BIField(SFINT32,	"zoom 1{rangeFrom 1}", DO_NOT_DISPLY_NAME | DO_NOT_DISPLY_VALUE, "zoom"),
	BIField(SFBOOL,	"convert FALSE", DO_NOT_SAVE | PUSH , 0),
	BIField(SFINT32,	"cMark 0", DO_NOT_DISPLY_NAME, "dec/inc cycle"),
	BIField(SFINT32,	"nextCycle 0", DO_NOT_DISPLY_NAME, "next cycle"),
	BIField(SFBOOL,	"addMark FALSE", DO_NOT_SAVE | PUSH | DO_NOT_DISPLY_NAME, "add cycle mark"),
	BIField(SFBOOL,	"deleteMark FALSE", DO_NOT_SAVE | PUSH | DO_NOT_DISPLY_NAME, "delete cycle mark"),
	BIField(MFINT32,	"cycle []", DISABLE),
	BIField(SFBOOL,	"sawTooth TRUE", 0),
	BIField(SFINT32, 	"windowSize 8192", 0),
	BIField(SFBOOL,	"overlap FALSE", 0),
	BIField(SFINT32, 	"partials 1024", 0),
	BIField(SFFLOAT, 	"pitchLimit 440", 0),
	BIField(SFBOOL,	"vocoder FALSE", 0),
	BIField(SFBOOL,	"clear FALSE", DO_NOT_SAVE | PUSH | DO_NOT_DISPLY_NAME, "clear cycles"),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_part_part)),
	BIField(SFNODE,	"osc NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_track_sound))
);

static const BIProto14 midiFormant_ = BIProto14(
	"MidiFormant", "childrem", Vec2i(16,16), Vec2i(112,192),
	BIField(SFBOOL,		"select FALSE", PUSH),
	BIField(SFSTRING,	"chord \"\"", 0),
	BIField(SFINT32,	"uOffset 0", 0),
	BIField(SFINT32,	"uCount -1", 0),
	BIField(MFINT32,	"pitch []{beginGroup TRUE}", 0),
	BIField(MFINT32,	"range []", 0),
	BIField(MFINT32,	"cent []{initElVal \"[64]\"}", SAME_AS_LAST),
	BIField(MFINT32,	"vLine []", SAME_AS_LAST),
	BIField(MFINT32,	"before []", 0),
	BIField(MFINT32,	"after []", 0),
	BIField(MFINT32,	"bw []{initElVal \"[64]\"}", SAME_AS_LAST),
	BIField(MFINT32,	"trough []", 0),
	BIField(MFINT32,	"peak []", 0),
	BIField(SFNODE,	"patch NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_midiFormant_patch))
);

static const BIProto1 filter_ = BIProto1(
	"Filter", "childrem", Vec2i(16,16), Vec2i(112,192),
	BIField(MFNODE,		"formant []", 0,	INIT_PROMPT_NODE(promptNode_filter_formant))
);

static const BIProto5 formant_ = BIProto5(
	"Formant", "formant", Vec2i(16,16), Vec2i(112,192),
	BIField(MFFLOAT,	"freq []{beginGroup TRUE}", 0),
	BIField(MFFLOAT,	"amp []", 0),
	BIField(MFFLOAT,	"bw []", 0),
	BIField(MFINT32,	"uCount []", 0),
	BIField(SFNODE,	"patch NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_fuseSurface_patch))
);

static const BIProto7 lowPass_ = BIProto7(
	"LowPass", "formant", Vec2i(16,16), Vec2i(112,192),
	BIField(MFFLOAT,	"freq []{beginGroup TRUE}", 0),
	BIField(MFFLOAT,	"pass []", 0),
	BIField(MFFLOAT,	"resonance []", 0),
	BIField(MFFLOAT,	"leak []", 0),
	BIField(MFFLOAT,	"bw []", 0),
	BIField(MFINT32,	"uCount []", 0),
	BIField(SFNODE,	"patch NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_fuseSurface_patch))
);

static const BIProto6 multiFormant_ = BIProto6(
	"MultiFormant", "formant", Vec2i(16,16), Vec2i(112,192),
	BIField(MFFLOAT,	"freq []{beginGroup TRUE}", 0),
	BIField(MFFLOAT,	"peak []", 0),
	BIField(MFFLOAT,	"trough []", 0),
	BIField(MFFLOAT,	"bw []", 0),
	BIField(MFINT32,	"uCount []", 0),
	BIField(SFNODE,	"patch NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_fuseSurface_patch))
);

static const BIProto7 track_ = BIProto7(
	"Track", "track", Vec2i(16,16), Vec2i(64,192), //Track
	//BIField(SFBOOL,		"output FALSE", PUSH | DISABLE),
	BIField(SFINT32,	"voices 1", 0),
	BIField(SFFLOAT, 	"detune 0", 0),
	BIField(SFINT32,	"echos 0", 0),
	BIField(SFFLOAT,	"echoDelay 0", 0),
	BIField(SFFLOAT,	"decay 0", 0),
	BIField(SFFLOAT,	"volume 1", 0),
	BIField(MFNODE,		"sound []", 0,	INIT_PROMPT_NODE(promptNode_track_sound))
);

static const BIProto6 composition_ = BIProto6(
	"Composition", "children", Vec2i(16,16), Vec2i(48,192), //Composition
	BIField(SFBOOL,		"output FALSE", PUSH),
	BIField(SFFLOAT,	"volume 1", 0),
	BIField(SFBOOL,		"addDelay TRUE", 0),
	BIField(SFFLOAT,	"delay 0", 0),
	BIField(SFFLOAT,	"duration 10", 0),
	BIField(MFNODE,		"track []", 0,	INIT_PROMPT_NODE(promptNode_composition_track))
);

static const BIProto3 cPMorpher_ = BIProto3(
	"CPMorpher", "children", Vec2i(16,16), Vec2i(96,128), //CPMorpher
	BIField(MFFLOAT,	"key []", DISABLE_LEN),
	BIField(MFVEC3F,	"keyValue []", DISABLE_LEN),
	BIField(MFINT32,	"cpIndex []", DISABLE_LEN)
);

static const BIProto13 hair_ = BIProto13(
	"Hair", "build", Vec2i(16,16), Vec2i(224,160), //Hair
	BIField(SFFLOAT,	"radius .0005", 0),
	BIField(SFINT32,	"staves 8", 0),
	BIField(SFINT32,	"bands 4", 0),
	BIField(SFINT32,	"strands 0", 0),
	BIField(MFVEC2F,	"hole []{beginGroup TRUE}", 0),
	BIField(MFINT32,	"scpi []", 0),
	BIField(MFINT32,	"beginIndex []{beginGroup TRUE}", 0),
	BIField(MFINT32,	"numOfHoles []", 0),
	BIField(SFNODE,	"part NULL", BLOCK_NEW,	INIT_PROMPT_NODE(promptNode_bezierLatheBuild_part)),
	BIField(MFNODE,	"strand []",  BLOCK_USE, INIT_PROMPT_NODE(promptNode_hair_strand)),
	BIField(MFNODE,	"lock []",  BLOCK_USE, INIT_PROMPT_NODE(promptNode_hair_lock)),
	BIField(MFNODE,		"textureMapper []", 0,	INIT_PROMPT_NODE(promptNode_sphereBuild_textureMapper)),
	BIField(SFNODE,	"setup NULL", 0,	INIT_PROMPT_NODE(promptNode_stem_setup))
);

static const BIProto2 strand_ = BIProto2(
	"Strand", "strand", Vec2i(16,16), Vec2i(160+16*3,192), //Strand
	BIField(MFVEC3F,	"point []", DISABLE_LEN),
	BIField(SFBOOL,		"cage TRUE", DISABLE)
);

static const BIProto21 lock_ = BIProto21(
	"Lock", "lock", Vec2i(16,16), Vec2i(160,192), //Lock
	BIField(MFVEC3F,	"point []{beginGroup TRUE}", DISABLE),
	BIField(MFVEC3F,	"point []", DISABLE_LEN | DO_NOT_SAVE),
	BIField(MFFLOAT,	"weight []", 0),
	BIField(MFFLOAT,	"uKnot []{beginGroup TRUE}", DISABLE),
	BIField(MFFLOAT,	"vKnot []{beginGroup TRUE}", DISABLE),
	BIField(SFVEC2F,	"order 3 3", DISABLE),

	BIField(SFBOOL,		"object FALSE", DISABLE | DO_NOT_SAVE),
	BIField(SFBOOL,		"cage TRUE", DISABLE),
	BIField(SFBOOL,		"showAll FALSE", 0),
	BIField(SFBOOL,		"mirror FALSE", 0),
	BIField(SFVEC2F,	"dim 0 0", 0),

	BIField(SFBOOL,		"endSP FALSE", 0),
	BIField(SFBOOL,		"uJoinEnds TRUE", 0),
	BIField(SFBOOL,		"beginSP FALSE", 0),

	BIField(MFVEC2F,	"hole []{beginGroup TRUE}", 0),
	BIField(MFINT32,	"scpi []", 0),
	BIField(MFVEC2F,	"border []{beginGroup TRUE}", 0),
	BIField(MFINT32,	"region []", 0),
	//BIField(SFBOOL,		"dummy FALSE", 0),
	BIField(MFNODE,		"lock []",  BLOCK_USE, INIT_PROMPT_NODE(promptNode_hair_lock)),
	BIField(MFNODE,		"unify []",  BLUE_DOT, INIT_PROMPT_NODE(promptNode_hair_lock)),
	BIField(MFNODE,		"textureMapper []", 0,	INIT_PROMPT_NODE(promptNode_sphereBuild_textureMapper))
);


static const BIProto1 g_S_Bool_ = BIProto1(
	"G_S_Bool", 0, Vec2i(16,16), Vec2i(112,96), //G_S_Bool
	BIField(SFBOOL,	"value FALSE", 0)
);
static const BIProto1 g_S_Color_ = BIProto1(
	"G_S_Color", 0, Vec2i(16,16), Vec2i(112,96), //G_S_Color
	BIField(SFCOLOR,	"value 1 1 1", 0)
);
static const BIProto1 g_S_Float_ = BIProto1(
	"G_S_Float", 0, Vec2i(16,16), Vec2i(112,96), //G_S_Float
	BIField(SFFLOAT,	"value 0", 0)
);
static const BIProto0 g_S_Image_ = BIProto0(
	"G_S_Image", 0, Vec2i(16,16), Vec2i(112,96) //G_S_Image
);
static const BIProto1 g_S_Int32_ = BIProto1(
	"G_S_Int32", 0, Vec2i(16,16), Vec2i(112,96), //G_S_Int32
	BIField(SFINT32,	"value 0", 0)
);
static const BIProto1 g_NodePtr_ = BIProto1(
	"G_NodePtr", 0, Vec2i(16,16), Vec2i(112,96), //G_NodePtr
	BIField(SFNODE,	"value NULL", 0)
);
static const BIProto1 g_S_Rotation_ = BIProto1(
	"G_S_Rotation", 0, Vec2i(16,16), Vec2i(112,96), //G_Rotation
	BIField(SFROTATION,	"value 0 0 1 0", 0)
);
static const BIProto1 g_S_String_ = BIProto1(
	"G_S_String", 0, Vec2i(16,16), Vec2i(112,96), //G_S_String
	BIField(SFSTRING,	"value \"\"", 0)
);
static const BIProto1 g_S_Time_ = BIProto1(
	"G_S_Time", 0, Vec2i(16,16), Vec2i(112,96), //G_S_Time
	BIField(SFTIME,	"value 0", 0)
);
static const BIProto1 g_S_Vec2f_ = BIProto1(
	"G_S_Vec2f", 0, Vec2i(16,16), Vec2i(112,96), //G_S_Vec2f
	BIField(SFVEC2F,	"value 0 0", 0)
);
static const BIProto1 g_S_Vec3f_ = BIProto1(
	"G_S_Vec3f", 0, Vec2i(16,16), Vec2i(112,96), //G_S_Vec3f
	BIField(SFVEC3F,	"value 0 0 0", 0)
);
static const BIProto1 g_M_Bool_ = BIProto1(
	"G_M_Bool", 0, Vec2i(16,16), Vec2i(112,96), //G_M_Bool
	BIField(MFBOOL,	"value []", 0)
);
static const BIProto1 g_M_Color_ = BIProto1(
	"G_M_Color", 0, Vec2i(16,16), Vec2i(112,96), //G_M_Color
	BIField(MFCOLOR,	"value []", 0)
);
static const BIProto1 g_M_Float_ = BIProto1(
	"G_M_Float", 0, Vec2i(16,16), Vec2i(112,96), //G_M_Float
	BIField(MFFLOAT,	"value []", 0)
);
static const BIProto0 g_M_Image_ = BIProto0(
	"G_M_Image", 0, Vec2i(16,16), Vec2i(112,96) //G_M_Image
);
static const BIProto1 g_M_Int32_ = BIProto1(
	"G_M_Int32", 0, Vec2i(16,16), Vec2i(112,96), //G_M_Int32
	BIField(MFINT32,	"value []", 0)
);
static const BIProto1 g_M_Node_ = BIProto1(
	"G_M_Node", 0, Vec2i(16,16), Vec2i(112,96), //G_M_Node
	BIField(MFNODE,	"value []", 0)
);
static const BIProto1 g_M_Rotation_ = BIProto1(
	"G_M_Rotation", 0, Vec2i(16,16), Vec2i(112,96), //G_M_Rotation
	BIField(MFROTATION,	"value []", 0)
);
static const BIProto1 g_M_String_ = BIProto1(
	"G_M_String", 0, Vec2i(16,16), Vec2i(112,96), //G_M_String
	BIField(MFSTRING,	"value []", 0)
);
static const BIProto1 g_M_Time_ = BIProto1(
	"G_M_Time", 0, Vec2i(16,16), Vec2i(112,96), //G_M_Time
	BIField(MFDOUBLE,	"value []", 0)
);
static const BIProto1 g_M_Vec2f_ = BIProto1(
	"G_M_Vec2f", 0, Vec2i(16,16), Vec2i(112,96), //G_M_Vec2f
	BIField(MFVEC2F,	"value []", 0)
);
static const BIProto1 g_M_Vec3f_ = BIProto1(
	"G_M_Vec3f", 0, Vec2i(16,16), Vec2i(112,96), //G_M_Vec3f
	BIField(MFVEC3F,	"value []", 0)
);

static const BIProto4 file_ = BIProto4(
	"File", 0, Vec2i(16,16), Vec2i(240,96), //File
	BIField(SFBOOL,		"sortByDate FALSE", 0),
	BIField(SFSTRING,	"filename \"\"", 0),
	BIField(SFBOOL,		"open FALSE", PUSH),
	BIField(MFNODE,		"folder []", 0)
);
static const BIProto2 floorHeight_ = BIProto2(
	"FloorHeight", 0, Vec2i(16,16), Vec2i(224,160), //FloorHeight
	BIField(SFBOOL,		"zero_height FALSE", PUSH),
	BIField(SFBOOL,		"blaxxun_height FALSE", PUSH)
);

static const BIProto2 message1Button_ = BIProto2(
	"Message1Button", 0, Vec2i(16,16), Vec2i(224,160), //Message1Button
	BIField(SFSTRING,	"msg \"\"", 0),
	BIField(SFBOOL,		"ok FALSE", PUSH)
);

static const BIProto4 questionMessage3Button_ = BIProto4(
	"QuestionMessage3Button", 0, Vec2i(16,16), Vec2i(224,160), //QuestionMessage3Button
	BIField(SFSTRING,	"msg \"\"", 0),
	BIField(SFBOOL,		"yes FALSE", PUSH),
	BIField(SFBOOL,		"no FALSE", PUSH),
	BIField(SFBOOL,		"cancel FALSE", PUSH)
);
static const BIProto6 recentFiles_ = BIProto6(
	"RecentFiles", 0, Vec2i(16,16), Vec2i(224,160), //RecentFiles
	BIField(SFBOOL,		"sel0 FALSE", LINK | DISABLE),
	BIField(SFBOOL,		"sel1 FALSE", LINK | DISABLE),
	BIField(SFBOOL,		"sel2 FALSE", LINK | DISABLE),
	BIField(SFBOOL,		"sel3 FALSE", LINK | DISABLE),
	BIField(SFBOOL,		"sel4 FALSE", LINK | DISABLE),
	BIField(SFBOOL,		"sel5 FALSE", LINK | DISABLE)
);

static const BIProto0 radicalPatch_ = BIProto0(
	"RadicalPatch", 0, Vec2i(16,16), Vec2i(224,160)
);


static const BIProto1 radial_Tug_ = BIProto1(
	"Radial Tug", 0, Vec2i(16,16), Vec2i(336,144), //Radial_Tug
	BIField(SFFLOAT,	"radius .2", 0)
);

static const BIProto13 palette_ = BIProto13(
	"Palette", 0, Vec2i(16,16), Vec2i(224,160), //COLOR_SELECTOR
	BIField(SFBOOL,		"insert FALSE", PUSH | DO_NOT_SAVE),
	BIField(SFBOOL,		"delete FALSE", PUSH | DO_NOT_SAVE),
	BIField(SFINT32,	"coords 0", DO_NOT_SAVE),
	BIField(SFFLOAT,	"weight 0", DO_NOT_SAVE),
	BIField(SFBOOL,		"autoWeightDivide TRUE", DO_NOT_SAVE),
	BIField(SFCOLOR,	"color 1 1 1", DO_NOT_DISPLY_NAME),
	BIField(SFBOOL,		"define FALSE", DO_NOT_SAVE),
	BIField(SFBOOL,		"dropper FALSE", DO_NOT_SAVE),
	BIField(SFBOOL,		"rectangle FALSE", DO_NOT_SAVE),
	BIField(SFBOOL,		"fill FALSE", DO_NOT_SAVE),
	BIField(SFBOOL,		"vertex FALSE", DO_NOT_SAVE),
	BIField(SFBOOL,		"triangle FALSE", DO_NOT_SAVE),
	BIField(MFCOLOR,	"customColor []", 0)
);

static const BIProto3 commonSpec_ = BIProto3(
	"CommonSpec", 0, Vec2i(16,16), Vec2i(224,160), //COMMON_SPEC
	BIField(SFBOOL,		"mirror FALSE", DISABLE),
	BIField(MFCOLOR,	"color []", 0),
	BIField(MFSTRING,	"alternative []", 0)
);

static const BIProto4 brush_ = BIProto4(
	"Brush", 0, Vec2i(16,16), Vec2i(224,160), //Brush
	BIField(SFFLOAT,	"fade .5", 0),
	BIField(SFFLOAT,	"opacity 1", 0),
	BIField(SFFLOAT,	"radius .2", 0),
	BIField(SFBOOL,		"smudge FALSE", 0)
);

static const BIProto4 fill_ = BIProto4(
	"Fill", 0, Vec2i(16,16), Vec2i(224,160), //Fill
	BIField(SFBOOL,		"model TRUE", 0),
	BIField(SFBOOL,		"part FALSE", 0),
	BIField(SFBOOL,		"patch FALSE", 0),
	BIField(SFFLOAT,	"tolerance 0", 0)
);

static const BIProto1 colorPicker_ = BIProto1(
	"ColorPicker", 0, Vec2i(16,16), Vec2i(224,160), //ColorPicker
	BIField(SFBOOL,		"blend TRUE", 0)
);

static const BIProto32 settings_ = BIProto32(
	"Settings", 0, Vec2i(16,16), Vec2i(224,160), //Settings
	//BIField(SFBOOL,		"autoOpen FALSE", 0),
	BIField(SFBOOL,		"screenshotWindows FALSE", 0),
	BIField(SFBOOL,		"screenshotMouse FALSE", 0),
	BIField(SFBOOL,		"multiPointLever TRUE", 0),
	BIField(SFBOOL,		"saveGZipped TRUE", 0),
	BIField(SFBOOL,		"invertWindowColor FALSE", 0),
	BIField(SFBOOL,		"showFPS FALSE", 0),
	BIField(SFBOOL,		"showToolBar TRUE", 0),
	BIField(SFBOOL,		"extendNodeBar FALSE", 0),
	BIField(SFBOOL,		"loadColorCodePresets FALSE", PUSH),
	BIField(MFSTRING,	"searchEngine []", DISABLE_LEN),
	BIField(SFBOOL,		"backfaceIgnore TRUE", DISABLE),
	//BIField(SFBOOL,		"perspective FALSE", 0),
	BIField(SFFLOAT,	"incDecRes .01", 0),
	BIField(SFFLOAT,	"incDecResCtrl .001", 0),
	BIField(SFFLOAT,	"backupDelay .4", 0),
	BIField(SFINT32,	"freeCPU 0", 0),
	BIField(SFBOOL,		"record FALSE", DISABLE),
	//BIField(SFBOOL,		"shiftDragTranslate FALSE", DISABLE),//phase out?
	//BIField(SFFLOAT,	"tugSlope 1", 0),
	//BIField(SFFLOAT,	"tugRadius 0", 0),
	BIField(SFBOOL,		"multiPartTug TRUE", 0),
	BIField(SFSTRING,	"htmlBrowser \"\"", 0),
	BIField(SFFLOAT,	"version 0", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(MFSTRING,	"file []", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(SFSTRING,	"importFile \"\"", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(SFBOOL,		"sortByDate FALSE", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(SFVEC2F,	"win3dPos 0 0", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(SFVEC2F,	"win3dSize 1280 720", UNREADABLE | UNWRITEABLE),
	BIField(SFVEC2F,	"sceneTreePos 500 25", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(SFVEC2F,	"sceneTreeSize 300 480", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(SFVEC2F,	"nodeWinPos 400 400", UNREADABLE | UNWRITEABLE | DISABLE | DO_NOT_SAVE),
	BIField(SFVEC2F,	"toolBarPos 0 670", UNREADABLE | UNWRITEABLE | DISABLE),
	BIField(SFFLOAT,	"roundCoordsSave 6", 0),
	BIField(SFBOOL,		"wacomPackets, TRUE", 0),
	BIField(SFNODE,		"palette NULL", 0),
	BIField(MFNODE,		"commonSpec []", 0,	INIT_PROMPT_NODE(promptNode_settings_commonSpec))
);

static const BIProto3 stretchCurve_ = BIProto3(
	"StretchCurve", 0, Vec2i(16,16), Vec2i(224,160), //StretchCurve
	BIField(SFBOOL,	"doIt FALSE", PUSH),
	BIField(SFVEC2F,	"at 1 1", 0),
	BIField(SFVEC2F,	"add 0 .01", 0)
);
static const BIProto12 openNetFile_ = BIProto12(
	"OpenNetFile", 0, Vec2i(24,24), Vec2i(224,160), //OpenNetFile
	BIField(SFBOOL,		"worldList FALSE", 0),
	BIField(SFSTRING,	"worldUrl \"\"", 0),
	BIField(SFSTRING,	"serverUrl \"\"", 0),
	BIField(SFSTRING,	"nickname \"\"", 0),
	BIField(SFSTRING,	"avatarUrl \"\"", 0),
	BIField(SFFLOAT,	"landOffset 0", 0),
	BIField(SFBOOL,		"male FALSE", 0),
	BIField(SFSTRING,	"voice \"\"", 0),
	BIField(SFSTRING,	"alarmFile \"\"", 0),
	BIField(SFBOOL,		"go FALSE", PUSH),
	BIField(SFVEC2F,	"chatPos 175, 83", DISABLE),
	BIField(SFVEC2F,	"chatSize 400, 132", DISABLE)
);
static const BIProto5 demo_ = BIProto5(
	"Demo", 0, Vec2i(16,16), Vec2i(224,160), //Demo
	BIField(SFINT32,	"operation 0", 0),
	BIField(SFBOOL,	"play FALSE", PUSH),
	BIField(SFBOOL,	"stop FALSE", PUSH),
	BIField(SFBOOL,	"express FALSE", 0),
	BIField(SFSTRING,	"status \"\"", 0)
);
static const BIProto7 convertToPatch_ = BIProto7(
	"ConvertToPatch", 0, Vec2i(16,16), Vec2i(160,192), //ConvertToPatch
	BIField(SFINT32,	"control_staves 4{rangeFrom 1 rangeTo 128}", 0),
	BIField(SFINT32,	"control_bands 4{rangeFrom 1 rangeTo 128}", 0),
	BIField(SFINT32,	"control_z_staves 4{rangeFrom 1 rangeTo 128}", 0),
	BIField(SFBOOL,	"stitched TRUE", 0),
	BIField(SFBOOL,	"stem FALSE", 0),
	BIField(SFBOOL,	"set_all_weights_to_1 TRUE", 0),
	BIField(SFBOOL,	"done FALSE", PUSH)
);
static const BIProto2 reduceCapturedPoses_ = BIProto2(
	"ReduceCapturedPoses", 0, Vec2i(16,16), Vec2i(160,192), //ReduceCapturedPoses
	BIField(SFINT32,	"keepCaptureForEvery 2", 0),
	BIField(SFBOOL,	"reduce FALSE", PUSH)
);

static const BIProto5 transformObj_ = BIProto5(
	"TransformObj", 0, Vec2i(16,16), Vec2i(224,160), //TransformObj
	BIField(SFVEC3F,	"pivotPoint 0 0 0", 0),
	BIField(SFROTATION,	"scaleOrientation 0 0 1 0", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0)
);

static const BIProto23 transformCP_ = BIProto23(
	"TransformCP", 0, Vec2i(16,16), Vec2i(224,160), //TransformCP
	BIField(SFBOOL,		"xTug TRUE", DISABLE),
	BIField(SFBOOL,		"yTug TRUE", DISABLE),
	BIField(SFBOOL,		"zTug TRUE", DISABLE),
	BIField(SFINT32,	"leverSelection 3", 0),

	BIField(SFBOOL,		"leverNeutral FALSE", DISABLE),
	BIField(SFBOOL,		"leverRotate FALSE", DISABLE),
	BIField(SFBOOL,		"leverStretch FALSE", DISABLE),
	BIField(SFBOOL,		"leverRotateAndStretch FALSE", DISABLE),
	BIField(SFBOOL,		"leverScale FALSE", DISABLE),
	BIField(SFBOOL,		"leverTranslate FALSE", DISABLE),


	BIField(SFBOOL,		"selectObject FALSE", DISABLE),
	//BIField(SFBOOL,		"selectRing FALSE", 0),
	//BIField(SFBOOL,		"selectRings FALSE", 0),
	BIField(SFBOOL,		"sameScreenPosition FALSE", DISABLE),
	BIField(SFBOOL,		"rotAxisX FALSE", 0),
	BIField(SFBOOL,		"rotAxisY FALSE", 0),
	BIField(SFBOOL,		"rotAxisZ FALSE", 0),
	BIField(SFVEC3F,	"pivotPoint 0 0 0", 0),
	BIField(MFROTATION,	"scaleOrientation {beginGroup TRUE}", 0),
	BIField(MFVEC3F,	"scale []{initElVal \"1 1 1\"}", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0),
	BIField(SFINT32,	"leverIdx 0", DISABLE),
    BIField(SFFLOAT,	"leverAngle 0", DISABLE),
	BIField(MFFLOAT,	"leverDistance[]", DISABLE)
);

static const BIProto2 animLever_ = BIProto2(
	"AnimLever", 0, Vec2i(16,16), Vec2i(224,160), //TransformCP
    BIField(SFFLOAT,	"leverAngle 0", DISABLE),
	BIField(MFFLOAT,	"leverDistance[]", DISABLE)
);

static const BIProto18 anim_ = BIProto18(
	"Anim", "children", Vec2i(16,16), Vec2i(32,192), //Anim
	BIField(SFFLOAT,	"slider 0", DO_NOT_OUTPUT | DO_NOT_DISPLY_NAME | DO_NOT_RESIZE, "drag slider to move to a point in time"),
	BIField(SFINT32,	"timeLine 0", DO_NOT_OUTPUT | UNREADABLE | UNWRITEABLE | DO_NOT_DISPLY_NAME | DO_NOT_DISPLY_VALUE, "click time-line to move to a point in time"),
	BIField(SFINT32,	"select 0", DO_NOT_OUTPUT | UNREADABLE | UNWRITEABLE | DO_NOT_DISPLY_NAME | DO_NOT_DISPLY_VALUE, "drag to select section"),
	BIField(SFBOOL,	"play FALSE",  PUSH | DO_NOT_DISPLY_NAME, "play/stop"),
	BIField(SFBOOL,	"pause TRUE", PUSH | DO_NOT_DISPLY_NAME, "pause/resume"),
	BIField(SFBOOL,	"loop FALSE", PUSH | DO_NOT_DISPLY_NAME, "loop/play once"),
	BIField(SFTIME,	"zoom 4", DO_NOT_DISPLY_NAME | DO_NOT_RESIZE | DO_NOT_DISPLY_VALUE, "zoom time line"),
	BIField(SFTIME,	"period 1", DO_NOT_DISPLY_NAME | DO_NOT_RESIZE | DO_NOT_DISPLY_VALUE, "add second to timeline"),
	BIField(SFINT32,	"stepsPerSecond 25{rangeFrom 1 rangeTo 70}", DO_NOT_OUTPUT | DO_NOT_DISPLY_NAME | DO_NOT_RESIZE, "steps per second"),
	BIField(SFFLOAT,	"speed 1", DO_NOT_OUTPUT | DO_NOT_DISPLY_NAME | DO_NOT_RESIZE, "speed"),
	
	BIField(SFFLOAT,	"playFrom FALSE", DO_NOT_DISPLY_NAME | DO_NOT_RESIZE | DO_NOT_DISPLY_VALUE, "mark where to play from"),
	BIField(SFBOOL,	"copy FALSE", PUSH | DO_NOT_DISPLY_NAME, "copy"),
	BIField(SFBOOL,	"paste FALSE", PUSH | DO_NOT_DISPLY_NAME, "paste"),
	BIField(SFBOOL,	"remove FALSE", PUSH | DO_NOT_DISPLY_NAME, "delete"),
	BIField(SFNODE,	"video NULL", DO_NOT_OUTPUT,	INIT_PROMPT_NODE(promptNode_anim_video)),
	BIField(MFNODE,	"soundTrack []", DO_NOT_OUTPUT,	INIT_PROMPT_NODE(promptNode_anim_soundTrack)),
	BIField(MFNODE,	"children []", 0,	INIT_PROMPT_NODE(promptNode_anim_children)),
	BIField(SFNODE,	"onFrame NULL", DO_NOT_OUTPUT,	INIT_PROMPT_NODE(promptNode_anim_onFrame))
);

static const BIProto19 timelineFilter_ = BIProto19(
	"TimelineFilter", "children", Vec2i(16,16), Vec2i(224,160), //AnimEditOptions
	BIField(SFBOOL,	"morph TRUE", 0),
	BIField(SFBOOL,	"rotation TRUE", 0),
	BIField(SFBOOL,	"translation TRUE", 0),
	BIField(SFBOOL,	"nik TRUE", 0),
	BIField(SFBOOL,	"posCompensation TRUE", 0),
	BIField(SFBOOL,	"fka TRUE", 0),
	BIField(SFBOOL,	"fkb TRUE", 0),
	BIField(SFBOOL,	"live TRUE", 0),
	BIField(SFBOOL,	"zoom TRUE", 0),
	BIField(SFBOOL,	"fieldOfView TRUE", 0),
	BIField(SFBOOL,	"orientation TRUE", 0),
	BIField(SFBOOL,	"lookAt TRUE", 0),
	BIField(SFBOOL,	"up TRUE", 0),
	BIField(SFBOOL,	"position TRUE", 0),
	BIField(SFBOOL,	"tickMorphFieldOnly FALSE", PUSH),
	BIField(SFBOOL,	"tickPartFieldsOnly FALSE", PUSH),
	BIField(SFBOOL,	"tickViewpointFieldsOnly FALSE", PUSH),
	BIField(SFBOOL,	"selectAllFields FALSE", PUSH),
	BIField(SFBOOL,	"ok FALSE", PUSH)
);


static const BIProto16 pasteOptions_ = BIProto16(
	"PasteOptions", "children", Vec2i(16,16), Vec2i(224,160), //PasteSection
	BIField(SFBOOL,	"filter FALSE", PUSH),
	BIField(SFBOOL,	"insert FALSE", 0),
	BIField(SFBOOL,	"beginEdge TRUE", 0),
	BIField(SFBOOL,	"endEdge TRUE", 0),
	BIField(SFINT32, "steps 0", 0),
	BIField(SFBOOL,	"quantize TRUE", 0),
	BIField(SFBOOL,	"transform FALSE", 0),
	BIField(SFBOOL,	"calcRot FALSE", 0),
	BIField(SFVEC3F,	"pivotPoint 0 0 0", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotate 0 0 1 0", 0),
	BIField(SFVEC3F,	"translate 0 0 0", 0),
	BIField(SFBOOL,	"mirror FALSE", 0),
	BIField(SFBOOL,	"curve FALSE", 0),
	BIField(SFFLOAT, "curveDistance 0", 0),
	BIField(SFBOOL,	"ok FALSE", PUSH)
);

static const BIProto5 deleteOptions_ = BIProto5(
	"DeleteOptions", "children", Vec2i(16,16), Vec2i(224,160), //DeleteSection
	BIField(SFBOOL,	"filter FALSE", PUSH),
	BIField(SFBOOL,	"time FALSE", 0),
	BIField(SFBOOL,	"beginEdge TRUE", 0),
	BIField(SFBOOL,	"endEdge TRUE", 0),
	BIField(SFBOOL,	"ok FALSE", PUSH)
);

static const BIProto3 region_ = BIProto3(
	"Region", "region", Vec2i(16,16), Vec2i(224,160), //Region
	BIField(SFVEC2F,	"borderPoint 0 0", 0),
	BIField(SFINT32,	"borderPoints 0", 0),
	BIField(SFINT32,	"referencedBorderPoints 0", 0)
);

static const BIProto2 seed_ = BIProto2(
	"Seed", "seed", Vec2i(16,16), Vec2i(224,160), //Seed
	BIField(SFINT32,	"number_of_strands 10", 0),
	BIField(SFBOOL,	"ok FALSE", PUSH)
);

static const BIProto2 thinner_ = BIProto2(
	"Thinner", "thinner", Vec2i(16,16), Vec2i(224,160), //Thinner
	BIField(SFINT32,	"number_of_strands 10", 0),
	BIField(SFBOOL,	"ok FALSE", PUSH)
);

static const BIProto2 insertLock_ = BIProto2(
	"InsertLock", "insertLock", Vec2i(16,16), Vec2i(224,160),
	BIField(SFINT32,	"insertions 1{rangeFrom 1 rangeTo 128}", 0),
	BIField(SFBOOL,	"ok FALSE", PUSH)
);

static const BIProto6 extrude_ = BIProto6(
	"Extrude", "extrude", Vec2i(16,16), Vec2i(224,160), //Extrude
	BIField(SFFLOAT,	"distance 0", 0),
	BIField(SFINT32,	"insertions 1{rangeFrom 1 rangeTo 128}", 0),
	BIField(SFBOOL,	"cap FALSE", 0),
	BIField(SFBOOL,	"hole FALSE", 0),
	BIField(SFBOOL,	"spiderWebCenter TRUE", 0),
	BIField(SFBOOL,	"done FALSE", PUSH)
);

static const BIProto1 insertComplete_ = BIProto1(
	"insertComplete", "insertComplete", Vec2i(16,16), Vec2i(224,160), //InsertComplete
	BIField(SFBOOL,	"done FALSE", PUSH)
	//BIField(SFBOOL,	"loop FALSE", 0)
);

static const BIProto3 cutComplete_ = BIProto3(
	"cutComplete", "cutComplete", Vec2i(16,16), Vec2i(224,160), //InsertComplete
	BIField(SFBOOL,	"done FALSE", PUSH),
	BIField(SFBOOL,	"unstitch FALSE", 0),
	BIField(SFBOOL,	"continue FALSE", 0)
);

static const BIProto1 addLathe_ = BIProto1(
	"AddLathe", "children", Vec2i(16,16), Vec2i(224,160), //AddLathe
	BIField(SFBOOL,	"done FALSE", PUSH)
);

static const BIProto3 addNurbsLine_ = BIProto3(
	"AddNurbsLine", "children", Vec2i(16,16), Vec2i(224,160), //AddNurbsLine
	BIField(SFBOOL,	"palette FALSE", PUSH),
	BIField(SFBOOL,	"done FALSE", PUSH),
	BIField(SFINT32,	"control_points 5{rangeFrom 1 rangeTo 128}", 0)
);

static const BIProto11 drawNurbsLine_ = BIProto11(
	"DrawNurbsLine", "children", Vec2i(16,16), Vec2i(224,160), //AddNurbsLine
	BIField(SFBOOL,	"palette FALSE", PUSH),
	BIField(SFBOOL,	"done FALSE", PUSH),
	BIField(SFINT32,"control_points 6{rangeFrom 3 rangeTo 512}", 0),
	BIField(SFBOOL,	"thickLine FALSE", 0),
	BIField(SFBOOL,	"colorPressure FALSE", 0),
	BIField(SFFLOAT,"colorMult 1", 0),
	BIField(SFBOOL,	"thickPressure FALSE", 0),
	BIField(SFFLOAT,"thickness .05", 0),
	BIField(SFFLOAT,"thickMult 0", 1),
	BIField(SFBOOL,	"loop FALSE", PUSH),
	BIField(SFBOOL,	"evolve FALSE", PUSH | DO_NOT_SAVE)
);

static const BIProto1 addBone_ = BIProto1(
	"AddBone", "children", Vec2i(16,16), Vec2i(224,160), //AddBone
	BIField(SFBOOL,	"done FALSE", PUSH)
);

static const BIProto17 grid_ = BIProto17(
	"Grid", "children", Vec2i(16,16), Vec2i(224,160), //Grid
	BIField(SFFLOAT, "scale 1", 0),
	BIField(SFINT32,	"squares 16{rangeFrom 1}", 0),
	BIField(SFINT32,	"subSquares 2", 0),
	BIField(SFBOOL,	"xzPlane FALSE", 0),
	BIField(SFBOOL,	"xyPlane FALSE", 0),
	BIField(SFBOOL,	"yzPlane FALSE", 0),
	BIField(SFROTATION, "orientation 0 0 1 0", 0),
	BIField(SFVEC3F, "position 0 0 0", 0),
	BIField(SFBOOL, "polar FALSE", 0),
	BIField(SFINT32, "sections 16", 0),
	BIField(SFINT32, "subSections 2", 0),
	BIField(SFBOOL, "snapLever FALSE", 0),
	BIField(SFBOOL,	"snap FALSE", 0),
	BIField(SFBOOL, "xDim TRUE", 0),
	BIField(SFBOOL, "yDim TRUE", 0),
	BIField(SFBOOL, "zDim TRUE", 0),
	BIField(SFBOOL,	"show TRUE", 0)
);

static const BIProto5 duplicate_ = BIProto5(
	"Duplicate", "children", Vec2i(16,16), Vec2i(224,160), //Duplicate
	BIField(SFBOOL,	"run FALSE", PUSH),
	BIField(SFINT32,	"iterations 1", 0),
	BIField(SFVEC3F,	"scale 1 1 1", 0),
	BIField(SFROTATION,	"rotation 0 0 1 0", 0),
	BIField(SFVEC3F,	"translation 0 0 0", 0)
);

static const BIProto2 insideOutPart_ = BIProto2(
	"InsideOutPart", "children", Vec2i(16,16), Vec2i(224,160), //InsideOutPart
	BIField(SFBOOL,	"add FALSE", 0),
	BIField(SFBOOL,	"ok FALSE", PUSH)
);

static const BIProto7 exportMovie_ = BIProto7(
	"ExportMovie", 0, Vec2i(16,16), Vec2i(224,160), //ExportMovie
	BIField(SFBOOL,	"output FALSE", PUSH),
	BIField(SFBOOL,	"openOutputFolder FALSE", PUSH),
	BIField(SFBOOL,	"povRay FALSE", 0),

	BIField(SFBOOL,	"avi FALSE", 0),
	BIField(SFBOOL,	"mpg FALSE", 0),
	BIField(SFBOOL,	"mp4 FALSE", 0),
	BIField(SFBOOL,	"flv FALSE", 0)
);

static const BIProto14 exportVrmlX3d_ = BIProto14(
	"ExportVrmlX3d", 0, Vec2i(16,16), Vec2i(224,160), //ExportVrmlX3d
	BIField(SFBOOL,	"output FALSE", PUSH),
	BIField(SFBOOL,	"openOutputFolder FALSE", PUSH),
	BIField(SFBOOL,	"gZip TRUE", 0),
	BIField(SFBOOL,	"vrml TRUE", 0),
	BIField(SFBOOL,	"x3dVrmlClassic FALSE", 0),
	BIField(SFBOOL,	"x3dXml FALSE", 0),

	BIField(SFBOOL,	"genHAnimNodes FALSE", 0),
	BIField(SFBOOL,	"genIdxTriSetNodes FALSE", 0),
	BIField(SFBOOL,	"genContactSBVT TRUE", 0),
	BIField(SFBOOL,	"genNormals TRUE", 0),
	BIField(SFBOOL,	"genColorIndex FALSE", 0),
	BIField(SFBOOL,	"segmentedParts FALSE", 0),
	BIField(SFFLOAT,"roundCoords 3", 0),
	BIField(SFBOOL,	"scriptAsRoutes FALSE", 0)
);

static const BIProto6 ringletGenerator_ = BIProto6(
	"RingletGenerator", "ringletGenerator", Vec2i(16,16), Vec2i(112,192), //RevolutionGenerator
	BIField(SFVEC2F,	"size .01, .03", 0),
	BIField(SFFLOAT,	"radius .1", 0),
	BIField(SFBOOL,		"clockwise FALSE", 0),
	BIField(SFFLOAT,	"distance .5", 0),
	BIField(SFINT32,	"revolutions 2", 0),
	BIField(SFINT32,	"insertions 4", 0)
);

static const BIProto6 robotSave_ = BIProto6(//toggleAccumulate
	"RobotSave", "robotSave", Vec2i(16,16), Vec2i(112,192), //robotSave
	BIField(SFINT32,	"mode 0", 0),
	BIField(SFINT32,	"selSmls 0", 0),
	BIField(MFINT32,	"pointList []", 0),
	BIField(MFVEC3F,	"beginDragPos []", 0),
	BIField(SFNODE,	"transformCP NULL", DISABLE,	0),
	BIField(SFNODE,	"animLever NULL", DISABLE,	0)
);



const BIProto* builtInProtoList[]={
	&mesh_,
    &scene_,
    &seamless_,
    &colorEffect_,
    &textureEffect_,
    &monoColorEffect_,
    &part_,
    &immediatePart_,
	&groupBuild_,
	&stem_,
	&textureMapper_,
    &colorSweep_,
	&colorMapper_,
    &surfaceGenerator_,
    &cCLathe_,
	&nurbsPatch_,
	&colorPatch_,
    &shape_,
    &appearance_,
    &material_,
    &indexedLineSet_,
    &coordinate_,
    &color_,

    &protoDeclare_,
    &protoInterface_,
    &protoBody_,
    &transform_,


	&nullNode_,
	&avatarSocket_,
	&indexedFaceSet_,
	&imageTexture_,
	&navigationInfo_,
	&group_,
	&noOutputGroup_,
	&gestureGroup_,
	&mobility_,
	&cCBend_,
	&stem2_,
	&polymap_,
	&branchJoin_,
	&generator_,
	&surfaceTransform_,
	&place_,
	&directionalLight_,
	&cCScale_,
	&cCTranslate_,
	&cCRingScale_,
	&bezierLathe_,
	&eCLathe_,
	&eCScale_,
	&eCShift_,
	&eCBend_,
	&eCSyncBend_,
	&eCTwist_,
	&background_,
	&sCGroupBuild_,
	&joinCloseVertex_,
	&centerJoint_,
	&delPoly_,
	&joinVertex_,
	&copyPart_,
	&viewpoint_,
	&transferBCyl_,
	&transferOwnership_,
	&transferVertex_,
	&normal_,
	&textureCoordinate_,
	&sineTug_,
	&simulSineTug_,
	&genPoly_,
	&script_,
	&branchJoinNegX_,
	&branchJoinPosX_,
	&branchJoinNegY_,
	&branchJoinPosY_,
	&branchJoinNegZ_,
	&branchJoinPosZ_,
	&projectOntoTri_,
	&evolveCoord_,
	&function_,
	&memberFunction_,
	&codeLine_,
	&morphPart_,
	&morpher_,
	&textureTransform_,
	&protoInstance_,
	&movie_,
	&video_,
	&audio_,
	&anim_,
	&timeSensor_,
	&orientationInterpolator_,
	&positionInterpolator_,
	&nurbsTrack_,
	&scalarInterpolator_,
	&posCompensator_,
	&nikInterpolator_,
	&collision_,
	&colorInterpolator_,
	&coordinateInterpolator_,
	&normalInterpolator_,
	&modelSequence_,
	&modelMorpher_,
	&worldInfo_,
	&touchSensor_,
	&cCPath_,
	&bspTree_,
	&inline_,
	&fog_,
	&fog2_,
	&anchor_,
	&audioClip_,
	&billboard_,
	&box_,
	&cone_,
	&cylinder_,
	&cylinderSensor_,
	&elevationGrid_,
	&extrusion_,
	&fontStyle_,
	&nurbsSurface_,
	&lOD_,
	&movieTexture_,
	&pixelTexture_,
	&planeSensor_,
	&pointLight_,
	&pointSet_,
	&proximitySensor_,
	&sound_,
	&sphere_,
	&sphereSensor_,
	&spotLight_,
	&switch_,
	&text_,
	&visibilitySensor_,
	&exitBuild_,
	&splitTri_,
	&swapTriDivide_,
	&placeTexture_,
	&hAnimHumanoid_,
	&hAnimJoint_,
	&hAnimDisplacer_,
	&hAnimSegment_,
	&hAnimSite_,
	&booleanFilter_,
	&CameraSelector_,
	&booleanToggle_,
	&booleanTrigger_,
	&integerSequencer_,
	&integerTrigger_,
	&timeTrigger_,
	&staticGroup_,
	&floatAccelerator_,
	&rotAccelerator_,
	&copyToMorph_,
	&relateMorpher_,
	&delPlane_,
	&delSphere_,
	&radialTug_,
	&tapeMeasure_,
	&posAccelerator_,
	&animByAnkle_,
	&animByWrist_,
	&rotFilter_,
	&rotMult_,
	&multiUser_,
	&indexedTriangleSet_,
	&polyline2D_,
	&sphereBuild_,
	&cylinderBuild_,
	&coneBuild_,
	&torusBuild_,
	&boxBuild_,
	&rectangleBuild_,
	&bezierLatheBuild_,
	&mirrorEdit_,
	&mirrorGroup_,
	&polyToNurbs_,
	&ringBuild_,
	&bezierScale_,
	&bezierCircumScale_,
	&bezierPatch_,
	&feedback_,
	&povLight_,
	&povFog_,
	&bezierShift_,
	&bezierTwist_,
	&bezierTape_,
	&bezierBend_,
	&nurbsLine_,
	&nurbsLathe_,
	&bendyLathe_,
	&fuseSurface_,
	&subdivision_,
	&oscillator_,
	&envelope_,
	&amplitudeModulator_,
	&frequencyModulator_,
	&chords_,
	&larynx_,
	&phrase_,
	&harmonic_,
	&harmonicPatch_,
	&sample_,
	&waveToPoly_,
	&midiFormant_,
	&filter_,
	&formant_,
	&lowPass_,
	&multiFormant_,
	&track_,
	&composition_,
	&cPMorpher_,
	&hair_,
	&strand_,
	&lock_,
	&g_S_Bool_,
	&g_S_Color_,
	&g_S_Float_,
	&g_S_Image_,
	&g_S_Int32_,
	&g_NodePtr_,
	&g_S_Rotation_,
	&g_S_String_,
	&g_S_Time_,
	&g_S_Vec2f_,
	&g_S_Vec3f_,
	&g_M_Bool_,
	&g_M_Color_,
	&g_M_Float_,
	&g_M_Image_,
	&g_M_Int32_,
	&g_M_Node_,
	&g_M_Rotation_,
	&g_M_String_,
	&g_M_Time_,
	&g_M_Vec2f_,
	&g_M_Vec3f_,
	&file_,
	&floorHeight_,
	&message1Button_,
	&questionMessage3Button_,
	&recentFiles_,
	&radicalPatch_,
	&radial_Tug_,
	&palette_,
	&commonSpec_,
	&brush_,
	&fill_,
	&colorPicker_,
	&settings_,
	&stretchCurve_,
	&openNetFile_,
	&demo_,
	&convertToPatch_,
	&reduceCapturedPoses_,
	&transformObj_,
	&transformCP_,
	&animLever_,
	&timelineFilter_,
	&pasteOptions_,
	&deleteOptions_,
	&region_,
	&seed_,
	&thinner_,
	&insertLock_,
	&extrude_,
	&insertComplete_,
	&cutComplete_,
	&addLathe_,
	&addNurbsLine_,
	&drawNurbsLine_,
	&addBone_,
	&grid_,
	&duplicate_,
	&insideOutPart_,
	&exportMovie_,
	&exportVrmlX3d_,
	&ringletGenerator_,
	&robotSave_,
};

int numOfBuiltInNodes = sizeof(builtInProtoList)/sizeof(const BIProto**);


void DynamicMField::read2(Field *&field, Properties& sprop, bool& wasMF, const BIField& bi){
	if(bi.type==MFBOOL){
		wasMF=true;
		field=new MFBool(M_Bool());
	}
	else if(bi.type==MFCOLOR){
		wasMF=true;
		field=new MFColor(M_Color3f());
	}
	else if(bi.type==MFFLOAT){
		wasMF=true;
		field=new MFFloat(M_Float());
	}
	else if(bi.type==MFINT32){
		wasMF=true;
		field=new MFInt32(M_Int());
	}
	else if(bi.type==MFNODE){
		wasMF=true;
		field=new MFNode(M_Node());
	}
	else if(bi.type==MFROTATION){
		wasMF=true;
		field=new MFRotation(M_Rotation());
	}
	else if(bi.type==MFSTRING){
		wasMF=true;
		field=new MFString(M_String());
	}
	else if(bi.type==MFDOUBLE){
		wasMF=true;
		field=new MFTime(M_Double());
	}
	else if(bi.type==MFVEC2F){
		wasMF=true;
		field=new MFVec2f(M_Vec2f());
	}
	else if(bi.type==MFVEC3F){
		wasMF=true;
		field=new MFVec3f(M_Vec3f());
	}
	else if(bi.type==SFBOOL){
		field=new SFBool(false);
	}
	else if(bi.type==SFCOLOR){
		field=new SFColor(Color3f());
	}
	else if(bi.type==SFFLOAT){
		field=new SFFloat(0);
	}
	else if(bi.type==SFINT32){
		field=new SFInt32(0);
	}
	else if(bi.type==SFNODE){
		field=new SFNode(NodePtr());
	}
	else if(bi.type==SFROTATION){
		field=new SFRotation(Rotation());
	}
	else if(bi.type==SFSTRING){
		field=new SFString(String());
	}
	else if(bi.type==SFTIME){
		field=new SFTime(0);
	}
	else if(bi.type==SFVEC2F){
		field=new SFVec2f(Vec2f());
	}
	else if(bi.type==SFVEC3F){
		field=new SFVec3f(Vec3f());
	}
	else throw;

}
void DynamicMField::read(M_Properties& properties, const BIProto& binode, String& str){
	bool groupHasBegan=false;
	int groupCntr=-1;
	for(int k=0; k<binode.numOf; k++){
		Properties sprop;
		bool wasMF=false;
		const BIField* biPtr=binode.bifieldRef;
		const BIField& bi=biPtr[k];
		sprop.flags=bi.flags;
		sprop.promptNode.length=bi.initPromptNode.numOf;
		for(int j=0; j<bi.initPromptNode.numOf; j++){
			sprop.promptNode[j]=bi.initPromptNode.list[j];
		}
		str=bi.str;
		int i=0;
		sprop.name=str.getWord(i);

		Field* field=0;
		sprop.tip=bi.tip;

		read2(field, sprop, wasMF, bi);

		if(sprop.initializeOnly() || sprop.inputOutput()){
			field->read(str,i);
		}
		*this+=field;

		bool beginGroup=false;
		bool endGroup=false;
		str.stb(i);
		if(i<str.length){
			sprop.read(*field, str, i, beginGroup, endGroup);
		}
		if( wasMF&&(sprop.isReadable()^sprop.isWritable())==false&&field->getType()!=MFNODE){
			if(groupHasBegan==false&&beginGroup==false){
				sprop.groupIndex=++groupCntr;
			}
			else if(beginGroup){
				sprop.groupIndex=++groupCntr;
				groupHasBegan=true;
			}
			else if(groupHasBegan){
				sprop.groupIndex=groupCntr;
			}
			if(endGroup){
				groupHasBegan=false;
			}
		}
		properties+=sprop;
	}
}



M_ProtoInterface g_lookupProtoInterface;
extern String g_protoInterfaceListStr;
extern String g_protoInterfaceTips;
extern String g_protoInterfaceNodePromptStr;

void initLookup(){
	String str;
	nodeLookUpStr.length=0;
	for(int i=0; i < numOfBuiltInNodes; i++){
		nodeLookUpStr += builtInProtoList[i]->ntype;
	}
	g_lookupProtoInterface.length = numOfBuiltInNodes;
	containerFieldLookUpStr.length = numOfBuiltInNodes;

	for(int k=0; k < numOfBuiltInNodes; k++){
		ProtoInterface* protoInterface = new ProtoInterface;
		g_lookupProtoInterface[k] = protoInterface;
		if(builtInProtoList[k]->containerField){
			containerFieldLookUpStr[k] = builtInProtoList[k]->containerField;
		}
		protoInterface->read(protoInterface->properties, *(builtInProtoList[k]), str);
	}
	g_userProtoBegin = g_lookupProtoInterface.length;
}
