/* Seamless3d

Copyright (C) 2001-2017 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#include <windows.h>


#include "seamless3d.h"
#include "script.h"
#include "gui.h"
#include "scene_tree.h"


#define numOfTypes 22

bool controlKeyState();
bool altKeyState();
bool shiftKeyState();

bool getKeyWord(Function& f, M_Node& clParent, int& codeLineIdx, CodeLine& cl, const String& str, int& i, int& getKeyWord);

M_<Ptr<Function> > g_functionList;
M_Local g_local2ndCompile;
M_ConstData g_const2ndCompile;
M_Asm2CppAddy g_asm2CppAddy;
M_<Variable*> g_postIncList;
M_<Variable*> g_postDecList;
M_Int breakStack;
M_Int Function::continueStack;

extern AnimCurve g_animCurve;
extern AnimCopy g_animCopy;
extern Ptr<TimelineFilter> g_timelineFilter;
extern Ptr<PasteOptions> g_pasteOptions;
extern Ptr<DeleteOptions> g_deleteOptions;

bool Function::reset_m_word=true;

extern bool g_dragActuallyHappened;
extern ScenePtr g_load_scene;

int g_callOffsetCntr=0;


const int sizeTypeLookUp[VARABLETYPE_LOOKUPLEN]={
1,
sizeof(Color3f),
sizeof(float),
sizeof(Image),
sizeof(int),
sizeof(NodePtr),
sizeof(Rotation),
sizeof(String),
sizeof(double),
sizeof(Vec2f),
sizeof(Vec3f),
sizeof(M_Bool),
sizeof(M_Color3f),
sizeof(M_Float),
sizeof(M_Image),
sizeof(M_Int),
sizeof(M_Node),
sizeof(M_Rotation),
sizeof(M_String),
sizeof(M_Double),
sizeof(M_Vec2f),
sizeof(M_Vec3f),
0,
};
int getValueSize(Variable* v){
	return sizeTypeLookUp[v->getValueType()];
}
unsigned char writeElement_doOp_M_Int32_ref[]={
0x8B, 0xBC, 0x24, 0,0,0,0,	//mov         edi,[esp+12] 
0x8B, 0x16,					//mov         edx,dword ptr [esi] 
0x83, 0xC6, sizeof(int),	//add         esi,sizeof(int)
0x8B, 0x1E,					//mov         ebx,dword ptr [esi] 
0x83, 0xC6, sizeof(int),	//add         esi,4 
0x39, 0x1F,					//cmp         dword ptr [edi+bufLen],ebx 
0x77, 0x06,					//ja          here
0x50,						//push        eax  
0xE8, 0,0,0,0,				//call        Exception
							//here:
0x8B, 0x4F, 8,				//mov         ecx,dword ptr [edi+pointer] 
0x89, 0x14, 0x99,			//mov         dword ptr [ecx+ebx*4],edx 
};
unsigned char writeElement_doOp_M_Vec2f_ref[]={
0x8B, 0xBC, 0x24, 0,0,0,0,	//mov         edi,[esp+12] 
0x8B, 0x4E, sizeof(Vec2f),	//mov         ecx,dword ptr [esi+sizeof(Vec2f)] 
0x39, 0x0F,					//cmp         dword ptr [edi],ecx 
0x77, 0x06,					//ja          6
0x50,						//push        eax  
0xE8, 0,0,0,0,				//call        Exception
							//here:
0xB8, sizeof(Vec2f),0,0,0,	//mov         eax,sizeof(Vec2f)
0xF7, 0xE1,					//mul         eax,ecx 
0x03, 0x47, 8,				//add         eax,dword ptr [edi+pointer]

0x8B, 0x0E,					//mov         ecx,dword ptr [esi] 
0x83, 0xC6, sizeof(float),	//add         esi,4 
0x89, 0x08,					//mov         dword ptr [eax],ecx 
0x8B, 0x0E,					//mov         ecx,dword ptr [esi] 
0x83, 0xC6, sizeof(float)+sizeof(int),	  //add         esi,8 
0x89, 0x48, sizeof(float),	//mov         dword ptr [eax+4],ecx 
};
unsigned char writeElement_doOp_M_Vec3f_ref[]={
0x8B, 0xBC, 0x24, 0,0,0,0,	//mov         edi,[esp+12] 
0x8B, 0x4E, sizeof(Vec3f),	//mov         ecx,dword ptr [esi+sizeof(Vec3f)] 
0x39, 0x0F,					//cmp         dword ptr [edi],ecx 
0x77, 0x06,					//ja          6
0x50,						//push        eax  
0xE8, 0,0,0,0,				//call        Exception
							//here:
0xB8, sizeof(Vec3f),0,0,0,	//mov         eax,sizeof(Vec3f)
0xF7, 0xE1,					//mul         eax,ecx 
0x03, 0x47, 8,				//add         eax,dword ptr [edi+pointer]

0x8B, 0x0E,					//mov         ecx,dword ptr [esi] 
0x83, 0xC6, sizeof(float),	//add         esi,4 
0x89, 0x08,					//mov         dword ptr [eax],ecx 
0x8B, 0x0E,					//mov         ecx,dword ptr [esi] 
0x83, 0xC6, sizeof(float),	//add         esi,4 
0x89, 0x48, sizeof(float),	//mov         dword ptr [eax+4],ecx 
0x8B, 0x0E,					//mov         ecx,dword ptr [esi] 
0x83, 0xC6, sizeof(float)+sizeof(int),		//add         esi,8 
0x89, 0x48, sizeof(float)*2 //mov         dword ptr [eax+8],ecx 
};

unsigned char writeElement_doOp_M_Rotation_ref[]={
0x8B, 0xBC, 0x24, 0,0,0,0,	//mov         edi,[esp+12] 
0x8B, 0x4E, sizeof(Rotation),//mov        ecx,dword ptr [esi+sizeof(Rotation)] 
0x39, 0x0F,					//cmp         dword ptr [edi],ecx 
0x77, 0x06,					//ja          6
0x50,						//push        eax  
0xE8, 0,0,0,0,				//call        Exception
							//here:
0xB8, sizeof(Rotation),0,0,0,	//mov         eax,sizeof(Rotation)
0xF7, 0xE1,					//mul         eax,ecx 
0x03, 0x47, 8,				//add         eax,dword ptr [edi+pointer]

0x8B, 0x0E,					//mov ecx,[esi] ;element source data
0x83, 0xC6, sizeof(float),	//add esi,sizeof float
0x89, 0x08,					//mov	[eax].x, ecx

0x8B, 0x0E,					//mov ecx,[esi] ;element source data
0x83, 0xC6, sizeof(float),	//add esi,sizeof float
0x89, 0x48, sizeof(float)*1,//mov	[eax].y, ecx

0x8B, 0x0E,					//mov ecx,[esi] ;element source data
0x83, 0xC6, sizeof(float),	//add esi,sizeof float
0x89, 0x48, sizeof(float)*2,//mov	[eax].z, ecx

0x8B, 0x0E,					//mov ecx,[esi] ;element source data
0x83, 0xC6, sizeof(float)+sizeof(int),//add esi,sizeof float + sizeof int
0x89, 0x48, sizeof(float)*3 //mov	[eax].angle, ecx
};

unsigned char writeElement_morpherMorph_doOp_void_ref[]={
0x68, 0, 0, 0, 0,			//push        7FFFFFFFh 
0xFF, 0x36,					//push        dword ptr [esi] 
0x68, 0, 0, 0, 0,			//push        7FFFFFFFh 
0xE8, 0, 0, 0, 0,			//call        0E3EECAA 
0x83, 0xC4, 0x0C,			//add         esp,0Ch 
0x83, 0xC6, sizeof(float),	//add         esi,4 
};


#define	copyMachineCodeM(M)copyMachineCode(M, sizeof(M))
#define	copyMachineCodeFM(M)f.copyMachineCode(M, sizeof(M))

template <> inline void Function::copyCode_writeElement_ref_insert<M_Bool>(){copyMachineCodeM(writeElement_doOp_M_Int32_ref);}
template <> inline void Function::copyCode_writeElement_ref_insert<M_Color3f>(){copyMachineCodeM(writeElement_doOp_M_Vec3f_ref);}
template <> inline void Function::copyCode_writeElement_ref_insert<M_Float>(){copyMachineCodeM(writeElement_doOp_M_Int32_ref);}
template <> inline void Function::copyCode_writeElement_ref_insert<M_Image>(){ throw;}
template <> inline void Function::copyCode_writeElement_ref_insert<M_Int>(){copyMachineCodeM(writeElement_doOp_M_Int32_ref);}
template <> inline void Function::copyCode_writeElement_ref_insert<M_Node>(){ throw;}
template <> inline void Function::copyCode_writeElement_ref_insert<M_Rotation>(){copyMachineCodeM(writeElement_doOp_M_Rotation_ref);}
template <> inline void Function::copyCode_writeElement_ref_insert<M_String>(){ throw;}
template <> inline void Function::copyCode_writeElement_ref_insert<M_Double>(){copyMachineCodeM(writeElement_doOp_M_Vec2f_ref);}
template <> inline void Function::copyCode_writeElement_ref_insert<M_Vec2f>(){copyMachineCodeM(writeElement_doOp_M_Vec2f_ref);}
template <> inline void Function::copyCode_writeElement_ref_insert<M_Vec3f>(){copyMachineCodeM(writeElement_doOp_M_Vec3f_ref);}


unsigned char copyGlobalLengthToScratchPad_doOp[]={
0xBF, 0,0,0,0,	//mov edi,0 
0x8B, 0x47, 4,	//mov eax,dword ptr [edi+4] 
0x83, 0xEE, 4,	//sub esi,4 
0x89, 0x06		//mov dword ptr [esi],eax 
};
unsigned char copyLengthToScratchPad_doOp_ref[]={
0x8B, 0xBC, 0x24, 0,0,0,0,	//mov         edi,[esp+12] 
0x8B, 0x47, 4,	//mov eax,dword ptr [edi+4] 
0x83, 0xEE, 4,	//sub esi,4 
0x89, 0x06		//mov dword ptr [esi],eax 
};
unsigned char copy_GM_TO_GM_doOp[]={
0x68, 0,0,0,0,	//push src object address
0xB9, 0,0,0,0,	//mov  ecx, dst object address 
0xE8, 0,0,0,0	//call M_<Vec3f>::operator= (428EA0h) 
};

enum NodeVTable{
	virtual_getProperties=4*0,
	virtual_renderNode=4*1,
	virtual_toString=4*2,
	virtual_toXMLString=4*3,
	virtual_toCPPString=4*4,
	virtual_getField=4*5,
	virtual_getFieldIndex=4*6,
	virtual_processFields=4*7,
	virtual_getFields=4*8,
	virtual_decodeVoid=4*9,
	virtual_paste=4*10,
	virtual_getFieldGroupIndex=4*11,
	virtual_setFieldGroupIndex=4*12,
	virtual_getIndexPtr=4*13,
	virtual_updateBuild=4*14,
	virtual_senceTug=4*15,
	virtual_catInverseRotation=4*16,
	virtual_resetRotations=4*17,
	virtual_processForOutput=4*18,
	virtual_getFLen=4*19,
	virtual_getType=4*20,
	virtual_scalerDestructor=4*21,
	//Build
	virtual_doBuild=4*22,
};

unsigned char copy_memberCall_doOp[]={
	0xB9, 0, 0, 0, 0,				//mov	ecx,offset ddummy 
	0x8B, 0x11,						//mov	edx,[ecx]
	0x8B, 0x42, virtual_doBuild,	//mov	eax,[edx+virtual_doBuild]
	0xFF, 0xD0						//call	eax  
};


const char typeNameLookUp[VARABLETYPE_LOOKUPLEN][11]={
"bool",
"Color3f",
"float",
"Image",
"int",
"NodePtr",
"Rotation",
"String",
"double",
"Vec2f",
"Vec3f",
"M_Bool",
"M_Color3f",
"M_Float",
"M_Image",
"M_Int",
"M_Node",
"M_Rotation",
"M_String",
"M_Double",
"M_Vec2f",
"M_Vec3f",
"void",
};

const char ecmaScriptTypeNameLookUp[VARABLETYPE_LOOKUPLEN][11]={
"SFBool",
"SFColor",
"SFFloat",
"SFImage",
"SFInt32",
"SFNode",
"SFRotation",
"SFString",
"SFTime",
"SFVec2f",
"SFVec3f",
"MFBool",
"MFColor",
"MFFloat",
"MFImage",
"MFInt32",
"MFNode",
"MFRotation",
"MFString",
"MFTime",
"MFVec2f",
"MFVec3f",
"void",
};

inline bool isLegalCppNameChar(const char& ch){
	if( ch>='a'&&ch<='z' ||
		ch>='A'&&ch<='Z' ||
		ch>='0'&&ch<='9' ||
		ch=='_'){
		return true;
	}
	return false;
}
bool makeNameLegalForCpp(String& str){
	bool flag=false;
	for(int i=0; i<str.length; i++){
		if(	isLegalCppNameChar(str[i])==false){
			str[i]='_';
			flag=true;
		}
	}
	return flag;
}
void String::endOfCppLabel(int& index)const{
	char* p=&pointer->rawChar[index];
	int newLength=pointer->rawLen-index;
	int i=0;
	while(i < newLength	&& isLegalCppNameChar(*p) ){
		p++;
	}
	index=p-pointer->rawChar;
}
String String::extractCppLabel(int& i)const{
	stbCpp(i);
	int start=i;
	endOfCppLabel(i);
	return substr(start,i-start);
}

NodePtr getNodeFromName(const String str){
   NodePtr node;
   for(int k=0; k < g_scene->defList.length; k++){
      if(g_scene->defList[k]->name==str){
         node = g_scene->defList[k];
		 break;
      }
   }
   return node;
}

Function* isFunctionCall(const String& label){
	Function* f=0;
	for(int k=g_functionList.length; --k>=0; ){
		f=g_functionList[k];
		if(f->functionName==label){
			return f;
		}
	}
	return 0;
}
L_Variable* Function::isLocalVariable(const String& label){
	for(int k=m_local.length; --k>=0; ){
		if(m_local[k]->name==label){
			return m_local[k];
		}
	}
	return 0;
}
template <class T> void V_<T,Single,Global>::decode(Function& f, const String& str, int& i){
	T& value=this->value.refValue();
	switch(str[i++]){
		case '=':{
			f.m_word+=new W_Assignment();
			f.decodeAssign(*this, str, i);
			break;
		}
		case '.':{
			str.stbCpp(i);
			f.m_word+=new W_Dot();
			f.writeProperty<T>(*this, str, i);
			break;
		}
		case '[':{
			throw;
			break;
		}
		case '-':{
			switch (str[i++]){
				case '-':{
					f.m_word+=new W_MinusMinus();
					preDec_insert2(f);
					break;
				}
				case '=':{
					f.m_word+=new W_MinusAssignment();
					f.decodeSubAssign<T>(*this, str, i);
					break;
				}
				default: throw;
			}
			break;
		}
		case '+':{
			switch (str[i++]){
				case '+':{
					f.m_word+=new W_PlusPlus();
					preInc_insert2(f);
					break;
				}
				case '=':{
					f.m_word+=new W_PlusAssignment();
					f.decodeAddAssign<T>(*this, str, i);
					break;
				}
				default: throw;
			}
			break;
		}
		default:{
			throw;
		}
	}
}
template <class T> void V_<T,Single,Local>::decode(Function& f, const String& str, int& i){
	switch(str[i++]){
		case '=':{
			f.m_word+=new W_Assignment();
			f.decodeAssign(*this, str, i);
			break;
		}
		case '.':{
			str.stbCpp(i);
			f.m_word+=new W_Dot();
			f.writeProperty<T>(*this, str, i);
			break;
		}
		case '[':{
			throw;
			break;
		}
		case '-':{
			switch (str[i++]){
				case '-':{
					f.m_word+=new W_MinusMinus();
					preDec_insert2(f);
					break;
				}
				case '=':{
					f.m_word+=new W_MinusAssignment();
					f.decodeSubAssign<T>(*this, str, i);
					break;
				}
				default: throw;
			}
			break;
		}
		case '+':{
			switch (str[i++]){
				case '+':{
					f.m_word+=new W_PlusPlus();
					preInc_insert2(f);
					break;
				}
				case '=':{
					f.m_word+=new W_PlusAssignment();
					f.decodeAddAssign<T>(*this, str, i);
					break;
				}
				default: throw;
			}
			break;
		}
		default:{
			throw;
		}
	}
}
template <class T> void V_<T,Multiple,Local>::decode(Function& f, const String& str, int& i){
	switch(str[i++]){
		case '[':{
			str.stbCpp(i);
			f.refAssignElement<T>(*this, str, i);
			break;
		}
		default:{
			throw;
		}
	}
}

template <class T> void V_<T,Multiple,Global>::decode(Function& f, const String& str, int& i){
	M_<T>& value=this->value.refValue();

	switch(str[i++]){
		case '=':{
			f.m_word+=new W_Assignment();
			f.decodeAssign(*this, str, i);
			break;
		}
		case '.':{
			str.stbCpp(i);
			f.m_word+=new W_Dot();
			f.writeProperty<T>(*this, str, i);
			break;
		}
		case '[':{
			str.stbCpp(i);
			f.globalAssignElement<T>(value, str, i);
			break;
		}
		case '-':{
			switch (str[i++]){
				case '-':{
					f.m_word+=new W_MinusMinus();
					preDec_insert2(f);
					break;
				}
				default: throw;
			}
			break;
		}
		case '+':{
			switch (str[i++]){
				case '+':{
					f.m_word+=new W_PlusPlus();
					preInc_insert2(f);
					break;
				}
				case '=':{
					f.m_word+=new W_PlusAssignment();
					f.decodeAddAssign(*this, str,i);
					break;
				}
				default: throw;
			}
			break;
		}
		default:{
			throw;
		}
	}
}

void morphCpp(CPMorpher* morpher, float v, Art* part){
	morpher->morph(v, *part);
}
void calcNikCpp(NikInterpolator* nikInterpolator, float v, Art* part){
	nikInterpolator->calc(v, *part);
}
void compCpp(PosCompensator* compensator, float v, Art* part){
	compensator->comp(v, *part);
}

void Function::decodeCppLine(M_Node& clParent, int& codeLineIdx, CodeLine& cl, const String& str, int& i, int& bracketCntr){
	g_postIncList.length = 0;
	g_postDecList.length = 0;

//	String name;
	int fieldIdx;
	String label;
	str.stbnlCpp(i);
	switch (str[i++]){
		case '\n':{
			return;
		}
		case '/':{
			if(str[i++]=='/'){
				static String commentStr;
				commentStr="//";
				while(i<str.length && (str[i]>=' '||str[i]=='\t')){
					commentStr+=str[i++];
				}
				m_word+=new W_Comment(commentStr);
			}
			else throw;
			break;
		}
		case '-':{
			if(str[i++]=='-'){
				m_word+=new W_MinusMinus();
				label=str.extractCppLabel(i);
				if(L_Variable* lv=isLocalVariable(label)){
					m_word+=new W_Local(this, lv->varIndex);
					lv->preDec_insert2(*this);
					str.stbCpp(i);
					if(str[i]==')'){
						i++;
					}
				}
				else {
					Node* node=getNodeFromName(label);
					if(G_Variable* gv=dynamic_cast<G_Variable*>(node)){
						m_word+=new W_Global(gv);
						gv->preDec_insert2(*this);
					}
					else throw;
				}

				break;
			}
			else throw;
			break;
		}
		case '+':{
			if(str[i++]=='+'){
				m_word+=new W_PlusPlus();
				label=str.extractCppLabel(i);
				if(L_Variable* lv=isLocalVariable(label)){
					m_word+=new W_Local(this, lv->varIndex);
					lv->preInc_insert2(*this);
					str.stbCpp(i);
					if(str[i]==')'){
						i++;
					}
				}
				else {
					Node* node=getNodeFromName(label);
					if(G_Variable* gv=dynamic_cast<G_Variable*>(node)){
						m_word+=new W_Global(gv);
						gv->preInc_insert2(*this);
					}
					else throw;
				}
				break;
			}
			else throw;
			break;
		}//case
		default: {//does not start with operator
			--i;
			if(getKeyWord(*this, clParent, codeLineIdx, cl, str, i, bracketCntr)){
			}
			else{
				label=str.extractCppLabel(i);
				if(L_Variable* lv=isLocalVariable(label)){
					m_word+=new W_Local(this, lv->varIndex);
					str.stbCpp(i);
					lv->decode(*this, str, i);
				}
				else if(Node* node=getNodeFromName(label)){

					if(G_Variable* gv=dynamic_cast<G_Variable*>(node)){
						m_word+=new W_Global(gv);
						str.stbCpp(i);
						gv->decode(*this, str, i);
					}
					else{
						if(str[i++]!='.')throw this;
						str.stbCpp(i);
						label=str.getWord2(i);
						str.stbCpp(i);
						char ch=str[i++];
						if(ch=='('){
							str.stbCpp(i);
							if(str[i]==')'){
								i++;
								if(label=="doBuild"){
									m_word+=new W_Node(node);
									m_word+=new W_Dot();
									m_word+=new W_MemberCall();
									int mark=compiled.length+1;
									copyMachineCodeM(copy_memberCall_doOp);
									insertDWord(mark, reinterpret_cast<int>(node));
								}
								else throw;
							}
							else if(label=="morph"){

								m_word+=new W_Node(node);
								m_word+=new W_Dot();
								m_word+=new W_Morph();
								m_word+=new W_OpenBracket();


								int opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
								if(opType!=S_FLOAT)throw;
								if(str[i++]!=','){
									throw;
								}
								m_word+=new W_Comma();
								str.stbCpp(i);
								label=str.getWord2(i);
								str.stbCpp(i);
								Node* part = getNodeFromName(label);
								if(part == 0){
									throw;
								}
								m_word += new W_Node(part);
								if(str[i++]!=')'){
									throw;
								}
								m_word += new W_CloseBracket();

								int mark=compiled.length+1;
								copyMachineCodeM(writeElement_morpherMorph_doOp_void_ref);
								insertDWord(mark, reinterpret_cast<int>(part));
								insertDWord(mark + 7, reinterpret_cast<int>(node));
								g_asm2CppAddy+=Asm2CppAddy(morphCpp, mark + 12);
								opType=_VOID;
							}
							else if(label=="calc"){//niki.calc(v, r_hip)

								m_word+=new W_Node(node);
								m_word+=new W_Dot();
								m_word+=new W_Calc();
								m_word+=new W_OpenBracket();


								int opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
								if(opType!=S_FLOAT)throw;
								if(str[i++]!=','){
									throw;
								}
								m_word+=new W_Comma();
								str.stbCpp(i);
								label=str.getWord2(i);
								str.stbCpp(i);
								Node* part = getNodeFromName(label);
								if(part == 0){
									throw;
								}
								m_word += new W_Node(part);
								if(str[i++]!=')'){
									throw;
								}
								m_word += new W_CloseBracket();

								int mark=compiled.length+1;
								copyMachineCodeM(writeElement_morpherMorph_doOp_void_ref);
								insertDWord(mark, reinterpret_cast<int>(part));
								insertDWord(mark + 7, reinterpret_cast<int>(node));
								g_asm2CppAddy+=Asm2CppAddy(calcNikCpp, mark + 12);
								opType=_VOID;
							}
							else if(label=="comp"){

								m_word+=new W_Node(node);
								m_word+=new W_Dot();
								m_word+=new W_Comp();
								m_word+=new W_OpenBracket();


								int opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
								if(opType!=S_FLOAT)throw;
								if(str[i++]!=','){
									throw;
								}
								m_word+=new W_Comma();
								str.stbCpp(i);
								label=str.getWord2(i);
								str.stbCpp(i);
								Node* part = getNodeFromName(label);
								if(part == 0){
									throw;
								}
								m_word += new W_Node(part);
								if(str[i++]!=')'){
									throw;
								}
								m_word += new W_CloseBracket();

								int mark=compiled.length+1;
								copyMachineCodeM(writeElement_morpherMorph_doOp_void_ref);
								insertDWord(mark, reinterpret_cast<int>(part));
								insertDWord(mark + 7, reinterpret_cast<int>(node));
								g_asm2CppAddy+=Asm2CppAddy(compCpp, mark + 12);
								opType=_VOID;
							}
							else throw;
						}
						else{
							fieldIdx=node->getProperties().getIndex(label);
							m_word+=new W_NoOutputNode(node);
							m_word+=new W_NoOutputDot();
							m_word+=new W_Field(node,fieldIdx);
							Field* fieldPtr=node->getField(fieldIdx);
							switch(ch){
								case '=':{
									m_word+=new W_Assignment();
									fieldPtr->decodeAssign(*this, str,i);
									break;
								}
								case '+':{
									if(str[i++]=='='){
										m_word+=new W_PlusAssignment();
										fieldPtr->decodeAddAssign(*this, str,i);
										break;
									}
									else throw;
								}
								default: throw;
							}
						}
					}
				}
				else if(Function* functionCall=isFunctionCall(label)){
					int bracketCntr=0;
					getArg(*functionCall, str, i, bracketCntr);
				}
				else throw;
			}
		}
	}//switch
	for(int k=0; k<g_postDecList.length; k++){
		g_postDecList[k]->preDec_insert2(*this);
	}
	for(int k=0; k<g_postIncList.length; k++){
		g_postIncList[k]->preInc_insert2(*this);
	}
}
void Function::addFunctionToList(const String& str, int& i){
	str.stbCpp(i);
	functionName=str.extractCppLabel(i);
	g_functionList+=this;
}
void Function::insertDWord(int mark, int data){
	for(int i=4; --i>=0;){
		compiled[mark++]=static_cast<char>(data);
		data>>=8;
	}
}
void Function::catConstQWord(unsigned char* data){
	for(int i=8; --i>=0;){
		constData+=*data;
		data++;
	}
}
void goToEndOfBody(const String& str, int& i){
	char ch;
	while((ch=str[i++])!='}'){
		if(ch=='{'){
			goToEndOfBody(str, i);
		}
	}
}
void onlyFunctionsToString(String& str){
	for(int k=g_functionList.length; --k>=0;){
		g_functionList[k]->toString(str);
	}
}
void initCompileForBody(){
	g_callOffsetCntr=0;
	for(int k=g_functionList.length; --k>=0;){
		Function& f=*(g_functionList[k]);
		//f.codeLine.refValue().length=0;
		f.m_local.length=f.paramLen;
		f.stackOffset.length=f.paramLen;
		f.localSize=0;
		f.call2ndCompile.length=0;
		for(int k=f.paramLen; --k>=0;){ //pushed on back to front real time for extra speed and so
			//localSize+=
			f.stackOffset[k]=getValueSize(f.m_local[k]);
		}
	}
	g_local2ndCompile.length=0;
	g_const2ndCompile.length=0;
	
	g_asm2CppAddy.length=0;
	Function::compiled.length=0;
	Function::constData.length=0;
}
void initCompile(){
	g_functionList.length=0;
}
void doFrontBrace(const String& str, int& i, int& bracketCntr){
	str.stbnlCpp(i);
	if(str[i++]!='{')throw;
//	bracketCntr++;
	str.stbnlCpp(i);
	if(str[i]=='\n'){
		i++;
	}
	str.stbnlCpp(i);
}
void Function::newDecodeHeader(const String& str, int& i, int& bracketCntr){
	m_local.length=0;
	stackOffset.length=0;
	g_local2ndCompile.length=0;
	g_const2ndCompile.length=0;
	call2ndCompile.length=0;
	g_asm2CppAddy.length=0;
	compiled.length=0;
	constData.length=0;
	if(reset_m_word){
		m_word.length=0;
	}
	localSize=0;

	str.stbCpp(i);
	if(str[i]=='('){
		i++;
		str.stbCpp(i);
	}
	else throw;
	while(str[i]!=')'&&i<str.length){
		extractParameter(str, i);
	}
	paramLen=m_local.length;

	if(i>=str.length) throw this;
	if(str[i]!=')')throw this;
	str.stbCpp(++i);
	if(i>=str.length) throw this;
	doFrontBrace(str, i, bracketCntr);
	bodyTextBegin=i;
	goToEndOfBody(str, i);
	str.stbCpp(i);
}
void Function::decodeHeader(const String& str, int& i, int& bracketCntr){
//	Operation* op=readInitLocal<float>(*this,str, i);
	m_local.length=0;
	stackOffset.length=0;
	g_local2ndCompile.length=0;
	g_const2ndCompile.length=0;
	call2ndCompile.length=0;
	g_asm2CppAddy.length=0;
	compiled.length=0;
	constData.length=0;
	if(reset_m_word && selectedLine==0){
		m_word.length=0;
	}
	localSize=0;

	if(str[i]=='}'){
		str.stbCpp(++i);
		return;
	}
	String label=str.extractCppLabel(i);

	if(label=="void"){		returnType=_VOID;}
	else if(label=="bool"){	returnType=S_BOOL;}
	else if(label=="Color3f"){	returnType=S_COLOR3F;}
	else if(label=="float"){	returnType=S_FLOAT;}
	else if(label=="Image"){	returnType=S_IMAGE;}
	else if(label=="int"){	returnType=S_INT32;}
	else if(label=="NodePtr"){	returnType=NODEPTR;}
	else if(label=="Rotation"){returnType=S_ROTATION;}
	else if(label=="String"){	returnType=S_STRING;}
	else if(label=="double"){	returnType=S_DOUBLE;}
	else if(label=="Vec2f"){	returnType=S_VEC2F;}
	else if(label=="Vec3f"){	returnType=S_VEC3F;}
	else if(label=="M_Bool"){	returnType=M_BOOL;}
	else if(label=="M_Color3f"){	returnType=M_COLOR;}
	else if(label=="M_Float"){	returnType=M_FLOAT;}
	else if(label=="M_Image"){	returnType=M_IMAGE;}
	else if(label=="M_Int"){	returnType=M_INT32;}
	else if(label=="M_Node"){	returnType=M_NODE;}
	else if(label=="M_Rotation"){returnType=M_ROTATION;}
	else if(label=="M_String"){returnType=M_STRING;}
	else if(label=="M_Double"){	returnType=M_TIME;}
	else if(label=="M_Vec2f"){	returnType=M_VEC2F;}
	else if(label=="M_Vec3f"){	returnType=M_VEC3F;}
	else throw;

	if(selectedLine){
		if(selectedLine==this){
			if(false==isInList(g_functionList, this)){
				addFunctionToList(str, i);
			}
			else{
				str.stbCpp(i);
				functionName=str.extractCppLabel(i);
			}
		}
	}
	else{
		addFunctionToList(str, i);
	}
	str.stbCpp(i);
	if(str[i]=='('){
		i++;
		str.stbCpp(i);
	}
	while(str[i]!=')'&&i<str.length){
		extractParameter(str, i);
	}
	paramLen=m_local.length;

	if(i>=str.length) throw this;
	if(str[i++]!=')')throw this;
	if(selectedLine==this){
		return;
	}
	str.stbCpp(i);
	if(i>=str.length) throw this;
	doFrontBrace(str, i, bracketCntr);
	bodyTextBegin=i;
	goToEndOfBody(str, i);
	str.stbCpp(i);
}
void Function::doCall2ndCompile(){
	for(int k=call2ndCompile.length; --k>=0;){
		int mark=call2ndCompile[k].byteIndex;
		Function& cf=*(call2ndCompile[k].function);
		
		insertDWord(mark, cf.localSize);

		mark+=5;

		unsigned char* callAddr=&compiled[mark];
		unsigned char* funcAddr=cf.startCodePtr;
		int data=funcAddr-callAddr-4;
		insertDWord(mark,data);

		mark+=6;
		insertDWord(mark, cf.sizeOfLocalAndParam);

	}
}
void decodeFunctions(const String& str){
	Function::compiled.enableWrite();

	for(int k=g_functionList.length; --k>=0;){
		g_functionList[k]->decodeBody(str);
	}
	for(int k=g_functionList.length; --k>=0;){
		Function* f=g_functionList[k];
		f->startCodePtr=&f->compiled[f->startCodeIdx];
	}
	for(int k=g_functionList.length; --k>=0;){
		g_functionList[k]->doCall2ndCompile();
	}

	int beginConstData=Function::compiled.length;
	Function::compiled+=Function::constData;
	for(int k=g_const2ndCompile.length; --k>=0;){
		Function::insertDWord(g_const2ndCompile[k].codeIdx, reinterpret_cast<int>(&Function::compiled[beginConstData+g_const2ndCompile[k].constIdx]));
	}
	for(int k=g_asm2CppAddy.length; --k>=0;){
		int mark=g_asm2CppAddy[k].byteIndex;
	
		unsigned char* callAddr=&Function::compiled[mark];
		unsigned char* addy=reinterpret_cast<unsigned char*>(g_asm2CppAddy[k].addy);
		int data=addy-callAddr-4;
		Function::insertDWord(mark,data);
	}

	Function::compiled.protect();
}
inline bool Function::dealWithEndOfLine(const String& str, int& i, CodeLine*& codeLine, const int& remBracketCntr, int& bracketCntr){
	if(codeLine->codeLine.length == 0 && codeLine->isIfElseForEtc(this) == false){
		codeLine->opTo=compiled.length;
		codeLine->wordTo=m_word.length;
	}
	str.stbnlCpp(i);
	codeLine->name="";
	for(int w=codeLine->wordFrom; w<codeLine->wordTo; w++){
		m_word[w]->toString(codeLine->name);
	}
	if(str[i]==';'){
		str.stbnlCpp(++i);
		if(str[i]=='\n')i++;
	}
	else if(str[i]=='}'){
		str.stbnlCpp(++i);
		if(str[i]=='\n')i++;
		if(--bracketCntr==remBracketCntr){
			return true;
		}
	}
	str.stbnlCpp(i);
	return false;
}
inline bool forEOLQuestion(const String& str, int& i, const int& remBracketCntr, int& bracketCntr){
	if(str[i]=='}'){
		str.stbnlCpp(++i);
		if(str[i]=='\n')i++;
		if(--bracketCntr==remBracketCntr){
			return false;
		}
		return true;
	}
	return true;
}
void Function::decodeBody(const String& str){
	int i=bodyTextBegin;

	g_local2ndCompile.length=0;
	startCodeIdx=compiled.length;

	
	CodeLine* codeLine;
	int remBracketCntr=0;
	int bracketCntr=1;
	for(int codeLineIdx=0; forEOLQuestion(str, i, remBracketCntr, bracketCntr); codeLineIdx++){
		if(selectedLine){
			codeLine=dynamic_cast<CodeLine*>(this->codeLine[codeLineIdx].p);
		}
		else{
			codeLine=new CodeLine;
			this->codeLine.refValue()+=codeLine;
		}
		codeLine->opFrom=compiled.length;
		codeLine->wordFrom=m_word.length;

		decodeCppLine(this->codeLine, codeLineIdx, *codeLine, str, i, bracketCntr);

		if(dealWithEndOfLine(str, i, codeLine, remBracketCntr, bracketCntr)){
			break;
		}
	}
	compiled+=0xc3; //ret
	str.stbCpp(++i);
	sizeOfLocalAndParam=4;
	{
		for(int k=stackOffset.length; --k>=0;){ //pushed on back to front real time for extra speed and so
			int h=stackOffset[k];
			stackOffset[k]=sizeOfLocalAndParam;
			sizeOfLocalAndParam+=h;
		}
	}
	sizeOfLocalAndParam-=4;
	{
		for(int k=g_local2ndCompile.length; --k>=0;){
			//if(k==2)continue;
			insertDWord(g_local2ndCompile[k].byteIndex, stackOffset[g_local2ndCompile[k].variableIndex] + g_local2ndCompile[k].callOffset );
		}
	}
	name=toCodeLineString();
}

String Function::toCodeLineString(){
	String str=typeNameLookUp[returnType];
	str+=' ';
	str+=functionName;
	str+='(';

	if(paramLen)for(int k=0;;){
		str+=String(typeNameLookUp[m_local[k]->getType()]);
		switch(m_local[k]->getType()){
			case M_BOOL:
			case M_COLOR:
			case M_FLOAT:
			case M_IMAGE:
			case M_INT32:
			case M_NODE:
			case M_ROTATION:
			case M_STRING:
			case M_TIME:
			case M_VEC2F:
			case M_VEC3F:{
				str+='&';
				break;
			}
		}
		str+=' ';
		str+=m_local[k]->name;
		if(++k>=paramLen)break;
		str+=", ";
	}
	str+=')';
	return str;
}
bool CodeLine::isIfElseForEtc(Function* f){
	if(f){
		M_Word& m_word=f->m_word;
		if(wordFrom < m_word.length){
			if(dynamic_cast<W_If*>(m_word[wordFrom]) || dynamic_cast<W_Else*>(m_word[wordFrom]) || dynamic_cast<W_Else*>(m_word[wordFrom]) || dynamic_cast<W_For*>(m_word[wordFrom]) || dynamic_cast<W_While*>(m_word[wordFrom])|| dynamic_cast<W_Do*>(m_word[wordFrom])){
				return true;
			}
		}
	}
	return false;
}

bool addTextCell(String& str, bool insertCurly);
void addTextCellForFunctionHeader(String& str);

void CodeLine::doTillClosingBracket(String& str, M_Word& m_word, int& w, int& bracketCounter){
	int remBracketCounter = bracketCounter;
	for(; w<wordTo; w++){
		Word* wrd=m_word[w];
		outputToString(str, m_word, w, bracketCounter);
		if(dynamic_cast<W_OpenBracket*>(wrd)){
			bracketCounter++;
		}
		else if(dynamic_cast<W_CloseBracket*>(wrd)){
			if(--bracketCounter==remBracketCounter){
				break;
			}
		}
	}
}
void CodeLine::doTillClosingSquareBracket(String& str, M_Word& m_word, int& w, int& bracketCounter){
	int remBracketCounter = bracketCounter;
	bracketCounter++;
	for(; w<wordTo; w++){
		Word* wrd=m_word[w];
		outputToString(str, m_word, w, bracketCounter);
		if(dynamic_cast<W_OpenBracket*>(wrd)){
			bracketCounter++;
		}
		else if(dynamic_cast<W_CloseSquareBracket*>(wrd)){
			if(--bracketCounter==remBracketCounter){
				break;
			}
		}
	}
}

void CodeLine::outputRotation(String& str, M_Word& m_word, int& w, int& bracketCounter){
	if(w+1 < wordTo){
		Word* wrd=m_word[w+1];
		if(dynamic_cast<W_Multiply*>(wrd)){
			if(dynamic_cast<W_S_Rotation2*>(m_word[w+2])){
				str+=".multiply(";
				w+=2;
				m_word[w++]->toString(str);
				doTillClosingBracket(str, m_word, w, bracketCounter);
				str+=')';
				outputRotation(str, m_word, w, bracketCounter);
			}
			else if(dynamic_cast<W_S_Vec3f2*>(m_word[w+2])){
				str+=".multVec(";
				w+=2;
				m_word[w++]->toString(str);
				doTillClosingBracket(str, m_word, w, bracketCounter);
				str+=')';
				outputRotation(str, m_word, w, bracketCounter);
			}
			else if(W_Global* w_g=dynamic_cast<W_Global*>(m_word[w+2])){
				if(dynamic_cast<V_<Rotation,Single>*>(w_g->variable)){
					str+=".multiply(";
					w+=2;
					m_word[w]->toString(str);
					str+=')';
					outputRotation(str, m_word, w, bracketCounter);
				}
			}
			else if(W_Local* w_g=dynamic_cast<W_Local*>(m_word[w+2])){
				if(dynamic_cast<V_<Rotation,Single,Local>*>(w_g->getVar())){
					str+=".multiply(";
					w+=2;
					m_word[w]->toString(str);
					str+=')';
					outputRotation(str, m_word, w, bracketCounter);
				}
			}
			else{
				throw;
			}
		}
	}
	return;
}
void CodeLine::outputM_Rotation(String& str, M_Word& m_word, int& w, int& bracketCounter){
	if(w+1 < wordTo){
		Word* wrd=m_word[w+1];
		if(dynamic_cast<W_OpenSquareBracket*>(wrd)){
			m_word[++w]->toString(str);
			doTillClosingSquareBracket(str, m_word, ++w, bracketCounter);
			Word* wrd=m_word[w+1];
			if(dynamic_cast<W_Multiply*>(wrd)){
				if(dynamic_cast<W_S_Rotation2*>(m_word[w+2])){
					str+=".multiply(";
					w+=2;
					m_word[w++]->toString(str);
					doTillClosingBracket(str, m_word, w, bracketCounter);
					str+=')';
					outputRotation(str, m_word, w, bracketCounter);
				}
				else if(dynamic_cast<W_S_Vec3f2*>(m_word[w+2])){
					str+=".multVec(";
					w+=2;
					m_word[w++]->toString(str);
					doTillClosingBracket(str, m_word, w, bracketCounter);
					str+=')';
					outputRotation(str, m_word, w, bracketCounter);
				}
				else if(W_Global* w_g=dynamic_cast<W_Global*>(m_word[w+2])){
					if(dynamic_cast<V_<Rotation,Single>*>(w_g->variable)){
						str+=".multiply(";
						w+=2;
						m_word[w]->toString(str);
						str+=')';
						outputRotation(str, m_word, w, bracketCounter);
					}
					else if(dynamic_cast<V_<Rotation,Multiple>*>(w_g->variable)){
						str+=".multiply(";
						w+=2;
						m_word[w]->toString(str);
						str+=')';
						outputM_Rotation(str, m_word, w, bracketCounter);
					}
					else if(dynamic_cast<V_<Vec3f,Multiple>*>(w_g->variable)){
						str+=".multVec(";
						w+=2;
						m_word[w]->toString(str);
						outputM_Vec3f(str, m_word, w, bracketCounter);
						str+=')';
					}
				}
				else if(W_Local* w_g=dynamic_cast<W_Local*>(m_word[w+2])){
					if(dynamic_cast<V_<Rotation,Single,Local>*>(w_g->getVar())){
						str+=".multiply(";
						w+=2;
						m_word[w]->toString(str);
						str+=')';
						outputRotation(str, m_word, w, bracketCounter);
					}
					if(dynamic_cast<V_<Vec3f,Single,Local>*>(w_g->getVar())){
						str+=".multVec(";
						w+=2;
						m_word[w]->toString(str);
						str+=')';
						outputRotation(str, m_word, w, bracketCounter);
					}
				}
				else{
					throw;
				}
			}
		}
	}
	return;
}
bool CodeLine::outputVec3f_2(String& str, M_Word& m_word, int& w, int& bracketCounter){
	if(w+1 < wordTo){
		W_Dot* dot=dynamic_cast<W_Dot*>(m_word[w+1]);
		if(w < m_word.length - 3 &&
			dynamic_cast<W_Dot*>(m_word[w+1]) &&
			(
				dynamic_cast<W_X*>(m_word[w+2])||
				dynamic_cast<W_Y*>(m_word[w+2])||
				dynamic_cast<W_Z*>(m_word[w+2])
			)
		){
			return false;
		}
	}
	return true;
}

void CodeLine::outputVec3f(String& str, M_Word& m_word, int& w, int& bracketCounter){//to be phased out
	if(w+1 < wordTo){
		Word* wrd=m_word[w+1];
		if(dynamic_cast<W_Add*>(wrd)){
			if(dynamic_cast<W_S_Rotation2*>(m_word[w+2])){
				str+=".add(";
				w+=2;
				m_word[w++]->toString(str);
				doTillClosingBracket(str, m_word, w, bracketCounter);
				str+=')';
			}
			else if(W_Global* w_g=dynamic_cast<W_Global*>(m_word[w+2])){
				if(dynamic_cast<V_<Rotation,Single>*>(w_g->variable)){
					str+=".add(";
					w+=2;
					m_word[w]->toString(str);
					outputRotation(str, m_word, w, bracketCounter);
					str+=')';
				}
			}
			else if(W_Local* w_g=dynamic_cast<W_Local*>(m_word[w+2])){
				if(dynamic_cast<V_<Rotation,Single,Local>*>(w_g->getVar())){
					str+=".add(";
					w+=2;
					m_word[w]->toString(str);
					outputRotation(str, m_word, w, bracketCounter);
					str+=')';
				}
			}
			else{
				throw;
			}
		}
	}
}
bool CodeLine::outputM_Vec3f(String& str, M_Word& m_word, int& w, int& bracketCounter){
	if(w+1 < wordTo){
		Word* wrd=m_word[w+1];
		if(dynamic_cast<W_OpenSquareBracket*>(wrd)){
			m_word[++w]->toString(str);
			doTillClosingSquareBracket(str, m_word, ++w, bracketCounter);
			W_Dot* dot=dynamic_cast<W_Dot*>(m_word[w+1]);
			if(w < m_word.length - 3 &&
				dynamic_cast<W_Dot*>(m_word[w+1]) &&
				(
					dynamic_cast<W_X*>(m_word[w+2])||
					dynamic_cast<W_Y*>(m_word[w+2])||
					dynamic_cast<W_Z*>(m_word[w+2])
				)
			){
				return false;
			}
		}
	}
	/*
			Word* wrd=m_word[w+1];
			if(dynamic_cast<W_Add*>(wrd)){
				if(dynamic_cast<W_S_Rotation2*>(m_word[w+2])){
					str+=".add(";
					w+=2;
					m_word[w++]->toString(str);
					doTillClosingBracket(str, m_word, w, bracketCounter);
					str+=')';
				}
				else if(W_Global* w_g=dynamic_cast<W_Global*>(m_word[w+2])){
					if(dynamic_cast<V_<Rotation,Single>*>(w_g->variable)){
						str+=".add(";
						w+=2;
						m_word[w]->toString(str);
						outputRotation(str, m_word, w, bracketCounter);
						str+=')';
					}
					else if(dynamic_cast<V_<Vec3f,Multiple>*>(w_g->variable)){
						str+=".add(";
						w+=2;
						m_word[w]->toString(str);
						outputM_Vec3f(str, m_word, w, bracketCounter);
						str+=')';
					}
				}
				else if(W_Local* w_g=dynamic_cast<W_Local*>(m_word[w+2])){
					if(dynamic_cast<V_<Rotation,Single,Local>*>(w_g->getVar())){
						str+=".add(";
						w+=2;
						m_word[w]->toString(str);
						outputRotation(str, m_word, w, bracketCounter);
						str+=')';
					}
				}
				else{
					throw;
				}
			}
		}
	}
	*/
	return true;
}
void CodeLine::outputToString(String& str, M_Word& m_word, int& w, int& bracketCounter){
	Word* wrd=m_word[w];
	if(dynamic_cast<W_Minus*>(wrd)){
		if(dynamic_cast<W_NoOutputNode*>(m_word[w+1])){
			W_Field* w_field=dynamic_cast<W_Field*>(m_word[w+3]);
			if(dynamic_cast<SFRotation*>(w_field->node->getField(w_field->fieldIdx))){
				w_field->toString(str);
				str+=".inverse()";
				w+=3;
				return;
			}
			else if(dynamic_cast<SFVec3f*>(w_field->node->getField(w_field->fieldIdx))){
				w_field->toString(str);
				str+=".negate()";
				w+=3;
				return;
			}
		}
		else if(W_Local* w_local=dynamic_cast<W_Local*>(m_word[w+1])){
			if(dynamic_cast<V_<Rotation,Single,Local>*>(w_local->getVar())){
				w_local->toString(str);
				str+=".inverse()";
				w++;
				return;
			}
			else if(dynamic_cast<V_<Vec3f,Single,Local>*>(w_local->getVar())){
				w_local->toString(str);
				str+=".negate()";
				w++;
				return;
			}
		}
		wrd->toString(str);
	}
	else if(dynamic_cast<W_VecDivide*>(wrd)){
		str+=".divide(";
		w++;
		m_word[w++]->toString(str);
		doTillClosingBracket(str, m_word, w, bracketCounter);
		str+=')';
		return;
	}
	else if(dynamic_cast<W_PlusAssignment*>(wrd)){
		if(W_Global* w_global=dynamic_cast<W_Global*>(m_word[w-1])){
			if(G_Variable* gv=dynamic_cast<G_Variable*>(w_global->variable)){
				int type=gv->getValueType();
				if(type >= FIRST_M_VALUE_TYPE && type <= LAST_VALUE_TYPE){
					str+='[';
					m_word[w-1]->toString(str);
					str+=".length] = ";
					return;
				}
			}
		}
		wrd->toString(str);
	}
	else if(dynamic_cast<W_Add*>(wrd)){
		if(W_Global* w_g=dynamic_cast<W_Global*>(m_word[w+1])){
			if(dynamic_cast<V_<Vec3f,Single>*>(w_g->variable)){
				String str2;
				m_word[++w]->toString(str2);
				if(outputVec3f_2(str2, m_word, w, bracketCounter)){
					str+=".add(";
					str+=str2;
					str+=')';
				}
				else{
					str+=" + ";
					str+=str2;
				}
				return;
			}
			else if(dynamic_cast<V_<Vec3f,Multiple>*>(w_g->variable)){
				String str2;
				m_word[++w]->toString(str2);
				if(outputM_Vec3f(str2, m_word, w, bracketCounter)){
					str+=".add(";
					str+=str2;
					str+=')';
				}
				else{
					str+=" + ";
					str+=str2;
				}
				return;
			}
		}
		else if(W_Local* w_l=dynamic_cast<W_Local*>(m_word[w+1])){
			if(dynamic_cast<V_<Vec3f,Single,Local>*>(w_l->getVar())){
				String str2;
				m_word[++w]->toString(str2);
				if(outputVec3f_2(str2, m_word, w, bracketCounter)){
					str+=".add(";
					str+=str2;
					str+=')';
				}
				else{
					str+=" + ";
					str+=str2;
				}
				return;
			}
			else if(dynamic_cast<V_<Vec3f,Multiple,Local>*>(w_l->getVar())){
				String str2;
				m_word[++w]->toString(str2);
				if(outputM_Vec3f(str2, m_word, w, bracketCounter)){
					str+=".add(";
					str+=str2;
					str+=')';
				}
				else{
					str+=" + ";
					str+=str2;
				}
				return;
			}
		}
		wrd->toString(str);
	}
	else if(dynamic_cast<W_S_Rotation2*>(wrd)){
		m_word[w++]->toString(str);
		doTillClosingBracket(str, m_word, w, bracketCounter);
		outputRotation(str, m_word, w, bracketCounter);
	}
	else if(W_Global* w_g=dynamic_cast<W_Global*>(wrd)){
		wrd->toString(str);
		if(dynamic_cast<V_<Rotation,Single>*>(w_g->variable)){
			outputRotation(str, m_word, w, bracketCounter);
		}
		if(dynamic_cast<V_<Rotation,Multiple>*>(w_g->variable)){
			outputM_Rotation(str, m_word, w, bracketCounter);
		}
//		if(dynamic_cast<V_<Vec3f,Multiple>*>(w_g->variable)){
//			outputM_Vec3f(str, m_word, w, bracketCounter);
//		}
	}
	else if(dynamic_cast<W_S_Vec3f2*>(wrd)){
		m_word[w++]->toString(str);
		doTillClosingBracket(str, m_word, w, bracketCounter);
		outputVec3f(str, m_word, w, bracketCounter);
	}
	else{
		wrd->toString(str);
	}
}
void CodeLine::toString(String& str, Function* lastFunction){
	if(this==Function::selectedLine && addTextCell(str, codeLine.length == 0)){
	}
	else{
		int bracketCounter=0;
		M_Word& m_word=lastFunction->m_word;
		for(int w=wordFrom; w<wordTo; w++){
			if(g_outputMode){
				outputToString(str, m_word, w, bracketCounter);
			}
			else{
				m_word[w]->toString(str);
			}
		}
		if(wordFrom == wordTo){
			str+="\n";
		}
		else if(codeLine.length==0 && wordFrom < lastFunction->m_word.length && isIfElseForEtc(lastFunction)){
			str+="{}\n";
		}
		else if(codeLine.length==0){
			str+=";\n";
		}
	}
	if(codeLine.length){
		str+="{\n";
		for(int k=0; k<codeLine.length; k++){
			CodeLine* clPtr=dynamic_cast<CodeLine*>(codeLine[k].p);
			clPtr->toString(str, lastFunction);
		}
		str+="}\n";
	}
}
void Function::toString(String& str, bool newList){
/*
	if(g_outputMode && readyForOutput == false){
		Script* script=getOutputScriptPtr();
		script->function+=this;
		return;
	}
*/
//	str+=getTypeStr();
//	str+="{\n";
	int remStrLen =str.length;
	if(selectedLine==this){
		int i =str.length;
		addTextCellForFunctionHeader(str);
	}
	else{
		headerToString(str);
	}
	str+="{\n";
	bodyTextBegin=str.length;

	for(int k=0; k<codeLine.length; k++){
		CodeLine* clPtr=dynamic_cast<CodeLine*>(codeLine[k].p);
		clPtr->toString(str, this);
	}
	str+="}\n";
	if(selectedLine==this){
		int bracketCntr=0;
		decodeHeader(str,  remStrLen, bracketCntr);
	}

}

bool String::cmpCppWord(const char* str,int& index)const{
	const char* p2=c_str()+index;
	while(*str!='\0'){
		if(*str++!=*p2++){
			return false;
		};
	}
	if(isLegalCppNameChar(*p2)){
		return false;
	}
	index= p2-c_str();
	return true;
}
bool String::cmpCppWord(const String& str,int& index)const{
	return cmpCppWord(str.pointer->rawChar, index);
}

typedef const char LkKSg[11] ;
typedef void (*LookupFunction)(Function& f, M_Node& clParent, int& codeLineIdx, CodeLine& cl, const String& str, int& i, int& bracketCntr);
template<class T, class S> void putInLocal(Function& f, M_Node& clParent, int& codeLineIdx, CodeLine& cl, const String& str, int& i, int& bracketCntr){

	str.stbCpp(i);

	int varIndex=f.m_local.length;
	f.m_word+=new W_ConstructLocal<T,S>();
	f.m_word+=new W_Local(&f, varIndex);
	f.m_word+=new W_Assignment();

	f.addNewLocal<T,Single>(str.extractCppLabel(i));

	f.stackOffset+=sizeof(T);
	f.localSize+=sizeof(T);

	str.stbCpp(i);
	switch(str[i++]){
		case '=':{
			int bracketCntr=0;
			int destOpType=f.getFirstOperand(str, i,bracketCntr, templateType<T,Single>());
			f.copyScratchPadToLocal<T,S>(varIndex, destOpType, str, i);
			break;
		}
		default: throw;
	}
}
template<class T, class S> void Function::copyScratchPadToLocal(const int& varIndex, const int& destOpType, const String& str, int& i){
	if(destOpType==templateType<T,Single>()){
		copyScratchPadToLocal_insert2<T>(varIndex);
	}
	else throw;
}
template<> void Function::copyScratchPadToLocal<float,Single>(const int& varIndex, const int& destOpType, const String& str, int& i){
	switch(destOpType){
		case S_FLOAT:{
			copyScratchPadToLocal_insert2<float>(varIndex);
			break;
		}
		case S_INT32:{
			copyMachineCode(&copyScratchPadToLocal_ele_S_Float_with_S_Int32);
			g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, compiled.length-4);
			break;
		}
		case S_DOUBLE:{
			copyMachineCode(&copyScratchPadToLocal_ele_S_Float_with_S_Time);
			g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, compiled.length-4);
			break;
		}
		default: throw;
	}
}
template<> void Function::copyScratchPadToLocal<int,Single>(const int& varIndex, const int& destOpType, const String& str, int& i){
	switch(destOpType){
		case S_FLOAT:{
			copyMachineCode(&copyScratchPadToLocal_ele_S_Int32_with_S_Float);
			g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, compiled.length-4);
			break;
		}
		case S_INT32:{
			copyScratchPadToLocal_insert2<int>(varIndex);
			break;
		}
		case S_DOUBLE:{
			copyMachineCode(&copyScratchPadToLocal_ele_S_Int32_with_S_Time);
			g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, compiled.length-4);
			break;
		}
		default: throw;
	}
}
template<> void Function::copyScratchPadToLocal<double,Single>(const int& varIndex, const int& destOpType, const String& str, int& i){
	switch(destOpType){
		case S_FLOAT:{
			copyMachineCode(&copyScratchPadToLocal_ele_S_Time_with_S_Float);
			g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, compiled.length-4);
			break;
		}
		case S_INT32:{
			copyMachineCode(&copyScratchPadToLocal_ele_S_Time_with_S_Int32);
			g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, compiled.length-4);
			break;
		}
		case S_DOUBLE:{
			copyScratchPadToLocal_insert2<double>(varIndex);
			break;
		}
		default: throw;
	}
}
////////////global
template<class T, class S> void Function::copyScratchPadToGlobal(V_<T,Single,Global>& v, const int& destOpType, const String& str, int& i){
	if(destOpType==templateType<T,Single>()){
		copyScratchPadToGlobal_insert2<T>(&v.value.refValue());
	}
	else throw;
}
template<> void Function::copyScratchPadToGlobal<float,Single>(V_<float,Single,Global>& v, const int& destOpType, const String& str, int& i){
	void* valAddress=&v.value.refValue();
	switch(destOpType){
		case S_FLOAT:{
			copyScratchPadToGlobal_insert2<float>(valAddress);
			break;
		}
		case S_INT32:{
			copyMachineCode(&copyScratchPadToGlobal_ele_S_Float_with_S_Int32);
			insertDWord(compiled.length-4, reinterpret_cast<int>(valAddress));
			break;
		}
		case S_DOUBLE:{
			copyMachineCode(&copyScratchPadToGlobal_ele_S_Float_with_S_Time);
			insertDWord(compiled.length-4, reinterpret_cast<int>(valAddress));
			break;
		}
		default: throw;
	}
}
template<> void Function::copyScratchPadToGlobal<int,Single>(V_<int,Single,Global>& v, const int& destOpType, const String& str, int& i){
	void* valAddress=&v.value.refValue();
	switch(destOpType){
		case S_FLOAT:{
			copyMachineCode(&copyScratchPadToGlobal_ele_S_Int32_with_S_Float);
			insertDWord(compiled.length-4, reinterpret_cast<int>(valAddress));
			break;
		}
		case S_INT32:{
			copyScratchPadToGlobal_insert2<int>(valAddress);
			break;
		}
		case S_DOUBLE:{
			copyMachineCode(&copyScratchPadToGlobal_ele_S_Int32_with_S_Time);
			insertDWord(compiled.length-4, reinterpret_cast<int>(valAddress));
			break;
		}
		default: throw;
	}
}
template<> void Function::copyScratchPadToGlobal<double,Single>(V_<double,Single,Global>& v, const int& destOpType, const String& str, int& i){
	void* valAddress=&v.value.refValue();
	switch(destOpType){
		case S_FLOAT:{
			copyMachineCode(&copyScratchPadToGlobal_ele_S_Time_with_S_Float);
			insertDWord(compiled.length-4, reinterpret_cast<int>(valAddress));
			break;
		}
		case S_INT32:{
			copyMachineCode(&copyScratchPadToGlobal_ele_S_Time_with_S_Int32);
			insertDWord(compiled.length-4, reinterpret_cast<int>(valAddress));
			break;
		}
		case S_DOUBLE:{
			copyScratchPadToGlobal_insert2<double>(valAddress);
			break;
		}
		default: throw;
	}
}
///////////////
/*
void Function::putInElse(int& codeLineIdx, CodeLine& cl, const String& str, int& i){
	cl.wordTo=m_word.length;

	str.stbCpp(i);
	if(str[i++]!='{')throw;

	doBody(cl, str, i);
}
*/
inline void Function::setUpElse(int& bypassMark, int& exitMark){

	copyMachineCode(&goto_ele);
	exitMark=compiled.length;

	insertDWord(bypassMark-4, compiled.length-bypassMark);

}
void Function::genCppLine(CodeLine& codeLine){
	codeLine.name="";
	for(int w=codeLine.wordFrom; w<codeLine.wordTo; w++){
		m_word[w]->toString(codeLine.name);
	}
}
inline void Function::doBody(CodeLine& cl, const String& str, int& i, int& bracketCntr){
	CodeLine* codeLine;
	const int remBracketCntr=bracketCntr++;
	for(int codeLineIdx=0; forEOLQuestion(str, i, remBracketCntr, bracketCntr); codeLineIdx++){
		if(selectedLine){
			codeLine=dynamic_cast<CodeLine*>(cl.codeLine.refValue()[codeLineIdx].p);
		}
		else{
			codeLine=new CodeLine;
			cl.codeLine.refValue()+=codeLine;
		}

		codeLine->opFrom=compiled.length;
		codeLine->wordFrom=m_word.length;

		decodeCppLine(cl.codeLine, codeLineIdx, *codeLine, str, i, bracketCntr);

		if(dealWithEndOfLine(str, i, codeLine, remBracketCntr, bracketCntr)){
			return;
		}
	}
}
void putInIf(Function& f, M_Node& clParent, int& codeLineIdx, CodeLine& cl, const String& str, int& i, int& bracketCntr){
	f.m_word+=new W_If();
	f.m_word+=new W_OpenBracket();
	str.stbCpp(i);
	if(str[i++]!='('){
		throw;
	}
	f.getFirstOperand(str, i, bracketCntr);
	while(true){
		if(dynamic_cast<W_And*>(f.m_word[f.m_word.length-1])){
			f.getFirstOperand(str, i, bracketCntr);
			str.stbCpp(i);
			f.copyMachineCode(&and_ele);
			if(str[i]!=')'){
				f.getFirstOperand(str, i, bracketCntr);
				continue;
			}
		}
		else if(dynamic_cast<W_Or*>(f.m_word[f.m_word.length-1])){
			f.getFirstOperand(str, i, bracketCntr);
			str.stbCpp(i);
			f.copyMachineCode(&or_ele);
			if(str[i]!=')'){
				f.getFirstOperand(str, i, bracketCntr);
				continue;
			}
		}
		break;
	};
	if(str[i++]!=')')throw;
	f.m_word+=new W_CloseBracket();
	str.stbCpp(i);

	doFrontBrace(str, i, bracketCntr);

	cl.opTo=f.compiled.length;
	cl.wordTo=f.m_word.length;

	f.copyMachineCode(&if_ele);
	int bypassMark=f.compiled.length;

	f.doBody(cl, str, i, bracketCntr);
//	bracketCntr--;
//	int remI=i;
//	str.stbnlCpp(++i);//we really need to check if else is immediately after if (what if there is a new line between if and else)
//	if(str[i]=='\n'){
//		str.stbnlCpp(++i);
//	}
	str.stbnlCpp(i);
	if(str.cmpCppWord("else",i)){
		str.stbCpp(i);

		CodeLine* codeLine;
		if(f.selectedLine){//if we are compiling codeLines that already exist
			codeLine=dynamic_cast<CodeLine*>(clParent[++codeLineIdx].p);
		}
		else{
			codeLine=new CodeLine;
			clParent+=codeLine;
		}

		codeLine->wordFrom=f.m_word.length;
		f.m_word+=new W_Else();

		int exitMark=0;	//exit mark does not get resolved untill all elseIfs and esles have been coded

		if(str.cmpCppWord("if",i)){
			f.setUpElse(bypassMark, exitMark);
			putInIf(f, clParent, codeLineIdx, *codeLine, str, i, bracketCntr);
		}
		else{
			f.setUpElse(bypassMark, exitMark);
//			f.putInElse(*codeLine, str, i);
			codeLine->wordTo=f.m_word.length;

			doFrontBrace(str, i, bracketCntr);

			f.doBody(*codeLine, str, i, bracketCntr);
//			bracketCntr--;
//			str.stbnlCpp(++i);
//			if(str[i]=='\n'){
//				str.stbnlCpp(++i);
//			}
		}
		f.genCppLine(*codeLine);
		f.insertDWord(exitMark-4, f.compiled.length-exitMark);
	}
	else{
//		i=remI;
		f.insertDWord(bypassMark-4, f.compiled.length-bypassMark);
	}
}
void changeFromToWord(CodeLine* cl, int add){
	cl->wordFrom+=add;
	cl->wordTo+=add;
	for(int k=0; k<cl->codeLine.length; k++){
		changeFromToWord(dynamic_cast<CodeLine*>(cl->codeLine[k].p), add);
	}
}
void goToNextSemicolon(const String& str, int& i){
	while(str[i++]!=';'){
	}
}
void goToEndBracket(const String& str, int& i){
	while(str[i++]!=')'){
	}
}
void goToIndentInCode(const String& str, int& i){
	while(str[i++]!=')'){
	}
	while(str[i++]!='{'){
	}
}
void putInFor(Function& f, M_Node& clParent, int& codeLineIdx, CodeLine& cl, const String& str, int& i, int& bracketCntr){
	int beginBreakStack=breakStack.length;
	int beginContinueStack=breakStack.length;
	f.m_word+=new W_For();
	f.m_word+=new W_OpenBracket();
	str.stbCpp(i);
	if(str[i++]!='('){
		throw;
	}
	//initilize for variables
	while(str[i]!=';'){
		f.decodeCppLine(clParent, codeLineIdx, cl, str, i, bracketCntr);
		str.stbCpp(i);
		if(str[i]==','){
		f.m_word+=new W_Comma();
			i++;
		}
		str.stbCpp(i);
	}

	if(str[i++]!=';')throw;

	f.m_word+=new W_ForSemicolon();
	cl.wordTo=f.m_word.length;
	int remWordLen=f.m_word.length;

	f.copyMachineCode(&goto_ele);
	int mark=f.compiled.length;

	int remBeginCondition=i;
	goToNextSemicolon(str, i);
	str.stbCpp(i);
	int remEndLoop=i;
	goToIndentInCode(str, i);

	cl.opTo=f.compiled.length;

	str.stbCpp(i);

	f.doBody(cl, str, i, bracketCntr);
//	str.stbnlCpp(++i);
//	if(str[i]=='\n'){
//		str.stbnlCpp(++i);
//	}
	int continueMark=f.compiled.length;
//	do post loop variables
	int remWordLen2=f.m_word.length;
	while(str[remEndLoop]!=')'){
		f.decodeCppLine(clParent, codeLineIdx, cl, str, remEndLoop, bracketCntr);
		str.stbCpp(remEndLoop);
		if(str[remEndLoop]==','){
		f.m_word+=new W_Comma();
			remEndLoop++;
		}
		str.stbCpp(remEndLoop);
	}
	f.m_word+=new W_CloseBracket();

	f.insertDWord(mark-4, f.compiled.length-mark);

	static M_Word copyWord;

	int wordDiff=f.m_word.length-remWordLen2;
	copyElements(remWordLen2, wordDiff, f.m_word, copyWord);
	f.m_word.length-=wordDiff;
	insertElements(remWordLen, 0, f.m_word, copyWord);
	cl.wordFrom-=wordDiff;
	changeFromToWord(&cl,wordDiff);

	int remWordLen3=f.m_word.length;
	int opTypeBool=f.getFirstOperand(str, remBeginCondition, bracketCntr);

	f.m_word+=new W_ForSemicolon();

	wordDiff=f.m_word.length-remWordLen3;
	copyElements(remWordLen3, wordDiff, f.m_word, copyWord);
	f.m_word.length-=wordDiff;
	insertElements(remWordLen, 0, f.m_word, copyWord);
	cl.wordFrom-=wordDiff;
	changeFromToWord(&cl,wordDiff);
	f.m_word+=new W_CloseBracket();

	f.copyMachineCode(&not_if_ele);
	f.insertDWord(f.compiled.length-4, mark-f.compiled.length);

	for(int i = breakStack.length; --i>=beginBreakStack; ){
		f.insertDWord(breakStack[i]-4, f.compiled.length-breakStack[i]);
		breakStack.length--;
	}
	for(int i = f.continueStack.length; --i>=beginContinueStack; ){
		f.insertDWord(f.continueStack[i]-4, continueMark-f.continueStack[i]);
		f.continueStack.length--;
	}
}
void putInWhile(Function& f, M_Node& clParent, int& codeLineIdx, CodeLine& cl, const String& str, int& i, int& bracketCntr){
	int beginBreakStack=breakStack.length;
	int beginContinueStack=breakStack.length;
	f.m_word+=new W_While();
	f.m_word+=new W_OpenBracket();
	str.stbCpp(i);
	if(str[i++]!='('){
		throw;
	}

	cl.wordTo=f.m_word.length;
	int remWordLen=f.m_word.length;

	f.copyMachineCode(&goto_ele);
	int mark=f.compiled.length;

	int remBeginCondition=i;
	goToEndBracket(str, i);
	str.stbCpp(--i);
	int remEndLoop=i;
	goToIndentInCode(str, i);

	cl.opTo=f.compiled.length;

	str.stbCpp(i);

	f.doBody(cl, str, i, bracketCntr);
//	str.stbnlCpp(++i);
//	if(str[i]=='\n'){
//		str.stbnlCpp(++i);
//	}
	int continueMark=f.compiled.length;
//	do post loop variables
	int remWordLen2=f.m_word.length;
	if(str[remEndLoop]!=')'){
		throw;
	}
	f.m_word+=new W_CloseBracket();

	f.insertDWord(mark-4, f.compiled.length-mark);

	static M_Word copyWord;

	int wordDiff=f.m_word.length-remWordLen2;
	copyElements(remWordLen2, wordDiff, f.m_word, copyWord);
	f.m_word.length-=wordDiff;
	insertElements(remWordLen, 0, f.m_word, copyWord);
	cl.wordFrom-=wordDiff;
	changeFromToWord(&cl,wordDiff);

	int remWordLen3=f.m_word.length;
	int opTypeBool=f.getFirstOperand(str, remBeginCondition, bracketCntr);

	wordDiff=f.m_word.length-remWordLen3;
	copyElements(remWordLen3, wordDiff, f.m_word, copyWord);
	f.m_word.length-=wordDiff;
	insertElements(remWordLen, 0, f.m_word, copyWord);
	cl.wordFrom-=wordDiff;
	changeFromToWord(&cl,wordDiff);
	f.m_word+=new W_CloseBracket();

	f.copyMachineCode(&not_if_ele);
	f.insertDWord(f.compiled.length-4, mark-f.compiled.length);

	for(int i = breakStack.length; --i>=beginBreakStack; ){
		f.insertDWord(breakStack[i]-4, f.compiled.length-breakStack[i]);
		breakStack.length--;
	}
	for(int i = f.continueStack.length; --i>=beginContinueStack; ){
		f.insertDWord(f.continueStack[i]-4, continueMark-f.continueStack[i]);
		f.continueStack.length--;
	}
}
void putInDo(Function& f, M_Node& clParent, int& codeLineIdx, CodeLine& cl, const String& str, int& i, int& bracketCntr){
	f.m_word+=new W_Do();
	str.stbCpp(i);
	doFrontBrace(str, i, bracketCntr);

	cl.opTo=f.compiled.length;
	cl.wordTo=f.m_word.length;

	int mark=f.compiled.length;

	while(str[i]!='}'){
		CodeLine* codeLine=new CodeLine;
		cl.codeLine.refValue()+=codeLine;
		codeLine->opFrom=f.compiled.length;
		codeLine->wordFrom=f.m_word.length;

		f.decodeCppLine(cl.codeLine, codeLineIdx, *codeLine, str, i, bracketCntr);

		codeLine->opTo=f.compiled.length;
		codeLine->wordTo=f.m_word.length;
		str.stbCpp(i);
		codeLine->name="";
		for(int w=codeLine->wordFrom; w<codeLine->wordTo; w++){
			f.m_word[w]->toString(codeLine->name);
		}
		str.stbCpp(++i);
	}
	CodeLine* codeLine=new CodeLine;
	clParent+=codeLine;
	codeLine->opFrom=f.compiled.length;
	codeLine->wordFrom=f.m_word.length;

	String label=str.extractCppLabel(++i);
	if(label!="while") throw;
	f.m_word+=new W_While();
	str.stbCpp(i);
	if(str[i++]!='('){
		throw;
	}
//	int bracketCntr=0;
	int opTypeBool=f.getFirstOperand(str, i, bracketCntr);
	f.copyMachineCode(&not_if_ele);
	int data=mark-f.compiled.length;
	f.insertDWord(f.compiled.length-4,data);

	codeLine->opTo=f.compiled.length;
	codeLine->wordTo=f.m_word.length;
	codeLine->name="";
	for(int w=codeLine->wordFrom; w<codeLine->wordTo; w++){
		f.m_word[w]->toString(codeLine->name);
	}
}
void putInBreak(Function& f, M_Node& clParent, int& codeLineIdx, CodeLine& cl, const String& str, int& i, int& bracketCntr){
	f.m_word+=new W_Break;
	f.copyMachineCode(&goto_ele);
	breakStack+=f.compiled.length;
}
void putInContinue(Function& f, M_Node& clParent, int& codeLineIdx, CodeLine& cl, const String& str, int& i, int& bracketCntr){
	f.m_word+=new W_Continue;
	f.copyMachineCode(&goto_ele);
	f.continueStack+=f.compiled.length;
}
void putInReturn(Function& f, M_Node& clParent, int& codeLineIdx, CodeLine& cl, const String& str, int& i, int& bracketCntr){
	f.m_word+=new W_ReturnVal();
	if(f.returnType==_VOID){
		f.compiled+=0xc3; //ret
	}
	else{

		int bracketCntr=0;

		int opTypeBool=f.getFirstOperand(str, i, bracketCntr);
		switch (f.returnType){
			case S_BOOL:{
				if(opTypeBool!=S_BOOL) throw;
				break;
			}
			case S_COLOR3F:{
				if(opTypeBool!=S_COLOR3F) throw;
				break;
			}
			case S_FLOAT:{
				if(opTypeBool!=S_FLOAT) throw;
				break;
			}
			case S_IMAGE:{
				if(opTypeBool!=S_IMAGE) throw;
				break;
			}
			case S_INT32:{
				if(opTypeBool!=S_INT32) throw;
				break;
			}
			case NODEPTR:{
				if(opTypeBool!=NODEPTR) throw;
				break;
			}
			case S_ROTATION:{
				if(opTypeBool!=S_ROTATION) throw;
				break;
			}
			case S_STRING:{
				if(opTypeBool!=S_STRING) throw;
				break;
			}
			case S_DOUBLE:{
				if(opTypeBool!=S_DOUBLE) throw;
				break;
			}
			case S_VEC2F:{
				if(opTypeBool!=S_VEC2F) throw;
				break;
			}
			case S_VEC3F:{
				if(opTypeBool!=S_VEC3F) throw;
				break;
			}
	//					case M_BOOL:	{returnOperand=new MyReturn<M_Bool>(getOperand(M_Bool(),str, i)); break;}
	//					case M_COLOR:	{returnOperand=new MyReturn<M_Color3f>(getOperand(M_Color3f(),str, i)); break;}
	//					case M_FLOAT:	{returnOperand=new MyReturn<M_Float>(getOperand(M_Float(),str, i)); break;}
	//					case M_IMAGE:	{returnOperand=new MyReturn<M_Image>(getOperand(M_Image(),str, i)); break;}
	//					case M_INT32:	{returnOperand=new MyReturn<M_Int>(getOperand(M_Int(),str, i)); break;}
	//					case M_NODE:	{returnOperand=new MyReturn<M_Node>(getOperand(M_Node(),str, i)); break;}
	//					case M_ROTATION:{returnOperand=new MyReturn<M_Rotation>(getOperand(M_Rotation(),str, i)); break;}
	//					case M_STRING:	{returnOperand=new MyReturn<M_String>(getOperand(M_String(),str, i)); break;}
	//					case M_TIME:	{returnOperand=new MyReturn<M_Double>(getOperand(M_Double(),str, i)); break;}
	//					case M_VEC2F:	{returnOperand=new MyReturn<M_Vec2f>(getOperand(M_Vec2f(),str, i)); break;}
	//					case M_VEC3F:	{returnOperand=new MyReturn<M_Vec3f>(getOperand(M_Vec3f(),str, i)); break;}
			default:throw;
		}
	}
	f.compiled+=0xc3; //ret
}

struct S_KeyWordLookUp {
	LkKSg name;
	LkKSg next;
	LookupFunction lookupFunction;
};
S_KeyWordLookUp keyWordLookUp[]={

	{"if", "",			putInIf},
	{"while", "",		putInWhile},
	{"for", "",			putInFor},
	{"do", "",			putInDo},
	{"break", "",		putInBreak},
	{"continue", "",	putInContinue},
	{"return", "",		putInReturn},
	{"M_Bool", "",		putInLocal<S_Bool,Multiple>},
	{"M_Color3f", "",		putInLocal<Color3f,Multiple>},
	{"M_Float", "",		putInLocal<float,Multiple>},
	{"M_Image", "",		0},
	{"M_Int", "",		putInLocal<int,Multiple>},
	{"M_Node", "",		putInLocal<NodePtr,Multiple>},
	{"M_Rotation", "",  putInLocal<Rotation,Multiple>},
	{"M_String", "",	putInLocal<String,Multiple>},
	{"M_Double", "",		putInLocal<double,Multiple>},
	{"M_Vec2f", "",		putInLocal<Vec2f,Multiple>},
	{"M_Vec3f", "",		putInLocal<Vec3f,Multiple>},
	{"bool", "",		putInLocal<S_Bool,Single>},
	{"Color3f", "",		putInLocal<Color3f,Single>},
	{"float", "",		putInLocal<float,Single>},
	{"Image", "",		0},
	{"int", "",		putInLocal<int,Single>},
	{"NodePtr", "",		putInLocal<NodePtr,Single>},
	{"Rotation", "",  putInLocal<Rotation,Single>},
	{"String", "",	putInLocal<String,Single>},
	{"double", "",		putInLocal<double,Single>},
	{"Vec2f", "",		putInLocal<Vec2f,Single>},
	{"Vec3f", "",		putInLocal<Vec3f,Single>},
};
//#define KEYWORD_LOOKUPLEN 28
#define KEYWORD_LOOKUPLEN (sizeof(keyWordLookUp)/sizeof(S_KeyWordLookUp))
inline bool checkSingleKeyWord(const char*& name, const char* str, int& i){
	for(int c=0, c2=i; c<sizeof(LkKSg); c++, c2++){
		if(str[c2]!=name[c]||name[c] == 0){
			if(name[c]==0 && !isLegalCppNameChar(str[c2])){
				i=c2;
				return true;
			}
			return false;
		}
	}
	return false;
}
bool getKeyWord(Function& f, M_Node& clParent, int& codeLineIdx, CodeLine& cl, const String& str, int& i, int& bracketCntr){
	int remI=i;
	for(int r=0; r<KEYWORD_LOOKUPLEN; r++){
		const char* name =keyWordLookUp[r].name;
		if(checkSingleKeyWord(name, str.c_str(), i)){
			keyWordLookUp[r].lookupFunction(f, clParent, codeLineIdx, cl, str,i, bracketCntr);
			return true;
		}
	}
	i=remI;
	return false;
}

/*
template <class T, class S> void Function::getHiddenBrackOperator(const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	switch(str[i++]){
		case '-':
		case '+':{
			--i;
			return;
		}
		case '/':{
			m_word+=new W_Divide();
			getOperand<T,S>(str, i,bracketCntr);
			str.stbCpp(i);
			copyMachineCode(division_insert<T>());
			if(str[i]=='+'||str[i]=='-'||str[i]==';'||str[i]==')'||str[i]==','){
				return;
			}
			break;
		}
		case '*':{
			m_word+=new W_Multiply();
			getOperand<T,S>(str, i,bracketCntr);
			str.stbCpp(i);
			copyMachineCode(multiplication_insert<T>());
			if(str[i]=='+'||str[i]=='-'||str[i]==';'||str[i]==')'||str[i]==','){
				return;
			}
			break;
		}
		case ')':
			--bracketCntr;
			m_word+=new W_CloseBracket();
			return;
		case ';':{
			//m_word+=new W_Semicolon();
			--i;
			return;
		}
		default:{
			throw;
		}
	}
	getHiddenBrackOperator<T,S>(str, i, bracketCntr);
}
template <class T, class S> void Function::getNextOperator(const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	switch(str[i++]){
		case '-':{
			m_word+=new W_Sub();
			getOperand<T,S>(str,i, bracketCntr);
			str.stbCpp(i);
			if(str[i]=='*'||str[i]=='/'){//if * must be hidden bracket + (getOperand * ?
				getHiddenBrackOperator<T,S>(str, i, bracketCntr);
			}
			copyMachineCode(subtraction_insert<T>());
			break;
		}
		case '+':{
			m_word+=new W_Add();
			getOperand<T,S>(str,i, bracketCntr);
			str.stbCpp(i);
			if(str[i]=='*'||str[i]=='/'){//if * must be hidden bracket + (getOperand * ?
				getHiddenBrackOperator<T,S>(str, i, bracketCntr);
			}
			copyMachineCode(addition_insert<T>());
			break;
		}
		case '/':{
			m_word+=new W_Divide();
			getOperand<T,S>(str,i, bracketCntr);
			str.stbCpp(i);
			copyMachineCode(division_insert<T>());
			break;
		}
		case '*':{
			m_word+=new W_Multiply();
			getOperand<T,S>(str,i, bracketCntr);
			str.stbCpp(i);
			copyMachineCode(multiplication_insert<T>());
			break;
		}
		case '<':
			m_word+=new W_LessThan();
			getFirstOperand<T,S>(str,i, bracketCntr);
			str.stbCpp(i);
			copyMachineCode(lessThan_insert<T>());
			return;
		case '>':
			m_word+=new W_GreaterThan();
			getFirstOperand<T,S>(str,i, bracketCntr);
			str.stbCpp(i);
			copyMachineCode(greaterThan_insert<T>());
			return;
		case '=':
			if(str[i]=='='){
				++i;
				m_word+=new W_Equality();
				getFirstOperand<T,S>(str,i, bracketCntr);
				str.stbCpp(i);
				copyMachineCode(equality_insert<T>());
				return;
			}
			else throw;
		case '!':
			if(str[i]=='='){
				++i;
				m_word+=new W_NonEquality();
				getFirstOperand<T,S>(str,i, bracketCntr);
				str.stbCpp(i);
				copyMachineCode(nonEquality_insert<T>());
				return;
			}
			else throw;
		case ')':
		case ']':
		case '{':
		case ',':
			--i;
			return;
		case ';':{
			if(!dynamic_cast<W_Semicolon*>(m_word[m_word.length-1])){
				//m_word+=new W_Semicolon();
			}
			--i;
			return;
		}
		default:{
			throw;
		}
	}
	getNextOperator<T,S>(str, i, bracketCntr);
}

template <class T, class S> void Function::getOperand(const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	char ch=str[i];
	if(ch>='0'&&ch<='9'||ch=='.'){
		CopyImmediateToScratchPad_insert<T>(str,i);
	}
	else if(ch=='('){
		m_word+=new W_OpenBracket();
		getFirstOperand<T,S>(str, ++i, ++bracketCntr);
		if(str[i++]!=')')throw;
		--bracketCntr;
		m_word+=new W_CloseBracket();
	}
	else {
		String label;
		label=str.extractCppLabel(i);
		if(label=="true"){
			CopyImmediateToScratchPad_insert(true);
		}
		else if(label=="false"){
			CopyImmediateToScratchPad_insert(false);
		}
		else if(label=="Vec2f"){
			CopyImmediateToScratchPad_insert<T>(str,i);
		}
		else if(label=="_Vec3f"){
			CopyImmediateToScratchPad_insert<T>(str,i);
		}
		else if(label=="PI"){
			copyPiToScratchPad_insert<T>(str,i);
		}
		else if(L_Variable* lv=isLocalVariable(label)){
			m_word+=new W_Local(this, lv->varIndex);
			decodeSource<T,S>(lv, str, i, bracketCntr);
		}
		else if(builtInFunction<T>(label,str, i, bracketCntr)){
		}
		else if(Function* functionCall=isFunctionCall(label)){
			getArg(*functionCall, str, i, bracketCntr);
			getNextOperator<T,S>(str,i, bracketCntr);
		}
		else if(Node* node=getNodeFromName(label)){
			if(G_Variable* gv=dynamic_cast<G_Variable*>(node)){
				m_word+=new W_Global(node);
				decodeSource<T,S>(gv, str, i, bracketCntr);
			}
			else{
				if(str[i++]!='.')throw this;
				str.stbCpp(i);
				int fieldIdx=node->getProperties().getIndex(str.getWord2(i));
				m_word+=new W_NodeDotField(node,fieldIdx);
				Field* fieldPtr=node->getField(fieldIdx);
				decodeField<T,S>(fieldPtr, str, i, bracketCntr);
			};
		}
		else throw;//'label' : undeclared identifier, //label has not been defined or declared
	}
}

template <class T, class S> inline void Function::getFirstOperand(const String& str, int& i, int& bracketCntr ){
	int remBracketCntr=bracketCntr;
	str.stbCpp(i);
	int remI=i;
	if(str[i]=='-'){
		if(str[++i]=='-'){
			throw;
		}
		else{
			str.stbCpp(i);
			char ch=str[i];
			if(ch>='0'&&ch<='9'||ch=='.'){
				i=remI;
				CopyImmediateToScratchPad_insert<T>(str,i);
			}
			else{
				m_word+=new W_Name("-");
				getOperand<T,S>(str,i, bracketCntr);
				copyMachineCode(negate_insert<T>());
			}
		}
	}
	else{
		getOperand<T,S>(str,i, bracketCntr);
	}
	while(true){
		str.stbCpp(i);
		if(str[i]!=';' && str[i]!='{' && str[i]!=')' && str[i]!=']' && str[i]!=','){
			remBracketCntr=bracketCntr;
			getNextOperator<T,S>(str,i, bracketCntr);
		}
		else break;
	}
}
template <> inline void Function::getFirstOperand<S_Bool,Single>(const String& str, int& i,int& bracketCntr){
	getOperand<S_Bool,Single>(str,i, bracketCntr);
	while(1){
		str.stbCpp(i);
		if(str[i]!=';' && str[i]!=')' && str[i]!=',' ){
			getNextOperator<S_Bool,Single>(str,i, bracketCntr);
		}
		else break;
	}
}
template <class T, class S> inline void Function::getOperandAfterLocal(const int& varIndex, const String& str, int& i, int& bracketCntr ){
	g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, compiled.length+3);
	copyMachineCode(CopyLocalToScratchPad_insert<T>());
	while(1){
		str.stbCpp(i);
		if(str[i]!=';' && str[i]!=')'){
			getNextOperator<T,S>(str,i, bracketCntr);
		}
		else break;
	}
}
*/
////end here
inline void Function::CopyImmediateToScratchPad_insert(const int& data){
	m_word+=new W_Immediate<S_Bool>(data!=0);
	copyMachineCode(&CopyImmediateToScratchPad_ele_S_Bool);
	compiled[compiled.length-4]=data;
}
template <> inline bool Function::builtInFunction<float>(const String& label, const String& str, int& i, int& bracketCntr, int& opType){
	if(label=="sqrt"){
		str.stbCpp(i);
		if(str[i++]!='('){
			throw;
		}
		m_word+=new W_Sqrt();
		m_word+=new W_OpenBracket();
		int opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
		if(opType!=S_FLOAT)throw;
		if(str[i++]!=')'){
			throw;
		}
		m_word+=new W_CloseBracket();
		copyMachineCode(&sqrt_ele_S_Float);
		opType=S_FLOAT;
	}
	else if(label=="fabs"){
		str.stbCpp(i);
		if(str[i++]!='('){
			throw;
		}
		m_word+=new W_Fabs();
		m_word+=new W_OpenBracket();
		int opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
		if(opType!=S_FLOAT)throw;
		if(str[i++]!=')'){
			throw;
		}
		m_word+=new W_CloseBracket();
		copyMachineCode(&fabs_ele_S_Float);
		opType=S_FLOAT;
	}
	else if(label=="sin"){
		str.stbCpp(i);
		if(str[i++]!='('){
			throw;
		}
		m_word+=new W_Sin();
		m_word+=new W_OpenBracket();
		int opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
		if(opType!=S_FLOAT)throw;
		if(str[i++]!=')'){
			throw;
		}
		m_word+=new W_CloseBracket();
		copyMachineCode(&sin_ele_S_Float);
		opType=S_FLOAT;
	}
	else if(label=="cos"){
		str.stbCpp(i);
		if(str[i++]!='('){
			throw;
		}
		m_word+=new W_Cos();
		m_word+=new W_OpenBracket();
		int opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
		if(opType!=S_FLOAT)throw;
		if(str[i++]!=')'){
			throw;
		}
		m_word+=new W_CloseBracket();
		copyMachineCode(&cos_ele_S_Float);
		opType=S_FLOAT;
	}
	else if(label=="asin"){
		str.stbCpp(i);
		if(str[i++]!='('){
			throw;
		}
		m_word+=new W_Asin();
		m_word+=new W_OpenBracket();
		int opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
		if(opType!=S_FLOAT)throw;
		if(str[i++]!=')'){
			throw;
		}
		m_word+=new W_CloseBracket();
		g_asm2CppAddy+=Asm2CppAddy(asinf, compiled.length+3);
		copyMachineCode(&asin_ele_S_Float);
		opType=S_FLOAT;
	}
	else if(label=="acos"){
		str.stbCpp(i);
		if(str[i++]!='('){
			throw;
		}
		m_word+=new W_Acos();
		m_word+=new W_OpenBracket();
		int opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
		if(opType!=S_FLOAT)throw;
		if(str[i++]!=')'){
			throw;
		}
		m_word+=new W_CloseBracket();
		g_asm2CppAddy+=Asm2CppAddy(acosf, compiled.length+3);
		copyMachineCode(&asin_ele_S_Float);
		opType=S_FLOAT;
	}
	else if(label=="ceil"){
		str.stbCpp(i);
		if(str[i++]!='('){
			throw;
		}
		m_word+=new W_Ceil();
		m_word+=new W_OpenBracket();
		int opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
		if(opType!=S_FLOAT)throw;
		if(str[i++]!=')'){
			throw;
		}
		m_word+=new W_CloseBracket();
		g_asm2CppAddy+=Asm2CppAddy(ceilf, compiled.length+3);
		copyMachineCode(&asin_ele_S_Float);
		opType=S_FLOAT;
	}
	else if(label=="floor"){
		str.stbCpp(i);
		if(str[i++]!='('){
			throw;
		}
		m_word+=new W_Floor();
		m_word+=new W_OpenBracket();
		int opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
		if(str[i++]!=')'){
			throw;
		}
		m_word+=new W_CloseBracket();
		if(opType==S_FLOAT){
			g_asm2CppAddy+=Asm2CppAddy(floorf, compiled.length+3);
			copyMachineCode(&asin_ele_S_Float);
			opType=S_FLOAT;
		}
		else if(opType==S_DOUBLE){
			g_asm2CppAddy+=Asm2CppAddy(0, compiled.length+3); ///needs to be resolved
			copyMachineCode(&asin_ele_S_Time);
			opType=S_DOUBLE;
		}
		else throw;
	}
	else if(label=="pow"){
		str.stbCpp(i);
		if(str[i++]!='('){
			throw;
		}
		m_word+=new W_Pow();
		m_word+=new W_OpenBracket();
		int opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
		if(opType!=S_FLOAT)throw;
		if(str[i++]!=','){
			throw;
		}
		m_word+=new W_Comma();
		opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
		if(opType!=S_FLOAT)throw;
		if(str[i++]!=')'){
			throw;
		}
		m_word+=new W_CloseBracket();
		g_asm2CppAddy+=Asm2CppAddy(powf, compiled.length+8);
		copyMachineCode(&pow_ele_S_Float);
		opType=S_FLOAT;
	}
	else if(label=="powf"){
		str.stbCpp(i);
		if(str[i++]!='('){
			throw;
		}
		m_word+=new W_Powf();
		m_word+=new W_OpenBracket();
		int opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
		if(opType!=S_FLOAT)throw;
		if(str[i++]!=','){
			throw;
		}
		m_word+=new W_Comma();
		opType=getFirstOperand(str, i, bracketCntr,S_FLOAT);
		if(opType!=S_FLOAT)throw;
		if(str[i++]!=')'){
			throw;
		}
		m_word+=new W_CloseBracket();
		g_asm2CppAddy+=Asm2CppAddy(powf, compiled.length+8);
		copyMachineCode(&pow_ele_S_Float);
		opType=S_FLOAT;
	}
	else if(label=="getTime"){
		str.stbCpp(i);
		if(str[i++]!='('){
			throw;
		}
		m_word+=new W_GetTime();
		m_word+=new W_OpenBracket();
		str.stbCpp(i);
		if(str[i++]!=')'){
			throw;
		}
		m_word+=new W_CloseBracket();

		int mark=compiled.length+2;
		copyMachineCode(&getTime_ele_S_Time);
		insertDWord(mark, reinterpret_cast<int>(&g_time));
		opType=S_DOUBLE;
	}
	else return false;
	return true;
}


template <class T, class S> inline void Function::passToFunctionByVal(const String& str, int& i, int& bracketCntr, int& justThisFunctionCallOffset){
	int opType=getFirstOperand(str, i, bracketCntr, templateType<T,S>());
	if(templateType<T,S>()!=opType)throw;
	copyMachineCode(pushScPA_insert<T, S>());
	if(str[i]==','){
		i++;
	}
	justThisFunctionCallOffset+=sizeof(T);
	g_callOffsetCntr+=sizeof(T);
}
//template <class T> inline int getAddress(V_<T,S>* var);
template <class T, class S> inline int getAddress(V_<T,S>* var){
	return reinterpret_cast<int>(&var->value.refValue());
}
template <class T, class S> inline void Function::passToFunctionByRef(const String& str, int& i, int& bracketCntr, int& justThisFunctionCallOffset){
	str.stbCpp(i);
	String label=str.extractCppLabel(i);
	if(Node* node=getNodeFromName(label)){
		if(V_<T,S>* gv=dynamic_cast<V_<T,S>*>(node)){
			m_word+=new W_Global(gv);
			if(templateType<T,S>()!=gv->getValueType())throw;
			int mark=compiled.length+1;
			copyMachineCode(pushScPA_insert<T, S>());
			insertDWord(mark, getAddress<T,S>(gv));
		}
		else{
			throw;
		}
	}
	else{
		throw;
	}
	if(str[i]==','){
		i++;
	}
	justThisFunctionCallOffset+=REF_SIZE;
	g_callOffsetCntr+=REF_SIZE;
}
void Function::getArg(Function& callFunction, const String& str, int& i, int& bracketCntr){
	int justThisFunctionCallOffset=0;
	m_word+=new W_Call(&callFunction);
	str.stbCpp(i);
	if(str[i++]!='(')throw this;
	m_word+=new W_OpenBracket();
	str.stbCpp(i);
	int variableIndex=0;
	for(; str[i]!=')' && i<str.length; variableIndex++){
		if(variableIndex!=0){
			m_word+=new W_Comma();
		}
		switch(callFunction.m_local[variableIndex]->getType()){
			case S_BOOL:{
				passToFunctionByVal<S_Bool,Single>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case S_COLOR3F:{
				passToFunctionByVal<Color3f,Single>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case S_FLOAT:{
				passToFunctionByVal<float,Single>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case S_IMAGE:{
				//passToFunctionByVal<Image,Single>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case S_INT32:{
				passToFunctionByVal<int,Single>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case NODEPTR:{
				passToFunctionByVal<NodePtr,Single>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case S_ROTATION:{
				passToFunctionByVal<Rotation,Single>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case S_STRING:{
				passToFunctionByVal<String,Single>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case S_DOUBLE:{
				passToFunctionByVal<double,Single>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case S_VEC2F:{
				passToFunctionByVal<Vec2f,Single>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case S_VEC3F:{
				passToFunctionByVal<Vec3f,Single>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case M_BOOL:{
				//passToFunctionByRef<S_Bool,Single>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case M_COLOR:{
				passToFunctionByRef<Color3f,Multiple>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case M_FLOAT:{
				passToFunctionByRef<float,Multiple>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case M_IMAGE:{
				//passToFunctionByRef<Image,Multiple>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case M_INT32:{
				passToFunctionByRef<int,Multiple>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case M_NODE:{
				passToFunctionByRef<NodePtr,Multiple>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case M_ROTATION:{
				passToFunctionByRef<Rotation,Multiple>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case M_STRING:{
				passToFunctionByRef<String,Multiple>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case M_TIME:{
				passToFunctionByRef<double,Multiple>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case M_VEC2F:{
				passToFunctionByRef<Vec2f,Multiple>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			case M_VEC3F:{
				passToFunctionByRef<Vec3f,Multiple>(str, i, bracketCntr, justThisFunctionCallOffset);
				break;
			}
			default: throw this;
		}
	}
	call2ndCompile+=ACall(&callFunction,compiled.length+2);
	copyMachineCode(&callVal_ele);

	++i;
	m_word+=new W_CloseBracket();

	g_callOffsetCntr-=justThisFunctionCallOffset;
}


void Function::copyMachineCode(unsigned char** address){
	unsigned char* eleFrom=address[0];
	unsigned char* eleTo=address[1];
	while(eleFrom<eleTo){
		compiled+=*eleFrom;
		eleFrom++;
	}
}
void Function::copyMachineCode(unsigned char* address, int length){
	for(int i=length; --i>=0;){
		compiled+=*address;
		address++;
	}
}

template <> inline void Function::CopyImmediateToScratchPad_insert<int>(const String& str, int& i){
	int data=getInt2(str, i);
	m_word+=new W_Immediate<int>(data);
	copyMachineCode(&CopyImmediateToScratchPad_ele_S_Int32);
	insertDWord(compiled.length-sizeof(int), data);
}
template <> inline void Function::CopyImmediateToScratchPad_insert<float>(const String& str, int& i){
	float data=getFloat2(str, i);
	m_word+=new W_Immediate<float>(data);
	copyMachineCode(&CopyImmediateToScratchPad_ele_S_Float);
	insertDWord(compiled.length-sizeof(int), *(int*)&data);
}
template <> inline void Function::CopyImmediateToScratchPad_insert<double>(const String& str, int& i){
	double data=getDouble(str, i);
	m_word+=new W_Immediate<double>(data);
	g_const2ndCompile+=ConstData(constData.length, compiled.length+2);
	copyMachineCode(&CopyImmediateToScratchPad_ele_S_Time);
	catConstQWord(reinterpret_cast<unsigned char*>(&data));
}
void Function::getFloatOperand(const String& str, int& i, int& bracketCntr){
	switch(getFirstOperand(str, i, bracketCntr, S_FLOAT)){
		case S_FLOAT:{
			break;
		}
		case S_INT32:{
			copyMachineCode(typeToType_insert<int,float>());
			break;
		}
		case S_DOUBLE:{
			copyMachineCode(typeToType_insert<double,float>());
			break;
		}
		default: throw;
	}
}
template <> inline void Function::CopyImmediateToScratchPad_insert<Rotation>(const String& str, int& i){
	
	m_word+=new W_S_Rotation2();
	m_word+=new W_OpenBracket();
	str.stbCpp(i);
	if(str[i++]!='(')throw;
	int bracketCntr=0;
	getFloatOperand(str, i, bracketCntr);
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	getFloatOperand(str, i, bracketCntr);
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	getFloatOperand(str, i, bracketCntr);
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	getFloatOperand(str, i, bracketCntr);
	if(str[i++]!=')')throw;
	copyMachineCode(&swapOrder_ele_S_Rotation);
	m_word+=new W_CloseBracket();
};
template <> inline void Function::CopyImmediateToScratchPad_insert<Vec2f>(const String& str, int& i){
	m_word+=new W_S_Vec2f2();
	m_word+=new W_OpenBracket();
	str.stbCpp(i);
	if(str[i++]!='(')throw;
	int bracketCntr=0;
	getFloatOperand(str, i, bracketCntr);
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	getFloatOperand(str, i, bracketCntr);
	if(str[i++]!=')')throw;
	copyMachineCode(&swapOrder_ele_S_Vec2f);
	m_word+=new W_CloseBracket();
};
template <> inline void Function::CopyImmediateToScratchPad_insert<Vec3f>(const String& str, int& i){
	m_word+=new W_S_Vec3f2();
	m_word+=new W_OpenBracket();
	str.stbCpp(i);
	if(str[i++]!='(')throw;
	int bracketCntr=0;
	getFloatOperand(str, i, bracketCntr);
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	getFloatOperand(str, i, bracketCntr);
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	getFloatOperand(str, i, bracketCntr);
	if(str[i++]!=')')throw;
	copyMachineCode(&swapOrder_ele_S_Vec3f);
	m_word+=new W_CloseBracket();
};
template <> inline void Function::CopyImmediateToScratchPad_insert<Color3f>(const String& str, int& i){
	m_word+=new W_S_Color2();
	m_word+=new W_OpenBracket();
	str.stbCpp(i);
	if(str[i++]!='(')throw;
	int bracketCntr=0;
	getFloatOperand(str, i, bracketCntr);
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	getFloatOperand(str, i, bracketCntr);
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	getFloatOperand(str, i, bracketCntr);
	if(str[i++]!=')')throw;
	copyMachineCode(&swapOrder_ele_S_Vec3f);
	m_word+=new W_CloseBracket();
};

template <> inline void Function::CopyImmediateToScratchPad_insert<S_Bool>(const String& str, int& i){
	CopyImmediateToScratchPad_insert<float>(str, i);
}

template <> inline void Function::copyPiToScratchPad_insert<int>(const String& str, int& i){
	int data=PI;
	m_word+=new W_PI;
	copyMachineCode(&CopyImmediateToScratchPad_ele_S_Int32);
	insertDWord(compiled.length-sizeof(int), data);
}
template <> inline void Function::copyPiToScratchPad_insert<float>(const String& str, int& i){
	float data=PI;
	m_word+=new W_PI;
	copyMachineCode(&CopyImmediateToScratchPad_ele_S_Float);
	insertDWord(compiled.length-sizeof(int), *(int*)&data);
}
template <> inline void Function::copyPiToScratchPad_insert<double>(const String& str, int& i){
	double data=PI;
	m_word+=new W_PI;
	g_const2ndCompile+=ConstData(constData.length, compiled.length+2);
	copyMachineCode(&CopyImmediateToScratchPad_ele_S_Time);
	catConstQWord(reinterpret_cast<unsigned char*>(&data));
}
template <class T> inline void Function::copyScratchPadToGlobal_insert2(void* valAddress){
	int mark=compiled.length+1;
	copyMachineCode(CopyScratchPadToGlobal_insert<T>());
	insertDWord(mark, reinterpret_cast<int>(valAddress));
}
template <class T> inline void Function::addScratchPadToGlobal_insert2(void* valAddress){
	int mark=compiled.length+1;
	copyMachineCode(addScratchPadToGlobal_insert<T>());
	insertDWord(mark, reinterpret_cast<int>(valAddress));
}
template <class T> inline void Function::subScratchPadToGlobal_insert2(void* valAddress){
	int mark=compiled.length+1;
	copyMachineCode(subScratchPadToGlobal_insert<T>());
	insertDWord(mark, reinterpret_cast<int>(valAddress));
}
template <class T> inline void Function::copyScratchPadToLocal_insert2(const int& variableIndex){
	copyMachineCode(CopyScratchPadToLocal_insert<T>());
	g_local2ndCompile+=Local2ndCompile(variableIndex, g_callOffsetCntr, compiled.length-4);
}
template <class T> inline void Function::copyScratchPadToLocal_insert2(const int& variableIndex, const int& offset){
	copyMachineCode(CopyScratchPadToLocal_insert<T>());
	g_local2ndCompile+=Local2ndCompile(variableIndex, g_callOffsetCntr + offset, compiled.length-4);
}
template <> inline void Function::copyScratchPadToLocal_insert2<Color3f>(const int& variableIndex){
	g_local2ndCompile+=Local2ndCompile(variableIndex,g_callOffsetCntr, compiled.length+3);
	copyMachineCode(CopyScratchPadToLocal_insert<Color3f>());
}
template <> inline void Function::copyScratchPadToLocal_insert2<Rotation>(const int& variableIndex){
	g_local2ndCompile+=Local2ndCompile(variableIndex,g_callOffsetCntr, compiled.length+3);
	copyMachineCode(CopyScratchPadToLocal_insert<Rotation>());
}
template <> inline void Function::copyScratchPadToLocal_insert2<Vec2f>(const int& variableIndex){
	g_local2ndCompile+=Local2ndCompile(variableIndex,g_callOffsetCntr, compiled.length+3);
	copyMachineCode(CopyScratchPadToLocal_insert<Vec2f>());
}
template <> inline void Function::copyScratchPadToLocal_insert2<Vec3f>(const int& variableIndex){
	g_local2ndCompile+=Local2ndCompile(variableIndex,g_callOffsetCntr, compiled.length+3);
	copyMachineCode(CopyScratchPadToLocal_insert<Vec3f>());
}
template <class T> inline void Function::subScratchPadToLocal_insert2(const int& variableIndex){
	g_local2ndCompile+=Local2ndCompile(variableIndex, g_callOffsetCntr, compiled.length+3);
	copyMachineCode(subScratchPadToLocal_insert<T>());
}
template <class T> inline void Function::addScratchPadToLocal_insert2(const int& variableIndex){
	g_local2ndCompile+=Local2ndCompile(variableIndex, g_callOffsetCntr, compiled.length+3);
	copyMachineCode(addScratchPadToLocal_insert<T>());
}
template <class T> inline void Function::addScratchPadToLocal_insert2(const int& variableIndex, const int& offset){
	g_local2ndCompile+=Local2ndCompile(variableIndex, g_callOffsetCntr+offset, compiled.length+3);
	copyMachineCode(addScratchPadToLocal_insert<T>());
}

template <> inline void Function::copyToScratchPad<S_Bool,Single>(L_Variable* v, const String& str, int& i, int& bracketCntr){
	const int varIndex=v->varIndex;
	switch(m_local[varIndex]->getType()){
		case S_BOOL:{getOperandAfterLocal<S_Bool,Single>(varIndex, str, i, bracketCntr );break;}
		case S_COLOR3F:{getOperandAfterLocal<Color3f,Single>(varIndex, str, i, bracketCntr );break;}
		case S_FLOAT:{getOperandAfterLocal<float,Single>(varIndex, str, i, bracketCntr );break;}
		case S_IMAGE:{throw;}
		case S_INT32:{getOperandAfterLocal<int,Single>(varIndex, str, i, bracketCntr );break;}
		case NODEPTR:{getOperandAfterLocal<NodePtr,Single>(varIndex, str, i, bracketCntr );break;}
		case S_ROTATION:{getOperandAfterLocal<Rotation,Single>(varIndex, str, i, bracketCntr );break;}
		case S_STRING:{getOperandAfterLocal<String,Single>(varIndex, str, i, bracketCntr );break;}
		case S_DOUBLE:{getOperandAfterLocal<double,Single>(varIndex, str, i, bracketCntr );break;}
		case S_VEC2F:{getOperandAfterLocal<Vec2f,Single>(varIndex, str, i, bracketCntr );break;}
		case S_VEC3F:{getOperandAfterLocal<Vec3f,Single>(varIndex, str, i, bracketCntr );break;}
		case M_BOOL:{getOperandAfterLocal<S_Bool,Multiple>(varIndex, str, i, bracketCntr );break;}
		case M_COLOR:{getOperandAfterLocal<Color3f,Multiple>(varIndex, str, i, bracketCntr );break;}
		case M_FLOAT:{getOperandAfterLocal<float,Multiple>(varIndex, str, i, bracketCntr );break;}
		case M_IMAGE:{throw;}
		case M_INT32:{getOperandAfterLocal<int,Multiple>(varIndex, str, i, bracketCntr );break;}
		case M_NODE:{getOperandAfterLocal<NodePtr,Multiple>(varIndex, str, i, bracketCntr );break;}
		case M_ROTATION:{getOperandAfterLocal<Rotation,Multiple>(varIndex, str, i, bracketCntr );break;}
		case M_STRING:{getOperandAfterLocal<String,Multiple>(varIndex, str, i, bracketCntr );break;}
		case M_TIME:{getOperandAfterLocal<double,Multiple>(varIndex, str, i, bracketCntr );break;}
		case M_VEC2F:{getOperandAfterLocal<Vec2f,Multiple>(varIndex, str, i, bracketCntr );break;}
		case M_VEC3F:{getOperandAfterLocal<Vec3f,Multiple>(varIndex, str, i, bracketCntr );break;}
		default: throw this;
	}
}
template <class T,class S> inline void Function::copyToScratchPad(G_Variable* v, const String& str, int& i, int& bracketCntr){
	if(v->getValueType()==templateType<T,Single>()){
		int mark=compiled.length+1;
		copyMachineCode(copyGlobalToScratchPad_insert<T>());
		insertDWord(mark, reinterpret_cast<int>(&dynamic_cast<V_<T,Single,Global>*>(v)->value.refValue()));
		str.stbCpp(i);
		if(str[i]=='+' && str[i+1]=='+'){
			i+=2;
			m_word+=new W_PlusPlus();
			g_postIncList+=v;
		}
		else if(str[i]=='-' && str[i+1]=='-'){
			i+=2;
			m_word+=new W_MinusMinus();
			g_postDecList+=v;
		}
	}
	else{
		convertVar<T,S>(v, str, i, bracketCntr);
	}
}
template <class T,class S> inline void Function::copyToScratchPad(Field* fieldPtr, const String& str, int& i, int& bracketCntr){
	if(fieldPtr->getType()==templateType<T,Single>()){
		int mark=compiled.length+1;
		copyMachineCode(copyGlobalToScratchPad_insert<T>());
		insertDWord(mark, reinterpret_cast<int>(&dynamic_cast<FD<T,S>*>(fieldPtr)->refValue()));
	}
	else{
		throw;
		//convertVar<T,S>(fieldPtr, str, i, bracketCntr);
	}
}

template <class T,class S> inline void Function::copyToScratchPad(L_Variable* v, const String& str, int& i, int& bracketCntr){

	if(v->getValueType()==templateType<T,Single>()){
		g_local2ndCompile+=Local2ndCompile(v->varIndex, g_callOffsetCntr, compiled.length+3);
		copyMachineCode(CopyLocalToScratchPad_insert<T>());

		str.stbCpp(i);
		if(str[i]=='+' && str[i+1]=='+'){
			i+=2;
			m_word+=new W_PlusPlus();
			g_postIncList+=v;
		}
		else if(str[i]=='-' && str[i+1]=='-'){
			i+=2;
			m_word+=new W_MinusMinus();
			g_postDecList+=v;
		}
	}
	else{
		convertVar<T,S>(v, str, i, bracketCntr);
	}
}


void normalize_S_Vec3fCpp(Vec3f* par){
	*par=par->normalize();
}


template <> inline void Function::copyToScratchPad<Vec3f,Single>(G_Variable* v, const String& str, int& i, int& bracketCntr){
	V_<Vec3f>* matchingType = dynamic_cast<V_<Vec3f>*>(v);
	if(matchingType){
		str.stbCpp(i);
		switch(str[i++]){
			case '.':{
				m_word+=new W_Dot();
				str.stbCpp(i);
				String label=str.extractCppLabel(i);
				if(label=="normalize"){
					str.stbCpp(i);
					if(str[i++]!='(')throw;
					str.stbCpp(i);
					if(str[i++]!=')')throw;

					m_word+=new W_Normalize();

					int mark=compiled.length+1;
					copyMachineCode(copyGlobalToScratchPad_insert<Vec3f>());
					insertDWord(mark, reinterpret_cast<int>(&matchingType->value.refValue()));

					g_asm2CppAddy+=Asm2CppAddy(normalize_S_Vec3fCpp, compiled.length+2);
					copyMachineCode(&normalize_insert_S_Vec3f);
					break;
				}
				else{
					throw;
				}
			}
			default:{
				--i;
				int mark=compiled.length+1;
				copyMachineCode(copyGlobalToScratchPad_insert<Vec3f>());
				insertDWord(mark, reinterpret_cast<int>(&matchingType->value.refValue()));
			}
		}
	}
	else{
		throw;
	}
}
template <> inline void Function::copyToScratchPad<Vec3f,Single>(L_Variable* v, const String& str, int& i, int& bracketCntr){
	if(v->getValueType()==S_VEC3F){
		str.stbCpp(i);
		switch(str[i++]){
			case '.':{
				m_word+=new W_Dot();
				str.stbCpp(i);
				String label=str.extractCppLabel(i);
				if(label=="normalize"){
					str.stbCpp(i);
					if(str[i++]!='(')throw;
					str.stbCpp(i);
					if(str[i++]!=')')throw;

					m_word+=new W_Normalize();

					g_local2ndCompile+=Local2ndCompile(v->varIndex, g_callOffsetCntr, compiled.length+3);
					copyMachineCode(CopyLocalToScratchPad_insert<Vec3f>());

					g_asm2CppAddy+=Asm2CppAddy(normalize_S_Vec3fCpp, compiled.length+2);
					copyMachineCode(&normalize_insert_S_Vec3f);
					break;
				}
				else{
					throw;
				}
			}
			default:{
				--i;
				g_local2ndCompile+=Local2ndCompile(v->varIndex, g_callOffsetCntr, compiled.length+3);
				copyMachineCode(CopyLocalToScratchPad_insert<Vec3f>());
			}
		}
	}
	else{
		throw; //can not convert Vec3f to anything
	}
}

template <class T, class T2, class T3, class GV> inline void Function::convertVar(GV*& v, const String& str, int& i, int& bracketCntr){
	if(v->getValueType()==templateType<T2,Single>()){
		copyToScratchPad<T2,Single>(v, str, i, bracketCntr);
		copyMachineCode(typeToType_insert<T2,T>());
	}
	else if(v->getValueType()==templateType<T3,Single>()){
		copyToScratchPad<T3,Single>(v, str, i, bracketCntr);
		copyMachineCode(typeToType_insert<T3,T>());
	}
	else{
		throw;
	}
};

template <> inline void Function::convertVar<float,Single>(G_Variable* v, const String& str, int& i, int& bracketCntr){
	convertVar<float, int, double, G_Variable>(v, str, i, bracketCntr);
};
template <> inline void Function::convertVar<float,Single>(L_Variable* v, const String& str, int& i, int& bracketCntr){
	convertVar<float, int, double, L_Variable>(v, str, i, bracketCntr);
};
template <> inline void Function::convertVar<int,Single>(G_Variable* v, const String& str, int& i, int& bracketCntr){
	convertVar<int, float, double, G_Variable>(v, str, i, bracketCntr);
};
template <> inline void Function::convertVar<int,Single>(L_Variable* v, const String& str, int& i, int& bracketCntr){
	convertVar<int, float, double, L_Variable>(v, str, i, bracketCntr);
};
template <> inline void Function::convertVar<double,Single>(G_Variable* v, const String& str, int& i, int& bracketCntr){
	convertVar<double, int, float, G_Variable>(v, str, i, bracketCntr);
};
template <> inline void Function::convertVar<double,Single>(L_Variable* v, const String& str, int& i, int& bracketCntr){
	convertVar<double, int, float, L_Variable>(v, str, i, bracketCntr);
};


template <> inline void Function::copyToScratchPad<S_Bool,Single>(G_Variable* v, const String& str, int& i, int& bracketCntr){
	throw;//v->boolCopyVarToScratchPad(*this, str, i, bracketCntr);
}
template <> inline void Function::copyToScratchPad<S_Bool,Single>(Field* v, const String& str, int& i, int& bracketCntr){
	throw;//v->boolCopyFieldToScratchPad(*this, str, i, bracketCntr);
}
/*
template <class T> void V_<T,Single>::boolCopyVarToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	f.getOperandAfterGlobal<T>(this, str,i,bracketCntr);
}

template <class T> void V_<T,Multiple>::boolCopyVarToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
template <class T> void V_<T,Single,Local>::boolCopyVarToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
template <class T> void V_<T,Multiple,Local>::boolCopyVarToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}

//////////////////////
template <class T> void FD<T,Single>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	f.getOperandAfterGlobal<T,Single>(this->refValue(), str,i,bracketCntr);
}
void FD<Color3f,Single>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){throw;}
void FD<Image,Single>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<NodePtr,Single>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<Rotation,Single>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<String,Single>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<Vec2f,Single>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<Vec3f,Single>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<S_Bool,Multiple>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<Color3f,Multiple>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<float,Multiple>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<Image,Multiple>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<int,Multiple>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<NodePtr,Multiple>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<Rotation,Multiple>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<String,Multiple>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<double,Multiple>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<Vec2f,Multiple>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
void FD<Vec3f,Multiple>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
template <class T> void FD<T,Multiple>::boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
	//f.getOperandAfterGlobal<T>(this, str,i,bracketCntr);
}
*/
template <class T, class S> inline void Function::getOperandAfterGlobal(const T& value, const String& str, int& i, int& bracketCntr ){
	int mark=compiled.length+1;
	copyMachineCode(copyGlobalToScratchPad_insert<T>());
	insertDWord(mark, reinterpret_cast<int>(&value));
	while(1){
		str.stbCpp(i);
		if(str[i]!=';' && str[i]!=')'){
			getNextOperator<T,S>(str,i, bracketCntr);
		}
		else break;
	}
}
////////////////////////
template <class T, class S> inline void Function::getOperandAfterGlobal(V_<T,S>* node, const String& str, int& i, int& bracketCntr ){
	int mark=compiled.length+1;
	copyMachineCode(copyGlobalToScratchPad_insert<T>());
	insertDWord(mark, reinterpret_cast<int>(&node->value.refValue()));
	while(1){
		str.stbCpp(i);
		if(str[i]!=';' && str[i]!=')'){
			getNextOperator<T,S>(str,i, bracketCntr);
		}
		else break;
	}
}
template <class T> inline void V_<T,Single,Local>::preDec_insert2(Function& f){
	g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, f.compiled.length+3);
	f.copyMachineCode(preDecLocal_insert<T>());
}
template <class T> inline void V_<T>::preDec_insert2(Function& f){
	int mark=f.compiled.length+2;
	f.copyMachineCode(preDecGlobal_insert<T>());
	f.insertDWord(mark, reinterpret_cast<int>(&this->value.refValue()));
	f.insertDWord(f.compiled.length-4, reinterpret_cast<int>(&this->value.refValue()));
}
template <class T> inline void V_<T,Single,Local>::preInc_insert2(Function& f){
	g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, f.compiled.length+3);
	f.copyMachineCode(preIncLocal_insert<T>());
}
template <class T> inline void V_<T>::preInc_insert2(Function& f){
	int mark=f.compiled.length+2;
	f.copyMachineCode(preIncGlobal_insert<T>());
	f.insertDWord(mark, reinterpret_cast<int>(&this->value.refValue()));
	f.insertDWord(f.compiled.length-4, reinterpret_cast<int>(&this->value.refValue()));
}
template <class T> inline void V_<T,Multiple,Local>::preDec_insert2(Function& f){
	throw;
}
template <> void V_<int>::preDec_insert2(Function& f){
	int mark=f.compiled.length+2;
	f.copyMachineCode(preDecGlobal_insert<int>());
	f.insertDWord(mark, reinterpret_cast<int>(&this->value.refValue()));
}
template <> void V_<Image>::preDec_insert2(Function& f){
	throw;
}
template <> void V_<Image>::preInc_insert2(Function& f){
	throw;
}
template <> void V_<Image,Multiple>::preDec_insert2(Function& f){
	throw;
}
template <> void V_<Image,Multiple>::preInc_insert2(Function& f){
	throw;
}

template <class T> void V_<T,Multiple>::preDec_insert2(Function& f){
	throw;
}
template <class T> void V_<T,Multiple>::preInc_insert2(Function& f){
	throw;
}

template <class T> inline void V_<T,Multiple,Local>::preInc_insert2(Function& f){
	throw;
}


template <class T> void FD<T,Single>::decodeAssign(Function& f, const String& str, int& i){
	int bracketCntr=0;
	int opType=f.getFirstOperand(str,i,bracketCntr);
	if(opType!=templateType<T,Single>())throw;

	int markLd=f.compiled.length+2;
	f.copyMachineCode(assignVal2FieldDirect_insert<FD<T,Single> >());
	f.insertDWord(markLd, reinterpret_cast<int>(this));
}
void FD<Image>::decodeAssign(Function& f, const String& str, int& i){
	throw;
}
void FD<Color3f>::decodeAssign(Function& f, const String& str, int& i){
	int bracketCntr=0;
	int opType=f.getFirstOperand(str, i, bracketCntr);
	if(opType!=S_COLOR3F)throw;

	int markLd=f.compiled.length+2;
	f.copyMachineCode(assignVal2FieldDirect_insert<FD<Color3f,Single> >());
	f.insertDWord(markLd, reinterpret_cast<int>(this));
}
void FD<NodePtr>::decodeAssign(Function& f, const String& str, int& i){
	throw;
}
void FD<Rotation>::decodeAssign(Function& f, const String& str, int& i){
	int bracketCntr=0;
	int opType=f.getFirstOperand(str, i, bracketCntr);
	if(opType!=S_ROTATION)throw;

	int markLd=f.compiled.length+2;
	f.copyMachineCode(assignVal2FieldDirect_insert<FD<Rotation,Single> >());
	f.insertDWord(markLd, reinterpret_cast<int>(this));
}
void FD<String>::decodeAssign(Function& f, const String& str, int& i){
	throw;
}
void FD<Vec2f>::decodeAssign(Function& f, const String& str, int& i){
	int bracketCntr=0;
	int opType=f.getFirstOperand(str, i, bracketCntr);
	if(opType!=S_VEC2F)throw;

	int markLd=f.compiled.length+2;
	f.copyMachineCode(assignVal2FieldDirect_insert<FD<Vec2f,Single> >());
	f.insertDWord(markLd, reinterpret_cast<int>(this));
}
void FD<Vec3f>::decodeAssign(Function& f, const String& str, int& i){
	int bracketCntr=0;
	int opType=f.getFirstOperand(str, i, bracketCntr);
	if(opType!=S_VEC3F)throw;

	int markLd=f.compiled.length+2;
	f.copyMachineCode(assignVal2FieldDirect_insert<FD<Vec3f,Single> >());
	f.insertDWord(markLd, reinterpret_cast<int>(this));
}
void FD<Image,Multiple>::decodeAssign(Function& f, const String& str, int& i){
	throw;
}
template <class T> void FD<T,Multiple>::decodeAssign(Function& f, const String& str, int& i){
	str.stbCpp(i);
	String label=str.extractCppLabel(i);
	Node* node=getNodeFromName(label);
	M_<T>* source=0;
	if(G_Variable* gv=dynamic_cast<G_Variable*>(node)){
		f.m_word+=new W_Global(gv);
		source=&dynamic_cast<V_<T,Multiple>*>(gv)->value.refValue();
	}
	else{
		if(str[i++]!='.')throw this;
		str.stbCpp(i);
		int fieldIdx=node->getProperties().getIndex(str.getWord2(i));
		f.m_word+=new W_NoOutputNode(node);
		f.m_word+=new W_NoOutputDot();
		f.m_word+=new W_Field(node,fieldIdx);
		FD<T,Multiple>* mfFieldPtr=dynamic_cast<FD<T,Multiple>*>(node->getField(fieldIdx));
		source=&mfFieldPtr->refValue();
		str.stbCpp(i);
	}
	int markPush=f.compiled.length+1;
	int markLd=f.compiled.length+6;
	f.copyMachineCode(assignVal2FieldDirect_insert<FD<T,Multiple> >());
	f.insertDWord(markPush, reinterpret_cast<int>(source));
	f.insertDWord(markLd, reinterpret_cast<int>(this));
}
template <class T> int FD<T>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	int mark=f.compiled.length+1;
	f.copyMachineCode(copyGlobalToScratchPad_insert<T>());
	f.insertDWord(mark, reinterpret_cast<int>(&refValue()));
	return getType();
}
int FD<String>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
int FD<NodePtr>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
int FD<Vec2f>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
int FD<Vec3f>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	int mark=f.compiled.length+1;
	f.copyMachineCode(copyGlobalToScratchPad_insert<Vec3f>());
	f.insertDWord(mark, reinterpret_cast<int>(&refValue()));
	return S_VEC3F;
}
int FD<Rotation>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	int mark=f.compiled.length+1;
	f.copyMachineCode(copyGlobalToScratchPad_insert<Rotation>());
	f.insertDWord(mark, reinterpret_cast<int>(&refValue()));
	return S_ROTATION;
}
int FD<Color3f>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	int mark=f.compiled.length+1;
	f.copyMachineCode(copyGlobalToScratchPad_insert<Color3f>());
	f.insertDWord(mark, reinterpret_cast<int>(&refValue()));
	return S_COLOR3F;
}
template <class T> int FD<T,Multiple>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	void* valAddress=&refValue();
	switch(str[i++]){
		case '.':{
			f.m_word+=new W_Dot();
			str.stbCpp(i);
			String label=str.extractCppLabel(i);
			if(label=="length"){
				f.m_word+=new W_Length();
				str.stbCpp(i);
				int mark=f.compiled.length+1;
				copyMachineCodeFM(copyGlobalLengthToScratchPad_doOp);
				f.insertDWord(mark, reinterpret_cast<int>(valAddress));
			}
			else throw;
			return S_INT32;
			break;
		}
		case '[':{
			f.m_word+=new W_OpenSquareBracket();
			int indexOpType=f.getFirstOperand(str,i,bracketCntr,S_INT32);
			switch(indexOpType){
				case S_FLOAT:{
					f.copyMachineCode(typeToType_insert<float,int>());
					break;
				}
				case S_INT32:{
					break;
				}
				case S_DOUBLE:{
					f.copyMachineCode(typeToType_insert<double,int>());
					break;
				}
				default:{
					throw;
				}
			}
			if(str[i++]!=']')throw;
			f.m_word+=new W_CloseSquareBracket();
			str.stbCpp(i);
			switch(str[i++]){
				case '.':{
					f.m_word+=new W_Dot();
//				template <class T> void V_<T, Multiple>::copyFloatElementComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
					str.stbCpp(i);
					if(str.cmpCppWord("x",i)){
						f.m_word+=new W_X;

						int mark=f.compiled.length+1;
						f.copyMachineCode(&copyGlobalComponent_x_toScratchPad_ele_M_Vec3f);
						f.insertDWord(mark, reinterpret_cast<int>(valAddress));
					}
					else if(str.cmpCppWord("y",i)){
						f.m_word+=new W_Y;

						int mark=f.compiled.length+1;
						f.copyMachineCode(&copyGlobalComponent_y_toScratchPad_ele_M_Vec3f);
						f.insertDWord(mark, reinterpret_cast<int>(valAddress));
					}
					else if(str.cmpCppWord("z",i)){
						f.m_word+=new W_Z;

						int mark=f.compiled.length+1;
						f.copyMachineCode(&copyGlobalComponent_z_toScratchPad_ele_M_Vec3f);
						f.insertDWord(mark, reinterpret_cast<int>(valAddress));
					}
//				}
					return S_FLOAT;
				}
				default:{
					--i;
					int mark=f.compiled.length+1;
					//f.copyMachineCode(&copyGlobalElement_toScratchPad_ele_M_Int32);
					f.copyMachineCode(copyGlobalElement_toScratchPad_insert<M_<T> >());
					f.insertDWord(mark, reinterpret_cast<int>(valAddress));

					return templateType<T,Single>();
				}
			}
			
			break;
		}
		default:{
			--i;
			//copyAddressToScratchPad(f, str, i);
			return templateType<T,Multiple>();
		}
	}
}

////////////
template <class T> void FD<T>::decodeAddAssign(Function& f, const String& str, int& i){
	throw;
}
void FD<String>::decodeAddAssign(Function& f, const String& str, int& i){
	throw;
}
void FD<NodePtr>::decodeAddAssign(Function& f, const String& str, int& i){
	throw;
}
void FD<Vec2f>::decodeAddAssign(Function& f, const String& str, int& i){
	throw;
}
void FD<Vec3f>::decodeAddAssign(Function& f, const String& str, int& i){
	throw;
}
void FD<Rotation>::decodeAddAssign(Function& f, const String& str, int& i){
	throw;
}
void FD<Color3f>::decodeAddAssign(Function& f, const String& str, int& i){
	throw;
}

template <class T> void FD<T,Multiple>::decodeAddAssign(Function& f, const String& str, int& i){
	int bracketCntr=0;
	int opType=f.getFirstOperand(str,i,bracketCntr);
	if(opType!=templateType<T,Single>())throw;
	int mark=f.compiled.length+1;
	f.copyMachineCode(addAssignElement2FieldDirect_insert<FD<T,Multiple> >());
	f.insertDWord(mark, reinterpret_cast<int>(this));
}
template <class T>inline void Function::decodeAssign(V_<T,Single,Global>& v, const String& str, int& i){
	int bracketCntr=0;
	int opType=getFirstOperand(str, i, bracketCntr, templateType<T,Single>());
	copyScratchPadToGlobal<T,Single>(v, opType, str, i);
}
template <>inline void Function::decodeAssign(V_<Vec3f,Single,Local>& v, const String& str, int& i){
	int bracketCntr=0;
	getFirstOperand(str, i,bracketCntr, S_VEC3F);
	copyScratchPadToLocal_insert2<Vec3f>(v.varIndex);
}

template <class T>inline void Function::decodeAssign(V_<T,Multiple,Global>& v, const String& str, int& i){
	int bracketCntr=0;

	String label=str.extractCppLabel(i);

	if(Node* node=getNodeFromName(label)){

		M_<T>* source=0;
		if(G_Variable* gv=dynamic_cast<G_Variable*>(node)){
			m_word+=new W_Global(gv);
			source=&dynamic_cast<V_<T,Multiple>*>(gv)->value.refValue();
		}
		else{
			if(str[i++]!='.')throw this;
			str.stbCpp(i);
			int fieldIdx=node->getProperties().getIndex(str.getWord2(i));
			m_word+=new W_NoOutputNode(node);
			m_word+=new W_NoOutputDot();
			m_word+=new W_Field(node,fieldIdx);
			FD<T,Multiple>* mfFieldPtr=dynamic_cast<FD<T,Multiple>*>(node->getField(fieldIdx));
			source=&mfFieldPtr->refValue();
			str.stbCpp(i);
		}
		int markPush=compiled.length+1;
		int markLd=compiled.length+6;
		copyMachineCodeM(copy_GM_TO_GM_doOp);
		insertDWord(markPush, reinterpret_cast<int>(source));
		insertDWord(markLd, reinterpret_cast<int>(&v.value.refValue()));
		g_asm2CppAddy+=Asm2CppAddy(m_assignOperator_addressLookUp[templateType<T,Single>()], compiled.length-4);
	}
	else throw;
}

template <class T>inline void Function::decodeAssign(V_<T,Single,Local>& v, const String& str, int& i){
	int bracketCntr=0;
	int opType=getFirstOperand(str, i,bracketCntr,templateType<T,Single>());
	if(opType!=templateType<T,Single>())throw;
	copyScratchPadToLocal_insert2<T>(v.varIndex);
}
template <class T>inline void Function::decodeAddAssign(V_<T,Single,Global>& v, const String& str, int& i){
	int bracketCntr=0;
	int opType=getFirstOperand(str, i,bracketCntr,templateType<T,Single>());
	if(opType!=templateType<T,Single>())throw;
	addScratchPadToGlobal_insert2<T>(&v.value.refValue());
}
template <class T>inline void Function::decodeAddAssign(V_<T,Single,Local>& v, const String& str, int& i){
	int bracketCntr=0;
	int opType=getFirstOperand(str, i,bracketCntr,templateType<T,Single>());
	if(opType!=templateType<T,Single>())throw;
	addScratchPadToLocal_insert2<T>(v.varIndex);
}
template <class T>inline void Function::decodeSubAssign(V_<T,Single,Global>& v, const String& str, int& i){
	int bracketCntr=0;
	int opType=getFirstOperand(str, i,bracketCntr,templateType<T,Single>());
	if(opType!=templateType<T,Single>())throw;
	subScratchPadToGlobal_insert2<T>(&v.value.refValue());
}
template <class T>inline void Function::decodeSubAssign(V_<T,Single,Local>& v, const String& str, int& i){
	int bracketCntr=0;
	int opType=getFirstOperand(str, i,bracketCntr,templateType<T,Single>());
	if(opType!=templateType<T,Single>())throw;
	subScratchPadToLocal_insert2<T>(v.varIndex);
}
M_<Vec3f>& operator_addAssignElement_S_Vec3f(M_<Vec3f>& owner, Vec3f& par){
	owner=par;
	return owner;
}
template <class T> M_<T>& operator_addAssignElement_ele(M_<T>& owner, T& par){
	owner+=par;
	return owner;
}
template <class T> inline void* operator_addAssignElement_insert(){
	return &operator_addAssignElement_ele<T>;
}
template <class T> inline void Function::decodeAddAssign(V_<T,Multiple,Global>& v, const String& str, int& i){
	int bracketCntr=0;
	if(getFirstOperand(str, i, bracketCntr, templateType<T,Single>())!=templateType<T,Single>()){
//		throw;
	}
	int mark=compiled.length+2;
	copyMachineCode(addAssignElement2ValDirect_insert<M_<T> >());
	insertDWord(mark, reinterpret_cast<int>(&v.value.refValue()));
	g_asm2CppAddy+=Asm2CppAddy(operator_addAssignElement_insert<T>(),compiled.length-4-3);
}
//	copyMachineCode(typeToType_insert<int,float>());


int V_<S_Bool,Single>::getType()const{return G_S_BOOL;}
int V_<Color3f,Single>::getType()const{return G_S_COLOR;}
int V_<float,Single>::getType()const{return G_S_FLOAT;}
int V_<int,Single>::getType()const{return G_S_INT32;}
int V_<Image,Single>::getType()const{return G_S_IMAGE;}
int V_<NodePtr,Single>::getType()const{return G_NODEPTR;}
int V_<Rotation,Single>::getType()const{return G_S_ROTATION;}
int V_<String,Single>::getType()const{return G_S_STRING;}
int V_<double,Single>::getType()const{return G_S_TIME;}
int V_<Vec2f,Single>::getType()const{return G_S_VEC2F;}
int V_<Vec3f,Single>::getType()const{return G_S_VEC3F;}

int V_<S_Bool,Multiple>::getType()const{return G_M_BOOL;}
int V_<Color3f,Multiple>::getType()const{return G_M_COLOR;}
int V_<float,Multiple>::getType()const{return G_M_FLOAT;}
int V_<int,Multiple>::getType()const{return G_M_INT32;}
int V_<Image,Multiple>::getType()const{return G_M_IMAGE;}
int V_<NodePtr,Multiple>::getType()const{return G_M_NODE;}
int V_<Rotation,Multiple>::getType()const{return G_M_ROTATION;}
int V_<String,Multiple>::getType()const{return G_M_STRING;}
int V_<double,Multiple>::getType()const{return G_M_TIME;}
int V_<Vec2f,Multiple>::getType()const{return G_M_VEC2F;}
int V_<Vec3f,Multiple>::getType()const{return G_M_VEC3F;}

void createLinkInfo(){
	new V_<S_Bool,Single>;
	new V_<Color3f,Single>;
	new V_<float,Single>;
	new V_<int,Single>;
	new V_<Image,Single>;
	new V_<NodePtr,Single>;
	new V_<Rotation,Single>;
	new V_<String,Single>;
	new V_<double,Single>;
	new V_<Vec2f,Single>;
	new V_<Vec3f,Single>;

	new V_<S_Bool,Multiple>;
	new V_<Color3f,Multiple>;
	new V_<float,Multiple>;
	new V_<int,Multiple>;
	new V_<Image,Multiple>;
	new V_<NodePtr,Multiple>;
	new V_<Rotation,Multiple>;
	new V_<String,Multiple>;
	new V_<double,Multiple>;
	new V_<Vec2f,Multiple>;
	new V_<Vec3f,Multiple>;
}
extern "C" int resizeContainer_call_M_Int32(M_Int& m_int32, const int newLength){
	return m_int32.length=newLength;
}
extern "C" int* newCpp(int newSize){
	return new int [newSize];
}
extern "C" double* newDoubleCpp(int newSize){
	return new double [newSize];
}
extern "C" Rotation* newRotationCpp(int newSize){
	return new Rotation [newSize];
}
extern "C" Vec2f* newS_Vec2fCpp(int newSize){
	return new Vec2f [newSize];
}
extern "C" Vec3f* newS_Vec3fCpp(int newSize){
	return new Vec3f [newSize];
}
extern "C" void deleteCpp(int* p){
	delete[] p;
}
extern "C" void deleteS_Vec2fCpp(Vec3f* p){
	delete[] p;
}
template <class T> inline void* newAddress(){
	return newCpp;
}
template <> inline void* newAddress<double>(){
	return newS_Vec3fCpp;
}
template <> inline void* newAddress<Rotation>(){
	return newS_Vec3fCpp;
}
template <> inline void* newAddress<Vec2f>(){
	return newS_Vec2fCpp;
}
template <> inline void* newAddress<Vec3f>(){
	return newS_Vec3fCpp;
}
template <class T> void Function::writeProperty(V_<T,Multiple,Global>& v, const String& str, int& i){
	int bracketCntr=0;
	void* valAddress=&v.value.refValue();

	str.stbCpp(i);
	if(str.cmpCppWord("length",i)==false)throw;
	m_word+=new W_Length();
	str.stbCpp(i);
	if(str[i++]!='=')throw;
	m_word+=new W_Assignment();
	int opType=getFirstOperand(str,i,bracketCntr,S_INT32);
	if(opType!=S_INT32)throw;

	int mark=compiled.length+1;
	g_asm2CppAddy+=Asm2CppAddy(newAddress<T>(),compiled.length+23);
	copyMachineCode(resizeContainer_insert<M_<T> >());
	insertDWord(mark, reinterpret_cast<int>(valAddress));
}
template <class T> void Function::writeProperty(V_<T,Single,Global>& v, const String& str, int& i){
	throw;
}
inline void Function::writeGlobalFloatComponent(unsigned char*& valAddress, Word* componentName, const int& offset, const String& str, int& i, int& bracketCntr){
	m_word+=componentName;
	str.stbCpp(i);
	switch(str[i++]){
		case '=':{
			m_word+=new W_Assignment();
			int opType=getFirstOperand(str,i,bracketCntr,S_FLOAT);
			if(opType!=S_FLOAT)throw;
			copyScratchPadToGlobal_insert2<float>(valAddress+sizeof(float)*offset);
			break;
		}
		case '+':{
			switch (str[i++]){
				case '+':{
					m_word+=new W_PlusPlus();
					throw;
					//preIncGlobal_insert2(this);
					break;
				}
				case '=':{
					m_word+=new W_PlusAssignment();
					int opType=getFirstOperand(str,i,bracketCntr,S_FLOAT);
					if(opType!=S_FLOAT)throw;
					addScratchPadToGlobal_insert2<float>(valAddress+sizeof(float)*offset);
					break;
				}
				default: throw;
			}
			break;
		}
		default: throw;
	}
}

template <> void Function::writeProperty(V_<Color3f,Single,Global>& v, const String& str, int& i){
	int bracketCntr=0;
	unsigned char* valAddress=(unsigned char*)&v.value.refValue();

	str.stbCpp(i);
	if(str.cmpCppWord("r",i)){
		writeGlobalFloatComponent(valAddress, new W_R, 0, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("g",i)){
		writeGlobalFloatComponent(valAddress, new W_G, 1, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("b",i)){
		writeGlobalFloatComponent(valAddress, new W_B, 2, str, i, bracketCntr);
	}
	else throw;
}
template <> void Function::writeProperty(V_<Rotation,Single,Global>& v, const String& str, int& i){
	int bracketCntr=0;
	unsigned char* valAddress=(unsigned char*)&v.value.refValue();

	str.stbCpp(i);
	if(str.cmpCppWord("x",i)){
		writeGlobalFloatComponent(valAddress, new W_X, 0, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("y",i)){
		writeGlobalFloatComponent(valAddress, new W_Y, 1, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("z",i)){
		writeGlobalFloatComponent(valAddress, new W_Z, 2, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("angle",i)){
		writeGlobalFloatComponent(valAddress, new W_Angle, 3, str, i, bracketCntr);
	}
	else throw;
}
template <> void Function::writeProperty(V_<Vec2f,Single,Global>& v, const String& str, int& i){
	int bracketCntr=0;
	unsigned char* valAddress=(unsigned char*)&v.value.refValue();

	str.stbCpp(i);
	if(str.cmpCppWord("x",i)){
		writeGlobalFloatComponent(valAddress, new W_X, 0, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("y",i)){
		writeGlobalFloatComponent(valAddress, new W_Y, 1, str, i, bracketCntr);
	}
	else throw;
}
template <> void Function::writeProperty(V_<Vec3f,Single,Global>& v, const String& str, int& i){
	int bracketCntr=0;
	unsigned char* valAddress=(unsigned char*)&v.value.refValue();

	str.stbCpp(i);
	if(str.cmpCppWord("x",i)){
		writeGlobalFloatComponent(valAddress, new W_X, 0, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("y",i)){
		writeGlobalFloatComponent(valAddress, new W_Y, 1, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("z",i)){
		writeGlobalFloatComponent(valAddress, new W_Z, 2, str, i, bracketCntr);
	}
	else throw;
}

/*
//this is like get operator
template <> void V_<Vec3f,Single>::boolCopyVarToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	
	str.stbCpp(i);
	if(str[i]=='.'){
		f.m_word+=new W_Dot();
		int opTypeA=S_FLOAT;
		i++;
		copyComponentToScratchPad(f, str, i, bracketCntr);
		str.stbCpp(i);
		switch(str[i++]){
			case '-':{
				throw;
			}
			case '+':{
				throw;
			}
			case '/':{
				throw;
			}
			case '*':{
				throw;
			}
			case '<':{
				f.m_word+=new W_LessThan();
				int opTypeB=f.getOperand(str, i, bracketCntr, opTypeA);
				
				if(opTypeA == S_FLOAT && opTypeB == S_FLOAT){
					f.copyMachineCode(lessThan_insert<float>());
					return;
				}
				else throw;
			}
			case '>':
				throw;
			case '=':
				throw;
			case '!':
				throw;
			case ')':
			case ']':
			case '{':
			case ',':
				--i;
				throw;
			case ';':{
				--i;
				throw;
			}
			default:{
				throw;
			}
		}
//		getNextOperator<T,S>(str, i, bracketCntr);
	}
	else {
		throw;
	}
}
*/
template <class T> void Function::writeProperty(V_<T,Single,Local>& v, const String& str, int& i){
	throw;
}
inline void Function::decodeFloatComponent(const int& varIndex, const char* componentName, const int& offset, const String& str, int& i, int& bracketCntr){
	m_word+=new W_Name(componentName);
	str.stbCpp(i);
	switch(str[i++]){
		case '=':{
			m_word+=new W_Assignment();
			int opType=getFirstOperand(str,i,bracketCntr,S_FLOAT);
			if(opType!=S_FLOAT)throw;
			copyScratchPadToLocal_insert2<float>(varIndex, sizeof(float)*offset);
			break;
		}
		case '+':{
			switch (str[i++]){
				case '+':{
					m_word+=new W_PlusPlus();
					throw;
					//preIncGlobal_insert2(this);
					break;
				}
				case '=':{
					m_word+=new W_PlusAssignment();
					int opType=getFirstOperand(str,i,bracketCntr,S_FLOAT);
					if(opType!=S_FLOAT)throw;
					addScratchPadToLocal_insert2<float>(varIndex, sizeof(float)*offset);
					break;
				}
				default: throw;
			}
			break;
		}
		default: throw;
	}
}
template <> void Function::writeProperty(V_<Color3f,Single,Local>& v, const String& str, int& i){
	int bracketCntr=0;
	int varIndex=v.varIndex;

	str.stbCpp(i);
	if(str.cmpCppWord("r",i)){
		decodeFloatComponent(varIndex, "r", 0, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("g",i)){
		decodeFloatComponent(varIndex, "g", 1, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("b",i)){
		decodeFloatComponent(varIndex, "b", 2, str, i, bracketCntr);
	}
}
template <> void Function::writeProperty(V_<Vec2f,Single,Local>& v, const String& str, int& i){
	int bracketCntr=0;
	int varIndex=v.varIndex;

	str.stbCpp(i);
	if(str.cmpCppWord("x",i)){
		decodeFloatComponent(varIndex, "x", 0, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("y",i)){
		decodeFloatComponent(varIndex, "y", 1, str, i, bracketCntr);
	}
}
template <> void Function::writeProperty(V_<Vec3f,Single,Local>& v, const String& str, int& i){
	int bracketCntr=0;
	int varIndex=v.varIndex;

	str.stbCpp(i);
	if(str.cmpCppWord("x",i)){
		decodeFloatComponent(varIndex, "x", 0, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("y",i)){
		decodeFloatComponent(varIndex, "y", 1, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("z",i)){
		decodeFloatComponent(varIndex, "z", 2, str, i, bracketCntr);
	}
}
template <> void Function::writeProperty(V_<Rotation,Single,Local>& v, const String& str, int& i){
	int bracketCntr=0;
	int varIndex=v.varIndex;

	str.stbCpp(i);
	if(str.cmpCppWord("x",i)){
		decodeFloatComponent(varIndex, "x", 0, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("y",i)){
		decodeFloatComponent(varIndex, "y", 1, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("z",i)){
		decodeFloatComponent(varIndex, "z", 2, str, i, bracketCntr);
	}
	else if(str.cmpCppWord("angle",i)){
		decodeFloatComponent(varIndex, "angle", 3, str, i, bracketCntr);
	}
}

template <class T> void Function::decodeGlobalElementProperty(M_<T>& value, const String& str, int& i){
	throw;
}

template <class C> inline void Function::decodeGlobalElementComponent(M_Vec3f& value, const String& str, int& i){
	int bracketCntr=0;
	void* valAddress=&value;
	str.stbCpp(i);

	switch(str[i++]){
		case '=':{
			m_word+=new W_Assignment();
			int opType=getFirstOperand(str,i,bracketCntr,S_FLOAT);
			if(opType!=S_FLOAT)throw;

			int mark=compiled.length+1;
			copyMachineCode(copyScratchPadToGlobalComponent_M_Vec3f_insert<C>());
			insertDWord(mark, reinterpret_cast<int>(valAddress));
			break;
		}
		case '-':{
		}
		case '+':{
			switch (str[i++]){
				case '+':{
					m_word+=new W_PlusPlus();
					throw;
					//preIncGlobal_insert2(this);
					break;
				}
				case '=':{
					m_word+=new W_PlusAssignment();
					int opType=getFirstOperand(str,i,bracketCntr,S_FLOAT);
					if(opType!=S_FLOAT)throw;

					int mark=compiled.length+1;
					copyMachineCode(addScratchPadToGlobalComponent_M_Vec3f_insert<C>());
					insertDWord(mark, reinterpret_cast<int>(valAddress));

					break;
				}
				default: throw;
			}
			break;
		}
		default: throw;
	}
}

template <> void Function::decodeGlobalElementProperty(M_Vec3f& value, const String& str, int& i){
	int bracketCntr=0;
	void* valAddress=&value;

	str.stbCpp(i);
	if(str.cmpCppWord("x",i)){
		m_word+=new W_X;
		decodeGlobalElementComponent<Component_X>(value, str, i);
	}
	else if(str.cmpCppWord("y",i)){
		m_word+=new W_Y;
		decodeGlobalElementComponent<Component_Y>(value, str, i);
	}
	else if(str.cmpCppWord("z",i)){
		m_word+=new W_Z;
		decodeGlobalElementComponent<Component_Z>(value, str, i);
	}
	else throw;
}
template <class T> inline void Function::refAssignElement(V_<T,Multiple,Local>& ref, const String& str, int& i){
	int bracketCntr=0;

	str.stbCpp(i);
	m_word+=new W_OpenSquareBracket();
	int opType=getFirstOperand(str,i,bracketCntr,S_INT32);
	if(opType!=S_INT32)throw;
	if(str[i++]!=']')throw;
	m_word+=new W_CloseSquareBracket();
	str.stbCpp(i);
	switch(str[i++]){
		case '=':{
			m_word+=new W_Assignment();
			int opType=getFirstOperand(str,i,bracketCntr,templateType<T,Single>());
			if(opType!=templateType<T,Single>()) throw;

			g_local2ndCompile+=Local2ndCompile(ref.varIndex, g_callOffsetCntr, compiled.length+MOV_MEM_ESP);
			//copyMachineCode(writeElement_ref_insert<M_<T> >());
			copyCode_writeElement_ref_insert<M_<T>>();

			break;
		}
		case '.':{
			throw;
			str.stbCpp(i);
			m_word+=new W_Dot();
			//decodeGlobalElementProperty<T>(value, str, i);
			break;
		}
	}
}
template <class T> inline void Function::globalAssignElement(M_<T>& value, const String& str, int& i){
	int bracketCntr=0;
	void* valAddress=&value;

	str.stbCpp(i);
	m_word+=new W_OpenSquareBracket();
	int opType=getFirstOperand(str,i,bracketCntr,S_INT32);
	if(opType!=S_INT32)throw;
	if(str[i++]!=']')throw;
	m_word+=new W_CloseSquareBracket();
	str.stbCpp(i);
	switch(str[i++]){
		case '=':{
			m_word+=new W_Assignment();
			int opType=getFirstOperand(str,i,bracketCntr,templateType<T,Single>());
			if(opType!=templateType<T,Single>()) throw;

			int mark=compiled.length+1;
			copyMachineCode(writeElement_insert<M_<T> >());
			insertDWord(mark, reinterpret_cast<int>(valAddress));
			break;
		}
		case '.':{
			str.stbCpp(i);
			m_word+=new W_Dot();
			decodeGlobalElementProperty<T>(value, str, i);
			break;
		}
	}
}
template <class T, class S> inline void Function::decodeSource(G_Variable* v, const String& str, int& i, int& bracketCntr){
	copyToScratchPad<T,S>(v, str, i, bracketCntr);
}
template <> inline void Function::decodeSource<float,Single>(G_Variable* v, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	switch(str[i++]){
		case '.':{
			m_word+=new W_Dot();
			v->copyComponentToScratchPad(*this, str, i, bracketCntr);
			break;
		}
		case '[':{
			m_word+=new W_OpenSquareBracket();
			int opType=getFirstOperand(str,i,bracketCntr,S_INT32);
			if(opType!=S_INT32)throw;
			if(str[i++]!=']')throw;
			m_word+=new W_CloseSquareBracket();
			str.stbCpp(i);
			switch(str[i++]){
				case '.':{
					m_word+=new W_Dot();
					v->copyFloatElementComponentToScratchPad(*this, str, i, bracketCntr);
					break;
				}
				default: throw;
			}
			
			break;
		}
		default:{
			--i;
			copyToScratchPad<float,Single>(v, str, i, bracketCntr);
		}
	}
}
template <class T, class S> inline void Function::decodeSource(L_Variable* v, const String& str, int& i, int& bracketCntr){
	copyToScratchPad<T,S>(v, str, i, bracketCntr);
}
template <> inline void Function::decodeSource<float,Single>(L_Variable* v, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	switch(str[i++]){
		case '.':{
			m_word+=new W_Dot();
			v->copyComponentToScratchPad(*this, str, i, bracketCntr);
			break;
		}
		case '[':{
			m_word+=new W_OpenSquareBracket();
			int opType=getFirstOperand(str,i,bracketCntr,S_FLOAT);
			if(opType!=S_FLOAT)throw;
			if(str[i++]!=']')throw;
			m_word+=new W_CloseSquareBracket();
			str.stbCpp(i);
			switch(str[i++]){
				case '.':{
					m_word+=new W_Dot();
					v->copyFloatElementComponentToScratchPad(*this, str, i, bracketCntr);
					break;
				}
				default: throw;
			}
			
			break;
		}
		default:{
			--i;
			copyToScratchPad<float,Single>(v, str, i, bracketCntr);
		}
	}
}

template <class T, class S> inline void Function::decodeMultipleGlobal(G_Variable* v, const String& str, int& i, int& bracketCntr){
	copyGlobalToScratchPad_insert2<T,S>(v, str, i, bracketCntr);
}
template <> inline void Function::decodeMultipleGlobal<float,Multiple>(G_Variable* v, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	if(str[i]=='.'){
		m_word+=new W_Dot();
		i++;
		v->copyComponentToScratchPad(*this, str, i, bracketCntr);
	}
	else{
		copyToScratchPad<float,Single>(v, str, i, bracketCntr);
	}
}
template <class S, class T> inline void Function::decodePostSquareBrack(G_Variable* v, const String& str, int& i, int& bracketCntr){
	throw;
}
template <> inline void Function::decodePostSquareBrack<float,Multiple>(G_Variable* v, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	if(str[i]=='.'){
		m_word+=new W_Dot();
		i++;
		v->copyComponentToScratchPad(*this, str, i, bracketCntr);
	}
	else{
		copyToScratchPad<float,Single>(v, str, i, bracketCntr);
	}
}
template <class T> int V_<T,Single,Global>::copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
template <> int V_<Rotation,Single,Global>::copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	unsigned int valAddress=reinterpret_cast<unsigned int>(&value.refValue());
	if(str.cmpCppWord("x",i)){
		f.m_word+=new W_X;
		int mark=f.compiled.length+1;
		f.copyMachineCode(copyGlobalToScratchPad_insert<float>());
		f.insertDWord(mark, valAddress+sizeof(float)*0);
	}
	else if(str.cmpCppWord("y",i)){
		f.m_word+=new W_Y;
		int mark=f.compiled.length+1;
		f.copyMachineCode(copyGlobalToScratchPad_insert<float>());
		f.insertDWord(mark, valAddress+sizeof(float)*1);
	}
	else if(str.cmpCppWord("z",i)){
		f.m_word+=new W_Z;
		int mark=f.compiled.length+1;
		f.copyMachineCode(copyGlobalToScratchPad_insert<float>());
		f.insertDWord(mark, valAddress+sizeof(float)*2);
	}
	else if(str.cmpCppWord("angle",i)){
		f.m_word+=new W_Angle;
		int mark=f.compiled.length+1;
		f.copyMachineCode(copyGlobalToScratchPad_insert<float>());
		f.insertDWord(mark, valAddress+sizeof(float)*3);
	}
	else if(str.cmpCppWord("normalize",i)){
		str.stbCpp(i);
		if(str[i++]!='(')throw;
		str.stbCpp(i);
		if(str[i++]!=')')throw;

		f.m_word+=new W_Normalize();

		int mark=f.compiled.length+1;
		f.copyMachineCode(copyGlobalToScratchPad_insert<Vec3f>());
		f.insertDWord(mark, reinterpret_cast<int>(&value.refValue()));

		g_asm2CppAddy+=Asm2CppAddy(normalize_S_Vec3fCpp, f.compiled.length+2);
		f.copyMachineCode(&normalize_insert_S_Vec3f);
		return S_VEC3F;
	}
	else throw;
	return S_FLOAT;
}
template <> int V_<Vec2f,Single,Global>::copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	unsigned int valAddress=reinterpret_cast<unsigned int>(&value.refValue());
	if(str.cmpCppWord("x",i)){
		f.m_word+=new W_X;
		int mark=f.compiled.length+1;
		f.copyMachineCode(copyGlobalToScratchPad_insert<float>());
		f.insertDWord(mark, valAddress+sizeof(float)*0);
	}
	else if(str.cmpCppWord("y",i)){
		f.m_word+=new W_Y;
		int mark=f.compiled.length+1;
		f.copyMachineCode(copyGlobalToScratchPad_insert<float>());
		f.insertDWord(mark, valAddress+sizeof(float)*1);
	}
	else throw;
	return S_FLOAT;
}
template <> int V_<Vec2f,Single,Local>::copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	if(str.cmpCppWord("x",i)){
		f.m_word+=new W_X;

		g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr + sizeof(float)*0, f.compiled.length+MOV_MEM_ESP);
		f.copyMachineCode(&copyLocalComponent_x_toScratchPad_ele_S_Vec2f);
	}
	else if(str.cmpCppWord("y",i)){
		f.m_word+=new W_Y;

		g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr + sizeof(float)*1, f.compiled.length+MOV_MEM_ESP);
		f.copyMachineCode(&copyLocalComponent_y_toScratchPad_ele_S_Vec2f);
	}
	else throw;
	return S_FLOAT;
}
template <> int V_<Vec3f,Single,Global>::copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	unsigned int valAddress=reinterpret_cast<unsigned int>(&value.refValue());
	if(str.cmpCppWord("x",i)){
		f.m_word+=new W_X;
		int mark=f.compiled.length+1;
		f.copyMachineCode(copyGlobalToScratchPad_insert<float>());
		f.insertDWord(mark, valAddress+sizeof(float)*0);
	}
	else if(str.cmpCppWord("y",i)){
		f.m_word+=new W_Y;
		int mark=f.compiled.length+1;
		f.copyMachineCode(copyGlobalToScratchPad_insert<float>());
		f.insertDWord(mark, valAddress+sizeof(float)*1);
	}
	else if(str.cmpCppWord("z",i)){
		f.m_word+=new W_Z;
		int mark=f.compiled.length+1;
		f.copyMachineCode(copyGlobalToScratchPad_insert<float>());
		f.insertDWord(mark, valAddress+sizeof(float)*2);
	}
	else if(str.cmpCppWord("normalize",i)){
		str.stbCpp(i);
		if(str[i++]!='(')throw;
		str.stbCpp(i);
		if(str[i++]!=')')throw;

		f.m_word+=new W_Normalize();

		int mark=f.compiled.length+1;
		f.copyMachineCode(copyGlobalToScratchPad_insert<Vec3f>());
		f.insertDWord(mark, reinterpret_cast<int>(&value.refValue()));

		g_asm2CppAddy+=Asm2CppAddy(normalize_S_Vec3fCpp, f.compiled.length+2);
		f.copyMachineCode(&normalize_insert_S_Vec3f);
		return S_VEC3F;
	}
	else throw;
	return S_FLOAT;
}
template <> int V_<Rotation,Single,Local>::copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	if(str.cmpCppWord("x",i)){
		f.m_word+=new W_X;

		g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr + sizeof(float)*0, f.compiled.length+MOV_MEM_ESP);
		f.copyMachineCode(&copyLocalComponent_x_toScratchPad_ele_S_Rotation);
		return S_FLOAT;
	}
	else if(str.cmpCppWord("y",i)){
		f.m_word+=new W_Y;

		g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr + sizeof(float)*1, f.compiled.length+MOV_MEM_ESP);
		f.copyMachineCode(&copyLocalComponent_y_toScratchPad_ele_S_Rotation);
		return S_FLOAT;
	}
	else if(str.cmpCppWord("z",i)){
		f.m_word+=new W_Z;

		g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr + sizeof(float)*2, f.compiled.length+MOV_MEM_ESP);
		f.copyMachineCode(&copyLocalComponent_z_toScratchPad_ele_S_Rotation);
		return S_FLOAT;
	}
	else if(str.cmpCppWord("angle",i)){
		f.m_word+=new W_Angle;

		g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr + sizeof(float)*3, f.compiled.length+MOV_MEM_ESP);
		f.copyMachineCode(&copyLocalComponent_z_toScratchPad_ele_S_Rotation);
		return S_FLOAT;
	}
	else throw;
}
template <> int V_<Vec3f,Single,Local>::copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	if(str.cmpCppWord("x",i)){
		f.m_word+=new W_X;

		g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr + sizeof(float)*0, f.compiled.length+MOV_MEM_ESP);
		f.copyMachineCode(&copyLocalComponent_x_toScratchPad_ele_S_Vec3f);
		return S_FLOAT;
	}
	else if(str.cmpCppWord("y",i)){
		f.m_word+=new W_Y;

		g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr + sizeof(float)*1, f.compiled.length+MOV_MEM_ESP);
		f.copyMachineCode(&copyLocalComponent_y_toScratchPad_ele_S_Vec3f);
		return S_FLOAT;
	}
	else if(str.cmpCppWord("z",i)){
		f.m_word+=new W_Z;

		g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr + sizeof(float)*2, f.compiled.length+MOV_MEM_ESP);
		f.copyMachineCode(&copyLocalComponent_z_toScratchPad_ele_S_Vec3f);
		return S_FLOAT;
	}
	else if(str.cmpCppWord("normalize",i)){
		str.stbCpp(i);
		if(str[i++]!='(')throw;
		str.stbCpp(i);
		if(str[i++]!=')')throw;

		f.m_word+=new W_Normalize();

		g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, f.compiled.length+3);
		f.copyMachineCode(CopyLocalToScratchPad_insert<Vec3f>());

		g_asm2CppAddy+=Asm2CppAddy(normalize_S_Vec3fCpp, f.compiled.length+2);
		f.copyMachineCode(&normalize_insert_S_Vec3f);
		return S_VEC3F;
	}
	else throw;
}
template <class T> int V_<T, Multiple>::copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
template <class T> void V_<T>::copyFloatElementComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
template <class T> void V_<T, Multiple>::copyFloatElementComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	if(str.cmpCppWord("x",i)){
		f.m_word+=new W_X;

		int mark=f.compiled.length+1;
		f.copyMachineCode(&copyGlobalComponent_x_toScratchPad_ele_M_Vec3f);
		f.insertDWord(mark, reinterpret_cast<int>(&value.refValue()));
	}
	else if(str.cmpCppWord("y",i)){
		f.m_word+=new W_Y;

		int mark=f.compiled.length+1;
		f.copyMachineCode(&copyGlobalComponent_y_toScratchPad_ele_M_Vec3f);
		f.insertDWord(mark, reinterpret_cast<int>(&value.refValue()));
	}
	else if(str.cmpCppWord("z",i)){
		f.m_word+=new W_Z;

		int mark=f.compiled.length+1;
		f.copyMachineCode(&copyGlobalComponent_z_toScratchPad_ele_M_Vec3f);
		f.insertDWord(mark, reinterpret_cast<int>(&value.refValue()));
	}
}
template <class T> int V_<T,Single,Local>::copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
template <class T> int V_<T,Multiple,Local>::copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
template <class T> void V_<T,Single,Local>::copyFloatElementComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
template <class T> void V_<T,Multiple,Local>::copyFloatElementComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	throw;
}
template <class T, class S> inline void Function::decodeField(Field* fieldPtr, const String& str, int& i, int& bracketCntr){
	copyToScratchPad<T,S>(fieldPtr, str, i, bracketCntr);
}

int L_S_Bool::getType()const{ return S_BOOL;}
int L_S_Color::getType()const{return S_COLOR3F;}
int L_S_Float::getType()const{return S_FLOAT;}
int L_S_Image::getType()const{return S_IMAGE;}
int L_S_Int32::getType()const{return S_INT32;}
int L_NodePtr::getType()const{return NODEPTR;}
int L_S_Rotation::getType()const{return S_ROTATION;}
int L_S_String::getType()const{return S_STRING;}
int L_S_Time::getType()const{return S_DOUBLE;}
int L_S_Vec2f::getType()const{return S_VEC2F;}
int L_S_Vec3f::getType()const{return S_VEC3F;}
int L_M_Bool::getType()const{return M_BOOL;}
int L_M_Color::getType()const{return M_COLOR;}
int L_M_Float::getType()const{return M_FLOAT;}
int L_M_Image::getType()const{return M_IMAGE;}
int L_M_Int32::getType()const{return M_INT32;}
int L_M_Node::getType()const{return M_NODE;}
int L_M_Rotation::getType()const{return M_ROTATION;}
int L_M_String::getType()const{return M_STRING;}
int L_M_Time::getType()const{return M_TIME;}
int L_M_Vec2f::getType()const{return M_VEC2F;}
int L_M_Vec3f::getType()const{return M_VEC3F;}


int G_S_Bool::getValueType()const{ return S_BOOL;}
int G_S_Color::getValueType()const{return S_COLOR3F;}
int G_S_Float::getValueType()const{return S_FLOAT;}
int G_S_Image::getValueType()const{return S_IMAGE;}
int G_S_Int32::getValueType()const{return S_INT32;}
int G_NodePtr::getValueType()const{return NODEPTR;}
int G_S_Rotation::getValueType()const{return S_ROTATION;}
int G_S_String::getValueType()const{return S_STRING;}
int G_S_Time::getValueType()const{return S_DOUBLE;}
int G_S_Vec2f::getValueType()const{return S_VEC2F;}
int G_S_Vec3f::getValueType()const{return S_VEC3F;}
int G_M_Bool::getValueType()const{return M_BOOL;}
int G_M_Color::getValueType()const{return M_COLOR;}
int G_M_Float::getValueType()const{return M_FLOAT;}
int G_M_Image::getValueType()const{return M_IMAGE;}
int G_M_Int::getValueType()const{return M_INT32;}
int G_M_Node::getValueType()const{return M_NODE;}
int G_M_Rotation::getValueType()const{return M_ROTATION;}
int G_M_String::getValueType()const{return M_STRING;}
int G_M_Time::getValueType()const{return M_TIME;}
int G_M_Vec2f::getValueType()const{return M_VEC2F;}
int G_M_Vec3f::getValueType()const{return M_VEC3F;}

int L_S_Bool::getValueType()const{ return S_BOOL;}
int L_S_Color::getValueType()const{return S_COLOR3F;}
int L_S_Float::getValueType()const{return S_FLOAT;}
int L_S_Image::getValueType()const{return S_IMAGE;}
int L_S_Int32::getValueType()const{return S_INT32;}
int L_NodePtr::getValueType()const{return NODEPTR;}
int L_S_Rotation::getValueType()const{return S_ROTATION;}
int L_S_String::getValueType()const{return S_STRING;}
int L_S_Time::getValueType()const{return S_DOUBLE;}
int L_S_Vec2f::getValueType()const{return S_VEC2F;}
int L_S_Vec3f::getValueType()const{return S_VEC3F;}
int L_M_Bool::getValueType()const{return M_BOOL;}
int L_M_Color::getValueType()const{return M_COLOR;}
int L_M_Float::getValueType()const{return M_FLOAT;}
int L_M_Image::getValueType()const{return M_IMAGE;}
int L_M_Int32::getValueType()const{return M_INT32;}
int L_M_Node::getValueType()const{return M_NODE;}
int L_M_Rotation::getValueType()const{return M_ROTATION;}
int L_M_String::getValueType()const{return M_STRING;}
int L_M_Time::getValueType()const{return M_TIME;}
int L_M_Vec2f::getValueType()const{return M_VEC2F;}
int L_M_Vec3f::getValueType()const{return M_VEC3F;}



inline int Function::getNumber(const String& str, int& i, int& bracketCntr, int optimizeFor){
	float data=getFloat2(str, i);
	if(optimizeFor==S_DOUBLE){
		double ddata=data;
		m_word+=new W_Immediate<double>(data);
		g_const2ndCompile+=ConstData(constData.length, compiled.length+2);
		copyMachineCode(&CopyImmediateToScratchPad_ele_S_Time);
		catConstQWord(reinterpret_cast<unsigned char*>(&ddata));
		return S_DOUBLE;
	}
	if(optimizeFor==S_FLOAT || data-floor(data)!=0){
		//float for now but should check to see if the number is higher than a float or ends with F
		m_word+=new W_Immediate<float>(data);
		copyMachineCode(&CopyImmediateToScratchPad_ele_S_Float);
		insertDWord(compiled.length-sizeof(float), *(int*)&data);
		return S_FLOAT;
	}
	else{
		//int
		int idata=data;
		m_word+=new W_Immediate<int>(data);
		copyMachineCode(&CopyImmediateToScratchPad_ele_S_Int32);
		insertDWord(compiled.length-sizeof(int), *(int*)&idata);
		return S_INT32;
	}
}

int Function::getOperand(const String& str, int& i, int& bracketCntr, int optimizeFor){
	str.stbCpp(i);
	char ch=str[i];
	if(ch>='0'&&ch<='9'||ch=='.'){
		return getNumber(str, i, bracketCntr, optimizeFor);
	}
	else if(ch=='('){
		m_word+=new W_OpenBracket();
		int opTypeB=getFirstOperand(str, ++i, ++bracketCntr, optimizeFor);
		if(str[i++]!=')')throw;
		--bracketCntr;
		m_word+=new W_CloseBracket();
		return opTypeB;
	}
	else {
		String label;
		label=str.extractCppLabel(i);
		if(label=="true"){
			CopyImmediateToScratchPad_insert(true);
			return S_BOOL;
		}
		else if(label=="false"){
			CopyImmediateToScratchPad_insert(false);
			return S_BOOL;
		}
		else if(label=="Rotation"){
			CopyImmediateToScratchPad_insert<Rotation>(str,i);
			return S_ROTATION;
		}
		else if(label=="Vec2f"){
			CopyImmediateToScratchPad_insert<Vec2f>(str,i);
			return S_VEC2F;
		}
		else if(label=="Vec3f"){
			CopyImmediateToScratchPad_insert<Vec3f>(str,i);
			return S_VEC3F;
		}
		else if(label=="Color3f"){
			CopyImmediateToScratchPad_insert<Color3f>(str,i);
			return S_COLOR3F;
		}
		else if(label=="PI"){
			if(optimizeFor==S_FLOAT){
				copyPiToScratchPad_insert<float>(str,i);
				return S_FLOAT;
			}
			else{
				copyPiToScratchPad_insert<double>(str,i);
				return S_DOUBLE;
			}
		}
		else if(builtInFunction<float>(label,str, i, bracketCntr, optimizeFor)){
			return optimizeFor;
		}
		else if(L_Variable* lv=isLocalVariable(label)){
			m_word+=new W_Local(this, lv->varIndex);
			return lv->copyToScratchPad(*this, str, i, bracketCntr);
		}
		else if(Node* node=getNodeFromName(label)){
			if(G_Variable* gv=dynamic_cast<G_Variable*>(node)){
				m_word+=new W_Global(gv);
				return gv->copyToScratchPad(*this, str, i, bracketCntr);
			}
			else{
				if(str[i++]!='.')throw this;
				str.stbCpp(i);
				int fieldIdx=node->getProperties().getIndex(str.getWord2(i));
				m_word+=new W_NoOutputNode(node);
				m_word+=new W_NoOutputDot();
				m_word+=new W_Field(node,fieldIdx);
				Field* fieldPtr=node->getField(fieldIdx);
				return fieldPtr->copyToScratchPad(*this, str, i, bracketCntr);

			};
		}
		else if(Function* functionCall=isFunctionCall(label)){
			getArg(*functionCall, str, i, bracketCntr);
			return functionCall->returnType;
		}
	}
	throw;
}
template <class T, class S> int Function::doHiddenBrackOperator(const String& str, int& i, int& bracketCntr){
	int opTypeB=templateType<T,Single>();
	str.stbCpp(i);
	switch(str[i++]){
		case '-':
		case '+':{
			--i;
			return opTypeB;
		}
		case '/':{
			m_word+=new W_Divide();
			opTypeB=getOperand(str,i, bracketCntr, opTypeB);
			str.stbCpp(i);
			opTypeB=doDivisionOperator<T,S>(opTypeB);
			if(str[i]=='+'||str[i]=='-'||str[i]==';'||str[i]==')'||str[i]==','){
				return opTypeB;
			}
			break;
		}
		case '*':{
			m_word+=new W_Multiply();
			opTypeB=getOperand(str,i, bracketCntr, opTypeB);
			str.stbCpp(i);
			opTypeB=doMultiplyOperator<T,S>(opTypeB);
			if(str[i]=='+'||str[i]=='-'||str[i]==';'||str[i]==')'||str[i]==','||str[i]=='<'||str[i]=='>'){
				return opTypeB;
			}
			break;
		}
		case ')':
			--bracketCntr;
			m_word+=new W_CloseBracket();
			return opTypeB;
		case ';':{
			//m_word+=new W_Semicolon();
			--i;
			return opTypeB;
		}
		default:{
			throw;
		}
	}
	doHiddenBrackOperator<T,S>(str, i, bracketCntr);
	return opTypeB;
}
template <class T> inline void Function::wordDivide(){
	m_word+=new W_Divide();
}
template <> inline void Function::wordDivide<Vec2f>(){
	m_word+=new W_VecDivide();
}
template <> inline void Function::wordDivide<Vec3f>(){
	m_word+=new W_VecDivide();
}
template <class T, class S> int Function::doOperator(const String& str, int& i, int& bracketCntr){
	int opTypeB=templateType<T,Single>();
	str.stbCpp(i);
	switch(str[i++]){
		case '-':{
			m_word+=new W_Sub();
			opTypeB=getOperand(str,i, bracketCntr, opTypeB);
			str.stbCpp(i);
			if(str[i]=='*'||str[i]=='/'){//if * must be hidden bracket + (getOperand * ?
				doHiddenBrackOperator<T,S>(str, i, bracketCntr);
			}
			return doSubtractionOperator<T,S>(opTypeB);
		}
		case '+':{
			m_word+=new W_Add();
			opTypeB=getOperand(str,i, bracketCntr, opTypeB);
			str.stbCpp(i);
			if(str[i]=='*'||str[i]=='/'){//if * must be hidden bracket + (getOperand * ?
				switch(opTypeB){
					case S_ROTATION:{
						opTypeB=doHiddenBrackOperator<Rotation,Single>(str, i, bracketCntr);
						break;
					}
					default:{
						doHiddenBrackOperator<T,S>(str, i, bracketCntr);
					}
				}
			}
			return doAdditionOperator<T,S>(opTypeB);
		}
		case '/':{
			wordDivide<T>();//m_word+=new W_Divide();
			opTypeB=getOperand(str,i, bracketCntr, opTypeB);
			str.stbCpp(i);
			return doDivisionOperator<T,S>(opTypeB);
		}
		case '%':{
			m_word+=new W_Modulo();
			opTypeB=getOperand(str,i, bracketCntr, opTypeB);
			str.stbCpp(i);
			return doModuloOperator<T,S>(opTypeB);
		}
		case '*':{
			m_word+=new W_Multiply();
			opTypeB=getOperand(str,i, bracketCntr, opTypeB);
			str.stbCpp(i);
			return doMultiplyOperator<T,S>(opTypeB);
		}
		case '<':{
			if(str[i]=='='){
				i++;
				m_word+=new W_LessEqual();
				opTypeB=getFirstOperand(str, i, bracketCntr, templateType<T,Single>());
				str.stbCpp(i);
				if(opTypeB == templateType<T,Single>()){
					copyMachineCode(lessEqual_insert<T>());
					return S_BOOL;
				}
				else throw;
			}
			else{
				m_word+=new W_LessThan();
				opTypeB=getFirstOperand(str, i, bracketCntr, templateType<T,Single>());
				str.stbCpp(i);
				if(opTypeB == templateType<T,Single>()){
					copyMachineCode(lessThan_insert<T>());
					return S_BOOL;
				}
				else throw;
			}
		}
		case '>':{
			if(str[i]=='='){
				i++;
				m_word+=new W_GreaterEqual();
				opTypeB=getFirstOperand(str, i, bracketCntr, templateType<T,Single>());
				str.stbCpp(i);
				if(opTypeB == templateType<T,Single>()){
					copyMachineCode(greaterEqual_insert<T>());
					return S_BOOL;
				}
				else throw;
			}
			else{
				m_word+=new W_GreaterThan();
				opTypeB=getFirstOperand(str, i, bracketCntr, templateType<T,Single>());
				str.stbCpp(i);
				if(opTypeB == templateType<T,Single>()){
					copyMachineCode(greaterThan_insert<T>());
					return S_BOOL;
				}
				else throw;
			}
		}
		case '=':{
			if(str[i]=='='){
				++i;
				m_word+=new W_Equality();
				opTypeB=getFirstOperand(str, i, bracketCntr, templateType<T,Single>());
				str.stbCpp(i);
				if(opTypeB == templateType<T,Single>()){
					copyMachineCode(equality_insert<T>());
					return S_BOOL;
				}
				else throw;
			}
			else throw;
		}
		case '!':{
			if(str[i]=='='){
				++i;
				m_word+=new W_NonEquality();
				opTypeB=getFirstOperand(str, i, bracketCntr, templateType<T,Single>());
				str.stbCpp(i);
				if(opTypeB == templateType<T,Single>()){
					copyMachineCode(nonEquality_insert<T>());
					return S_BOOL;
				}
				else throw;
			}
			else throw;
		}
		case '&':{
			if(str[i]=='&'){
				++i;
				m_word+=new W_And();
				return opTypeB;
			}
			else throw;
		}
		case '|':{
			if(str[i]=='|'){
				++i;
				m_word+=new W_Or();
				return opTypeB;
			}
			else throw;
		}
		case ')':
		case ']':
		case '{':
		case ',':
			--i;
			throw;
		case ';':{
			--i;
			throw;
		}
		default:{
			throw;
		}
	}
	return opTypeB;
}
template <class T, class S> inline int Function::doSubtractionOperator(int& opTypeB){
	if(templateType<T,Single>()==opTypeB){
		copyMachineCode(subtraction_insert<T>());
	}
	else throw;
	return templateType<T,Single>();
}
template <class T, class S> inline int Function::doAdditionOperator(int& opTypeB){
	if(templateType<T,Single>()==opTypeB){
		copyMachineCode(addition_insert<T>());
	}
	else throw;
	return templateType<T,Single>();
}
template <> inline int Function::doSubtractionOperator<float,Single>(int& opTypeB){
	switch(opTypeB){
		case S_FLOAT:{
			copyMachineCode(subtraction_insert<float>());
			return S_FLOAT;
		}
		case S_INT32:{
			copyMachineCode(&subtraction_ele_S_Float_S_Int32);
			return S_FLOAT;
		}
		case S_DOUBLE:{
			copyMachineCode(&subtraction_ele_S_Float_S_Time);
			return S_DOUBLE;
		}
		default: throw;
	}
}
template <> inline int Function::doSubtractionOperator<int,Single>(int& opTypeB){
	switch(opTypeB){
		case S_FLOAT:{
			copyMachineCode(&subtraction_ele_S_Int32_S_Float);
			return S_FLOAT;
		}
		case S_INT32:{
			copyMachineCode(subtraction_insert<int>());
			return S_INT32;
		}
		case S_DOUBLE:{
			copyMachineCode(&subtraction_ele_S_Int32_S_Time);
			return S_DOUBLE;
		}
		default: throw;
	}
}
template <> inline int Function::doSubtractionOperator<double,Single>(int& opTypeB){
	switch(opTypeB){
		case S_FLOAT:{
			copyMachineCode(&subtraction_ele_S_Time_S_Float);
			return S_DOUBLE;
		}
		case S_INT32:{
			copyMachineCode(&subtraction_ele_S_Time_S_Int32);
			return S_DOUBLE;
		}
		case S_DOUBLE:{
			copyMachineCode(subtraction_insert<double>());
			return S_DOUBLE;
		}
		default: throw;
	}
}
template <> inline int Function::doAdditionOperator<float,Single>(int& opTypeB){
	switch(opTypeB){
		case S_FLOAT:{
			copyMachineCode(addition_insert<float>());
			return S_FLOAT;
		}
		case S_INT32:{
			copyMachineCode(&addition_ele_S_Float_S_Int32);
			return S_FLOAT;
		}
		case S_DOUBLE:{
			copyMachineCode(&addition_ele_S_Float_S_Time);
			return S_DOUBLE;
		}
		default: throw;
	}
}
template <> inline int Function::doAdditionOperator<int,Single>(int& opTypeB){
	switch(opTypeB){
		case S_FLOAT:{
			copyMachineCode(&addition_ele_S_Int32_S_Float);
			return S_FLOAT;
		}
		case S_INT32:{
			copyMachineCode(addition_insert<int>());
			return S_INT32;
		}
		case S_DOUBLE:{
			copyMachineCode(&addition_ele_S_Int32_S_Time);
			return S_DOUBLE;
		}
		default: throw;
	}
}
template <> inline int Function::doAdditionOperator<double,Single>(int& opTypeB){
	switch(opTypeB){
		case S_FLOAT:{
			copyMachineCode(&addition_ele_S_Time_S_Float);
			return S_DOUBLE;
		}
		case S_INT32:{
			copyMachineCode(&addition_ele_S_Time_S_Int32);
			return S_DOUBLE;
		}
		case S_DOUBLE:{
			copyMachineCode(addition_insert<double>());
			return S_DOUBLE;
		}
		default: throw;
	}
}

///////////////
template <class T, class S> inline int Function::doDivisionOperator(int& opTypeB){
	if(templateType<T,Single>()==opTypeB){
		copyMachineCode(division_insert<T>());
	}
	else throw;
	return templateType<T,Single>();
}
template <> inline int Function::doDivisionOperator<Vec2f, Single>(int& opTypeB){
	if(opTypeB==S_INT32){
		copyMachineCode(typeToType_insert<int,float>());
	}
	else if(opTypeB==S_DOUBLE){
		copyMachineCode(typeToType_insert<double,float>());
	}
	else if(opTypeB!=S_FLOAT){
		throw;
	}
	copyMachineCode(division_insert<Vec2f>());
	return templateType<Vec2f,Single>();
}
template <> inline int Function::doDivisionOperator<Vec3f, Single>(int& opTypeB){
	if(opTypeB==S_INT32){
		copyMachineCode(typeToType_insert<int,float>());
	}
	else if(opTypeB==S_DOUBLE){
		copyMachineCode(typeToType_insert<double,float>());
	}
	else if(opTypeB!=S_FLOAT){
		throw;
	}
	copyMachineCode(division_insert<Vec3f>());
	return templateType<Vec3f,Single>();
}
template <class T, class S> inline int Function::doModuloOperator(int& opTypeB){
	if(templateType<T,Single>()==opTypeB){
		copyMachineCode(&modulo_ele_Int);
	}
	else throw;
	return templateType<T,Single>();
}
template <class T, class S> inline int Function::doMultiplyOperator(int& opTypeB){
	if(templateType<T,Single>()==opTypeB){
		copyMachineCode(multiplication_insert<T>());
	}
	else throw;
	return templateType<T,Single>();
}
void* s_RotationMultiplyS_Rotation_address;
void* s_RotationMultiplyS_Vec3f_address;
void* s_RotationMinus_address;

void* m_assignOperator_addressLookUp[VARABLETYPE_LOOKUPLEN/2];

typedef  Rotation (Rotation::*RotationMultiplyRotationFP)(const Rotation&)const; 
typedef  Vec3f (Rotation::*RotationMultiplyVec3fFP)(const Vec3f&)const; 
typedef  Rotation (Rotation::*RotationMinusFP)()const;

typedef  M_Bool& (M_Bool::*M_BoolAssignOperator)(const M_Bool&); 
typedef  M_Color3f& (M_Color3f::*M_Color3fAssignOperator)(const M_Color3f&); 
typedef  M_Float& (M_Float::*M_FloatAssignOperator)(const M_Float&); 
typedef  M_Image& (M_Image::*M_ImageAssignOperator)(const M_Image&); 
typedef  M_Int& (M_Int::*M_IntAssignOperator)(const M_Int&); 
typedef  M_Node& (M_Node::*M_NodeAssignOperator)(const M_Node&); 
typedef  M_Rotation& (M_Rotation::*M_RotationAssignOperator)(const M_Rotation&); 
typedef  M_String& (M_String::*M_StringAssignOperator)(const M_String&); 
typedef  M_Double& (M_Double::*M_DoubleAssignOperator)(const M_Double&); 
typedef  M_Vec2f& (M_Vec2f::*M_Vec2fAssignOperator)(const M_Vec2f&); 
typedef  M_Vec3f& (M_Vec3f::*M_Vec3fAssignOperator)(const M_Vec3f&); 

void getMemberAddresses(){
	void* memPtr;

	RotationMultiplyRotationFP rotationMultiplyRotationFP = &Rotation::operator*;
	memPtr = *reinterpret_cast<void**>(&rotationMultiplyRotationFP);
	s_RotationMultiplyS_Rotation_address = memPtr;

	RotationMultiplyVec3fFP rotationMultiplyVec3fFP = &Rotation::operator*;
	memPtr = *reinterpret_cast<void**>(&rotationMultiplyVec3fFP);
	s_RotationMultiplyS_Vec3f_address = memPtr;

	RotationMinusFP rotationMinusFP = &Rotation::operator-;
	memPtr = *reinterpret_cast<void**>(&rotationMinusFP);
	s_RotationMinus_address = memPtr;
///////////////////////////////
	M_BoolAssignOperator m_BoolAssignOperator = &M_Bool::operator=;
	memPtr = *reinterpret_cast<void**>(&m_BoolAssignOperator);
	m_assignOperator_addressLookUp[S_BOOL] = memPtr;

	M_Color3fAssignOperator m_Color3fAssignOperator = &M_Color3f::operator=;
	memPtr = *reinterpret_cast<void**>(&m_Color3fAssignOperator);
	m_assignOperator_addressLookUp[S_COLOR3F] = memPtr;

	M_FloatAssignOperator m_FloatAssignOperator = &M_Float::operator=;
	memPtr = *reinterpret_cast<void**>(&m_FloatAssignOperator);
	m_assignOperator_addressLookUp[S_FLOAT] = memPtr;

	M_ImageAssignOperator m_ImageAssignOperator = &M_Image::operator=;
	memPtr = *reinterpret_cast<void**>(&m_ImageAssignOperator);
	m_assignOperator_addressLookUp[S_IMAGE] = memPtr;

	M_IntAssignOperator m_IntAssignOperator = &M_Int::operator=;
	memPtr = *reinterpret_cast<void**>(&m_IntAssignOperator);
	m_assignOperator_addressLookUp[S_INT32] = memPtr;

	M_NodeAssignOperator m_NodeAssignOperator = &M_Node::operator=;
	memPtr = *reinterpret_cast<void**>(&m_NodeAssignOperator);
	m_assignOperator_addressLookUp[NODEPTR] = memPtr;

	M_RotationAssignOperator m_RotationAssignOperator = &M_Rotation::operator=;
	memPtr = *reinterpret_cast<void**>(&m_RotationAssignOperator);
	m_assignOperator_addressLookUp[S_ROTATION] = memPtr;

	M_StringAssignOperator m_StringAssignOperator = &M_String::operator=;
	memPtr = *reinterpret_cast<void**>(&m_StringAssignOperator);
	m_assignOperator_addressLookUp[S_STRING] = memPtr;

	M_DoubleAssignOperator m_DoubleAssignOperator = &M_Double::operator=;
	memPtr = *reinterpret_cast<void**>(&m_DoubleAssignOperator);
	m_assignOperator_addressLookUp[S_DOUBLE] = memPtr;

	M_Vec2fAssignOperator m_Vec2fAssignOperator = &M_Vec2f::operator=;
	memPtr = *reinterpret_cast<void**>(&m_Vec2fAssignOperator);
	m_assignOperator_addressLookUp[S_VEC2F] = memPtr;

	M_Vec3fAssignOperator m_Vec3fAssignOperator = &M_Vec3f::operator=;
	memPtr = *reinterpret_cast<void**>(&m_Vec3fAssignOperator);
	m_assignOperator_addressLookUp[S_VEC3F] = memPtr;

}

template <> inline int Function::doMultiplyOperator<Rotation, Single>(int& opTypeB){
	switch(opTypeB){
		case S_ROTATION:{
			int mark=compiled.length;
			copyMachineCode(&multiplication_ele_S_Rotation);
			g_asm2CppAddy+=Asm2CppAddy(s_RotationMultiplyS_Rotation_address, mark+11);
			return S_ROTATION;
		}
		case S_VEC3F:{
			int mark=compiled.length;
			copyMachineCode(&s_RotationMultiplyS_Vec3f_ele);
			g_asm2CppAddy+=Asm2CppAddy(s_RotationMultiplyS_Vec3f_address, mark+11);
			return S_VEC3F;
		}
		default: throw;
	}
}
template <> inline int Function::doDivisionOperator<float,Single>(int& opTypeB){
	switch(opTypeB){
		case S_FLOAT:{
			copyMachineCode(division_insert<float>());
			return S_FLOAT;
		}
		case S_INT32:{
			copyMachineCode(&division_ele_S_Float_S_Int32);
			return S_FLOAT;
		}
		case S_DOUBLE:{
			copyMachineCode(&division_ele_S_Float_S_Time);
			return S_DOUBLE;
		}
		default: throw;
	}
}
template <> inline int Function::doDivisionOperator<int,Single>(int& opTypeB){
	switch(opTypeB){
		case S_FLOAT:{
			copyMachineCode(&division_ele_S_Int32_S_Float);
			return S_FLOAT;
		}
		case S_INT32:{
			copyMachineCode(division_insert<int>());
			return S_INT32;
		}
		case S_DOUBLE:{
			copyMachineCode(&division_ele_S_Int32_S_Time);
			return S_DOUBLE;
		}
		default: throw;
	}
}
template <> inline int Function::doDivisionOperator<double,Single>(int& opTypeB){
	switch(opTypeB){
		case S_FLOAT:{
			copyMachineCode(&division_ele_S_Time_S_Float);
			return S_DOUBLE;
		}
		case S_INT32:{
			copyMachineCode(&division_ele_S_Time_S_Int32);
			return S_DOUBLE;
		}
		case S_DOUBLE:{
			copyMachineCode(division_insert<double>());
			return S_DOUBLE;
		}
		default: throw;
	}
}
template <> inline int Function::doMultiplyOperator<float,Single>(int& opTypeB){
	switch(opTypeB){
		case S_FLOAT:{
			copyMachineCode(multiplication_insert<float>());
			return S_FLOAT;
		}
		case S_INT32:{
			copyMachineCode(&multiplication_ele_S_Float_S_Int32);
			return S_FLOAT;
		}
		case S_DOUBLE:{
			copyMachineCode(&multiplication_ele_S_Float_S_Time);
			return S_DOUBLE;
		}
		default: throw;
	}
}
template <> inline int Function::doMultiplyOperator<int,Single>(int& opTypeB){
	switch(opTypeB){
		case S_FLOAT:{
			copyMachineCode(&multiplication_ele_S_Int32_S_Float);
			return S_FLOAT;
		}
		case S_INT32:{
			copyMachineCode(multiplication_insert<int>());
			return S_INT32;
		}
		case S_DOUBLE:{
			copyMachineCode(&multiplication_ele_S_Int32_S_Time);
			return S_DOUBLE;
		}
		default: throw;
	}
}
template <> inline int Function::doMultiplyOperator<double,Single>(int& opTypeB){
	switch(opTypeB){
		case S_FLOAT:{
			copyMachineCode(&multiplication_ele_S_Time_S_Float);
			return S_DOUBLE;
		}
		case S_INT32:{
			copyMachineCode(&multiplication_ele_S_Time_S_Int32);
			return S_DOUBLE;
		}
		case S_DOUBLE:{
			copyMachineCode(multiplication_insert<double>());
			return S_DOUBLE;
		}
		default: throw;
	}
}
int Function::getFirstOperand(const String& str, int& i, int& bracketCntr, int opTypeA){
	str.stbCpp(i);
	int remI=i;

	if(str[i]=='-'){
		if(str[++i]=='-'){
			throw;
		}
		else{
			str.stbCpp(i);
			char ch=str[i];
			if(ch>='0'&&ch<='9'||ch=='.'){
				i=remI;
				opTypeA=getNumber(str, i, bracketCntr, opTypeA);
			}
			else{
				m_word+=new W_Minus;
				opTypeA=getOperand(str, i, bracketCntr, opTypeA);
				switch(opTypeA){
					case S_FLOAT:{
						copyMachineCode(negate_insert<float>());
						break;
					}
					case S_INT32:{
						copyMachineCode(negate_insert<int>());
						break;
					}
					case S_ROTATION:{
						int mark=compiled.length;
						copyMachineCode(&negate_ele_S_Rotation);
						g_asm2CppAddy+=Asm2CppAddy(s_RotationMinus_address, mark+10);
						break;
					}
					case S_DOUBLE:{
						copyMachineCode(negate_insert<double>());
						break;
					}
					case S_VEC3F:{
						copyMachineCode(negate_insert<Vec3f>());
						break;
					}
					default:{
						throw;
					}
				}
			}
		}
	}
	else if(str[i]=='!'){
		str.stbCpp(++i);
		m_word+=new W_Not;
		opTypeA=getOperand(str, i, bracketCntr, opTypeA);
		switch(opTypeA){
			case S_BOOL:{
				copyMachineCode(not_insert<bool>());
				break;
			}
			default:{
				throw;
			}
		}
	}
	else{
		opTypeA=getOperand(str, i, bracketCntr, opTypeA);
		str.stbCpp(i);
	}
	while(true){
		str.stbCpp(i);
		if(str[i]!=';' && str[i]!='{' && str[i]!=')' && str[i]!=']' && str[i]!=',' && dynamic_cast<W_Logic*>(m_word[m_word.length-1])==0){
			switch(opTypeA){
				case S_BOOL:{
					opTypeA=doOperator<S_Bool,Single>(str,i, bracketCntr);
					break;
				}
				case S_COLOR3F:{
					opTypeA=doOperator<Vec3f,Single>(str,i, bracketCntr);
					break;
				}
				case S_FLOAT:{
					opTypeA=doOperator<float,Single>(str,i, bracketCntr);
					break;
				}
				case S_INT32:{
					opTypeA=doOperator<int,Single>(str,i, bracketCntr);
					break;
				}
				case S_ROTATION:{
					opTypeA=doOperator<Rotation,Single>(str,i, bracketCntr);
					break;
				}
				case S_DOUBLE:{
					opTypeA=doOperator<double,Single>(str,i, bracketCntr);
					break;
				}
				case S_VEC2F:{
					opTypeA=doOperator<Vec2f,Single>(str,i, bracketCntr);
					break;
				}
				case S_VEC3F:{
					opTypeA=doOperator<Vec3f,Single>(str,i, bracketCntr);
					break;
				}
				default: throw;
			}
		}
		else break;
	}
	return opTypeA;
}
template <class T> int V_<T,Single,Local>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, f.compiled.length+3);
	f.copyMachineCode(CopyLocalToScratchPad_insert<T>());

	str.stbCpp(i);
	if(str[i]=='+' && str[i+1]=='+'){
		i+=2;
		f.m_word+=new W_PlusPlus();
		g_postIncList+=this;
	}
	else if(str[i]=='-' && str[i+1]=='-'){
		i+=2;
		f.m_word+=new W_MinusMinus();
		g_postDecList+=this;
	}
	return templateType<T,Single>();
}
template <> int V_<Rotation,Single,Global>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	if(str[i]=='.'){
		f.m_word+=new W_Dot();
		i++;
		return copyComponentToScratchPad(f, str, i, bracketCntr);
	}
	else{
		int mark=f.compiled.length+1;
		f.copyMachineCode(copyGlobalToScratchPad_insert<Rotation>());
		f.insertDWord(mark, reinterpret_cast<int>(&value.refValue()));
		return S_ROTATION;
	}
}
template <class T> int V_<T,Single,Global>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	int mark=f.compiled.length+1;
	f.copyMachineCode(copyGlobalToScratchPad_insert<T>());
	f.insertDWord(mark, reinterpret_cast<int>(&value.refValue()));

	str.stbCpp(i);
	if(str[i]=='+' && str[i+1]=='+'){
		i+=2;
		f.m_word+=new W_PlusPlus();
		g_postIncList+=this;
	}
	else if(str[i]=='-' && str[i+1]=='-'){
		i+=2;
		f.m_word+=new W_MinusMinus();
		g_postDecList+=this;
	}
	return templateType<T,Single>();
}
template <> int V_<Rotation,Single,Local>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	if(str[i]=='.'){
		f.m_word+=new W_Dot();
		i++;
		copyComponentToScratchPad(f, str, i, bracketCntr);
		return S_FLOAT;
	}
	else{
		g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, f.compiled.length+MOV_MEM_ESP);
		f.copyMachineCode(CopyLocalToScratchPad_insert<Rotation>());
		return S_ROTATION;
	}
}
template <> int V_<Vec2f,Single,Local>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	if(str[i]=='.'){
		f.m_word+=new W_Dot();
		i++;
		copyComponentToScratchPad(f, str, i, bracketCntr);
		return S_FLOAT;
	}
	else{
		g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, f.compiled.length+MOV_MEM_ESP);
		f.copyMachineCode(CopyLocalToScratchPad_insert<Vec2f>());
		return S_VEC2F;
	}
}
template <> int V_<Vec3f,Single,Global>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	if(str[i]=='.'){
		f.m_word+=new W_Dot();
		i++;
		return copyComponentToScratchPad(f, str, i, bracketCntr);
	}
	else{
		int mark=f.compiled.length+1;
		f.copyMachineCode(copyGlobalToScratchPad_insert<Vec3f>());
		f.insertDWord(mark, reinterpret_cast<int>(&value.refValue()));
		return S_VEC3F;
	}
}

template <> int V_<Vec3f,Single,Local>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	str.stbCpp(i);
	if(str[i]=='.'){
		f.m_word+=new W_Dot();
		i++;
		return copyComponentToScratchPad(f, str, i, bracketCntr);
	}
	else{
		g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, f.compiled.length+MOV_MEM_ESP);
		f.copyMachineCode(CopyLocalToScratchPad_insert<Vec3f>());
		return S_VEC3F;
	}
}

template <class T> int V_<T,Multiple,Local>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	switch(str[i++]){
		case '.':{
			f.m_word+=new W_Dot();
			str.stbCpp(i);
			String label=str.extractCppLabel(i);
			if(label=="length"){
				f.m_word+=new W_Length();
				str.stbCpp(i);
				g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, f.compiled.length+MOV_MEM_ESP);
				copyMachineCodeFM(copyLengthToScratchPad_doOp_ref);
			}
			else throw;
			return S_INT32;
			break;
		}
		case '[':{
			f.m_word+=new W_OpenSquareBracket();
			int indexOpType=f.getFirstOperand(str,i,bracketCntr,S_INT32);
			switch(indexOpType){
				case S_FLOAT:{
					f.copyMachineCode(typeToType_insert<float,int>());
					break;
				}
				case S_INT32:{
					break;
				}
				case S_DOUBLE:{
					f.copyMachineCode(typeToType_insert<double,int>());
					break;
				}
				default:{
					throw;
				}
			}
			if(str[i++]!=']')throw;
			f.m_word+=new W_CloseSquareBracket();
			str.stbCpp(i);
			switch(str[i++]){
				case '.':{
					f.m_word+=new W_Dot();
					copyFloatElementComponentToScratchPad(f, str, i, bracketCntr);
					return S_FLOAT;
				}
				default:{
					--i;
					g_local2ndCompile+=Local2ndCompile(varIndex, g_callOffsetCntr, f.compiled.length+MOV_MEM_ESP);
					f.copyMachineCode(copyLocalElement_toScratchPad_insert<M_<T> >());
					return templateType<T,Single>();
				}
			}
			break;
		}
		default:{
			--i;
			//copyAddressToScratchPad(f, str, i);
			return templateType<T,Multiple>();
		}
	}
}
template <class T> int V_<T,Multiple,Global>::copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr){
	void* valAddress=&value.refValue();
	switch(str[i++]){
		case '.':{
			f.m_word+=new W_Dot();
			str.stbCpp(i);
			String label=str.extractCppLabel(i);
			if(label=="length"){
				f.m_word+=new W_Length();
				str.stbCpp(i);
				int mark=f.compiled.length+1;
				copyMachineCodeFM(copyGlobalLengthToScratchPad_doOp);
				f.insertDWord(mark, reinterpret_cast<int>(valAddress));
			}
			else throw;
			return S_INT32;
			break;
		}
		case '[':{
			f.m_word+=new W_OpenSquareBracket();
			int indexOpType=f.getFirstOperand(str,i,bracketCntr,S_INT32);
			switch(indexOpType){
				case S_FLOAT:{
					f.copyMachineCode(typeToType_insert<float,int>());
					break;
				}
				case S_INT32:{
					break;
				}
				case S_DOUBLE:{
					f.copyMachineCode(typeToType_insert<double,int>());
					break;
				}
				default:{
					throw;
				}
			}
			if(str[i++]!=']')throw;
			f.m_word+=new W_CloseSquareBracket();
			str.stbCpp(i);
			switch(str[i++]){
				case '.':{
					f.m_word+=new W_Dot();
					copyFloatElementComponentToScratchPad(f, str, i, bracketCntr);
					return S_FLOAT;
				}
				default:{
					--i;
					int mark=f.compiled.length+1;
					//f.copyMachineCode(&copyGlobalElement_toScratchPad_ele_M_Int32);
					f.copyMachineCode(copyGlobalElement_toScratchPad_insert<M_<T> >());
					f.insertDWord(mark, reinterpret_cast<int>(&value.refValue()));

					return templateType<T,Single>();
				}
			}
			
			break;
		}
		default:{
			--i;
			//copyAddressToScratchPad(f, str, i);
			return templateType<T,Multiple>();
		}
	}
}
M_DEP_Byte Function::compiled;
M_Byte Function::constData;
//M_Word Function::m_word;
bool Function::readyForOutput=false;
Node* Function::selectedLine=0;

template <class T, class S> W_Type* getWordType(){
	throw;
}
template <> W_Type* getWordType<bool, Single>(){return new W_S_Bool();}
template <> W_Type* getWordType<Color3f, Single>(){return new W_S_Color();}
template <> W_Type* getWordType<float, Single>(){return new W_S_Float();}
template <> W_Type* getWordType<Image, Single>(){return new W_S_Image();}
template <> W_Type* getWordType<int, Single>(){return new W_S_Int32();}
template <> W_Type* getWordType<NodePtr, Single>(){return new W_S_NodePtr();}
template <> W_Type* getWordType<Rotation, Single>(){return new W_S_Rotation();}
template <> W_Type* getWordType<String, Single>(){return new W_S_String();}
template <> W_Type* getWordType<double, Single>(){return new W_S_Time();}
template <> W_Type* getWordType<Vec2f, Single>(){return new W_S_Vec2f();}
template <> W_Type* getWordType<Vec3f, Single>(){return new W_S_Vec3f();}

template <> W_Type* getWordType<bool, Multiple>(){return new W_M_Bool();}
template <> W_Type* getWordType<Color3f, Multiple>(){return new W_M_Color();}
template <> W_Type* getWordType<float, Multiple>(){return new W_M_Float();}
template <> W_Type* getWordType<Image, Multiple>(){return new W_M_Image();}
template <> W_Type* getWordType<int, Multiple>(){return new W_M_Int32();}
template <> W_Type* getWordType<NodePtr, Multiple>(){return new W_M_NodePtr();}
template <> W_Type* getWordType<Rotation, Multiple>(){return new W_M_Rotation();}
template <> W_Type* getWordType<String, Multiple>(){return new W_M_String();}
template <> W_Type* getWordType<double, Multiple>(){return new W_M_Double();}
template <> W_Type* getWordType<Vec2f, Multiple>(){return new W_M_Vec2f();}
template <> W_Type* getWordType<Vec3f, Multiple>(){return new W_M_Vec3f();}

void W_IntListData::read(M_Word& m_word, const String& str,int& i){
	str.stbCpp(i);
	if(str[i++]=='('){
		m_word+=new W_OpenBracket;
		str.stbCpp(i);

		int ctr=0;

		for(int k=i; str[k]!=')';ctr++){
			getInt(str, k);
			str.stb(k);
		}
		value.length=ctr;

		for(ctr=0;str[i]!=')';ctr++){
			value[ctr]=getInt(str, i);
			str.stb(i);
		}
		m_word+=this;
		m_word+=new W_CloseBracket;
		str.stbCpp(++i);
	}
	else{
		throw;
	}
}
void W_FloatListData::read(M_Word& m_word, const String& str,int& i){
	str.stbCpp(i);
	if(str[i++]=='('){
		m_word+=new W_OpenBracket;
		str.stbCpp(i);

		int ctr=0;

		for(int k=i; str[k]!=')';ctr++){
			getFloat2(str, k);
			str.stb(k);
		}
		value.length=ctr;

		for(ctr=0;str[i]!=')';ctr++){
			value[ctr]=getFloat2(str, i);
			str.stb(i);
		}
		m_word+=this;
		m_word+=new W_CloseBracket;
		str.stbCpp(++i);
	}
	else{
		throw;
	}
}
template <class T, class L> void convertListToHigherType(M_<T>& dst, M_<L>& src){
	dst.length=src.length*sizeof(L)/sizeof(T);
	if(src.length % dst.length){
		throw;
	}
	for(int i=src.length; --i>=0;){
		reinterpret_cast<L*>(dst.getArray())[i]=src.getArray()[i];
	}
}

void V_<Image,Single>::getGlobalValue(const String& str, int& i, int& bracketCntr){}
void V_<NodePtr,Single>::getGlobalValue(const String& str, int& i, int& bracketCntr){}
void V_<String,Single>::getGlobalValue(const String& str, int& i, int& bracketCntr){}
void V_<bool,Multiple>::getGlobalValue(const String& str, int& i, int& bracketCntr){}
void V_<Color3f,Multiple>::getGlobalValue(const String& str, int& i, int& bracketCntr){
	if(str.cmpCppWord("FloatList", i)==false)throw;
	m_word+=new W_FloatList;
	W_FloatListData* w_listData=new W_FloatListData;
	w_listData->read(m_word, str, i);
	convertListToHigherType(value.refValue(), w_listData->value);
}
void V_<float,Multiple>::getGlobalValue(const String& str, int& i, int& bracketCntr){
	if(str.cmpCppWord("M_Float", i)==false)throw;
	m_word+=new  W_M_Float2;
	W_FloatListData* w_listData=new W_FloatListData;
	w_listData->read(m_word, str, i);
	value.refValue()=w_listData->value;
}
void V_<Image,Multiple>::getGlobalValue(const String& str, int& i, int& bracketCntr){}

void V_<int,Multiple>::getGlobalValue(const String& str, int& i, int& bracketCntr){
	if(str.cmpCppWord("M_Int", i)==false)throw;
	m_word+=new  W_M_Int2;
	W_IntListData* w_listData=new W_IntListData;
	w_listData->read(m_word, str, i);
	value.refValue()=w_listData->value;
}
void V_<NodePtr,Multiple>::getGlobalValue(const String& str, int& i, int& bracketCntr){}

void V_<Rotation,Multiple>::getGlobalValue(const String& str, int& i, int& bracketCntr){
	if(str.cmpCppWord("FloatList", i)==false)throw;
	m_word+=new W_FloatList;
	W_FloatListData* w_listData=new W_FloatListData;
	w_listData->read(m_word, str, i);
	convertListToHigherType(value.refValue(), w_listData->value);
}
void V_<String,Multiple>::getGlobalValue(const String& str, int& i, int& bracketCntr){}
void V_<double,Multiple>::getGlobalValue(const String& str, int& i, int& bracketCntr){}

void V_<Vec2f,Multiple>::getGlobalValue(const String& str, int& i, int& bracketCntr){
	if(str.cmpCppWord("FloatList", i)==false)throw;
	m_word+=new W_FloatList;
	W_FloatListData* w_listData=new W_FloatListData;
	w_listData->read(m_word, str, i);
	convertListToHigherType(value.refValue(), w_listData->value);
}

void V_<Vec3f,Multiple>::getGlobalValue(const String& str, int& i, int& bracketCntr){
	if(str.cmpCppWord("FloatList", i)==false)throw;
	m_word+=new W_FloatList;
	W_FloatListData* w_listData=new W_FloatListData;
	w_listData->read(m_word, str, i);
	convertListToHigherType(value.refValue(), w_listData->value);
}

void V_<bool,Single>::getGlobalValue(const String& str, int& i, int& bracketCntr){
	m_word+=new W_Immediate<bool>(value.refValue()=getCppBool(str, i));
}
void V_<double,Single>::getGlobalValue(const String& str, int& i, int& bracketCntr){
	m_word+=new W_Immediate<double>(value.refValue()=getFloat2(str, i));
}
void V_<float,Single>::getGlobalValue(const String& str, int& i, int& bracketCntr){
	m_word+=new W_Immediate<float>(value.refValue()=getFloat2(str, i));
}
void V_<int,Single>::getGlobalValue(const String& str, int& i, int& bracketCntr){
	m_word+=new W_Immediate<int>(value.refValue()=getInt2(str, i));
}

void V_<Color3f,Single>::getGlobalValue(const String& str, int& i, int& bracketCntr){
	if(str.cmpCppWord("Color3f", i)==false)throw;
	m_word+=new W_S_Color2();
	m_word+=new W_OpenBracket();
	str.stbCpp(i);
	if(str[i++]!='(')throw;
	m_word+=new W_Immediate<float>(value.r=getFloat2(str, i));
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	m_word+=new W_Immediate<float>(value.g=getFloat2(str, i));
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	m_word+=new W_Immediate<float>(value.b=getFloat2(str, i));
	if(str[i++]!=')')throw;
	m_word+=new W_CloseBracket();
}
void V_<Rotation,Single>::getGlobalValue(const String& str, int& i, int& bracketCntr){
	if(str.cmpCppWord("Rotation", i)==false)throw;
	m_word+=new W_S_Rotation2();
	m_word+=new W_OpenBracket();
	str.stbCpp(i);
	if(str[i++]!='(')throw;
	m_word+=new W_Immediate<float>(value.x=getFloat2(str, i));
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	m_word+=new W_Immediate<float>(value.y=getFloat2(str, i));
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	m_word+=new W_Immediate<float>(value.z=getFloat2(str, i));
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	m_word+=new W_Immediate<float>(value.angle=getFloat2(str, i));
	if(str[i++]!=')')throw;
	m_word+=new W_CloseBracket();
}
void V_<Vec2f,Single>::getGlobalValue(const String& str, int& i, int& bracketCntr){
	if(str.cmpCppWord("Vec2f", i)==false)throw;
	m_word+=new W_S_Vec2f2();
	m_word+=new W_OpenBracket();
	str.stbCpp(i);
	if(str[i++]!='(')throw;
	m_word+=new W_Immediate<float>(value.refValue().x=getFloat2(str, i));
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	m_word+=new W_Immediate<float>(value.refValue().y=getFloat2(str, i));
	if(str[i++]!=')')throw;
	m_word+=new W_CloseBracket();
}
void V_<Vec3f,Single>::getGlobalValue(const String& str, int& i, int& bracketCntr){
	if(str.cmpCppWord("Vec3f", i)==false)throw;
	m_word+=new W_S_Vec3f2();
	m_word+=new W_OpenBracket();
	str.stbCpp(i);
	if(str[i++]!='(')throw;
	m_word+=new W_Immediate<float>(value.x=getFloat2(str, i));
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	m_word+=new W_Immediate<float>(value.y=getFloat2(str, i));
	if(str[i++]!=',')throw;
	m_word+=new W_Comma();
	m_word+=new W_Immediate<float>(value.z=getFloat2(str, i));
	if(str[i++]!=')')throw;
	m_word+=new W_CloseBracket();
}
template <class T, class S> Node* decodeInitializedGlobal(const String& str, int& i, int& bracketCntr){
	static String name;
	name=str.extractCppLabel(i);
	str.stbCpp(i);

	if(str[i]=='('){
		Function* f=new Function;
		f->functionName=name;
		g_functionList+=f;
		f->returnType=templateType<T,S>();
		f->newDecodeHeader(str, i, bracketCntr);
		return f;
	}
	V_<T,S>* v=new V_<T,S>;
	v->m_word+=getWordType<T,S>();
	v->name=name;
	v->m_word+=new W_Global(v);
	g_load_scene->defList += v;

	str.stbCpp(i);

	if(str[i]=='='){
		i++;
		v->m_word+=new W_Assignment();
		str.stbCpp(i);
		v->getGlobalValue(str, i, bracketCntr);
		str.stbCpp(i);
	}
	if(str[i++]!=';'){
		throw;
	}
	
	return v;
}

template Node* decodeInitializedGlobal<S_Bool,Single>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<Color3f,Single>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<float,Single>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<Image,Single>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<int,Single>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<NodePtr,Single>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<Rotation,Single>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<String,Single>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<double,Single>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<Vec2f,Single>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<Vec3f,Single>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<S_Bool,Multiple>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<Color3f,Multiple>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<float,Multiple>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<Image,Multiple>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<int,Multiple>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<NodePtr,Multiple>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<Rotation,Multiple>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<String,Multiple>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<double,Multiple>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<Vec2f,Multiple>(const String& str, int& i, int& bracketCntr);
template Node* decodeInitializedGlobal<Vec3f,Multiple>(const String& str, int& i, int& bracketCntr);

template <> Node* decodeInitializedGlobal<void,Single>(const String& str, int& i, int& bracketCntr){
	Function* f=new Function;
	f->returnType=_VOID;
	f->addFunctionToList(str, i);
	f->newDecodeHeader(str, i, bracketCntr);
	return f;
}
void addFieldToScript(Seamless* smls, Node* node, int fieldIdx);
void convertRoutesToScriptForFunction(Seamless* smls){
	for(int k=g_functionList.length; --k>=0;){
		M_Word& m_word=g_functionList[k]->m_word;
		for(int k=m_word.length; --k>=0;){
			if(W_Field* w_field=dynamic_cast<W_Field*>(m_word[k])){
				if(Art* part=dynamic_cast<Art*>(w_field->node)){
					addFieldToScript(smls, part, w_field->fieldIdx);
				}
			}
		}
	}
}
void Function::decodeTextCell(const String& str, int& i, int& bracketCntr){
	static String name;
	name=str.extractCppLabel(i);
	str.stbCpp(i);

	if(str[i]=='('){
		functionName=name;
		g_functionList+=this;
		//returnType=templateType<T,S>();
		newDecodeHeader(str, i, bracketCntr);
	}
	else{
		throw;
	}
}
void G_Variable::commonDecodeTextCell(Word* wt, W_Global* wg, const String& str, int& i, int& bracketCntr){
	name=str.extractCppLabel(i);
	str.stbCpp(i);
	m_word.length=0;
	m_word+=wt;
	m_word+=wg;
	str.stbCpp(i);
	if(i==str.length){
		int type=getValueType();
		if(type<FIRST_M_VALUE_TYPE){
			throw;
		}
		return;
	}
	if(str[i++]!='='){
		int type=getValueType();
		if(type<FIRST_M_VALUE_TYPE){
			throw;
		}
		return;
	}
	m_word+=new W_Assignment();
	str.stbCpp(i);
}
void V_<S_Bool,Single>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_S_Bool(), new W_Global(this), str, i, bracketCntr);
	getGlobalValue(str, i, bracketCntr);
}
void V_<Color3f,Single>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_S_Color(), new W_Global(this), str, i, bracketCntr);
	getGlobalValue(str, i, bracketCntr);
}
void V_<float,Single>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_S_Float(), new W_Global(this), str, i, bracketCntr);
	getGlobalValue(str, i, bracketCntr);
}
void V_<Image,Single>::decodeTextCell(const String& str, int& i, int& bracketCntr){}
void V_<int,Single>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_S_Int32(), new W_Global(this), str, i, bracketCntr);
	getGlobalValue(str, i, bracketCntr);
}
void V_<NodePtr,Single>::decodeTextCell(const String& str, int& i, int& bracketCntr){}
void V_<Rotation,Single>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_S_Rotation(), new W_Global(this), str, i, bracketCntr);
	getGlobalValue(str, i, bracketCntr);
}
void V_<String,Single>::decodeTextCell(const String& str, int& i, int& bracketCntr){}
void V_<double,Single>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_S_Time(), new W_Global(this), str, i, bracketCntr);
	getGlobalValue(str, i, bracketCntr);
}
void V_<Vec2f,Single>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_S_Vec2f(), new W_Global(this), str, i, bracketCntr);
	getGlobalValue(str, i, bracketCntr);
}
void V_<Vec3f,Single>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_S_Vec3f(), new W_Global(this), str, i, bracketCntr);
	getGlobalValue(str, i, bracketCntr);
}
void V_<S_Bool,Multiple>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_M_Bool(), new W_Global(this), str, i, bracketCntr);
	if(i<str.length){
		getGlobalValue(str, i, bracketCntr);
	}
}
void V_<Color3f,Multiple>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_M_Color(), new W_Global(this), str, i, bracketCntr);
	if(i<str.length){
		getGlobalValue(str, i, bracketCntr);
	}
}
void V_<float,Multiple>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_M_Float(), new W_Global(this), str, i, bracketCntr);
	if(i<str.length){
		getGlobalValue(str, i, bracketCntr);
	}
}
void V_<Image,Multiple>::decodeTextCell(const String& str, int& i, int& bracketCntr){}
void V_<int,Multiple>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_M_Int32(), new W_Global(this), str, i, bracketCntr);
	if(i<str.length){
		getGlobalValue(str, i, bracketCntr);
	}
}
void V_<NodePtr,Multiple>::decodeTextCell(const String& str, int& i, int& bracketCntr){}
void V_<Rotation,Multiple>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_M_Rotation(), new W_Global(this), str, i, bracketCntr);
	if(i<str.length){
		getGlobalValue(str, i, bracketCntr);
	}
}
void V_<String,Multiple>::decodeTextCell(const String& str, int& i, int& bracketCntr){}
void V_<double,Multiple>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_M_Double(), new W_Global(this), str, i, bracketCntr);
	if(i<str.length){
		getGlobalValue(str, i, bracketCntr);
	}
}
void V_<Vec2f,Multiple>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_M_Vec2f(), new W_Global(this), str, i, bracketCntr);
	if(i<str.length){
		getGlobalValue(str, i, bracketCntr);
	}
}
void V_<Vec3f,Multiple>::decodeTextCell(const String& str, int& i, int& bracketCntr){
	commonDecodeTextCell(new W_M_Vec3f(), new W_Global(this), str, i, bracketCntr);
	if(i<str.length){
		getGlobalValue(str, i, bracketCntr);
	}
}

void outputUserScripts(String& str, bool& newList){
	if(g_outputScriptStackExist.length){
		Function::readyForOutput=true;
		if(g_outputScriptStackExist[g_outputScriptStackExist.length-1]){
			if(g_writeX3DXML){
				g_outputScriptStack[g_outputScriptStack.length-1]->toXMLString(str, newList, "children");
			}
			else{
				g_outputScriptStack[g_outputScriptStack.length-1]->toString(str, newList);
			}
		}
		g_outputScriptStack.length--;
		g_outputScriptStackExist.length--;
	}
}
Node* newMemberFunction(){
	return new MemberFunction;
}
void Anim::decodeVoid(const String& str, int& i, int& bracketCntr){
	if(str.cmpCppWord("onFrame",i)){// || str.cmpCppWord("onFrame",i)){
		Function* f=new MemberFunction;
		f->returnType=_VOID;
		f->addFunctionToList(str, i);
		f->functionName="onFrame";
		f->newDecodeHeader(str, i, bracketCntr);
		onFrame.refValue() = f;
	}
}
void TimeSensor::decodeVoid(const String& str, int& i, int& bracketCntr){
	if(str.cmpCppWord("on_fraction_changed",i)){
		Function* f=new MemberFunction;
		f->returnType=_VOID;
		f->addFunctionToList(str, i);
		f->functionName="on_fraction_changed";
		f->newDecodeHeader(str, i, bracketCntr);
		on_fraction_changed.refValue() = f;
	}
	else throw;
}
void TouchSensor::decodeVoid(const String& str, int& i, int& bracketCntr){
	if(str.cmpCppWord("onTouchTime",i)){
		Function* f=new MemberFunction;
		f->returnType=_VOID;
		f->addFunctionToList(str, i);
		f->functionName="onTouchTime";
		f->newDecodeHeader(str, i, bracketCntr);
		onTouchTime.refValue() = f;
	}
	else throw;
}
void Mobility::decodeVoid(const String& str, int& i, int& bracketCntr){
	Function* f=new MemberFunction;
	f->returnType=_VOID;
	if(str.cmpCppWord("onRot",i)){
		f->addFunctionToList(str, i);
		f->functionName="onRot";
		f->newDecodeHeader(str, i, bracketCntr);
		onRot.refValue() = f;
	}
	else if(str.cmpCppWord("onTran",i)){
		f->addFunctionToList(str, i);
		f->functionName="onTran";
		f->newDecodeHeader(str, i, bracketCntr);
		onTran.refValue() = f;
	}
	else throw;
}
void Build::decodeVoid(const String& str, int& i, int& bracketCntr){
	if(str.cmpCppWord("setup",i)){
		Function* f=new MemberFunction;
		f->returnType=_VOID;
		f->addFunctionToList(str, i);
		f->functionName="setup";
		f->newDecodeHeader(str, i, bracketCntr);
		setup.refValue() = f;
	}
	else throw;
}
void G_Variable::toString(String& str, bool newList){

	if(g_outputMode){
		addToOutputScript();
	}
	if(!g_outputMode){
		for(int w=0; w<m_word.length; w++){
			Word* wrd=m_word[w];
			wrd->toString(str);
		}
		str+=";\n";
	}
}
FD<bool,Single>* V_<bool,Single>::getInitValue()const{
	return new FD<bool,Single>(dynamic_cast<W_Immediate<bool>*>(m_word[3])->value);
}
FD<Color3f,Single>* V_<Color3f,Single>::getInitValue()const{
	return new FD<Color3f,Single>(
		Color3f(
			dynamic_cast<W_Immediate<float>*>(m_word[5])->value,
			dynamic_cast<W_Immediate<float>*>(m_word[7])->value,
			dynamic_cast<W_Immediate<float>*>(m_word[9])->value
		)
	);
}
FD<float,Single>* V_<float,Single>::getInitValue()const{
	return new FD<float,Single>(dynamic_cast<W_Immediate<float>*>(m_word[3])->value);
}
FD<Image,Single>* V_<Image,Single>::getInitValue()const{
	throw;
}
FD<int,Single>* V_<int,Single>::getInitValue()const{
	return new FD<int,Single>(dynamic_cast<W_Immediate<int>*>(m_word[3])->value);
}
FD<NodePtr,Single>* V_<NodePtr,Single>::getInitValue()const{
	throw;
}
FD<Rotation,Single>* V_<Rotation,Single>::getInitValue()const{
		return new FD<Rotation,Single>(
		Rotation(
			dynamic_cast<W_Immediate<float>*>(m_word[5])->value,
			dynamic_cast<W_Immediate<float>*>(m_word[7])->value,
			dynamic_cast<W_Immediate<float>*>(m_word[9])->value,
			dynamic_cast<W_Immediate<float>*>(m_word[11])->value
		)
	);
}
FD<String,Single>* V_<String,Single>::getInitValue()const{
	return new FD<String,Single>(dynamic_cast<W_Immediate<String>*>(m_word[3])->value);
}
FD<double,Single>* V_<double,Single>::getInitValue()const{
	return new FD<double,Single>(dynamic_cast<W_Immediate<double>*>(m_word[3])->value);
}
FD<Vec2f,Single>* V_<Vec2f,Single>::getInitValue()const{
	return new FD<Vec2f,Single>(
		Vec2f(
			dynamic_cast<W_Immediate<float>*>(m_word[5])->value,
			dynamic_cast<W_Immediate<float>*>(m_word[7])->value
		)
	);
}
FD<Vec3f,Single>* V_<Vec3f,Single>::getInitValue()const{
	return new FD<Vec3f,Single>(
		Vec3f(
			dynamic_cast<W_Immediate<float>*>(m_word[5])->value,
			dynamic_cast<W_Immediate<float>*>(m_word[7])->value,
			dynamic_cast<W_Immediate<float>*>(m_word[9])->value
		)
	);
}

template <class T> void V_<T,Single,Global>::addToOutputScript()const{
	Script& script=*getOutputScriptPtr();
	script+=getInitValue();
	script.properties+=Properties(name);
}
template <class T> void V_<T,Multiple,Global>::addToOutputScript()const{
	Script& script=*getOutputScriptPtr();
	script+=new FD<T,Multiple>(value);
	script.properties+=Properties(name);
}
template <class T> void V_<T, Single, Local>::copyAddressToScratchPad(Function& f, const String& str, int& i){
	throw;
}
template <class T> void V_<T, Multiple, Local>::copyAddressToScratchPad(Function& f, const String& str, int& i){
	throw;
}
template <class T> void V_<T, Single>::copyAddressToScratchPad(Function& f, const String& str, int& i){
	throw;
}
template <class T> void V_<T, Multiple>::copyAddressToScratchPad(Function& f, const String& str, int& i){
	str.stbCpp(i);
	int mark=f.compiled.length;
	f.compiled.length+=4;
	f.insertDWord(mark, reinterpret_cast<int>(&value.refValue()));
}

bool getEquiveRoute(M_Word& m_word, W_Field*& w_dstField, W_Field*& w_srcField, M_Node& m_codeLine, int i){
	CodeLine* codeLine=dynamic_cast<CodeLine*>(m_codeLine[i].p);
	if(codeLine->wordTo - codeLine->wordFrom >= 2){
		int w=codeLine->wordFrom+2;
		if(w_dstField=dynamic_cast<W_Field*>(m_word[w++])){
			if(dynamic_cast<W_Assignment*>(m_word[w])){
				w+=3;
				if(w<m_word.length){
					if(w_srcField=dynamic_cast<W_Field*>(m_word[w])){
						return true;
					}
				}
			}
		}
	}
	return false;
}

bool getNodeFieldEqualsSomething(M_Word& m_word, W_Field*& w_dstField, M_Node& m_codeLine, int i){
	CodeLine* codeLine=dynamic_cast<CodeLine*>(m_codeLine[i].p);
	if(codeLine->wordTo - codeLine->wordFrom >= 2){
		int w=codeLine->wordFrom+2;
		if(w_dstField=dynamic_cast<W_Field*>(m_word[w++])){
			if(dynamic_cast<W_Assignment*>(m_word[w])){
				return true;
			}
		}
	}
	return false;
}

bool getMorphFunction(M_Word& m_word, W_Node*& w_morpher, W_Node*& w_part, M_Node& m_codeLine, int i){
	CodeLine* codeLine=dynamic_cast<CodeLine*>(m_codeLine[i].p);
	if(codeLine->wordTo - codeLine->wordFrom >= 2){
		int w = codeLine->wordFrom;
		if(dynamic_cast<W_Morph*>(m_word[w + 2])){
			if(w_morpher = dynamic_cast<W_Node*>(m_word[w])){
				if(w_part = dynamic_cast<W_Node*>(m_word[w + 6])){
					return true;
				}
			}
		}
	}
	return false;
}


bool getCalcFunction(M_Word& m_word, W_Node*& w_nikInterpolator, W_Node*& w_part, M_Node& m_codeLine, int i){
	CodeLine* codeLine=dynamic_cast<CodeLine*>(m_codeLine[i].p);
	if(codeLine->wordTo - codeLine->wordFrom >= 2){
		int w = codeLine->wordFrom;
		if(dynamic_cast<W_Calc*>(m_word[w + 2])){
			if(w_nikInterpolator = dynamic_cast<W_Node*>(m_word[w])){
				if(w_part = dynamic_cast<W_Node*>(m_word[w + 6])){
					return true;
				}
			}
		}
	}
	return false;
}

bool getCompFunction(M_Word& m_word, W_Node*& w_compensator, W_Node*& w_part, M_Node& m_codeLine, int i){
	CodeLine* codeLine=dynamic_cast<CodeLine*>(m_codeLine[i].p);
	if(codeLine->wordTo - codeLine->wordFrom >= 2){
		int w = codeLine->wordFrom;
		if(dynamic_cast<W_Comp*>(m_word[w + 2])){
			if(w_compensator = dynamic_cast<W_Node*>(m_word[w])){
				if(w_part = dynamic_cast<W_Node*>(m_word[w + 6])){
					return true;
				}
			}
		}
	}
	return false;
}

void Anim::captureMultiplePartPose(Art* selNode){
	Seamless* smls = 0;

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField=0;
		W_Field* w_srcField=0;
		W_Node* w_morpher = 0;
		W_Node* w_part = 0;

		for(int i=m_codeLine.length; --i>=0;){
			if(getEquiveRoute(m_word, w_dstField, w_srcField, m_codeLine, i)){
				if(Art* part=dynamic_cast<Art*>(w_dstField->node)){
					if((controlKeyState()) && part != selNode){
						continue;
					}
					if((altKeyState())){
						continue;
					}
					if(SamplePose* samplePose=dynamic_cast<SamplePose*>(w_srcField->node)){
						samplePose->take(slider, part, part->getField(w_dstField->fieldIdx));
					}
				}
			}
			else if(smls == 0){
				if(getMorphFunction(m_word, w_morpher, w_part, m_codeLine, i)){
					if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_morpher->node)){
						if(Art* part = dynamic_cast<Art*>(w_part->node)){
							if((controlKeyState()) && part != selNode){
								continue;
							}
							if((shiftKeyState())){
								continue;
							}
							smls = part->smls;
							if(smls->animCP){
								if(smls->workAnimCP){
									captureMorph(smls);
								}
							}
							else{
								smls->genDynamicMatricesAndPopulate();
								capturePolyMorph(smls);
							}
						}
					}
				}
			}
		}
	}
}

template<typename T>
inline void deleteSectionT(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime, M_Float& key, M_<T>& keyValue, SamplePose* samplePose){
	if(selectFrom > 1){
		selectFrom = 1;
	}
	if(selectTo > 1){
		selectTo = 1;
	}

	for(int k = 0; k < key.length; k++){
		if(roughGE(key[k], selectFrom)){
			int firstSample = k;
			while(roughLT(key[k], selectTo) && k < key.length - 1){
				k++;
			}
			int lastSample = k;
			if(firstSample == lastSample){
				if(roughGT(key[lastSample], selectTo)){
					break;
				}
			}
			else if(roughGT(key[lastSample], selectTo)){
				lastSample--;
			}

			if(selectFrom < selectTo){
				if(beginEdge == false && roughEQ(key[firstSample], selectFrom)){
					firstSample++;
				}
				if(endEdge == false && roughEQ(key[lastSample], selectTo)){
					lastSample--;
				}
			}

			int numOf = lastSample - firstSample + 1;
			if(numOf > 0){
				key.deleteElement(firstSample, numOf);
				keyValue.deleteElement(firstSample, numOf);
			}
			break;
		}
	}
	if(deleteTime){
		float diff = selectTo - selectFrom;
		samplePose->insertTime(selectFrom, selectFrom - diff, beginEdge);
	}
}

void OrientationInterpolator::deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime){
	deleteSectionT(selectFrom, selectTo, beginEdge, endEdge, deleteTime, key.refValue(), keyValue.refValue(), this);
}

void PositionInterpolator::deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime){
	deleteSectionT(selectFrom, selectTo, beginEdge, endEdge, deleteTime, key.refValue(), keyValue.refValue(), this);
}

void ScalarInterpolator::deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime){
	deleteSectionT(selectFrom, selectTo, beginEdge, endEdge, deleteTime, key.refValue(), keyValue.refValue(), this);
}

void CameraSelector::deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime){
	deleteSectionT(selectFrom, selectTo, beginEdge, endEdge, deleteTime, key.refValue(), keyValue.refValue(), this);
}

void CPMorpher::deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime){
	if(selectFrom > 1){
		selectFrom = 1;
	}
	if(selectTo > 1){
		selectTo = 1;
	}

	for(int k = 0; k < key.length; k++){
		if(roughGE(key[k], selectFrom)){
			int firstSample = k;
			while(roughLT(key[k], selectTo) && k < key.length - 1){
				k++;
			}
			int lastSample = k;
			if(firstSample == lastSample){
				if(roughGT(key[lastSample], selectTo)){
					break;
				}
			}
			else if(roughGT(key[lastSample], selectTo)){
				lastSample--;
			}

			if(selectFrom < selectTo){
				if(beginEdge == false && roughEQ(key[firstSample], selectFrom)){
					firstSample++;
				}
				if(endEdge == false && roughEQ(key[lastSample], selectTo)){
					lastSample--;
				}
			}

			int numOf = lastSample - firstSample + 1;
			if(numOf > 0){
				int numOfChannels = cpIndex.length;
				key.refValue().deleteElement(firstSample, numOf);
				keyValue.refValue().deleteElement(firstSample * numOfChannels, numOf * numOfChannels);
			}
			break;
		}
	}
	if(deleteTime){
		float diff = selectTo - selectFrom;
		insertTime(selectFrom, selectFrom - diff, beginEdge);
	}
}

void NikInterpolator::deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime){
	if(selectFrom > 1){
		selectFrom = 1;
	}
	if(selectTo > 1){
		selectTo = 1;
	}

	for(int k = 0; k < key.length; k++){
		if(roughGE(key[k], selectFrom)){
			int firstSample = k;
			while(roughLT(key[k], selectTo) && k < key.length - 1){
				k++;
			}
			int lastSample = k;
			if(firstSample == lastSample){
				if(roughGT(key[lastSample], selectTo)){
					break;
				}
			}
			else if(roughGT(key[lastSample], selectTo)){
				lastSample--;
			}

			if(selectFrom < selectTo){
				if(beginEdge == false && roughEQ(key[firstSample], selectFrom)){
					firstSample++;
				}
				if(endEdge == false && roughEQ(key[lastSample], selectTo)){
					lastSample--;
				}
			}

			int numOf = lastSample - firstSample + 1;
			if(numOf > 0){
				int numOfChannels = getSampleLength();
				key.refValue().deleteElement(firstSample, numOf);
				keyValue.refValue().deleteElement(firstSample * numOfChannels, numOf * numOfChannels);
			}
			break;
		}
	}
	if(deleteTime){
		float diff = selectTo - selectFrom;
		insertTime(selectFrom, selectFrom - diff, beginEdge);
	}
}


void NurbsTrack::deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime){
}

void PosCompensator::deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime){
	if(selectFrom > 1){
		selectFrom = 1;
	}
	if(selectTo > 1){
		selectTo = 1;
	}

	for(int k = 0; k < moment.length; k++){
		if(roughGE(moment[k].x, selectFrom)){
			int firstSample = k;
			while(roughLT(moment[k].x, selectTo) && k < moment.length - 1){
				k++;
			}
			int lastSample = k;
			if(firstSample == lastSample){
				if(roughGT(moment[lastSample].x, selectTo)){
					break;
				}
			}
			else if(roughGT(moment[lastSample].x, selectTo)){
				lastSample--;
			}
			if(selectFrom < selectTo){
				if(beginEdge == false && roughEQ(moment[firstSample].x, selectFrom)){
					firstSample++;
				}
				if(endEdge == false && roughEQ(moment[lastSample].x, selectTo)){
					lastSample--;
				}
			}

			int numOf = lastSample - firstSample + 1;
			if(numOf > 0){
				moment.refValue().deleteElement(firstSample, numOf);
				relPos.refValue().deleteElement(firstSample, numOf);
			}
			break;
		}
	}
	if(deleteTime){
		float diff = selectTo - selectFrom;
		insertTime(selectFrom, selectFrom - diff, beginEdge);
	}
}

void insertTime2(M_Float& key, double selectFrom, double selectTo, bool beginEdge){
	double diff = selectTo - selectFrom;
	double s = 1.0 / (1.0 + diff);

	if(selectFrom > 1){
		selectFrom = 1;
	}
	if(selectTo > 1){
		selectTo = 1;
	}
	double add = diff * s;
	for(int k = 0; k < key.length;){
		float ky = key[k];
		bool ready = beginEdge ? roughGE(ky, selectFrom) : roughGT(ky, selectFrom);
		if(ready){
			for(; k < key.length; k++){
				float ky = key[k];
				key[k] = static_cast<float>(ky * s + add);
			}
			return;
		}
		key[k++] = static_cast<float>(ky * s);
	}
}


void OrientationInterpolator::insertTime(double selectFrom, double selectTo, bool beginEdge){
	insertTime2(key.refValue(), selectFrom, selectTo, beginEdge);
}

void PositionInterpolator::insertTime(double selectFrom, double selectTo, bool beginEdge){
	insertTime2(key.refValue(), selectFrom, selectTo, beginEdge);
}

void ScalarInterpolator::insertTime(double selectFrom, double selectTo, bool beginEdge){
	insertTime2(key.refValue(), selectFrom, selectTo, beginEdge);
}

void CameraSelector::insertTime(double selectFrom, double selectTo, bool beginEdge){
	insertTime2(key.refValue(), selectFrom, selectTo, beginEdge);
}

void CPMorpher::insertTime(double selectFrom, double selectTo, bool beginEdge){
	insertTime2(key.refValue(), selectFrom, selectTo, beginEdge);
}

void NikInterpolator::insertTime(double selectFrom, double selectTo, bool beginEdge){
	insertTime2(key.refValue(), selectFrom, selectTo, beginEdge);
}

void NurbsTrack::insertTime(double selectFrom, double selectTo, bool beginEdge){
}

void PosCompensator::insertTime(double selectFrom, double selectTo, bool beginEdge){
	double diff = selectTo - selectFrom;
	float s = 1.0 / (1.0 + diff);

	if(selectFrom > 1){
		selectFrom = 1;
	}
	if(selectTo > 1){
		selectTo = 1;
	}
	double add = diff * s;
	for(int k = 0; k < moment.length;){
		Vec2f ky = moment[k];
		bool ready = beginEdge ? roughGE(ky.x, selectFrom) : roughGT(ky.x, selectFrom);
		if(ready){
			for(; k < moment.length; k++){
				Vec2f ky = moment[k];
				moment.refValue()[k] = ky * s + Vec2f(add, add);
			}
			return;
		}
		moment.refValue()[k++] = ky * s;
	}
}


void scaleTime2(double add, double period, M_Float& key){
	double s = period / (period + add);

	for(int k = 0; k < key.length;){
		float ky = key[k];
		key[k++] = static_cast<float>(ky * s);
	}
}

void OrientationInterpolator::scaleTime(double add, double period){
	scaleTime2(add, period, key.refValue());
}

void PositionInterpolator::scaleTime(double add, double period){
	scaleTime2(add, period, key.refValue());
}

void ScalarInterpolator::scaleTime(double add, double period){
	scaleTime2(add, period, key.refValue());
}

void CameraSelector::scaleTime(double add, double period){
	scaleTime2(add, period, key.refValue());
}

void CPMorpher::scaleTime(double add, double period){
	scaleTime2(add, period, key.refValue());
}

void NikInterpolator::scaleTime(double add, double period){
	scaleTime2(add, period, key.refValue());
}

void NurbsTrack::scaleTime(double add, double period){
	scaleTime2(add, period, key.refValue());
}

void PosCompensator::scaleTime(double add, double period){
	double s = period / (period + add);

	for(int k = 0; k < moment.length;){
		Vec2f ky = moment[k];
		moment.refValue()[k++] = ky * s;
	}
}


void take2(float slider, Vec3f pos, M_Int& index, M_Float& key, M_Vec3f& keyValue);
void take2(float slider, float s_float, M_Int& index, M_Float& key, M_Float& keyValue);
void take2(float slider, Node* node, M_Int& index, M_Float& key, M_Node& keyValue);
void take2(float slider, Rotation rot, M_Int& index, M_Float& key, M_Rotation& keyValue);
void take2(Vec2f slider, Vec3f pos, M_Int& index, M_Vec2f& moment, M_Vec3f& relPos);

void PasteOptions::pasteSection(SamplePose* samplePose, double slider, double period, int& firstSample, int& lastSample, int numOf, double& add, double& scaleTime){
	if(g_pasteOptions->insert2){
		samplePose->insertTime(slider, slider + selectedFrac2, beginEdge);
		scaleTime = 1.0 / (period + selectedPeriod * frameDiffRatio);
	}
	else{
		if(frameDiffRatio > 1.0){
			samplePose->deleteSection(slider, slider + selectedFrac, false, false, false);
			samplePose->insertTime(slider, slider + selectedFrac2, beginEdge);
			scaleTime = 1.0 / (period +  periodWarp);
		}
		else if(frameDiffRatio < 1.0){
			samplePose->deleteSection(slider, slider + selectedFrac, false, true, false);
			samplePose->deleteSection(slider, slider + periodWarp, false, true, true);
			if(period < selectedPeriod){
				scaleTime = 1.0 / (selectedPeriod * frameDiffRatio);
			}
			else{
				scaleTime = 1.0 / (period + periodWarp);
			}
		}
		else{
			samplePose->deleteSection(slider, slider + selectedFrac, false, false, false);
		}
	}

	firstSample = 0;
	lastSample = numOf - 1;
	if(firstSample < lastSample){
		if(beginEdge == false && g_animCopy.key[firstSample] == g_animCopy.selectFrom){
			firstSample++;
		}
		if(endEdge == false && g_animCopy.key[lastSample] == g_animCopy.selectTo){
			lastSample--;
		}
	}
	add = slider * period - g_animCopy.selectFrom * g_animCopy.period;
}

void OrientationInterpolator::pasteSection(double slider, double period, Node* node, int keyIndex, int keyValueIndex, int numOf){
	int firstSample = 0;
	int lastSample = 0;
	double scaleTime = 1.0 / period;
	double add = 0;
	g_pasteOptions->pasteSection(this, slider, period, firstSample, lastSample, numOf, add, scaleTime);

	

	for(int j = firstSample; j <= lastSample; j++){
		double t = (g_animCopy.key[keyIndex + j] * g_animCopy.period);
		double f = g_animCopy.selectFrom * g_animCopy.period;
		t = (t - f) * g_pasteOptions->frameDiffRatio + f;
		t = (t + add) * scaleTime;
		if(g_pasteOptions->quantize){
			t = quantize(t, g_pasteOptions->stepFrac);
		}

		Rotation r = g_animCopy.rot[keyValueIndex + j];
		if(Art* part = dynamic_cast<Art*>(node)){
			if(g_pasteOptions->mirror && part->mirror){
				r = Rotation(r.x, -r.y, -r.z, r.angle);
			}
			if(g_animCurve.processed && (part->ik || part == g_selNode.getRaw())){
				g_animCurve.generate(r, t);
			}
			if(g_pasteOptions->transform && (part->ik || part == g_selNode.getRaw()) && g_pasteOptions->rotate.angle != 0){
				r = r * g_pasteOptions->rotate.refValue();
			}
		}

		take2(t, r, index, key.refValue(), keyValue.refValue());
	}
}

void copyStraightForCurveFirst(int keyIndex, int keyValueIndex, int numOf);

void PositionInterpolator::pasteSection(double slider, double period, Node* node, int keyIndex, int keyValueIndex, int numOf){
	int firstSample = 0;
	int lastSample = 0;
	double scaleTime = 1.0 / period;
	double add = 0;
	g_pasteOptions->pasteSection(this, slider, period, firstSample, lastSample, numOf, add, scaleTime);


	for(int j = firstSample; j <= lastSample; j++){
		double t = (g_animCopy.key[keyIndex + j] * g_animCopy.period);
		double f = g_animCopy.selectFrom * g_animCopy.period;
		t = (t - f) * g_pasteOptions->frameDiffRatio + f;
		t = (t + add) * scaleTime;
		if(g_pasteOptions->quantize){
			t = quantize(t, g_pasteOptions->stepFrac);
		}

		Vec3f v = g_animCopy.pos[keyValueIndex + j];
		if(Art* part = dynamic_cast<Art*>(node)){
			if(g_pasteOptions->mirror && part->mirror){
				v.x = -v.x;
			}
			if(g_animCurve.processed && (part->ik || part == g_selNode.getRaw())){
				g_animCurve.generate(v, t);
			}

			if(g_pasteOptions->transform && (part->ik || part == g_selNode.getRaw())){
				v = g_pasteOptions->rotate.refValue() * ((v - g_pasteOptions->pivotPoint) * g_pasteOptions->scale) + g_pasteOptions->translate + g_pasteOptions->pivotPoint;
			}
		}
		else if(Viewpoint* view = dynamic_cast<Viewpoint*>(node)){
			if(g_pasteOptions->transform){
				v = g_pasteOptions->rotate.refValue() * ((v - g_pasteOptions->pivotPoint) * g_pasteOptions->scale) + g_pasteOptions->translate + g_pasteOptions->pivotPoint;
			}
		}

		take2(t, v, index, key.refValue(), keyValue.refValue());
	}
}

void ScalarInterpolator::pasteSection(double slider, double period, Node* node, int keyIndex, int keyValueIndex, int numOf){
	int firstSample = 0;
	int lastSample = 0;
	double scaleTime = 1.0 / period;
	double add = 0;
	g_pasteOptions->pasteSection(this, slider, period, firstSample, lastSample, numOf, add, scaleTime);

	for(int j = firstSample; j <= lastSample; j++){
		double t = (g_animCopy.key[keyIndex + j] * g_animCopy.period);
		double f = g_animCopy.selectFrom * g_animCopy.period;
		t = (t - f) * g_pasteOptions->frameDiffRatio + f;
		t = (t + add) * scaleTime;
		if(g_pasteOptions->quantize){
			t = quantize(t, g_pasteOptions->stepFrac);
		}

		take2(t, g_animCopy.scalar[keyValueIndex + j], index, key.refValue(), keyValue.refValue());
	}
}

void CameraSelector::pasteSection(double slider, double period, Node* node, int keyIndex, int keyValueIndex, int numOf){
	int firstSample = 0;
	int lastSample = 0;
	double scaleTime = 1.0 / period;
	double add = 0;
	g_pasteOptions->pasteSection(this, slider, period, firstSample, lastSample, numOf, add, scaleTime);

	for(int j = firstSample; j <= lastSample; j++){
		double t = (g_animCopy.key[keyIndex + j] * g_animCopy.period);
		double f = g_animCopy.selectFrom * g_animCopy.period;
		t = (t - f) * g_pasteOptions->frameDiffRatio + f;
		t = (t + add) * scaleTime;
		if(g_pasteOptions->quantize){
			t = quantize(t, g_pasteOptions->stepFrac);
		}

		take2(t, g_animCopy.m_node[keyValueIndex + j], index, key.refValue(), keyValue.refValue());
	}
}

void CPMorpher::pasteSection(double slider, double period, Art* part, int keyIndex, int keyValueIndex, int numOf, int srcNumOfChannels, M_Int& cpIndexSrc, int cpIndexSrcBase){
	int firstSample = 0;
	int lastSample = 0;
	double scaleTime = 1.0 / period;
	double add = 0;
	g_pasteOptions->pasteSection(this, slider, period, firstSample, lastSample, numOf, add, scaleTime);

	int numOfChannels = cpIndex.length;
	for(int j = firstSample; j <= lastSample; j++){
		double t = (g_animCopy.key[keyIndex + j] * g_animCopy.period);
		double f = g_animCopy.selectFrom * g_animCopy.period;
		t = (t - f) * g_pasteOptions->frameDiffRatio + f;
		t = (t + add) * scaleTime;
		if(g_pasteOptions->quantize){
			t = quantize(t, g_pasteOptions->stepFrac);
		}

		for(int i = 0; i < srcNumOfChannels; i++){
			//int channel = cpIndexSrc[cpIndexSrcBase + i];
			Vec3f v = g_animCopy.pos[keyValueIndex + j * srcNumOfChannels + i];
			if(g_pasteOptions->mirror && part->mirror){
				v.x = -v.x;
			}
			take(t, v, i, numOfChannels);
		}
	}
}

void NikInterpolator::pasteSection(double slider, double period, Node* node, int keyIndex, int keyValueIndex, int numOf){
	int firstSample = 0;
	int lastSample = 0;
	double scaleTime = 1.0 / period;
	double add = 0;
	g_pasteOptions->pasteSection(this, slider, period, firstSample, lastSample, numOf, add, scaleTime);

	int sampleLen = getSampleLength();

	for(int j = firstSample; j <= lastSample; j++){
		double t = (g_animCopy.key[keyIndex + j] * g_animCopy.period);
		double f = g_animCopy.selectFrom * g_animCopy.period;
		t = (t - f) * g_pasteOptions->frameDiffRatio + f;
		t = (t + add) * scaleTime;
		if(g_pasteOptions->quantize){
			t = quantize(t, g_pasteOptions->stepFrac);
		}

		for(int i = 0; i < sampleLen; i++){
			//int channel = cpIndexSrc[cpIndexSrcBase + i];
			Vec3f v = g_animCopy.pos[keyValueIndex + j * sampleLen + i];
			int offSet = i & 1;
			take(t, v, i >> 1, sampleLen, offSet);
		}
	}
}

void NurbsTrack::pasteSection(double slider, double period, Art* part, int keyIndex, int keyValueIndex, int numOf){
}

void PosCompensator::pasteSection(double slider, double period, Art* part, int keyIndex, int keyValueIndex, int numOf){
	int firstSample = 0;
	int lastSample = 0;
	double scaleTime = 1.0 / period;
	double add = 0;
	g_pasteOptions->pasteSection(this, slider, period, firstSample, lastSample, numOf, add, scaleTime);

	for(int j = firstSample; j <= lastSample; j++){
		//Vec2f t = Vec2f(g_copyKeytAnim[keyIndex + j * 2], g_copyKeytAnim[keyIndex + j * 2 + 1]) * g_copyPeriodAnim + Vec2f(add, add);

		Vec2f t = Vec2f(g_animCopy.key[keyIndex + j * 2], g_animCopy.key[keyIndex + j * 2 + 1]) * g_animCopy.period;
		Vec2f f = Vec2f(g_animCopy.selectFrom * g_animCopy.period, g_animCopy.selectFrom * g_animCopy.period);
		t = (t - f) * g_pasteOptions->frameDiffRatio + f;
		t = (t + Vec2f(add, add)) * scaleTime;
		if(g_pasteOptions->quantize){
			t.x = quantize(t.x, g_pasteOptions->stepFrac);
			t.y = quantize(t.y, g_pasteOptions->stepFrac);
		}

		Vec3f v = g_animCopy.pos[keyValueIndex + j];
		if(g_pasteOptions->mirror && part->mirror){
			v.x = -v.x;
		}
		if(g_animCurve.processed && (part->ik || part == g_selNode.getRaw())){
			g_animCurve.generate(v, t.x);
		}
		if(g_pasteOptions->transform && (part->ik || part == g_selNode.getRaw())){
			v = g_pasteOptions->rotate.refValue() * ((v - g_pasteOptions->pivotPoint) * g_pasteOptions->scale) + g_pasteOptions->translate + g_pasteOptions->pivotPoint;
		}

		take2(t * scaleTime, v, index, moment.refValue(), relPos.refValue());
	}
}

void OrientationInterpolator::copySection(double selectFrom, double selectTo, Node* node, int fieldIdx){

	double diff = selectTo - selectFrom;
	double s = 1.0;

	for(int k = 0; k < key.length; k++){
		if(roughGE(key[k], selectFrom)){
			int firstSample = k;
			while(roughLT(key[k], selectTo) && k < key.length - 1){
				k++;
			}
			int lastSample = k;
			if(firstSample == lastSample){
				if(roughGT(key[lastSample], selectTo)){
					break;
				}
			}
			else if(roughGT(key[lastSample], selectTo)){
				lastSample--;
			}

			int numOf = lastSample - firstSample + 1;
			if(numOf > 0){
				g_animCopy.numOf += numOf;
				g_animCopy.node += node;
				g_animCopy.field += fieldIdx;
				for(int j = firstSample; j <= lastSample; j++){
					g_animCopy.key += key[j];
					g_animCopy.rot += keyValue[j];
				}
			}
			break;
		}
	}

}

void PositionInterpolator::copySection(double selectFrom, double selectTo, Node* node, int fieldIdx){
	double diff = selectTo - selectFrom;
	double s = 1.0;

	for(int k = 0; k < key.length; k++){
		if(roughGE(key[k], selectFrom)){
			int firstSample = k;
			while(roughLT(key[k], selectTo) && k < key.length - 1){
				k++;
			}
			int lastSample = k;
			if(firstSample == lastSample){
				if(roughGT(key[lastSample], selectTo)){
					break;
				}
			}
			else if(roughGT(key[lastSample], selectTo)){
				lastSample--;
			}

			int numOf = lastSample - firstSample + 1;
			if(numOf > 0){
				g_animCopy.numOf += numOf;
				g_animCopy.node += node;
				g_animCopy.field += fieldIdx;
				for(int j = firstSample; j <= lastSample; j++){
					g_animCopy.key += key[j];
					g_animCopy.pos += keyValue[j];
				}
			}
			break;
		}
	}
}

void ScalarInterpolator::copySection(double selectFrom, double selectTo, Node* node, int fieldIdx){
	double diff = selectTo - selectFrom;
	double s = 1.0;

	for(int k = 0; k < key.length; k++){
		if(roughGE(key[k], selectFrom)){
			int firstSample = k;
			while(roughLT(key[k], selectTo) && k < key.length - 1){
				k++;
			}
			int lastSample = k;
			if(firstSample == lastSample){
				if(roughGT(key[lastSample], selectTo)){
					break;
				}
			}
			else if(roughGT(key[lastSample], selectTo)){
				lastSample--;
			}

			int numOf = lastSample - firstSample + 1;
			if(numOf > 0){
				g_animCopy.numOf += numOf;
				g_animCopy.node += node;
				g_animCopy.field += fieldIdx;
				for(int j = firstSample; j <= lastSample; j++){
					g_animCopy.key += key[j];
					g_animCopy.scalar += keyValue[j];
				}
			}
			break;
		}
	}
}

void CameraSelector::copySection(double selectFrom, double selectTo, Node* node, int fieldIdx){
	double diff = selectTo - selectFrom;
	double s = 1.0;

	for(int k = 0; k < key.length; k++){
		if(roughGE(key[k], selectFrom)){
			int firstSample = k;
			while(roughLT(key[k], selectTo) && k < key.length - 1){
				k++;
			}
			int lastSample = k;
			if(firstSample == lastSample){
				if(roughGT(key[lastSample], selectTo)){
					break;
				}
			}
			else if(roughGT(key[lastSample], selectTo)){
				lastSample--;
			}

			int numOf = lastSample - firstSample + 1;
			if(numOf > 0){
				g_animCopy.numOf += numOf;
				g_animCopy.node += node;
				g_animCopy.field += fieldIdx;
				for(int j = firstSample; j <= lastSample; j++){
					g_animCopy.key += key[j];
					g_animCopy.m_node += keyValue[j];
				}
			}
			break;
		}
	}
}

void CPMorpher::copySection(double selectFrom, double selectTo, Node* node, int fieldIdx){
	double diff = selectTo - selectFrom;
	double s = 1.0;

	for(int k = 0; k < key.length; k++){
		if(roughGE(key[k], selectFrom)){
			int firstSample = k;
			while(roughLT(key[k], selectTo) && k < key.length - 1){
				k++;
			}
			int lastSample = k;
			if(firstSample == lastSample){
				if(roughGT(key[lastSample], selectTo)){
					break;
				}
			}
			else if(roughGT(key[lastSample], selectTo)){
				lastSample--;
			}

			int numOf = lastSample - firstSample + 1;
			if(numOf > 0){
				g_animCopy.numOf += numOf;
				g_animCopy.field += TimelineFilter::MORPH;
				g_animCopy.node += node;
				int sampleLen = cpIndex.length;
				g_animCopy.numOf += sampleLen;
				g_animCopy.numOf += cpIndex;
				for(int j = firstSample; j <= lastSample; j++){
					g_animCopy.key += key[j];
					for(int i = 0; i < sampleLen; i++){
						g_animCopy.pos += keyValue[j * sampleLen + i];
					}
				}
			}
			break;
		}
	}
}

void NikInterpolator::copySection(double selectFrom, double selectTo, Node* node, int fieldIdx){
	double diff = selectTo - selectFrom;
	double s = 1.0;

	for(int k = 0; k < key.length; k++){
		if(roughGE(key[k], selectFrom)){
			int firstSample = k;
			while(roughLT(key[k], selectTo) && k < key.length - 1){
				k++;
			}
			int lastSample = k;
			if(firstSample == lastSample){
				if(roughGT(key[lastSample], selectTo)){
					break;
				}
			}
			else if(roughGT(key[lastSample], selectTo)){
				lastSample--;
			}

			int numOf = lastSample - firstSample + 1;
			if(numOf > 0){
				g_animCopy.numOf += numOf;
				g_animCopy.field += TimelineFilter::NIK;
				g_animCopy.node += node;
				int sampleLen = getSampleLength();
				//g_animCopy.numOf += nurbLineLen * 2;
				//g_animCopy.numOf += cpIndex;
				for(int j = firstSample; j <= lastSample; j++){
					g_animCopy.key += key[j];
					for(int i = 0; i < sampleLen; i++){
						g_animCopy.pos += keyValue[j * sampleLen + i];
					}
				}
			}
			break;
		}
	}
}

void NurbsTrack::copySection(double selectFrom, double selectTo, Node* node, int fieldIdx){
}

void PosCompensator::copySection(double selectFrom, double selectTo, Node* node, int fieldIdx){
	double diff = selectTo - selectFrom;
	double s = 1.0;

	for(int k = 0; k < moment.length; k++){
		if(roughGE(moment[k].x, selectFrom)){
			int firstSample = k;
			while(roughLT(moment[k].x, selectTo) && k < moment.length - 1){
				k++;
			}
			int lastSample = k;
			if(firstSample == lastSample){
				if(roughGT(moment[lastSample].x, selectTo)){
					break;
				}
			}
			else if(roughGT(moment[lastSample].x, selectTo)){
				lastSample--;
			}

			int numOf = lastSample - firstSample + 1;
			if(numOf > 0){
				g_animCopy.numOf += numOf;
				g_animCopy.node += node;
				g_animCopy.field += TimelineFilter::POSCOMPENSATION;
				for(int j = firstSample; j <= lastSample; j++){
					g_animCopy.key += moment[j].x;
					g_animCopy.key += moment[j].y;
					g_animCopy.pos += relPos[j];
				}
			}
			break;
		}
	}
}

void AnimCopy::initCopySection(Anim* anim){
	tranDiff = Vec3f();
	rotDiff = Rotation();
	
	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		anim->transformInterpolators(part);
	}


	node.length = 0;
	field.length = 0;
	numOf.length = 0;
	key.length = 0;
	rot.length = 0;
	pos.length = 0;
	scalar.length = 0;

	selectFrom = anim->selectFrom;
	selectTo = anim->selectTo;
	period = anim->period;
	stepsPerSecond = anim->stepsPerSecond;
}

void Anim::copySection(){
	Seamless* smls = 0;

	g_animCopy.initCopySection(this);

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField=0;
		W_Field* w_srcField=0;
		W_Node* w_dstNode = 0; //morpher/comp
		W_Node* w_part = 0;

		for(int i = 0; i < m_codeLine.length; i++){
			if(getNodeFieldEqualsSomething(m_word, w_dstField, m_codeLine, i)){
				if(CameraSelector* cameraSelector = dynamic_cast<CameraSelector*>(w_dstField->node)){
					cameraSelector->copySection(selectFrom, selectTo, w_dstField->node, w_dstField->fieldIdx);
					continue;
				}
			}
			if(getEquiveRoute(m_word, w_dstField, w_srcField, m_codeLine, i)){
				if(SamplePose* samplePose=dynamic_cast<SamplePose*>(w_srcField->node)){
					samplePose->copySection(selectFrom, selectTo, w_dstField->node, w_dstField->fieldIdx);
				}
			}
			else if(getMorphFunction(m_word, w_dstNode, w_part, m_codeLine, i)){
				if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_dstNode->node)){
					morpher->copySection(selectFrom, selectTo, w_part->node, 0);
				}
			}
			else if(getCalcFunction(m_word, w_dstNode, w_part, m_codeLine, i)){
				if(NikInterpolator* niki = dynamic_cast<NikInterpolator*>(w_dstNode->node)){
					niki->copySection(selectFrom, selectTo, w_part->node, 0);
				}
			}
			else if(getCompFunction(m_word, w_dstNode, w_part, m_codeLine, i)){
				if(PosCompensator* compensator = dynamic_cast<PosCompensator*>(w_dstNode->node)){
					compensator->copySection(selectFrom, selectTo, w_part->node, 0);
				}
			}
		}
	}
}

void Anim::scaleTime(double add){

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField=0;
		W_Field* w_srcField=0;
		W_Node* w_part = 0;
		W_Node* w_node = 0;

		for(int i = 0; i < m_codeLine.length; i++){
			if(getEquiveRoute(m_word, w_dstField, w_srcField, m_codeLine, i)){
				if(Art* part=dynamic_cast<Art*>(w_dstField->node)){
					if(SamplePose* samplePose=dynamic_cast<SamplePose*>(w_srcField->node)){
						samplePose->scaleTime(add, period);
					}
				}
			}
			else if(getMorphFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(Art* part = dynamic_cast<Art*>(w_part->node)){
					if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_node->node)){
						morpher->scaleTime(add, period);
					}
				}
			}
			else if(getCalcFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(Art* part = dynamic_cast<Art*>(w_part->node)){
					if(NikInterpolator* niki = dynamic_cast<NikInterpolator*>(w_node->node)){
						niki->scaleTime(add, period);
					}
				}
			}
			else if(getCompFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(Art* part=dynamic_cast<Art*>(w_part->node)){
					if(PosCompensator* compensator = dynamic_cast<PosCompensator*>(w_node->node)){
						compensator->scaleTime(add, period);
					}
				}
			}
		}
	}
}


OrientationInterpolator* Anim::getOrientationInterpolator(const Node* selNode, const Field* field){
	Seamless* smls = 0;

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField=0;
		W_Field* w_srcField=0;
		W_Node* w_part = 0;

		for(int i = m_codeLine.length; --i>=0;){
			if(getEquiveRoute(m_word, w_dstField, w_srcField, m_codeLine, i)){
				if(selNode == w_dstField->node){
					if(OrientationInterpolator* samplePose = dynamic_cast<OrientationInterpolator*>(w_srcField->node)){
						if(selNode->getField(w_dstField->fieldIdx) == field){
							return samplePose;
						}
					}
				}
			}
		}
	}
	return 0;
}

PositionInterpolator* Anim::getPositionInterpolator(const Node* selNode, const Field* field){
	Seamless* smls = 0;

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField=0;
		W_Field* w_srcField=0;
		W_Node* w_part = 0;

		for(int i = m_codeLine.length; --i>=0;){
			if(getEquiveRoute(m_word, w_dstField, w_srcField, m_codeLine, i)){
				if(selNode == w_dstField->node){
					if(PositionInterpolator* samplePose = dynamic_cast<PositionInterpolator*>(w_srcField->node)){
						if(selNode->getField(w_dstField->fieldIdx) == field){
							return samplePose;
						}
					}
				}
			}
		}
	}
	return 0;
}

ScalarInterpolator* Anim::getScalarInterpolator(const Node* selNode, const Field* field){
	Seamless* smls = 0;

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField=0;
		W_Field* w_srcField=0;
		W_Node* w_part = 0;

		for(int i = m_codeLine.length; --i>=0;){
			if(getEquiveRoute(m_word, w_dstField, w_srcField, m_codeLine, i)){
				if(selNode == w_dstField->node){
					if(ScalarInterpolator* samplePose = dynamic_cast<ScalarInterpolator*>(w_srcField->node)){
						if(selNode->getField(w_dstField->fieldIdx) == field){
							return samplePose;
						}
					}
				}
			}
		}
	}
	return 0;
}

CameraSelector* Anim::getCameraSelector(const Node* selNode, const Field* field){
	Seamless* smls = 0;

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField=0;
		W_Node* w_part = 0;

		for(int i = m_codeLine.length; --i>=0;){
			if(getNodeFieldEqualsSomething(m_word, w_dstField, m_codeLine, i)){
				if(selNode == w_dstField->node){
					if(CameraSelector* samplePose = dynamic_cast<CameraSelector*>(w_dstField->node)){
						if(selNode->getField(w_dstField->fieldIdx) == field){
							return samplePose;
						}
					}
				}
			}
		}
	}
	return 0;
}


PosCompensator* Anim::getPosCompensator(const Node* selNode){
	Seamless* smls = 0;

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Node* w_node = 0;
		W_Node* w_part = 0;



		for(int i = m_codeLine.length; --i>=0;){
			if(getCompFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(PosCompensator* compensator = dynamic_cast<PosCompensator*>(w_node->node)){
					if(selNode == w_part->node){
						return compensator;
					}
				}
			}
		}
	}
	return 0;
}

NikInterpolator* Anim::getNikInterpolator(const Node* selNode){
	Seamless* smls = 0;

	if(MemberFunction* function = dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine = function->codeLine;

		M_Word& m_word = function->m_word;
		W_Node* w_node = 0;
		W_Node* w_part = 0;



		for(int i = m_codeLine.length; --i>=0;){
			if(getCalcFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(NikInterpolator* niki = dynamic_cast<NikInterpolator*>(w_node->node)){
					if(selNode == w_part->node){
						return niki;
					}
				}
			}
		}
	}
	return 0;
}

CPMorpher* Anim::getCPMorpher(const Node* selNode){
	Seamless* smls = 0;

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Node* w_node = 0;
		W_Node* w_part = 0;



		for(int i = m_codeLine.length; --i>=0;){
			if(getMorphFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_node->node)){
					if(selNode == w_part->node){
						return morpher;
					}
				}
			}
		}
	}
	return 0;
}

void Anim::resetMorphers(M_Art& m_part){
	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Node* w_morpher = 0;
		W_Node* w_part = 0;

		for(int i=m_codeLine.length; --i>=0;){
			if(getMorphFunction(m_word, w_morpher, w_part, m_codeLine, i)){
				if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_morpher->node)){
					if(Art* part = dynamic_cast<Art*>(w_part->node)){
						for(int k = m_part.length; --k >= 0;){
							if(m_part[k] == part){
								morpher->cpIndex.refValue().length = 0;
							}
						}
					}
				}
			}
		}
	}
}


void Anim::morphAllParts(){

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField = 0;
		W_Field* w_srcField = 0;
		W_Node* w_node = 0;
		W_Node* w_part = 0;

		for(int i=m_codeLine.length; --i>=0;){
			if(getMorphFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_node->node)){
					if(Art* part = dynamic_cast<Art*>(w_part->node)){
						morpher->do_frame(part);
					}
				}
			}
		}
	}
}


void Anim::plotRecordNotch(AnimTimeLine* tl, Node* par){
	M_Node& children = this->children.refValue();

	if(dynamic_cast<Viewpoint*>(par)){
		for(int k = 0; k < children.length; k++){
			if(CameraSelector* cameraSelector = dynamic_cast<CameraSelector*>(children[k].p)){
				cameraSelector->plotRecordNotch(tl, this, cameraSelector, &cameraSelector->keyValue);
			}
		}
	}

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField = 0;
		W_Field* w_srcField = 0;
		W_Node* w_node = 0;
		W_Node* w_part = 0;

		for(int i=m_codeLine.length; --i>=0;){
			if(getEquiveRoute(m_word, w_dstField, w_srcField, m_codeLine, i)){
				if(Node* node=dynamic_cast<Node*>(w_dstField->node)){
					if(node == par){
						if(SamplePose* samplePose = dynamic_cast<SamplePose*>(w_srcField->node)){
							samplePose->plotRecordNotch(tl, this, node, node->getField(w_dstField->fieldIdx));
						}
					}
				}
			}
			else if(getMorphFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_node->node)){
					if(Art* part = dynamic_cast<Art*>(w_part->node)){
						if(part == par){
							if(g_timelineFilter->morph){
								morpher->plotRecordNotch(tl, this, part, 0);
							}
						}
					}
				}
			}
			else if(getCalcFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(NikInterpolator* niki = dynamic_cast<NikInterpolator*>(w_node->node)){
					if(Art* part = dynamic_cast<Art*>(w_part->node)){
						if(part == par){
							niki->plotRecordNotch(tl, this, part, 0);
						}
					}
				}
			}
			else if(getCompFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(PosCompensator* compensator = dynamic_cast<PosCompensator*>(w_node->node)){
					if(Art* part = dynamic_cast<Art*>(w_part->node)){
						if(part == par){
							if(g_timelineFilter->posCompensation){
								compensator->plotRecordNotch(tl, this, 0, 0);
							}
						}
					}
				}
			}
		}
	}
}

bool Anim::searchKeyFrameInTimeLine(AnimControlPanel& animCP, Art* par, float clickPos){
	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField = 0;
		W_Field* w_srcField = 0;
		W_Node* w_node = 0;
		W_Node* w_part = 0;

		for(int i = m_codeLine.length; --i >= 0;){
			if(getEquiveRoute(m_word, w_dstField, w_srcField, m_codeLine, i)){
				if(Art* part=dynamic_cast<Art*>(w_dstField->node)){
					if(part == par){
						if(SamplePose* samplePose = dynamic_cast<SamplePose*>(w_srcField->node)){
							if(samplePose->searchKeyFrameInTimeLine(animCP, clickPos)){
								return true;
							}
						}
					}
				}
			}
			else if(getMorphFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_node->node)){
					if(Art* part = dynamic_cast<Art*>(w_part->node)){
						if(part == par){
							if(morpher->searchKeyFrameInTimeLine(animCP, clickPos)){
								return true;
							}
						}
					}
				}
			}
			else if(getCalcFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(NikInterpolator* niki = dynamic_cast<NikInterpolator*>(w_node->node)){
					if(Art* part = dynamic_cast<Art*>(w_part->node)){
						if(part == par){
							if(niki->searchKeyFrameInTimeLine(animCP, clickPos)){
								return true;
							}
						}
					}
				}
			}
			else if(getCompFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(PosCompensator* pc = dynamic_cast<PosCompensator*>(w_node->node)){
					if(Art* part = dynamic_cast<Art*>(w_part->node)){
						if(part == par){
							if(pc->searchKeyFrameInTimeLine(animCP, clickPos)){
								return true;
							}
						}
					}
				}
			}
		}
	}
	return false;
}

void Anim::setMorphersForTransformDrag(Seamless* smls){

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine = function->codeLine;

		M_Word& m_word = function->m_word;
		W_Node* w_morpher = 0;
		W_Node* w_part = 0;

		for(int i = m_codeLine.length; --i >= 0;){
			if(getMorphFunction(m_word, w_morpher, w_part, m_codeLine, i)){
				if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_morpher->node)){
					if(Art* part = dynamic_cast<Art*>(w_part->node)){
						if(part->smls == smls){
							morpher->morph(slider, *part);
						}
					}
				}
			}
		}
	}
}

void commonEndForSetupAnim();

bool isInMultiList(NurbsPatch* patch, int idx){
	for(int k = MultiSelect::pointList.length; --k>=0;){
		if(MultiSelect::pointList[k].p == patch && MultiSelect::pointList[k].i == idx){
			return true;
		}
	}
	return false;
}

void Anim::captureMorph2(const int k, Art* part, Matrix& imatrix, int& channel, CPMorpher*& cpMorpher, M_<CPMorpher*>& morpherList, const M_ControlPoint& m_controlPoint){
	if(part->track == false || part->show == false){
		return;
	}
	const ControlPoint& controlPoint = m_controlPoint[k];
	int idx = controlPoint.i;
	NurbsPatch* patch = controlPoint.p;


	if(isInMultiList(patch, idx)){
		Vec3f pt = imatrix * patch->dpoint[idx];
		Vec3f sp = patch->tpoint[idx] / patch->weight[idx];
		Vec3f diff = pt - sp;
		//(
		//abs(diff.x) > .0001 ||
		//abs(diff.y) > .0001 ||
		//abs(diff.z) > .0001
		
		//){
		//if(part->name == "head" && patch->name == "crownNP"){
		//	setMathHiPresision();
		//	part->smls->catMatrix();
		//	part->smls->catIMatrix();
		//	Vec3f dp = patch->dpoint[idx];
		////	Vec3f tstore = part->matrix * patch->tpoint[idx];
		//	Vec3f pt3 = part->imatrix * dp;


		//	Vec3f pt2 = part->matrix * pt3;
		//	Vec3f diff2 = pt2 - dp;
		//	int test = 0;
		//}
		diff = diff * patch->weight[idx];
		if(cpMorpher == 0){
			cpMorpher = new CPMorpher;
			morpherList[part->plidx] = cpMorpher;
			M_Float& key = cpMorpher->key.refValue();
			M_Vec3f& keyValue = cpMorpher->keyValue.refValue();
			M_Int& cpIndex = cpMorpher->cpIndex.refValue();
			String name;
			if(connectPartToCPMorpher(part, name, cpMorpher)){
				commonEndForSetupAnim();
			}
		}
		M_Float& key = cpMorpher->key.refValue();
		M_Vec3f& keyValue = cpMorpher->keyValue.refValue();
		M_Int& cpIndex = cpMorpher->cpIndex.refValue();
		
		if(cpIndex.length == 0){
			cpIndex += k;
			keyValue.length = key.length = 2;
			key[1] = 1;
		}
		int numOfChannels = keyValue.length / key.length;

		if(channel == cpIndex.length){
			cpMorpher->insertChannel(channel, k, cpIndex, slider, diff, keyValue, numOfChannels, key.length);
		}
		else{
			bool taken = false;
			for(; channel < cpIndex.length; channel++){
				int cpIdx = cpIndex[channel];
				if(cpIdx == k){
					cpMorpher->take(slider, diff, channel, numOfChannels);
					channel++;
					taken = true;
					break;
				}
				else if(cpIdx > k){
					cpMorpher->insertChannel(channel, k, cpIndex, slider, diff, keyValue, numOfChannels, key.length);
					taken = true;
					break;
				}
			}
			if(taken == false){
				cpMorpher->insertChannel(channel, k, cpIndex, slider, diff, keyValue, numOfChannels, key.length);
			}
		}
	}
	//else if(cpMorpher){//make sure 0 0 0 is inserted if not different
	//	M_Float& key = cpMorpher->key.refValue();
	//	M_Vec3f& keyValue = cpMorpher->keyValue.refValue();
	//	int numOfChannels = 0;
	//	if(key.length){
	//		numOfChannels = keyValue.length / key.length;
	//	}
	//	M_Int& cpIndex = cpMorpher->cpIndex.refValue();

	//	diff = Vec3f();
	//	for(; channel < cpIndex.length; channel++){
	//		int cpIdx = cpIndex[channel];
	//		if(cpIdx == k){
	//			cpMorpher->take(slider, diff, channel, numOfChannels);
	//			channel++;
	//			break;
	//		}
	//		else if(cpIdx > k){
	//			break;
	//		}
	//	}
	//}
}

void getRenderedCrd(const Art* part, const int idx, Vec3f& crd);

inline void addMorpherPart(CPMorpher*& morpher, Art*& part, const float& slider){
	morpher->frame = slider;
	M_<CPMorpherPart>& cpMorpherPart = part->smls->cpMorpherPart;
	for(int r = cpMorpherPart.length; --r>=0;){
		if(cpMorpherPart[r].morpher == morpher && cpMorpherPart[r].part == part)return;
	}
	cpMorpherPart += CPMorpherPart(morpher, part);
}

extern M_Vec3f g_m_coord;

void Anim::capturePolyMorph2(const int k, Art* part, Matrix& imatrix, int& channel, CPMorpher*& cpMorpher, M_<CPMorpher*>& morpherList, M_Vec3f& coord){
	Vec3f sp = coord[k];
	Vec3f pt = g_m_coord[part->fromCoord + k];
	//getRenderedCrd(part, k, pt);
	Vec3f diff = pt - sp;
	if(	
		abs(diff.x) > .01 ||
		abs(diff.y) > .01 ||
		abs(diff.z) > .01
		){
		if(cpMorpher == 0){
			cpMorpher = new CPMorpher;
			morpherList[part->plidx] = cpMorpher;
			String name;
			if(connectPartToCPMorpher(part, name, cpMorpher)){
				commonEndForSetupAnim();
			}
		}
		addMorpherPart(cpMorpher, part, slider);

		M_Float& key = cpMorpher->key.refValue();
		M_Vec3f& keyValue = cpMorpher->keyValue.refValue();
		M_Int& cpIndex = cpMorpher->cpIndex.refValue();
		
		if(cpIndex.length == 0){
			cpIndex += k;
			keyValue.length = key.length = 2;
			key[1] = 1;
		}
		int numOfChannels = keyValue.length / key.length;

		if(channel == cpIndex.length){
			cpMorpher->insertChannel(channel, k, cpIndex, slider, diff, keyValue, numOfChannels, key.length);
		}
		else{
			for(; channel < cpIndex.length; channel++){
				int cpIdx = cpIndex[channel];
				if(cpIdx == k){
					cpMorpher->take(slider, diff, channel, numOfChannels);
					channel++;
					break;
				}
				else if(cpIdx > k){
					cpMorpher->insertChannel(channel, k, cpIndex, slider, diff, keyValue, numOfChannels, key.length);
					break;
				}
			}
		}
	}
	else if(cpMorpher){//make sure 0 0 0 is inserted if not different
		M_Float& key = cpMorpher->key.refValue();
		M_Vec3f& keyValue = cpMorpher->keyValue.refValue();
		int numOfChannels = 0;
		if(key.length){
			numOfChannels = keyValue.length / key.length;
		}
		M_Int& cpIndex = cpMorpher->cpIndex.refValue();

		diff = Vec3f();
		for(; channel < cpIndex.length; channel++){
			int cpIdx = cpIndex[channel];
			if(cpIdx == k){
				cpMorpher->take(slider, diff, channel, numOfChannels);
				channel++;
				break;
			}
			else if(cpIdx > k){
				break;
			}
		}
	}
}

void Anim::captureMorphLoop(Art* part, Matrix iCatMatrix, M_<CPMorpher*>& morpherList){
	Vec3f translation = part->translation.refValue();
	Vec3f scale = part->scale.refValue();
	Vec3f absolutePivotPoint = part->absolutePivotPoint;


	M_Int& coordGroup = part->coordGroup.refValue();
	M_Float& weight = part->weight.refValue();
	CPMorpher* cpMorpher = morpherList[part->plidx];
	const M_ControlPoint& m_controlPoint = part->controlPoint;
	int channel = 0;
	int q = 0;
	for(int cg = 0; cg < coordGroup.length; cg++){
		Rotation rotation = part->rotation;
		rotation = Rotation(rotation.x, rotation.y, rotation.z, rotation.angle * weight[cg]);
		Matrix fracIMatrix = iCatMatrix * translationMatrix(-translation - absolutePivotPoint) * rotationMatrix(-rotation) * scaleMatrix(Vec3f(1./scale.x, 1./scale.y, 1./scale.z)) * translationMatrix(absolutePivotPoint);

		for(int c = coordGroup[cg]; --c >= 0 && q < m_controlPoint.length; q++){
			captureMorph2(q, part, fracIMatrix, channel, cpMorpher, morpherList, m_controlPoint);
		}
	}
	Rotation rotation = part->rotation.refValue();
	iCatMatrix = iCatMatrix * translationMatrix(-translation - absolutePivotPoint) * rotationMatrix(-rotation) * scaleMatrix(Vec3f(1./scale.x, 1./scale.y, 1./scale.z)) * translationMatrix(absolutePivotPoint);

	for(; q < m_controlPoint.length; q++){
		captureMorph2(q, part, iCatMatrix, channel, cpMorpher, morpherList, m_controlPoint);
	}

	M_SArt& m_part = part->part.refValue();
	for(int k=0; k < m_part.length; k++){
		Art* part = m_part[k];
		captureMorphLoop(part, iCatMatrix, morpherList);
	}
}

//bool coordIsInList(Art* part, int idx, Vec3f& s_beginDragPos){ //this function could be made a lot more efficient if needed
//	for(int i = MultiSelect::coordList.length; --i>=0;){
//		if(MultiSelect::coordList[i].p == part && MultiSelect::coordList[i].i == idx){
//			s_beginDragPos = MultiSelect::beginDragPos[i];
//			return true;
//		}
//	}
//	return false;
//}

void Anim::capturePolyMorphLoop(Art* part, Matrix iCatMatrix, M_<CPMorpher*>& morpherList){
	Vec3f translation = part->translation.refValue();
	Vec3f scale = part->scale.refValue();
	Vec3f absolutePivotPoint = part->absolutePivotPoint;


	M_Int& coordGroup = part->coordGroup.refValue();
	M_Float& weight = part->weight.refValue();
	CPMorpher* cpMorpher = morpherList[part->plidx];
	const M_Vec3f& coord = part->coord;
	int channel = 0;
	int q = 0;
	for(int cg = 0; cg < coordGroup.length; cg++){
		Rotation rotation = part->rotation;
		rotation = Rotation(rotation.x, rotation.y, rotation.z, rotation.angle * weight[cg]);
		Matrix fracIMatrix = iCatMatrix * translationMatrix(-translation - absolutePivotPoint) * rotationMatrix(-rotation) * scaleMatrix(Vec3f(1./scale.x, 1./scale.y, 1./scale.z)) * translationMatrix(absolutePivotPoint);


		for(int c = coordGroup[cg]; --c >= 0 && q < coord.length; q++){
			capturePolyMorph2(q, part, fracIMatrix, channel, cpMorpher, morpherList, part->coord.refValue());
		}
	}
	Rotation rotation = part->rotation.refValue();
	iCatMatrix = iCatMatrix * translationMatrix(-translation - absolutePivotPoint) * rotationMatrix(-rotation) * scaleMatrix(Vec3f(1./scale.x, 1./scale.y, 1./scale.z)) * translationMatrix(absolutePivotPoint);

	for(; q < coord.length; q++){
		capturePolyMorph2(q, part, iCatMatrix, channel, cpMorpher, morpherList, part->coord.refValue());
	}

	M_SArt& m_part = part->part.refValue();
	for(int k=0; k < m_part.length; k++){
		Art* part = m_part[k];
		capturePolyMorphLoop(part, iCatMatrix, morpherList);
	}
}

void Anim::capturePolyMorph(Seamless* smls){

	const M_Art& partList = smls->partList;
	static M_<CPMorpher*> morpherList;
	morpherList.length = 0;
	morpherList.length = partList.length;

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine = function->codeLine;

		M_Word& m_word = function->m_word;
		W_Node* w_morpher = 0;
		W_Node* w_part = 0;

		for(int i = m_codeLine.length; --i >= 0;){
			if(getMorphFunction(m_word, w_morpher, w_part, m_codeLine, i)){
				if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_morpher->node)){
					if(Art* part = dynamic_cast<Art*>(w_part->node)){
						if(part->smls == smls){
							morpherList[part->plidx] = morpher;
						}
					}
				}
			}
		}
	}
	Vec3f translation = smls->translation.refValue();
	Rotation rotation = smls->rotation.refValue();
	Vec3f scale = smls->scale.refValue();
	Vec3f absolutePivotPoint;

	Matrix iCatMatrix = translationMatrix(-translation - absolutePivotPoint) * rotationMatrix( -rotation) * scaleMatrix(Vec3f(1./scale.x, 1./scale.y, 1./scale.z)) * translationMatrix(absolutePivotPoint);

	M_SArt& m_part = smls->skeleton.refValue();
	for(int k = 0; k < m_part.length; k++){
		Art* part = m_part[k];
		capturePolyMorphLoop(part, iCatMatrix, morpherList);
	}
	g_dragActuallyHappened = false;
}

void renderScene();

void Anim::deleteMorpherChannel(){

	Seamless* smls = 0;
	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine = function->codeLine;

		M_Word& m_word = function->m_word;
		W_Node* w_morpher = 0;
		W_Node* w_part = 0;

		for(int i = m_codeLine.length; --i >= 0;){
			if(getMorphFunction(m_word, w_morpher, w_part, m_codeLine, i)){
				if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_morpher->node)){
					if(Art* part = dynamic_cast<Art*>(w_part->node)){
						smls = part->smls;
						if(part->smls->animCP){
							morpher->deleteChannelPoints(part);
						}
						else{
							morpher->deleteChannelCoords(part);
						}
					}
				}
			}
		}
	}
	MultiSelect::pointList.length = 0;
	MultiSelect::coordList.length = 0;
	if(smls){
		smls->updateAllVertexInBuffers();
	}
	renderScene();
	if(play == false){
		update = true;
	}

}

void Anim::captureMorph(Seamless* smls){
	smls->catIMatrix();
	smls->catMatrix();

	const M_Art& partList = smls->partList;
	static M_<CPMorpher*> morpherList;
	morpherList.length = 0;
	morpherList.length = partList.length;

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine = function->codeLine;

		M_Word& m_word = function->m_word;
		W_Node* w_morpher = 0;
		W_Node* w_part = 0;

		for(int i = m_codeLine.length; --i >= 0;){
			if(getMorphFunction(m_word, w_morpher, w_part, m_codeLine, i)){
				if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_morpher->node)){
					if(Art* part = dynamic_cast<Art*>(w_part->node)){
						if(part->smls == smls){
							morpherList[part->plidx] = morpher;
						}
					}
				}
			}
		}
	}
	Vec3f translation = smls->translation.refValue();
	Rotation rotation = smls->rotation.refValue();
	Vec3f scale = smls->scale.refValue();
	Vec3f absolutePivotPoint;

	Matrix iCatMatrix = translationMatrix(-translation - absolutePivotPoint) * rotationMatrix( -rotation) * scaleMatrix(Vec3f(1./scale.x, 1./scale.y, 1./scale.z)) * translationMatrix(absolutePivotPoint);

	M_SArt& m_part = smls->skeleton.refValue();
	for(int k = 0; k < m_part.length; k++){
		Art* part = m_part[k];
		captureMorphLoop(part, iCatMatrix, morpherList);
	}
}
/*
void Anim::captureNiki(Seamless* smls){
	smls->catIMatrix();
	smls->catMatrix();

	const M_Art& partList = smls->partList;
	static M_<NikInterpolator*> nikiList;
	nikiList.length = 0;
	nikiList.length = partList.length;

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine = function->codeLine;

		M_Word& m_word = function->m_word;
		W_Node* w_niki = 0;
		W_Node* w_part = 0;

		for(int i = m_codeLine.length; --i >= 0;){
			if(getCalcFunction(m_word, w_niki, w_part, m_codeLine, i)){
				if(NikInterpolator* niki = dynamic_cast<NikInterpolator*>(w_niki->node)){
					if(Art* part = dynamic_cast<Art*>(w_part->node)){
						if(part->smls == smls){
							nikiList[part->plidx] = niki;
						}
					}
				}
			}
		}
	}
	Vec3f translation = smls->translation.refValue();
	Rotation rotation = smls->rotation.refValue();
	Vec3f scale = smls->scale.refValue();
	Vec3f absolutePivotPoint;

	Matrix iCatMatrix = translationMatrix(-translation - absolutePivotPoint) * rotationMatrix( -rotation) * scaleMatrix(Vec3f(1./scale.x, 1./scale.y, 1./scale.z)) * translationMatrix(absolutePivotPoint);

	M_SArt& m_part = smls->skeleton.refValue();
	for(int k = 0; k < m_part.length; k++){
		Art* part = m_part[k];
		captureMorphLoop(part, iCatMatrix, nikiList);
	}
}
*/

void Anim::captureCameraLive(Viewpoint* view){
	if(view->live){
		createUniqueName(view);
	}

	bool flag = false;

	M_Node& children = this->children.refValue();

	for(int k = 0; k < children.length; k++){
		if(CameraSelector* cameraSelector = dynamic_cast<CameraSelector*>(children[k].p)){
			cameraSelector->take(slider, view, &view->live);
			flag = true;
			break;
		}
	}
	if(flag == false){
		if(CameraSelectorPtr cameraSelector = connectNodeToCameraSelector(view, &view->live, _LIVE_)){
			cameraSelector->take(slider, view, &view->live);
		}
	}
}

void Anim::captureNikiPose(NikInterpolator* niki) {
	M_Vec3f& longPoint = niki->longPoint;
	M_Vec3f& shortPoint = niki->shortPoint;
	for (int k = longPoint.length; --k >= 0;) {
		niki->take(slider, longPoint[k], k, longPoint.length * 2, 0);
		niki->take(slider, shortPoint[k], k, longPoint.length * 2, 1);
	}
}

void Anim::captureSinglePartPose(Art* par, Field* field){
	bool flag = false;
	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine = function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField = 0;
		W_Field* w_srcField = 0;

		for(int i = m_codeLine.length; --i >= 0;){
			if(getEquiveRoute(m_word, w_dstField, w_srcField, m_codeLine, i)){
				if(Art* part=dynamic_cast<Art*>(w_dstField->node)){
					if(field == part->getField(w_dstField->fieldIdx)){
						if(SamplePose* samplePose=dynamic_cast<SamplePose*>(w_srcField->node)){
							samplePose->take(slider, part, field);
							flag = true;
						}
					}
				}
			}
		}
		//if(flag == false){
		//	if(field == &par->rotation || field == &par->ikRot){
		//		if(OrientationInterpolatorPtr orientationInterpolator = connectNodeToOrientationInterpolator(par, field)){
		//			orientationInterpolator->take(slider, par);
		//		}
		//	}
		//	else if(field == &par->translation || field == &par->ikPos){
		//		if(PositionInterpolatorPtr positionInterpolator = connectNodeToPositionInterpolator(par, field)){
		//			positionInterpolator->take(slider, par);
		//		}
		//	}
		//	else if(field == &par->fka || field == &par->fkb){
		//		connectNodeToScalarInterpolator(par, field);
		//	}
		//}
	}
	if(flag == false){
		if(field == &par->rotation || field == &par->ikRot){
			if(OrientationInterpolatorPtr orientationInterpolator = connectNodeToOrientationInterpolator(par, field, _ROTATION_)){
				orientationInterpolator->take(slider, par, field);
			}
		}
		else if(field == &par->translation || field == &par->ikPos){
			if(PositionInterpolatorPtr positionInterpolator = connectNodeToPositionInterpolator(par, field, _TRANSLATION_)){
				positionInterpolator->take(slider, par, field);
			}
		}
		else if(field == &par->fka){
			if(ScalarInterpolatorPtr scalarInterpolator = connectNodeToScalarInterpolator(par, field, _FA_)){
				scalarInterpolator->take(slider, par, field);
			}
		}
		else if(field == &par->fkb){
			if(ScalarInterpolatorPtr scalarInterpolator = connectNodeToScalarInterpolator(par, field, _FB_)){
				scalarInterpolator->take(slider, par, field);
			}
		}
	}
}

void Anim::removeCompensation(Art* part){
	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine = function->codeLine;
		M_Word& m_word = function->m_word;
		W_Node* w_compensator = 0;
		W_Node* w_part = 0;
		for(int i = m_codeLine.length; --i >= 0;){
			if(getCompFunction(m_word, w_compensator, w_part, m_codeLine, i)){
				if(part == w_part->node){
					if(PosCompensator* posCompensator = dynamic_cast<PosCompensator*>(w_compensator->node)){
						M_Vec2f& moment = posCompensator->moment.refValue();
						for(int k = moment.length; --k >= 0;){
							if(moment[k].x <= slider && moment[k].y >= slider){
								moment.deleteElement(k);
								posCompensator->relPos.refValue().deleteElement(k);
								return;
							}
						}
					}
				}
			}
		}
	}
}

void Anim::captureViewpoint(Viewpoint* selView){
	ScalarInterpolator* fieldOfViewNode = 0;
	ScalarInterpolator* zoomNode = 0;
	OrientationInterpolator* rotationNode = 0;
	PositionInterpolator* positionNode = 0;
	PositionInterpolator* lookAtNode = 0;
	PositionInterpolator* upNode = 0;
	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine = function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField = 0;
		W_Field* w_srcField = 0;

		for(int i = m_codeLine.length; --i >= 0;){
			if(getEquiveRoute(m_word, w_dstField, w_srcField, m_codeLine, i)){
				if(selView == w_dstField->node){
					if(Field* field = selView->getField(w_dstField->fieldIdx)){
						if(SamplePose* samplePose=dynamic_cast<SamplePose*>(w_srcField->node)){
							if(field == &selView->zoom){
								zoomNode = dynamic_cast<ScalarInterpolator*>(samplePose);
							}
							else if(field == &selView->fieldOfView){
								fieldOfViewNode = dynamic_cast<ScalarInterpolator*>(samplePose);
							}
							else if(field == &selView->orientation && selView->aim == false){
								rotationNode = dynamic_cast<OrientationInterpolator*>(samplePose);
							}
							else if(field == &selView->lookAt && selView->aim){
								lookAtNode = dynamic_cast<PositionInterpolator*>(samplePose);
							}
							else if(field == &selView->up && selView->aim){
								upNode = dynamic_cast<PositionInterpolator*>(samplePose);
							}
							else if(field == &selView->position){
								positionNode = dynamic_cast<PositionInterpolator*>(samplePose);
							}
							samplePose->take(slider, selView, field);
						}
					}
				}
			}
		}
	}
	if(zoomNode == 0){
		zoomNode = connectNodeToScalarInterpolator(selView, &selView->zoom, _ZOOM_);
		M_Float& key = zoomNode->key.refValue();
		M_Float& keyValue = zoomNode->keyValue.refValue();
		keyValue += selView->zoom;
		keyValue += selView->zoom;
		key.length = 2;
		key[1] = 1;
		zoomNode->take(slider, selView, &selView->zoom);
	}
	if(fieldOfViewNode == 0){
		fieldOfViewNode = connectNodeToScalarInterpolator(selView, &selView->fieldOfView, _FOV_);
		M_Float& key = fieldOfViewNode->key.refValue();
		M_Float& keyValue = fieldOfViewNode->keyValue.refValue();
		if(key.length == 0){
			keyValue += selView->fieldOfView;
			keyValue += selView->fieldOfView;
			key.length = 2;
			key[1] = 1;
		}
		fieldOfViewNode->take(slider, selView, &selView->fieldOfView);
	}
	if(rotationNode == 0 && selView->aim == false){
		rotationNode = connectNodeToOrientationInterpolator(selView, &selView->orientation, _ROTATION_);
		M_Float& key = rotationNode->key.refValue();
		M_Rotation& keyValue = rotationNode->keyValue.refValue();
		keyValue += selView->orientation;
		keyValue += selView->orientation;
		key.length = 2;
		key[1] = 1;
		rotationNode->take(slider, selView, &selView->orientation);
	}
	if(lookAtNode == 0 && selView->aim){
		lookAtNode = connectNodeToPositionInterpolator(selView, &selView->lookAt, _LOOK_AT_);
		M_Float& key = lookAtNode->key.refValue();
		M_Vec3f& keyValue = lookAtNode->keyValue.refValue();
		keyValue += selView->lookAt;
		keyValue += selView->lookAt;
		key.length = 2;
		key[1] = 1;
		lookAtNode->take(slider, selView, &selView->lookAt);
	}
	if(upNode == 0 && selView->aim){
		upNode = connectNodeToPositionInterpolator(selView, &selView->up, _UP_);
		M_Float& key = upNode->key.refValue();
		M_Vec3f& keyValue = upNode->keyValue.refValue();
		keyValue += selView->up;
		keyValue += selView->up;
		key.length = 2;
		key[1] = 1;
		upNode->take(slider, selView, &selView->up);
	}
	if(positionNode == 0){
		positionNode = connectNodeToPositionInterpolator(selView, &selView->position, _POSITION_);
		M_Float& key = positionNode->key.refValue();
		M_Vec3f& keyValue = positionNode->keyValue.refValue();
		keyValue += selView->position;
		keyValue += selView->position;
		key.length = 2;
		key[1] = 1;
		positionNode->take(slider, selView, &selView->position);
	}
}



void copySourceCP(M_<NurbsPatch*>& cpNodeList);
void copyDestToSourceCP(M_<NurbsPatch*>& cpNodeList){
	for(int k = 0; k < cpNodeList.length; k++){
		NurbsPatch* patch = cpNodeList[k];
		patch->tpoint = patch->point;
	}
}

void Anim::compileMorph(Seamless* smls){
	setMathHiPresision();
	copySourceCP(smls->cpNodeList);

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine = function->codeLine;

		M_Word& m_word = function->m_word;
		W_Node* w_morpher = 0;
		W_Node* w_part = 0;

		for(int i = m_codeLine.length; --i >= 0;){
			if(getMorphFunction(m_word, w_morpher, w_part, m_codeLine, i)){
				if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_morpher->node)){
					if(Art* part = dynamic_cast<Art*>(w_part->node)){
						if(part->smls == smls){
							morpher->frame = slider;
							morpher->do_frame(part);
							morpher->deleteSample(slider);
						}
					}
				}
			}
		}
	}
	copyDestToSourceCP(smls->cpNodeList); //comment out all above if u want transforms compiled too
}

//// stuff from shared stuff

#pragma warning( disable : 4731)
extern bool mouse3dMode;
extern M_String g_shared_touchSensors;

void houseKeepVisualClue();
extern Ptr<AnimLever> g_vcArtAnimBase;
extern Ptr<SceneTree> g_stw;

void Anim::renderNodeFirst(){
	setMathHiPresision();

	if(play || update){
		if(update == false){
			Video* video = dynamic_cast<Video*>(this->video.p);
				
			if(video && video->audioTimeMode){
			}
			else{
				if(loop.getValue() == false && g_time >= startTime + period / speed){
					play.refValue() = false;
					slider.refValue() = 1;
					//if(Composition* composition = dynamic_cast<Composition*>(soundTrack.p)){
					//	togglePlaySound(false, composition->ibuf, composition->bufLen);
					//}
					updateDisplayFrom();
					houseKeepVisualClue();

				}//else otherwise same as loop
				else{
					slider.refValue() = fmod((g_time - startTime) * speed, period) / period;
				}
			}
		}
		Function* function=dynamic_cast<Function*>(onFrame.p);
		if(function){
			unsigned char* func=function->startCodePtr;
			float v=slider;
			unsigned localSize=function->localSize;
			
			__asm{
				push ebp
				push ebx
				mov esi,esp
				sub esp,0x400
				sub esi,0x300
				mov edi,func
				push v
				sub esp,localSize
				call edi
				add esp,0x400 + 4;
				add esp,localSize;
				pop ebx
				pop ebp
			}
		}
//		if(ControlPanel* controlPanel = dynamic_cast<ControlPanel*>(Window::activeWindow.getActiveWindow())){
//			if(dynamic_cast<Anim*>(controlPanel->node.p)){
////				controlPanel->paint();
//		}


		if(play == false){
			update = false;

			if(g_vcArtAnimBase->anim == this){
				g_vcArtAnimBase->resetRotationLever();
				g_vcArtAnimBase->showLever = true;
				g_stw->cp->paint();
			}
		}
	}
}
void TimeSensor::renderNode(Matrix parentMatrix, Vec3f parentColScale){
	setMathHiPresision();
	//||(loop.getValue()==false&&g_time>startTime+cycleInterval)
	if(enabled.getValue()==false)return;
	if(loop.getValue()==false && g_time>=startTime+cycleInterval){
		if(fraction_changed==1)return;
		fraction_changed.refValue()=1;
		fraction_changed.sendOutRoute(1);
	}//else otherwise same as loop
	else{
		fraction_changed.sendOutRoute(fmod(g_time-startTime,cycleInterval)/cycleInterval);
	}
	Function* function=dynamic_cast<Function*>(on_fraction_changed.p);
	if(function){
		unsigned char* func=function->startCodePtr;
		float v=fraction_changed;
		unsigned localSize=function->localSize;
		
		__asm{
			push ebp
			push ebx
			mov esi,esp
			sub esp,0x400
			sub esi,0x300
			mov edi,func
			push v
			push dword ptr [g_time + 4]
			push dword ptr [g_time]
			sub esp,localSize
			call edi
			add esp,0x400 + 4 + 8;
			add esp,localSize;
			pop ebx
			pop ebp
		}
	}
}

inline bool checkSharedTouchSensor(const String& name){
	for(int i = g_shared_touchSensors.length; --i>=0;){
		if(g_shared_touchSensors[i] == name){
			return true;
		}
	}
	return false;
}

void TouchSensor::renderNode(Matrix parentMatrix, Vec3f parentColScale){
	setMathHiPresision();
	//||(loop.getValue()==false&&g_time>startTime+cycleInterval)
	if((mouse3dMode == false && hit == true) || checkSharedTouchSensor(name)){
		hit=false;
		Function* function=dynamic_cast<Function*>(onTouchTime.p);
		if(function){
			unsigned char* func=function->startCodePtr;
			unsigned localSize=function->localSize;
			
			__asm{
//				int 3
				push ebp
				push ebx
				mov esi,esp
				sub esp,0x400
				sub esi,0x300
				mov edi,func
				push dword ptr [g_time + 4]
				push dword ptr [g_time]
				push dword ptr [g_time + 4]
				push dword ptr [g_time]
				sub esp,localSize
				call edi
				add esp,0x400 + 8 + 8;
				add esp,localSize;
				pop ebx
				pop ebp
			}
		}
	}
}
void initialize();
//typedef void (Func)();
void initializeScript(){
	setMathHiPresision();
	int len=g_scene->sceneTreeList.length;
	for(int k=0; k<len; k++){
		Function* function=dynamic_cast<Function*>(g_scene->sceneTreeList[k].node.p);
		if(function && function->functionName=="initialize"){
			//function->doFunction();
			unsigned char* func=function->startCodePtr;
			__asm{
				push ebp
				push ebx
				mov esi,esp
				sub esp,0x400
				sub esi,0x300
				mov edi,func
				call edi
				add esp,0x400
				pop ebx
				pop ebp
			}
		}
	}
//	initialize();
}

void Build::doSetup(){
	if(Function* function=dynamic_cast<Function*>(setup.p)){
		unsigned char* func=function->startCodePtr;
		if(func==0){
			return;
		}
		__asm{
			push ebp
			push ebx
			mov esi,esp
			sub esp,0x400
			sub esi,0x300
			mov edi,func
			call edi
			add esp,0x400
			pop ebx
			pop ebp
		}
	}
}
